/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#ifndef _ArchMgrRun_h
#define _ArchMgrRun_h

///////////////////////////////////////////////////////////////////////////////
// OpenSTA includes.
#include <OSTAORB/OSTAORB.h>
#include <ArchMgr_i.h>

///////////////////////////////////////////////////////////////////////////////
// @class   This class contains all functionality required to implement a
//          Cyrano daemon process.  This includes logging facilities as well as
//          automatic library loading, etc.
//
//          This class only throws exceptions of type <c CyrDaemonException>.
//          All other exception types are caught within this class.
class ArchMgrRun
{
// @access Public Members
public:
    ArchMgrRun(CORBAServer* pServer, HANDLE hOSTADaemon, DWORD dwOSTADaemon,
        CString strOSTAHome);       // @cmember Constructor
    virtual ~ArchMgrRun();          // @cmember Destructor

    void Run();                     // @cmember Start ArchMgr Daemon execution
    void StartArchMgr();            // @cmember Start ArchMgr CORBA servant

    // @cmember,mfunc Returns true if ArchMgr Daemon is running.
    bool IsRunning() const {return m_bRunning;};

// @access Private Members
private:;
    string       m_strArchMgrNam;   // @cmember The name of the ArchMgr CORBA object.
    ArchMgrImpl* m_pArchMgr;        // @cmember Pointer to ArchMgr CORBA object.
    CORBAServer* m_pCorba;          // @cmember Pointer to CORBA server.
    bool         m_bRunning;        // @cmember True if the ArchMgr daemon is running.
    HANDLE       m_hOSTADaemon;     // @cmember HANDLE to OpenSTA Daemon process
    DWORD        m_dwOSTADaemonPid; // @cmember OpenSTA Daemon process PID
    CString      m_strOSTAHome;     // @cmember OPENSTA_HOME path
    BOOL         m_bTrace;          // @cmember Trace flag
};

#endif