/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#ifndef _ARCHMGR_I_H_
#define _ARCHMGR_I_H_

///////////////////////////////////////////////////////////////////////////////
// OpenSTA includes.
#include "ArchMgr.h"
#include <OSTAORB/OSTAORB.h>

///////////////////////////////////////////////////////////////////////////////
// Standard includes.
#include <string>

class ArchMgrImpl : public POA_ArchMgr, public PortableServer::RefCountServantBase
{
public:
    ArchMgrImpl(CORBAServer* pCorba, HANDLE hOSTADaemon);
    virtual ~ArchMgrImpl();

    // Architecture Manager Daemon methods:
    virtual char*    FileHostProperties(); // @cmember Write host properties to file
    virtual CORBA::Boolean    StartOSTADaemon();    // @cmember Start OpenSTA Dameon
    virtual CORBA::Boolean    StopOSTADaemon();     // @cmember Stop OpenSTA Daemon
    virtual CORBA::Boolean    RestartOSTADaemon();  // @cmember Restart OpenSTA Daemon
    virtual CORBA::Boolean    Stop(CORBA::Boolean bCleanNames); // @cmember Stop Architecture Manager and OpenSTA Daemons


    // Helpers.
    void                      Deactivate();

private:
    BOOL                      GetOSInfo(char *pOSName, char *pOSVersion);
    void                      GetSystemData(char *pArchitectureName, char *pNumberOfProcessors);
    void                      CleanNS();
    void                      CleanNSChunk(const CosNaming::BindingList &bl,
                                const CosNaming::NamingContext_var context);

    bool                      m_bOpen;      // true if ArchMgrDmn is open for incoming calls
    PortableServer::ObjectId* m_pObjid;
    CORBAServer*              m_pCorba;
    std::string               m_strRegisterName;
    HANDLE                    m_hOSTADaemon;    // @cmember HANDLE to OpenSTA Daemon process

    // Forbidden.
    ArchMgrImpl();
    ArchMgrImpl(const ArchMgrImpl&);
    operator=(const ArchMgrImpl&);
};

#endif  // _ARCHMGR_I_H_