/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     VDK_MODULE
// @module  Injector Service VDK | This module provides services for
//          controlling the execution of Task Groups on an Injector and
//          for obtaining status information for Executers running on an
//          Injector.
//
//          VDKs are loaded by the OpenSTA Daemon when it starts up.
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// OpenSTA includes.
#include <Daemon/CyrVdk.h>
#include "InjectorControl_i.h"
#include "HTTPInjectorStatus_i.h"
#include "StepByStep_Control_i.h"

///////////////////////////////////////////////////////////////////////////////
// Global variables
//
// @globalv InjectorControlImpl* | InjectorControl | Pointer to Injector
//          Control object.
InjectorControlImpl* InjectorControl = NULL;

// @globalv HTTPInjectorStatusImpl* | InjectorStatus | Pointer to InjectorStatus object.
// 
         
HTTPInjectorStatusImpl* InjectorStatus = NULL;
//InjectorStatusImpl* InjectorStatus = NULL;

// @globalv StepByStepControlImpl* | stepByStepControl | Pointer to step-by-step control
//          object.
StepByStepControlImpl* pStepByStepControl = NULL;

///////////////////////////////////////////////////////////////////////////////
// @func    This function initialises the OpenSTA Injector Service VDK.
//
// @rdesc   A standard OpenSTA error code.
//
// @parm    The name of the parent Factory that all Providers (allocated using
//          the Factory mechanism) within this DLL will register themselves
//          with.
// @parm    Pointer to CORBA server object.
//
VDK_EXP CyrVdkInitialise( const char* factory, CORBAServer* pServer )
{
#ifndef _WIN32
    CyrtoolsInitialize();
#endif
    // Create the InjectorControl object for this Injector
    InjectorControl = new InjectorControlImpl(pServer);

    // Create the HTTPInjectorStatus object for this Injector.
    InjectorStatus = new HTTPInjectorStatusImpl(pServer, *InjectorControl);
	//InjectorStatus = new InjectorStatusImpl(pServer, *InjectorControl);

    // Create the StepByStepControl object for this Injector
    pStepByStepControl = new StepByStepControlImpl(pServer, *InjectorControl);

    return (InjectorControl && InjectorStatus && pStepByStepControl ? CYR_SUCCESS
        : CYR_UNKNOWN_ERROR);
}

///////////////////////////////////////////////////////////////////////////////
// @func    This function stops the Injector Service VDK. It deletes the
//          InjectorControl and InjectorStaus objects.
//
// @rdesc   A standard OpenSTA error code.
//
VDK_EXP CyrVdkUninitialise()
{
    delete InjectorStatus;
    delete pStepByStepControl;
    delete InjectorControl;
#ifndef _WIN32
    CyrtoolsUninitialize();
#endif
    return CYR_SUCCESS;
}

#ifdef _WIN32
extern "C"
BOOL WINAPI DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID /*lpReserved*/)
{
    if (dwReason == DLL_PROCESS_ATTACH)
    {
        CyrtoolsInitialize();
    }
    else if (dwReason == DLL_PROCESS_DETACH)
    {
        CyrtoolsUninitialize();
    }
    return TRUE;
}
#endif
