/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include "HTTPInjectorStatus_i.h"


using namespace std;

const char* strNoValidHTTPExecuter = "NO_HTTP_EXECUTER_RUNNING"; 

///////////////////////////////////////////////////////////////////////////////
// @mfunc   Additional code extends InjectorStatusImpl to have additional Http stats
//
// @parm    Pointer to CORBA server object.
// @parm    Injector Control object reference.
//
HTTPInjectorStatusImpl::HTTPInjectorStatusImpl(CORBAServer* corba, InjectorControlImpl& ic)
    : InjectorStatusImpl(corba, ic)
{
	//remove the base IS from the name sever
	WriteLogMsg(TRCLEV_LOW,string("HTTPInjectorStatusImpl::HTTPInjectorStatusImpl - Remove InjectorStatusImpl IS from name server"));
	m_corba->UnNameObject(InjectorStatus::CLASS_NAME, m_registered_name.c_str());
	
	//add the new IS to the nameserver
	WriteLogMsg(TRCLEV_LOW,string("HTTPInjectorStatusImpl::HTTPInjectorStatusImpl - Register IS with name " + m_registered_name));
    m_corba->NameObject(this->_this(), InjectorStatus::CLASS_NAME, m_registered_name.c_str());
    WriteLogMsg(TRCLEV_LOW,string("HTTPInjectorStatusImpl::HTTPInjectorStatusImpl - Register IS with name complete"));

	m_bStopMonitoring=true;
	m_lInterval=-1;
	m_bCachedInjectorSnapList=false;
	m_bCachedHTTP_InjectorSnapshot=false;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   Destructor. 
//
HTTPInjectorStatusImpl::~HTTPInjectorStatusImpl()
{
	//if its the same name as injector status
	//then let the base class deregister it
}
///////////////////////////////////////////////////////////////////////////////
// @mfunc   Start Monitoring.Start saving to file, and poll after interval lPace
//          
//
// @parm    file name
// @parm    interval
//

//////////////////////////////////////////////////////////////////////////////

CORBA::Boolean HTTPInjectorStatusImpl::StartMonitoring(const char* strFileName, long lPace)
{
	try
	{
		m_lInterval=lPace; 
		
		m_bStopMonitoring=false;
		m_statsFileName=strFileName;
		m_pStatWriter = new omni_thread(&WriteToFile,this);
		m_pStatWriter->start();
		return true;
	}
	catch(...)
	{

	}
	return false;

}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   Stop Monitoring.stop saving to file, and stop polling after interval lPace
//          
//
//////////////////////////////////////////////////////////////////////////////

CORBA::Boolean HTTPInjectorStatusImpl::StopMonitoring()
{
	m_lInterval=-1;
	m_bStopMonitoring=true;
	return true;
}

HTTP_TestExecuterSnapshot* HTTPInjectorStatusImpl::HTTP_GetStatsSnapshot()
{
    HTTP_TestExecuterSnapshot* pResult;

	// Check if its cached, if it is return the latest value.
	if(m_bCachedHTTP_InjectorSnapshot)
	{
        // Must allocate pResult, since it's a return value.
	CTLockGuard<CTMutex>	guard(m_CacheInUseMutex);
        pResult = new HTTP_TestExecuterSnapshot(m_cachedHTTP_InjectorSnapshot);
	}
    else
    {
        // The HTTP_GetStatsSnapshot2, to which we delegate the call, will take care of the
        // allocation of pResult.
	    pResult = HTTP_GetStatsSnapshot2();
    }

	return pResult;
}

HTTP_TestExecuterSnapshot* HTTPInjectorStatusImpl::HTTP_GetStatsSnapshot2()
{
    HTTP_TestExecuterSnapshot* pResult = NULL;

	//the actual direct call
    CTLockGuard<CTMutex> lock(m_ic.m_TEMutex);

    try
	{
        for (InjectorControlImpl::TestExecuters_t::iterator ppTE = m_ic.m_TEs.begin();ppTE != m_ic.m_TEs.end();++ppTE)
        {
			if ((*ppTE)->m_tgtType == tgt_script)
			{
				//only one http executer per host
				HTTP_TestExecuter_var pHTE(HTTP_TestExecuter::_narrow((*ppTE)->m_pTE));
	            if(!CORBA::is_nil(pHTE))
				{
                    // Allocation of the actual data was done by the client runtime.
                    // It can be reliably used to initialize pResult.
					pResult = pHTE->HTTP_GetStatsSnapshot();
                    break;
				}
			}
		}
    }
    catch(...)
	{
        string strmsg ="Exception from: HTTPInjectorStatusImpl::HTTP_GetStatsSnapshot";
        LogError (TRCLEV_NONE,strmsg);
    }

	return pResult;
}

CORBA::Boolean HTTPInjectorStatusImpl::HTTP_GetBothStatsSnapshots(HTTP_TestExecuterSnapshot_out HTTP_InjectorSnapshot, TESnapList_out InjectorSnapList)
{	
	//check if its cached, if it is return the latest value
	bool one=false;
	bool two=false;

	//check if its cached, if it is return the latest value
	if(m_bCachedInjectorSnapList)
	{
        // Must allocate HTTP_InjectorSnapshot, since it's an out argument.
		CTLockGuard<CTMutex> guard(m_CacheInUseMutex);
		HTTP_InjectorSnapshot = new HTTP_TestExecuterSnapshot(m_cachedHTTP_InjectorSnapshot);
		one=true;
	}
	if(m_bCachedHTTP_InjectorSnapshot)
	{
        // Must allocate InjectorSnapList, since it's an out argument.
		CTLockGuard<CTMutex> guard(m_CacheInUseMutex);
		InjectorSnapList = new TESnapList(m_cachedInjectorSnapList);
		two=true;
	}
	if(one&&two)
		return true;

    // HTTP_GetBothStatsSnapshots2, to which we delegate the work, will take care
    // of allocation of HTTP_InjectorSnapshot and InjectorSnapList.
	return HTTP_GetBothStatsSnapshots2(HTTP_InjectorSnapshot, InjectorSnapList);
}

CORBA::Boolean HTTPInjectorStatusImpl::HTTP_GetBothStatsSnapshots2(HTTP_TestExecuterSnapshot_out HTTP_InjectorSnapshot, TESnapList_out InjectorSnapList)
{
	//the actual direct call
	CTLockGuard<CTMutex> lock(m_ic.m_TEMutex);
	string tenam;
	int nHTTPExecuters = 0;

    try
	{
        // Allocate the standard snapshots list.
        InjectorSnapList = new TESnapList;

        // Get the status snapshot for each Executer on the local Injector
        // and append it to the list of snapshots for the Injector.
        for(InjectorControlImpl::TestExecuters_t::iterator ppTE = m_ic.m_TEs.begin(); ppTE != m_ic.m_TEs.end(); ++ppTE)
		{
            bool bHTTP = false;
            tenam = (*ppTE)->m_strRegName;

			if((*ppTE)->m_tgtType == tgt_script)
			{
				//only one http executer per host
				HTTP_TestExecuter_var pHTE(HTTP_TestExecuter::_narrow((*ppTE)->m_pTE));
	            if(!CORBA::is_nil(pHTE))
				{
					HTTP_InjectorSnapshot = pHTE->HTTP_GetStatsSnapshot();
					string strmsg = "HTTPInjectorStatusImpl::HTTP_GetBothStatsSnapshot: HTTPStats - Got Stats!'"
					    + tenam + "'";
					WriteLogMsg(TRCLEV_MEDIUM,strmsg);
                    bHTTP = true;
					nHTTPExecuters ++;
				}
			}



            if(!bHTTP)
            {
                // This TE is not an HTTP TE.
                CORBA::ULong uLength = InjectorSnapList->length() + 1;
                InjectorSnapList->length(uLength);
			    if(!(*ppTE)->m_pTE->GetStatsSnapshot((*InjectorSnapList)[uLength-1]))
                {
                    string strmsg = "InjectorStatusImpl::GetStatsSnapshot: Failed to get snapshot from TG Executer '"
                        + tenam + "'";
                    LogError (TRCLEV_LOW,strmsg);
                    InjectorSnapList->length(uLength-1);
                }
            }
		}

		if(nHTTPExecuters <1)
		{
			//there is no valid HTTP_InjectorSnapshot!
			//this http snapshot is empty with executer name "NO_HTTP_EXECUTER_RUNNING"
			HTTP_InjectorSnapshot = new HTTP_TestExecuterSnapshot();
			HTTP_InjectorSnapshot->strTestExecuter = strNoValidHTTPExecuter;
		}
	}
	catch(...)
	{
        string strmsg = "HTTPInjectorStatusImpl::HTTP_GetBothStatsSnapshot: Exception getting both snapshots from TG Executer '"
                + tenam + "'; Could be TG stopped or completed";
        LogError (TRCLEV_NONE, strmsg);
		return false;
	}

	return true;
}

void* HTTPInjectorStatusImpl::WriteToFile(void* args)
{

	HTTPInjectorStatusImpl* pIS = reinterpret_cast<HTTPInjectorStatusImpl*>(args);

	long intervalBetweenCalls=pIS->m_lInterval;

	if(intervalBetweenCalls<=0)
	{
		pIS->WriteLogMsg(TRCLEV_MEDIUM,"HTTPInjectorStatusImpl::WriteToFile:Sample rate is <=0 so not sampling");
		return NULL;
	}

	pIS->WriteLogMsg(TRCLEV_MEDIUM,"HTTPInjectorStatusImpl::WriteToFile:Sample rate is >0 so sampling");

	//when this function is called get the latest snapshots and write them to the file with the id also
	//these are var's so that the out function in http executer can deallocate memory?

	HTTP_TestExecuterSnapshot_var HTTP_InjectorSnapshot;
	TESnapList_var InjectorSnapList;
	

	// put the file in the local engines directory
	char * pOpenStaHome = getenv("OPENSTA_HOME") ;
    if (!pOpenStaHome) return NULL;

    string strLocalPath = pOpenStaHome;
	#ifdef _WIN32
    strLocalPath += "Engines\\"; 
	#else
    strLocalPath += "Engines/"; 
	#endif

	ofstream statsfile;
	statsfile.open((strLocalPath+(pIS->m_statsFileName)).c_str(), ios::binary );

	if(!(statsfile.is_open()))
	{
		//error opening stream
		return NULL;
	}

	//add it to the list of files that need to be copied at the end of the test
	//pIS->m_ic.SetResultsFile((pIS->m_statsFileName).c_str());
	
	//first byte is integer version number
	const int version =0;
	statsfile.write((char*)&version,sizeof(int));

	const int buffSize=50000;	//50K assuming 200 task groups with 200 tasks 
	char buffer[buffSize+sizeof(long)+1];
	long temp=0;
	long pos=0;
	bool bFileHasBeenUsed=false;

	while(pIS->m_bStopMonitoring==false)
	{
		try
		{
			//call direct, dont use the cache as we need to get new stats
			if(pIS->HTTP_GetBothStatsSnapshots2(HTTP_InjectorSnapshot,InjectorSnapList))
			{	
				if(strcmp(HTTP_InjectorSnapshot->strTestExecuter,strNoValidHTTPExecuter)!=0)
				{
					//this executer name is not the special case which should be ignored!

					pos=0;	//reset to start of buffer again
					CTTime t;
				    t.now();
					temp=((t.getSeconds())-(pIS->m_ic.GetTimeOffset()));
					if(pos>buffSize) break;
					memcpy(buffer+pos,&temp,sizeof(long));
					pos+=sizeof(long);
				
					long strLength= strlen((char*)(HTTP_InjectorSnapshot->strTestExecuter))+1;
					if(pos>buffSize) break;
					memcpy(buffer+pos,&strLength,sizeof(long));
					pos+=sizeof(long);	

					if(pos+strLength>buffSize) break;
					memcpy(buffer+pos,(HTTP_InjectorSnapshot->strTestExecuter),((sizeof(char))*(strLength-1)));
					pos+=((sizeof(char))*(strLength-1));
	
					//ensure it's terminated
					char x='\0';
					if(pos>buffSize) break;
					memcpy(buffer+pos,&x,sizeof(char));
					pos+=sizeof(char);
	
					temp=HTTP_InjectorSnapshot->lAvgConnectionTime;
					if(pos>buffSize) break;
					memcpy(buffer+pos,&temp,sizeof(long));
					pos+=sizeof(long);
				
					long numberOfTaskGroupSnaps = HTTP_InjectorSnapshot->seqTaskGroupSnaps.length();
					if(pos>buffSize) break;
					memcpy(buffer+pos,&numberOfTaskGroupSnaps,sizeof(long));
					pos+=sizeof(long);
					
					for (long a=0; a<numberOfTaskGroupSnaps; a++)
					{
						temp=HTTP_InjectorSnapshot->seqTaskGroupSnaps[a].lTaskGroupId;
						if(pos>buffSize) break;
						memcpy(buffer+pos,&temp,sizeof(long));
						pos+=sizeof(long);
		
						temp=HTTP_InjectorSnapshot->seqTaskGroupSnaps[a].lCompletedIterations;
						if(pos>buffSize) break;
						memcpy(buffer+pos,&temp,sizeof(long));
						pos+=sizeof(long);	

						temp=HTTP_InjectorSnapshot->seqTaskGroupSnaps[a].lRunTime;
						if(pos>buffSize) break;
						memcpy(buffer+pos,&temp,sizeof(long));
						pos+=sizeof(long);
			
						temp=HTTP_InjectorSnapshot->seqTaskGroupSnaps[a].lTotalUsers;
						if(pos>buffSize) break;
						memcpy(buffer+pos,&temp,sizeof(long));
						pos+=sizeof(long);
			
						temp=HTTP_InjectorSnapshot->seqTaskGroupSnaps[a].lRequests;
						if(pos>buffSize) break;
						memcpy(buffer+pos,&temp,sizeof(long));
						pos+=sizeof(long);
			
						temp=HTTP_InjectorSnapshot->seqTaskGroupSnaps[a].lFailedRequests;
						if(pos>buffSize) break;
						memcpy(buffer+pos,&temp,sizeof(long));
						pos+=sizeof(long);

						temp=HTTP_InjectorSnapshot->seqTaskGroupSnaps[a].lBytesIn;
						if(pos>buffSize) break;
						memcpy(buffer+pos,&temp,sizeof(long));
						pos+=sizeof(long);

						temp=HTTP_InjectorSnapshot->seqTaskGroupSnaps[a].lBytesOut;
						if(pos>buffSize) break;
						memcpy(buffer+pos,&temp,sizeof(long));
						pos+=sizeof(long);
		
						temp=HTTP_InjectorSnapshot->seqTaskGroupSnaps[a].lMinRequestLatency;
						if(pos>buffSize) break;
						memcpy(buffer+pos,&temp,sizeof(long));
						pos+=sizeof(long);
	
						temp=HTTP_InjectorSnapshot->seqTaskGroupSnaps[a].lMaxRequestLatency;
						if(pos>buffSize) break;
						memcpy(buffer+pos,&temp,sizeof(long));
						pos+=sizeof(long);
				
						temp=HTTP_InjectorSnapshot->seqTaskGroupSnaps[a].lAvgRequestLatency;
						if(pos>buffSize) break;
						memcpy(buffer+pos,&temp,sizeof(long));
						pos+=sizeof(long);
	
						long numberOfTaskSnaps = HTTP_InjectorSnapshot->seqTaskGroupSnaps[a].seqTaskSnaps.length();
						if(pos>buffSize) break;
						memcpy(buffer+pos,&numberOfTaskSnaps,sizeof(long));
						pos+=sizeof(long);
				

						for(long b = 0; b < numberOfTaskSnaps; b++)
						{
							temp=HTTP_InjectorSnapshot->seqTaskGroupSnaps[a].seqTaskSnaps[b].lTaskId;
							if(pos>buffSize) break;
							memcpy(buffer+pos,&temp,sizeof(long));
							pos+=sizeof(long);
					
							temp=HTTP_InjectorSnapshot->seqTaskGroupSnaps[a].seqTaskSnaps[b].lActiveUsers;
							if(pos>buffSize) break;
							memcpy(buffer+pos,&temp,sizeof(long));
							pos+=sizeof(long);
						
							temp=HTTP_InjectorSnapshot->seqTaskGroupSnaps[a].seqTaskSnaps[b].lIterations;
							if(pos>buffSize) break;
							memcpy(buffer+pos,&temp,sizeof(long));
							pos+=sizeof(long);
						
							temp=HTTP_InjectorSnapshot->seqTaskGroupSnaps[a].seqTaskSnaps[b].lRuntime;
							if(pos>buffSize) break;
							memcpy(buffer+pos,&temp,sizeof(long));
							pos+=sizeof(long);
						}
					}

			
					if(pos>buffSize) break;
					//it was valid no buffer overRun
					
					statsfile.write(buffer,pos);
					statsfile.write("|Y",2);
					statsfile.flush();

					if(bFileHasBeenUsed==false)
					{
						//there is now data for the file so it should be retrieved at the end of the test
						//add it to the list of files that need to be copied at the end of the test
						pIS->m_ic.SetResultsFile((pIS->m_statsFileName).c_str());
						bFileHasBeenUsed=true;
					}
				}
				
				//only store in the cache if the file is being written - locked
				CTLockGuard<CTMutex> guard(pIS->m_CacheInUseMutex);
				pIS->m_cachedHTTP_InjectorSnapshot  = HTTP_InjectorSnapshot;
				pIS->m_cachedInjectorSnapList       = InjectorSnapList;
				
				pIS->m_bCachedHTTP_InjectorSnapshot = true;
				pIS->m_bCachedInjectorSnapList      = true;
			}
		}
		catch(...)
		{
		}
		#ifdef _WIN32
        ::Sleep(intervalBetweenCalls);
		#else
        ::sleep(intervalBetweenCalls / 1000);
		#endif
	}
	
	//close the file
	statsfile.flush();
	statsfile.close();
	

	//stop using the cache
	pIS->m_bCachedHTTP_InjectorSnapshot=false;
	pIS->m_bCachedInjectorSnapList=false;

	//stop the thread
	pIS->m_pStatWriter->exit();

	return NULL;
}


