/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// RemoteRepository.cpp: implementation of the CRemoteRepository class.
//
//////////////////////////////////////////////////////////////////////

#include "RemoteRepository.h"
#include <CECInetInfo.h>
#include <fstream>
#include "CECSimpleParse.h"
#include <iostream>
using namespace std;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CRemoteRepository::CRemoteRepository(CORBAServer* corba,const string& node,
									 string strTestName, string strTestRunFolder)
{
	m_pCorba = corba;
	m_strReposNode = node;
	m_strTest = strTestName;
	m_strTestRunFolder = strTestRunFolder;
	m_bReadTrace = false;

	Connect();
}

CRemoteRepository::~CRemoteRepository()
{
}

bool CRemoteRepository::CheckoutTofFile(const char* strTofFile)
{
	try
	{
		if (CORBA::is_nil(m_pFileTransfer))
			Connect();

		return m_pFileTransfer->GetObjectCodeFile(m_strReposNode.c_str(), strTofFile);
	}
	catch(...)
	{
	}

	return false;
}

bool CRemoteRepository::CheckoutProfileFile(const char* strProfileFile)
{
	try
	{
		if (CORBA::is_nil(m_pFileTransfer))
			Connect();

		return m_pFileTransfer->GetCollectorFile(m_strReposNode.c_str(), strProfileFile);
	}
	catch(...)
	{
	}
	return false;
}

bool CRemoteRepository::CheckoutDataFile(const char* strDataFile)
{
	try
	{
		if (CORBA::is_nil(m_pFileTransfer))
			Connect();

		return m_pFileTransfer->GetDataFile(m_strReposNode.c_str(), strDataFile);
	}
	catch(...)
	{
	}
	return false;
}

bool CRemoteRepository::CheckoutTraceSettingsFile()
{
    string strFileNam = "TraceSettings.txt";

    try
    {
		if (CORBA::is_nil(m_pFileTransfer))
			Connect();

		bool bRet =  m_pFileTransfer->GetRootFile(m_strReposNode.c_str(), strFileNam.c_str());

		string strDestPath = getenv("OPENSTA_HOME");
#ifdef _WIN32
		strDestPath  += "Engines\\";
#else
		strDestPath  += "Engines/";
#endif
		strDestPath += strFileNam;

        m_strTrcFilPath = strDestPath;  // Save Trace file name
		return bRet;
    }
    catch (...)
    {
    }
	return false;
}

bool CRemoteRepository::StoreResultsFile(const char* strResultsFile)
{
	try
	{
		if (CORBA::is_nil(m_pFileTransfer))
			Connect();
		return m_pFileTransfer->StoreResultsFile(m_strReposNode.c_str(), m_strTest.c_str(), m_strTestRunFolder.c_str(), strResultsFile);
	}
	catch(...)
	{
	}
	return false;

}

void CRemoteRepository::ReadTraceSettings()
{
    if (!CheckoutTraceSettingsFile()) m_bReadTrace = false;

	ifstream ifs (m_strTrcFilPath.c_str());

    while (ifs.good() && !ifs.eof())
	{
		string strline;
		getline(ifs, strline);
		CECSimpleParse parse(strline);
		
		string strName = parse.getNext(',');
		string strLevel = parse.getToEnd();
        m_TraceNameToLevel.insert(TraceNameToLevel_t::value_type(strName, strLevel));

    }
    m_bReadTrace = true;
	return;
}

string CRemoteRepository::GetTraceLevel(string name)
{
    string strTrcLev = "10";        // Assume default trace level

    // If we have a TraceSettings file set level from file, otherwise set
    // a hardwired default value.
    if (m_bReadTrace)
    {
        TraceNameToLevel_t::iterator pTrcI = m_TraceNameToLevel.find(name);
        if(pTrcI != m_TraceNameToLevel.end())
        {
            strTrcLev = pTrcI->second;
        }
    }
    return strTrcLev;
}

void CRemoteRepository::Connect()
{
    CECInetInfo info;
    string underscore_ip_name=info.GetIPAddressAsString();
	
    try
    {
		CORBA::Object_ptr obj = m_pCorba->FindObjectWithName(FileTransferNode::CLASS_NAME,underscore_ip_name.c_str());
		m_pFileTransfer = FileTransferNode::_narrow(obj);
	    ReadTraceSettings();
    }
    catch(...)
    {
    }

}
