/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include "TPSemaphore_i.h"
/////////////////////////////////////////////////////
// @mfunc Constructor
TPSemaphoreImpl::TPSemaphoreImpl(CORBAServer* pcorba,string SemaphoreName)
        :semaphore(1),m_regname(SemaphoreName),m_corba(pcorba)
{
    m_corba->ActivateObject(this);  
    m_corba->NameObject(this->_this(),TPSemaphore::CLASS_NAME,m_regname.c_str());   
}
/////////////////////////////////////////////////////
// @mfunc Destructor
TPSemaphoreImpl::~TPSemaphoreImpl()
{
    m_corba->UnNameObject(TPSemaphore::CLASS_NAME,m_regname.c_str());   
}
/////////////////////////////////////////////////////
// @mfunc Acquire Semaphore (attempt to decrement)
void TPSemaphoreImpl::acquire()
{
    static omni_mutex mutex;
    omni_mutex_lock guard(mutex);
    semaphore.wait();//wait for lock to become free
}
/////////////////////////////////////////////////////
// @mfunc Try and Acquire Semaphore (attempt to decrement)
// @rdesc true if acquired

CORBA::Boolean TPSemaphoreImpl::tryacquire()
{
    static omni_mutex mutex;
    omni_mutex_lock guard(mutex);
    int have_locked=semaphore.trywait();
    if(have_locked==1)//have mangaged to acquire semaphore
    {
        return true;
    }
    else //already acquired
    {
        return false;
    }
}
/////////////////////////////////////////////////////
// @mfunc release Mutex(increment count)
void TPSemaphoreImpl::release()
{
    static omni_mutex mutex;
    omni_mutex_lock guard(mutex);
    semaphore.post();
}
