/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     VarFactory
///////////////////////////////////////////////////////////////////////////////
#ifndef _VARFACTORY_I_H_
#define _VARFACTORY_I_H_
#include "TParamList.h"
#include "VarFactory.h"
#include <OSTAORB/OSTAORB.h>   
#include "TPMutex_i.h"
#include "TPSemaphore_i.h"
#include "GlobalVariable_i.h"  
#include "TPCYRSemaphore.h"
#ifdef WIN32
#pragma warning(disable:4786)
#pragma warning(disable:4503)
#endif
#include <map>
#include <list>
#include <string>
#include <vector>
#ifndef CYR_NO_STD_NAMESPACE
using namespace std;
#endif

// container of registered scripts, a scriptobject is basically the CORBA object associated
// with a process thread within a TestManager/TestExecuter i.e. a TestLevel or TestExecuterThread
// each of these must be registered with the factory so that when no more of a script is running
// then script variables can be disposed of.
// the container is in 2 parts
// 1. a map of idents for scriptobjects with a pointer to the object, this gives a way of identifying
// a CORBA pointer as CORBA object pointers can not reliably be compared for equivalence
// 2. a  map of the above maps per scriptname
// this gives us a way of tracking how many objects are running a particular script
// and removing the object from the list when required.
//typedef map< string, ScriptObject_ptr, less <string> > ScriptObjectMap_t;
//typedef map< string , ScriptObjectMap_t* , less<string> > ScriptMap_t;
//Map of names of scripts against the number currently running with that name
typedef map<string,int, less<string> > ScriptMap_t;
///////////////////////////////////////////////////////////////////////////////
//
// @class   VarFactoryImpl
//
///////////////////////////////////////////////////////////////////////////////
class VarFactoryImpl: public POA_VarFactory, public PortableServer::RefCountServantBase 
{
public:
    VarFactoryImpl(CORBAServer* pserver,const string &regname);
    virtual ~VarFactoryImpl();
    virtual CORBA::Long  RegisterScript(const char*  scriptname);
	virtual void ClearAll();
    virtual CORBA::Object_ptr  GetVariable( const char*  name,
                                            CORBA::Long  size,
                                            const char*  file,
                                            const char*  fileid,
                                            const char*  dimensions,
                                            const char*  values,
                                            CORBA::Long  options,
                                            CORBA::Long  seed,
                                            const char*  scriptname);
    virtual CORBA::Object_ptr  CreateMutex(const char*  name);
    virtual CORBA::Object_ptr  CreateSimpleSemaphore(const char*  name);
    virtual CORBA::Object_ptr  CreateRealSemaphore(const char*  name);

    virtual CORBA::Long  DeregisterScript(const char*  script, CORBA::Boolean  keepalive);
private:
    virtual void ConstructInit(CORBAServer* pserver,const string &regname);
    virtual CORBA::Long  DisposeVariables(const string& scriptname);
    virtual void    ClearGlobalVarList();
    virtual void    ClearScriptVarList();
    virtual void    ClearRegisteredScriptList();
    virtual void    ClearMutexList();
    virtual void    ClearSemaphoreList();
    CORBAServer* m_pServer;
    string  m_regname;
#ifdef HPUX
public:
#endif
    class VFVar;
    class VFVarDecl;
private:
    typedef map< string , VFVar* , less <string> >    VFVarMap_t;

    ScriptMap_t     m_RegisteredScriptList;
    VFVarMap_t    m_GlobalVarList;
    typedef map< string , VFVarMap_t* , less <string > > VFVarMapMap_t;
    VFVarMapMap_t     m_ScriptVarList;
    typedef vector<TPMutexImpl*>    mutexlist_t;
    mutexlist_t     m_mutexlist;
    typedef vector<TPCYRSemaphore*>    simplesemaphorelist_t;
    simplesemaphorelist_t   m_simplesemaphorelist;
    typedef vector<TPSemaphoreImpl*>    realsemaphorelist_t;
    realsemaphorelist_t     m_realsemaphorelist;
};
///////////////////////////////////////////////////////////////////////////////
//
// @class   VarFactoryImpl::VFVarDecl
//
///////////////////////////////////////////////////////////////////////////////
class VarFactoryImpl::VFVarDecl
{
public:
    VFVarDecl(const char*  name,
                CORBA::Long  size,
                const char*  file,
                const char*  fileid,
                const char*  dimensions,
                const char*  values,
                CORBA::Long  options,
                CORBA::Long  seed,
                const char*  scriptname);

    virtual ~VFVarDecl();
    virtual string GetUniqueName();
    virtual bool compare(const VFVarDecl& in);
    virtual bool operator==(const VFVarDecl& in);
    virtual bool operator!=(const VFVarDecl& in);

protected:
    string  m_name;
    CORBA::Long  m_size;
    string  m_file;
    string  m_fileid;
    string  m_dimensions;
    string  m_values;
    CORBA::Long  m_options;
    CORBA::Long  m_seed;
    string  m_scriptname;
private:
    string  m_unique_name;
};
///////////////////////////////////////////////////////////////////////////////
//
// @class   VarFactoryImpl::VFVar
//
///////////////////////////////////////////////////////////////////////////////
class VarFactoryImpl::VFVar:public VarFactoryImpl::VFVarDecl
{
public:

    VFVar(CORBAServer* pServer,
            string VFRegName,
            const char*  name,
            CORBA::Long  size,
            const char*  file,
            const char*  fileid,
            const char*  dimensions,
            const char*  values,
            CORBA::Long  options,
            CORBA::Long  seed,
            const char*  scriptname);
    VFVar(CORBAServer* pServer, string VFRegName,const VFVarDecl& VFVD);
    virtual ~VFVar();
    virtual CORBA::Object_ptr   GetVariable_ptr();
private:
    virtual void CreateVariable(CORBAServer* pServer,string VFRegName);
    virtual GlobalVariableImpl* CreateNewSimpleVariable();
    virtual GlobalVariableImpl* CreateNewRangeVariable(); 
    virtual GlobalVariableImpl* CreateNewListVariable();  
    virtual GlobalVariableImpl* CreateNewFileVariable();  
    virtual GlobalVariableImpl* CreateNewArrayVariable(); 
    CORBAServer* m_pServer;
    string m_regname;
    GlobalVariableImpl* m_CORBAvariable;
};
#endif
