/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc  CyrWrapLib
///////////////////////////////////////////////////////////////////////////////

#ifndef _CECOrbBorrow_h
#define _CECOrbBorrow_h

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include <OSTAORB/OSTAORB.h>
#include <Factory/Factory.h>
#include <Repository/CyrStRepository.h>

///////////////////////////////////////////////////////////////////////////////
// Constants
// @const long | DEFAULT_PERIOD | The default length of time an object will
//        exist for in periods.  One period is approximately ten minutes.
const long DEFAULT_PERIOD = 6;

///////////////////////////////////////////////////////////////////////////////
// @class   This class represents the base class for all borrowed objects.  A
//          borrowed object is one that is constantly being reaped by the
//          Factory reaper mechanism.  To prevent object destruction this class
//          allows such objects to extend their lifetimes.  When the Borrow
//          method is called the object's lease period will be extended.
class CECOrbBorrow
{
    // @access Public Members
public:
    // @cmember Constructor with only Factory name
    CECOrbBorrow( CORBAServer* corba, const char* factory );
    // @cmember Constructor with Factory reference
    CECOrbBorrow( CORBAServer* corba, CyrFactory_ptr factory );
    virtual ~CECOrbBorrow(); // @cmember Destructor

    // @cmember Registers the name and class of this object to be borrowed
    void Register( const char* name, const char* cls );
    void Borrow( long period );  // @cmember Extends lease on a borrowed item.

    CyrFactory_ptr  GetFactory();  // @cmember Returns factory reference
    CORBAServer* GetHelper();   // @cmember Returns CORBA helper class
    string    GetRegisteredName() const;   // @cmember Returns registered name
    string    GetRegisteredClass() const;  // @cmember Returns registered class

    // @access Protected Members
protected:
    CORBAServer*   m_corba;    // @cmember Pointer to CORBA functionality
    CyrFactory_var    m_factory;  // @cmember Reference to Factory object
    string      m_name;     // @cmember Registered name of class
    string      m_class;    // @cmember Registered class name
}
;

#endif
