/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc CyrWrapLib
///////////////////////////////////////////////////////////////////////////////

#ifndef _CECOrbProject_h
#define _CECOrbProject_h

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include <CyrWrapLib/CECOrbServer.h>

///////////////////////////////////////////////////////////////////////////////
// @class   CECOrbProject | This class is a wrapper around the CORBA interface
//          to OpenSTARep's Project objects.  It provides a basic set of methods
//          for creation and retrieval of Projects.  Specific Project
//          instances, such as an Impact Project, are derived from this class
//          and are expected to initialize themselves correctly.
// @base    public | CECOrbBorrow
class CECOrbProject : public CECOrbBorrow
{
    // @access Public Members
public:
    // @cmember Constructor for retrieving existing Project
    CECOrbProject( CECOrbServer* server, const char* name );

    string GetName()const;         // @cmember Returns Project name

    CyrStProject_ptr GetServant();  // @cmember Returns the underlying servant object
    CyrStServer_ptr  GetHiddenServer();  // @cmember Returns CyrStServer reference
    CECOrbServer*    GetParentServer();  // @cmember Returns CECServer reference

    // @access Protected Members
protected:
    CyrStProject_var m_servant;  // @cmember Reference to underlying CORBA servant
    CyrStServer_var m_hidden;    // @cmember Reference to CyrStServer
    CECOrbServer* m_parent;      // @cmember Reference to CECOrbServer
    string     m_cacheName;      // @cmember cached ProjectName to save on CORBA calls
}
;

#endif
