/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     SpawnProcess
///////////////////////////////////////////////////////////////////////////////

#ifndef _SpawnProc_i_h
#define _SpawnProc_i_h

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
extern "C"
{
#include "psl_macros.h"
#include "target_rtndef.h"
}

#include <Objects/SpawnProc.h>
#include <OSTAORB/OSTAORB.h>
#include <process.hpp>

///////////////////////////////////////////////////////////////////////////////
//
class SpawnProcessImpl : public POA_SpawnProcess,public PortableServer::RefCountServantBase
{
    // @access Public Members
public:
    // @cmember Constructor.
    SpawnProcessImpl( CORBAServer* pServer, const char* name );
    virtual ~SpawnProcessImpl();     // @cmember Destructor.

    long Start  ( const char* image_ptr );   // @cmember Starts the process.
    // @cmember Starts a CYRANO product defined process.
    long StartProductImage  (long TargetID, long TargetImage, const char* pProduct, const char* pParameters);
    // @cmember Waits for process to terminate
    long Wait();
    // @cmember,mfunc  This method retrieves the process id for the spawned process.
#ifdef _WIN32
    long GetPid()
    {
        return m_process->getActualPid();
    };
#else
    long GetPid()
    {
        return m_process->getPid();
    };
#endif


    // @access Private Members
private:
    // @cmember format an executable name given components
    int FormatExecutable    (enum  TARGETID TargetID ,enum  TARGETIMAGE TargetImage
                             ,const char* pProduct ,char* pImagePath);

    string m_name;        // @cmember Name of this object.
    CORBAServer* m_pServer;  // @cmember Pointer to COBRA functionality.
    CTProcess* m_process;    // @cmember Pointer to process functionality.
}
;

#endif
