/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

//==================================================================< Pragmas >
#ifdef  WIN32
#  pragma warning(disable : 4514)
#endif  // WIN32


//=================================================================< Includes >
#ifndef __DEFINITION__
#  include "definition.h"
#endif  // __DEFINITION__

#ifndef __FUNCTOR__
#  include "functor.hpp"
#endif  // __FUNCTOR__


//====================================================================< Class >

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////// Class CTFunctor //
///////////////////////////////////////////////////////////////////////////////
// Class representing the function invocation.

//============================================< Constructor(s) and destructor >
//-----------------------------------------------------------------------------
// CTFunctor::CTFunctor
//
// Default constructor.
//
// functorXX : reference to a template XX functor.
// Return    : none.
//-----------------------------------------------------------------------------
CTFunctor::CTFunctor (const CTFunctorXX& functorXX)
        : pFunctorXX_ (NULL)
{
    pFunctorXX_ = ( (CTFunctorXX&) functorXX).duplicate () ;
} // CTFunctor::CTFunctor

//-----------------------------------------------------------------------------
// CTFunctor::CTFunctor
//
// Copy constructor.
//
// functor : reference to a functor object.
// Return  : none.
//-----------------------------------------------------------------------------
CTFunctor::CTFunctor (const CTFunctor& functor)
        : pFunctorXX_ (NULL)
{
    pFunctorXX_ = functor.pFunctorXX_->duplicate () ;
} // CTFunctor::CTFunctor

//-----------------------------------------------------------------------------
// CTFunctor::~CTFunctor
//
// Destructor.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------
CTFunctor::~CTFunctor ()
{
    if (pFunctorXX_)
    {
        delete pFunctorXX_ ;
        pFunctorXX_ = NULL ;
    }
} // CTFunctor::CTFunctor

//================================================================< Operators >
//-----------------------------------------------------------------------------
// CTFunctor::operator=
//
// functor : reference to a functor object.
// Return  : *this.
//-----------------------------------------------------------------------------
CTFunctor& CTFunctor::operator= (const CTFunctor& functor)
{
    pFunctorXX_ = functor.pFunctorXX_->duplicate () ;

    return *this ;
} // CTFunctor::operator=

//=========================================================< Control methods >
//-----------------------------------------------------------------------------
// CTFunctor::call
//
// Call the function.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------
void CTFunctor::call ()
{
    // Verify if the functor is correct.
    if (pFunctorXX_ == NULL)
    {
        throw CTFunctorException ("call: pFunctorXX_ is NULL.", __FILE__, __LINE__) ;
    }

    // Call the function.
    pFunctorXX_->call () ;
} // CTFunctor::call

//-----------------------------------------------------------------------------
// CTFunctor::getResultPointer
//
// Get a pointer to the return value.
//
// Arguments : none.
// Return    : pointer to the return value.
//-----------------------------------------------------------------------------
void* CTFunctor::getResultPointer ()
{
    // Verify if the functor is correct.
    if (pFunctorXX_ == NULL)
    {
        throw CTFunctorException ("getResultPointer: pFunctorXX_ is NULL.", __FILE__, __LINE__) ;
    }

    // Get a pointer to the return value.
    return pFunctorXX_->getResultPointer () ;
} // CTFunctor::getResultPointer

///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////// End of Class CTFunctor //
///////////////////////////////////////////////////////////////////////////////
