/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#ifndef __LIBRARY__
#define __LIBRARY__

//
//  class CTLibrary : Should be used to load a library and get procedure address.
//  --------------- - constructor : Load a new library.
//                  - destructor : Unload the library.
//                  - getName : Get the library name.
//


//=================================================================< Includes >
#ifndef __DEFINITION__
#  include "definition.h"
#endif  // __DEFINITION__

#ifndef __EXCEPTION__
#  include "exception.hpp"
#endif  // __EXCEPTION__

#ifndef __MTSAFE__
#  include "mtsafe.hpp"
#endif  // __MTSAFE__


//====================================================================< Class >

///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////// Class CTLibraryException //
///////////////////////////////////////////////////////////////////////////////
// Class used to keep library classes exceptions.
class CTLibraryException : public CTException
{
    //-----------------------------------------------------------------------------
public:                     // Constructor(s) and destructor.
    CTLibraryException (const string& clMsg, const string& clFilename, short sLineno, long lNumber = -1L)
: CTException (clMsg, clFilename, sLineno, lNumber)
    { }


public:                     // Operators.
    void                    operator= (const CTLibraryException& exception)
    {
        CTException::operator= (exception) ;
    }

    //-----------------------------------------------------------------------------
}

; // CTLibraryException

///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////// End of Class CTLibraryException //
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////// Class CTLibrary //
///////////////////////////////////////////////////////////////////////////////
// Class used to load a library.
class CTLibrary : virtual public CTMTSafe
{
    //-----------------------------------------------------------------------------
public:                 // Types.
    enum enumFileType
    {
        UNKNOWN_TYPE,
        LIB_TYPE,
        EXE_TYPE
    } ;

    //-----------------------------------------------------------------------------
public:             // Constructor(s) and destructor.
    CTLibrary (const string& name) ;
    CTLibrary (const CTLibrary& library) ;
    ~CTLibrary () ;

public:             // Operators.
    CTLibrary&      operator= (const CTLibrary&) ;

public:             // Get methods.
    string      getName (void) ;
    string      getName (void) const ;
    void*           getInfo (void) const ;
    static string   getSystemFileName (const string& fileName, enumFileType eFileType = UNKNOWN_TYPE) ;
    static string   getFullPath (const string& fileName) ;
    static string   getBaseFileName (const string& fileName) ;
    static string   getDirectory (const string& fileName) ;

    //-----------------------------------------------------------------------------
private:            // Members.
    void*           pLibraryInfo_ ;

    //-----------------------------------------------------------------------------
}

; // CTLibrary

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////// End of Class CTLibrary //
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////// Class CTProcedure //
///////////////////////////////////////////////////////////////////////////////
// Class used to get a procedure address.
class CTProcedure : virtual public CTMTSafe
{
    //-----------------------------------------------------------------------------
public:             // Constructor(s) and destructor.
    CTProcedure (const CTLibrary& library, const string& name) ;
    CTProcedure (const CTProcedure& procedure) ;
    ~CTProcedure () ;

public:             // Operators.
    CTProcedure&    operator= (const CTProcedure&) ;

public:             // Get methods.
    string      getName (void) ;
    string      getName (void) const ;
    void*           getAddress (void) const ;
    CTLibrary&      getLibrary (void) ;
    void*           getInfo (void) const ;

    //-----------------------------------------------------------------------------
private:            // Members.
    void*           pProcedureInfo_ ;

    //-----------------------------------------------------------------------------
}

; // CTProcedure

///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////// End of Class CTProcedure //
///////////////////////////////////////////////////////////////////////////////

#endif  // __LIBRARY__
