/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#ifndef __SERVICE__
#define __SERVICE__

//==================================================================< Pragmas >
#ifdef  WIN32
#  pragma warning(disable : 4710)
#endif  // WIN32


//=================================================================< Includes >
#ifndef __DEFINITION__
#  include "definition.h"
#endif  // __DEFINITION__

#ifndef __EXCEPTION__
#  include <exception.hpp>
#endif  // __EXCEPTION__

#ifndef __MTSAFE__
#  include <mtsafe.hpp>
#endif  // __MTSAFE__



//====================================================================< Class >

///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////// Class CTServiceException //
///////////////////////////////////////////////////////////////////////////////
// Class used to keep service exception.
class CTServiceException : public CTException
{
    //-----------------------------------------------------------------------------
public:                 // Constructor(s) and destructor.
    CTServiceException (const string& clMsg, const string& clFilename, short sLineno, long lNumber = -1L)
: CTException (clMsg, clFilename, sLineno, lNumber)
    { }


public:                     // Operators.
    void                    operator= (const CTServiceException& exception)
    {
        CTException::operator= (exception) ;
    }

    //-----------------------------------------------------------------------------
}

; // CTServiceException

///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////// End of Class CTServiceException //
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////// Class CTService //
///////////////////////////////////////////////////////////////////////////////
class CTService : public CTMTSafe
{
    //-----------------------------------------------------------------------------
public:                 // Types.
    enum enumStartMethode
    {
        START_UNDEFINED = 0,
        START_CONSOLE,
        START_SERVICE
    } ;

    enum enumServiceControl
    {
        SRVCTRL_STOP = 0,
        SRVCTRL_PAUSE,
        SRVCTRL_CONTINUE
    } ;

public:                 // Constructor(s) and destructor.
    CTService (const string& programName, const string& serviceDisplayName) ;
    ~CTService () ;

public:                 // Get methods.
    enumStartMethode    getStartMode (void) ;
    const char*         getStartModeLabel (void)
    {
        return zStartModeLabel_[getStartMode ()] ;
    } ;

public:                 // Controls methods.
    void                start (const enumStartMethode eStartMethode = START_UNDEFINED) ;
    void                install (void) ;
    void                uninstall (void) ;
    void                control (enumServiceControl eServiceControl) ;

public:                 // User methods.
    virtual void        init (void) = 0 ;
    virtual void        main (void) = 0 ;
    virtual void        exit (void) = 0 ;
    virtual void        controlNotify (enumServiceControl) = 0 ;

    //-----------------------------------------------------------------------------
private:                // Members.
    void*               pServiceInfo_ ;
    enumStartMethode    eStartMethode_ ;
    static const char* const zStartModeLabel_[3] ;

    //-----------------------------------------------------------------------------
}

; // CTService

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////// End of Class CTService //
///////////////////////////////////////////////////////////////////////////////

#endif // __SERVICE__
