/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/*===========================================================================*/
/*                                       */
/*               **** C O M M O N ****                   */
/*                                       */
/*===========================================================================*/
/*                                       */
/*          Module Name: CHECK_DIGIT_ROUTINES            */
/*          ============                         */
/*                                       */
/* Description:                                  */
/* ============                                  */
/*                                       */
/*  Check Digit Routines.                            */
/*                                       */
/*===========================================================================*/

/*    Define useful Include structures   */

#include  <psl_macros.h>        /* PSL Standard Macros */

/*    Define Function Prototypes for all routines in this module */

#include  psl_common_file(psl_rtndef.h) /* PSL Function Prototypes */

/************************************************************************/
/*                                  */
/*  PSL_CALC_CHK_DIG - Calculate Check Digit for a string       */
/*                                  */
/*  Calling Convention:                     */
/*                                  */
/*     PSL_CALC_CHK_DIG (string_ptr, strlen_ptr, chkdig_ptr);   */
/*                                  */
/*  Where:                              */
/*                                  */
/*     string_ptr = Pointer to String to generate Check Digit   */
/*     strlen_ptr = Pointer to Length of string         */
/*     chkdig_ptr = Pointer to buffer to hold returned Check Digit  */
/*                                  */
/************************************************************************/

void    PSL_CALC_CHK_DIG (char *string_ptr,
                          short *strlen_ptr,
                          char *chkdig_ptr)
{
    /*
     *  Local Data.
     */
    short      jsub;       /* Loop Control */
    char       *wrk_ptr;       /* Work Pointer */
    unsigned long       sum = 0;        /* Sum Value */

    /*
     *  Calculate the sum of each digit multiplied by its offset
     *  (digit1 * 1) + (digit2 * 2) + ... + (digitn * n)
     */
    for (jsub = 0, wrk_ptr = string_ptr;
            jsub < *strlen_ptr;
            jsub++, wrk_ptr++)
    {
        sum += ((*wrk_ptr) * (jsub+1));
    }

    /*
     *  The Check Digit is the modulus (remainder) of the sum divided
     *  by the number of digits (including the check digit) plus 1
     *  i.e. *strlen_ptr + 2.
     */
    *chkdig_ptr = (char)(sum % (*strlen_ptr + 2));

    return;
}
