#ifndef PSL__SCL_RTNDEF_LOADED
#define PSL__SCL_RTNDEF_LOADED 1
/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/*
 *  scl_rtndef.h:
 *  =============
 *
 *  This module contains function prototypes for the SCL routines
 *  that may be used by multiple SCL modules.
 */

/*   Define Incomplete Structures for all referenced types */

#include <psl_structures.h>     /* Common Structures */
#include <vtest_structures.h>       /* V-TEST Structures */

/*
 *    Incomplete type declarations.
 */
struct scl_com;
struct scl_tpa;
struct fattbl;
struct labtbl;
struct subtbl;
struct symtbl;
struct oph;
struct opd;

/*
 *    Then the function prototypes for internal SCL routines.
 */
void        ALLOCATE_OPERAND    (struct scl_tpa *tpa_ptr
                                 ,struct oph     **oph_pptr
                                 ,struct opd     **opd_ptr);

void    ADD_DEP         (struct scl_com *com_ptr,
                         char *depnam,
                         enum deptyp_t deptyp,
                         char *depdat);

void    ADD_FILE_SYMBOL     (struct scl_com *com_ptr
                             ,struct fattbl  *fattbl_ptr);

void    ADD_LABEL       (struct tpaarg *argblk_ptr,
                         char *label_ptr,
                         unsigned long label_len);

void    ADD_OPR         (struct scl_com *com_ptr,
                         unsigned long opr);

void    ADD_TIMER       (struct tpaarg *argblk_ptr);

void    ADD_UNDEF_LABEL     (struct tpaarg *argblk_ptr,
                             char *label_ptr,
                             unsigned long label_len);

void    ADD_VARIABLE        (struct tpaarg *argblk_ptr);

unsigned long   CHK_SCL_DNAME       (struct tpaarg *argblk_ptr);

void    CHK_SUBRTN_DEF      (struct tpaarg *argblk_ptr);

void    CHK_SUBTST_DEF      (struct tpaarg *argblk_ptr);

unsigned long   CHK_THR_DEF     (struct tpaarg *argblk_ptr);

void    CREATE_ENCCHRIMM    (struct scl_com *com_ptr,
                             char *encbuf_ptr,
                             size_t encbuflen,
                             const char *rawval_ptr,
                             PSL_U_LONGWORD_T rawvallen);

void    CREATE_VAR_OPD      (struct   scl_tpa *tpa_ptr,
                             struct   symtbl  *var_ptr);

void    DEL_DEP_LL      (struct scl_com *com_ptr);

void    ENC_REC_FORMAT      (struct   scl_com *com_ptr
                             ,unsigned long    rec_count);

void    ENC_INI_VALUES      (struct   scl_com *com_ptr
                             ,unsigned long    area_type);

void    ENC_INTEGER     (struct scl_tpa *tpa_ptr, long value);

void    ENC_INTEGER_AT      (char *addr_ptr, long value);

void    ENC_LABEL       (struct tpaarg *argblk_ptr,
                         char *label_ptr,
                         unsigned long label_len);

void   ENC_OPERATOR (struct scl_tpa *tpa_ptr, long operator);

void   FREE_OPH_OPD        (struct oph *oph_ptr,
                            struct opd *opd_ptr);

void   INI_DEF (struct scl_com *com_ptr);

void   INI_OPERAND (struct scl_tpa *tpa_ptr,
                    unsigned long opd_no);

void   INI_THR_DEF (struct scl_com *com_ptr);

void   INI_VPSTATUS (struct scl_com *com_ptr);

void   LIST_TIMERS (struct scl_com *com_ptr);

void   LIST_VARIABLES (struct scl_com *com_ptr);

unsigned long   READ_NEXT_LINE (struct scl_com *com_ptr,
                       struct tpaarg *argblk_ptr,
                       unsigned long *line_size,
                       char *source_line,
                       char *parsable_line,
                       char *error_line,
                       unsigned long *eof_flag,
                       unsigned long unit);

void   REPORT_ERROR (struct tpaarg *, unsigned long errnum);

void   SCL_COMPILE     (struct scl_com *scl_com);

unsigned long   SCL_GETCMD (struct scl_com *scl_com_ptr,
                            long argc,
                            char *argv[]);

unsigned long   SCL_GETNEXT_FILE (struct scl_com *scl_com);

void   SCL_LIST_LINE (struct scl_com *com_ptr, char *buf_ptr);

void   SCL_CLOSE_SOURCE (struct scl_com *scl_com
                         ,unsigned long unit);

unsigned long   SET_TC_GOTO (struct tpaarg *argblk_ptr);

unsigned long   SCL_OPEN_INCLUDE (struct scl_com *scl_com,
                                  char *byte_script_name,
                                  unsigned long name_length,
                                  unsigned long unit);

unsigned long   SCL_OPEN_SOURCE (struct scl_com *scl_com, unsigned long unit);

void   SCL_REPLACE (struct scl_com *scl_com_ptr);

unsigned long   SCL_TPARSE (struct scl_com *scl_com_ptr);

void   UNPACK_INTEGER (char *addr_ptr, long *value);

/*
 *    DEFAULT handling routines.
 */
void    SCL_DEFAULT_TDF     (struct diagstk *diagstk_ptr
                             ,struct tdf     *tdf_ptr);

void    SCL_DEFAULT_TDP     (struct diagstk *diagstk_ptr
                             ,struct tdp     *tdp_ptr);

void    SCL_DEFAULT_PRM     (struct diagstk *diagstk_ptr
                             ,struct prm     *prm_ptr);

void    SCL_FREE_PRM        (struct prm     *src_prm_ptr);

void    SCL_FREE_TDF        (struct tdf     *src_tdf_ptr);


unsigned long   SCL_UNDEFAULT_TDF   (struct diagstk *diagstk_ptr
                                     ,struct tlh     *src_tlh_ptr
                                     ,struct prm     *src_prm_ptr
                                     ,struct tdf     *src_tdf_ptr
                                     ,struct tdf     *def_tdf_ptr
                                     ,struct tdf     *out_tdf_ptr);

unsigned long   SCL_UNDEFAULT_PRM   (struct diagstk *diagstk_ptr
                                     ,struct tlh     *src_tlh_ptr
                                     ,struct prm     *src_prm_ptr
                                     ,struct prm     *def_prm_ptr
                                     ,struct prm     *out_prm_ptr);

unsigned long   WRT_DEP_FILE        (struct scl_com *com_ptr);

unsigned long   WRT_SCRTOF_FILE     (struct scl_com *com_ptr);

unsigned long   WRT_THRTOF_FILES    (struct scl_com *com_ptr);

unsigned long   WRT_TOF_LINE        (struct scl_com *com_ptr,
                                     char *lin_ptr);
#endif  /* PSL__SCL_RTNDEF_LOADED */
