/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     CECResultsFile
// @module  CECResultsFile | File Access class
///////////////////////////////////////////////////////////////////////////////

#ifndef CYRANO_TP_STANDALONE
#include "CORBARepository.h"
#include "CECResultsFile.h"
#include "CECInetInfo.h"
#include "CyrHelper.inl"

///////////////////////////////////////////////////////////////////////////////
//@mfunc    constructor
//@parm	    CECOrbFolder TestRun
//@parm	    string filename
//@parm	    string full filename including path
CECResultsFile::CECResultsFile(CECOrbFolder* m_TestRun, const char* FileName,
                               const char* FullFileName)
{
    // We need to get the name of the current file with and without path
    // The full path version is used to identify the file to transfer
    CECInetInfo info;

    m_FileName = FileName;
    m_FullFileName = FullFileName;

    // the CECOrbFile constructor will throw a CyrRepositoryException
    // which will be caught in the CORBARepository (calling object)
    m_OrbFile = new CECOrbFile(m_TestRun, m_FileName.c_str(),"Results File");
}

///////////////////////////////////////////////////////////////////////////////
//@mfunc Destructor
CECResultsFile::~CECResultsFile()
{
    delete m_OrbFile;
}


void CECResultsFile::Store(bool del)
{

    // when we check in we need to pass in the full path to the local file
    try
    {
        m_OrbFile->Lock();
        m_OrbFile->CheckIn(m_FullFileName.c_str(),true);

        // now remove the local file
        if (del)
            CyrDeleteFile(m_FullFileName.c_str());
    }
    catch(...)
    {
    }

}

#endif
