/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
// TestManager_i.cpp: implementation of the TestManager_i class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ManagerMain.h"
#include "OstaTraceDef.h"
#include "TestManager_i.h"
#include "ctime.hpp"
#include "CECInetInfo.h"
#include <assert.h>
#include <process.h>
#include "VarFactory.h"

#include <iostream>
using namespace std;
extern long m_lSampleRate;
CTSemaphore blocking_semaphore(0);
CTMutex startTGmutex;

const char* strXMLPrefix = "WebRelay";
const char* strHTTPStatsFile = "HTTPDataFile";

//////////////////////////////////////////////////////////////////////
// Thread Functions
//////////////////////////////////////////////////////////////////////
 
void* TestManagerImpl_SCHEDULER(void* args)
{
	TestManagerImpl* pTM=reinterpret_cast<TestManagerImpl*>(args);
	assert(pTM);

	try
	{
		while (true)
		{
			CTTime tm;
			tm.now();

			POSITION pos = pTM->m_execInfo.GetStartPosition();
			long   i;
			TestManagerImpl::TGExecInfo info;

			while (pos != NULL)
			{
				pTM->m_execInfo.GetNextAssoc( pos, i, info );

				if (info.status == DELAYED && (unsigned)info.tgStartup.lStartTime <= tm.getSeconds())
				{
				    char strMsg[64];
				    sprintf(strMsg, "Starting scheduled TG: Group %d", info.tgStartup.execInfo.lTaskGroupId);
					LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, strMsg);

					// Activate the IC for use by this test manager instance...
					// localhost is converted in starttaskgroup() so strHost is valid

					info.pIC=NULL;
					info.pWebIC=NULL;

					info.pIC = pTM->ConnectToInjector(info.tgStartup.strHost);

					if (info.pIC == NULL || CORBA::is_nil(info.pIC))
					{
						// CAN'T CONNECT TO IT ON LOCAL LAN 

						info.pIC=NULL;
						info.pWebIC = pTM->ConnectToInjectorViaWeb(info.tgStartup.strHost);

						if(info.pWebIC == NULL || CORBA::is_nil(info.pWebIC))
						{
							//CAN'T CONNECT VIA WEB EITHER

							info.pIC=NULL;
							info.pWebIC=NULL;

							string strMsg="Failed to connect to Injector Control on host "+string(info.tgStartup.strHost);
							CECErrorLog err(0, pTM->m_strTestName.c_str(), "TestManager::Scheduler", strMsg.c_str());
							pTM->GetRepository()->m_pTR->AddTestOutput(CECErrorLog::OUTPUT_TYPE, err.ToString().c_str());

							LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Scheduler couldn't connect to injector");
							info.status = FAILED;
							pTM->m_execInfo[i]=info;

							CString strTaskNumber;
							strTaskNumber.Format("Task Group %d failed to start (Couldn't connect to host \"%s\")",info.tgStartup.execInfo.lTaskGroupId+1,info.tgStartup.strHost);
							CECTestLog audit(0,(const char*)strTaskNumber.GetBuffer(0));
							pTM->GetRepository()->m_pTR->AddTestOutput(CECTestLog::OUTPUT_TYPE, audit.ToString().c_str());
			
							if (pTM->m_execInfo[i].tgStartup.bStopOnComplete && pTM->m_bUseStopOnComplete)
							{
								pTM->m_nStopOnCompleteRemaining--;
								// Make sure that it does not get handled twice!
								pTM->m_execInfo[i].tgStartup.bStopOnComplete = FALSE;
								if (pTM->m_nStopOnCompleteRemaining == 0)
								{
									LOG_MESSAGE(TRCCAT_SHUTDOWN, TRCLEV_MEDIUM, "All TasksGroup with StopOnComplete have finished. Calling StopTest");
									pTM->StopTest(false, 30000);
								}
							}
						}
						else
						{						
							//CONNECTED BY XML RELAY

							TaskGroupExecInfo EI = info.tgStartup.execInfo;
							LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Calling StartTaskGroup now on the web");
							
							bool bStarted = FALSE;
							try
							{
								bStarted = info.pWebIC->StartTaskGroup(info.tgStartup.strHost, EI);
							}
							catch(...)
							{
								bStarted=FALSE;
							}

							if (bStarted)
							{
								CString strTaskNumber;
								strTaskNumber.Format("Task Group %d Started via the web",info.tgStartup.execInfo.lTaskGroupId+1);
								CECTestLog audit(0,(const char*)strTaskNumber.GetBuffer(0));
								pTM->GetRepository()->m_pTR->AddTestOutput(CECTestLog::OUTPUT_TYPE, audit.ToString().c_str());
							
								// Task group has started sucessfully, set to active!
								LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Started sucessfully! Now ACTIVE via the web");
								info.status = ACTIVE;
								pTM->m_execInfo[i] = info;
							}
							else
							{
								info.status = INACTIVE;
								pTM->m_execInfo[i] = info;
							
								CString strTaskNumber;
								strTaskNumber.Format("Task Group %d failed to start (Injector couldn't start Task Group)",info.tgStartup.execInfo.lTaskGroupId+1);
								CECTestLog audit(0,(const char*)strTaskNumber.GetBuffer(0));
								pTM->GetRepository()->m_pTR->AddTestOutput(CECTestLog::OUTPUT_TYPE, audit.ToString().c_str());
							}
						}
						
					}
					else
					{
						// IT'S ON THE LOCAL LAN

						info.pWebIC=NULL;

						TaskGroupExecInfo EI = info.tgStartup.execInfo;
						LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Calling IC->StartTaskGroup..");
						bool bStarted = info.pIC->StartTaskGroup(EI);
						if (bStarted)
						{
							CString strTaskNumber;
							strTaskNumber.Format("Task Group %d Started",info.tgStartup.execInfo.lTaskGroupId+1);
							CECTestLog audit(0,(const char*)strTaskNumber.GetBuffer(0));
							pTM->GetRepository()->m_pTR->AddTestOutput(CECTestLog::OUTPUT_TYPE, audit.ToString().c_str());
							
							// Task group has started sucessfully, set to active!
							LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Started sucessfully! Now ACTIVE on local LAN");
							info.status = ACTIVE;
							pTM->m_execInfo[i] = info;

						}
						else
						{
							info.status = INACTIVE;
							pTM->m_execInfo[i] = info;
							
							CString strTaskNumber;
							strTaskNumber.Format("Task Group %d failed to start (Injector couldn't start Task Group)",info.tgStartup.execInfo.lTaskGroupId+1);
							CECTestLog audit(0,(const char*)strTaskNumber.GetBuffer(0));
							pTM->GetRepository()->m_pTR->AddTestOutput(CECTestLog::OUTPUT_TYPE, audit.ToString().c_str());
						}
					}

					pTM->m_nDelayedCount--;
				}
			}
	
			::Sleep(1000);
		}
	}
	catch(...)
	{
		LOG_MESSAGE(TRCCAT_RUNNING, TRCLEV_NONE, "Exception: TestManagerImpl_SCHEDULER");
	}
	return NULL;
}

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
TestManagerImpl::TestManagerImpl(CORBAServer* pServer)
        :m_pServer(pServer), m_lCachedTotalVUs(0)
{
	LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Constructing TestManager Interface");
	m_nPID = _getpid();
	m_pRepos = NULL;
	m_pScheduler = NULL;
	m_nDelayedCount = 0;
	m_bUseStopOnComplete = FALSE;
	m_nStopOnCompleteRemaining = 0;
	m_execInfo.RemoveAll();

    char strReg[64];
    CECInetInfo inet;
    sprintf(strReg, "%s_%8.8lX", inet.GetIPAddressAsString().c_str(), getpid());
	m_strReg = strReg;

	//assuming tm  module is running on repository - this machine
	m_strXMLreg = strXMLPrefix;

    m_pServer->ActivateObject(this);
    m_pServer->NameObject(this->_this(),TestManager::CLASS_NAME,m_strReg.c_str());

	string strMsg = "TestManager Interface registered with name: ";
	strMsg += m_strReg;
	LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, strMsg);
}

TestManagerImpl::TestManagerImpl(CORBAServer* pServer,BOOL registerServer)
        :m_pServer(pServer), m_lCachedTotalVUs(0)
{
	LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Constructing TestManager Interface");
	m_nPID = _getpid();
	m_pRepos = NULL;
	m_pScheduler = NULL;
	m_nDelayedCount = 0;
	m_bUseStopOnComplete = FALSE;
	m_nStopOnCompleteRemaining = 0;
	m_execInfo.RemoveAll();

    char strReg[64];
    CECInetInfo inet;
    sprintf(strReg, "%s_%8.8lX", inet.GetIPAddressAsString().c_str(), getpid());
	m_strReg = strReg;

	//assuming tm  module is running on repository - this machine	
	m_strXMLreg = strXMLPrefix;

	if(registerServer)
	{
		m_pServer->ActivateObject(this);
		m_pServer->NameObject(this->_this(),TestManager::CLASS_NAME,m_strReg.c_str());
		string strMsg = "TestManager Interface registered with name: ";
		strMsg += m_strReg;
		LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, strMsg);
	}
}

TestManagerImpl::~TestManagerImpl()
{

}

BOOL TestManagerImpl::Initialize(string strTest)
{
	// Create the repository
	try
	{
		LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Create Repository connection");
		m_strTestName = strTest;
		m_pRepos = new CLocalRepository(m_pServer, strTest, m_nPID);
		m_strTestRunString = m_pRepos->GetTestRunString();

		CECTestLog audit(0,string("TestManager Initialized for Test "+strTest).c_str());

		m_pRepos->m_pTR->AddTestOutput(CECTestLog::OUTPUT_TYPE, audit.ToString().c_str());

		// Start Schedulder thread
		m_pScheduler = new omni_thread(&TestManagerImpl_SCHEDULER ,this);
	    m_pScheduler->start();

		// Start Stop Controller thread
	}
	catch(...)
	{
		LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_NONE, "Exception: Create Repository connection");
	}
	return true;
}

void TestManagerImpl::CloseDown()
{
	try
	{
		LOG_MESSAGE(TRCCAT_SHUTDOWN, TRCLEV_MEDIUM, "Clear Variables");
	    CECInetInfo inet;
		CORBA::Object_var Obj = m_pServer->FindObjectWithName(VarFactory::CLASS_NAME, inet.GetIPAddressAsString().c_str());
		VarFactory_var VF = VarFactory::_narrow(Obj);
		VF->ClearAll();

		LOG_MESSAGE(TRCCAT_SHUTDOWN, TRCLEV_MEDIUM, "Deleteing repository");
		if(m_pRepos)
			delete m_pRepos;
	}
	catch(...)
	{
		LOG_MESSAGE(TRCCAT_SHUTDOWN, TRCLEV_NONE, "Exception: Deleteing repository");
	}

	try
	{
		LOG_MESSAGE(TRCCAT_SHUTDOWN, TRCLEV_MEDIUM, "De-registering TestManager Interface");
	    m_pServer->UnNameObject(TestManager::CLASS_NAME,m_strReg.c_str());
	}
	catch(...)
	{
		LOG_MESSAGE(TRCCAT_SHUTDOWN, TRCLEV_NONE, "Exception: De-registering TestManager Interface");
	}
}

char* TestManagerImpl::GetTestRunFolderName()
{
	if(m_pRepos)
		return CORBA::string_dup(m_pRepos->GetRunFolderName().c_str());
	return CORBA::string_dup("");
}

CORBA::Boolean TestManagerImpl::StartTaskGroup(TaskGroupStartupInfo& tgInfo)
{
	LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "StartTaskGroup invoked");

	string strLev;
	if(tgInfo.execInfo.lStatsLevel == stalev_none)
	{
		strLev = "NONE";
	}
	else if(tgInfo.execInfo.lStatsLevel == stalev_low)
	{
		strLev = "LOW";
	}
	else if(tgInfo.execInfo.lStatsLevel == stalev_high)
	{
		strLev = "HIGH";
	}
	LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, string("TaskGroup StatLevel = ")+strLev);

	// Look for an existing entry in the map for this task group
	TGExecInfo info;
	BOOL bLookup = m_execInfo.Lookup(tgInfo.execInfo.lTaskGroupId, info);

	// If the entry exists....
	if(bLookup)
	{
		// ... and it's active or delayed -> return
		if (info.status == ACTIVE || info.status == DELAYED || info.status == STOPPING)
		{
			CString strTaskNumber;
			strTaskNumber.Format("Task Group %d failed to start (Task Group is already active)",tgInfo.execInfo.lTaskGroupId+1);
			CECTestLog audit(0,(const char*)strTaskNumber.GetBuffer(0));
			m_pRepos->m_pTR->AddTestOutput(CECTestLog::OUTPUT_TYPE, audit.ToString().c_str());

			return false;
		}
	}

	// ... if it is STOPPED/Failed/complete/waiting enabled then copy in the new execInfo (may have been changed)
	// and prepare to start the test

	CString strHost;
	strHost=tgInfo.strHost;
	strHost.MakeLower();	//case sensitive
	strHost.TrimRight();	//remove trailing whitespace

	if(strHost.IsEmpty())
	{
		//if no host entered use localhost
		strHost="localhost";
	}

	if (strHost == string("localhost").c_str())
	{
	    CECInetInfo inet;
	    tgInfo.strHost=inet.GetHostName().c_str();
	}
	info.tgStartup = tgInfo;
	info.status = INACTIVE;
	info.pIC = NULL;
	info.pWebIC = NULL;
	m_execInfo.SetAt(tgInfo.execInfo.lTaskGroupId, info);
			
	// Connect to the InjectorControl object on the host machine
	// host could have changed so reconnect

	if (tgInfo.lStartTime == 0)
	{
		//don't bother looking for delayed task groups
		info.pIC = ConnectToInjector(info.tgStartup.strHost);
		if(info.pIC == NULL || CORBA::is_nil(info.pIC))
		{
			// CAN'T CONNECT ON LAN

			info.pIC = NULL;
			info.pWebIC = ConnectToInjectorViaWeb(info.tgStartup.strHost);

			if(info.pWebIC == NULL || CORBA::is_nil(info.pWebIC))
			{
				//CAN'T CONNECT VIA WEB EITHER

				info.pWebIC = NULL;

				string strMsg="Failed to connect to Injector Control on host "+string(info.tgStartup.strHost);
				CECErrorLog err(0, m_strTestName.c_str(), "TestManager::StartTaskGroup", strMsg.c_str());
				m_pRepos->m_pTR->AddTestOutput(CECErrorLog::OUTPUT_TYPE, err.ToString().c_str());	

				info.status = FAILED;
				m_execInfo.SetAt(tgInfo.execInfo.lTaskGroupId, info);

				CString strTaskNumber;
				strTaskNumber.Format("Task Group %d failed to start (Couldn't connect to host \"%s\")",tgInfo.execInfo.lTaskGroupId+1,info.tgStartup.strHost);
				CECTestLog audit(0,(const char*)strTaskNumber.GetBuffer(0));
				m_pRepos->m_pTR->AddTestOutput(CECTestLog::OUTPUT_TYPE, audit.ToString().c_str());

			}
			else
			{
				//CONNECTED BY XML RELAY

				TaskGroupExecInfo EI = info.tgStartup.execInfo;

				LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Calling StartTaskGroup now on WEB");
							
				bool bStarted = FALSE;
				try
				{
					bStarted = info.pWebIC->StartTaskGroup(info.tgStartup.strHost, EI);
				}
				catch(...)
				{
					bStarted=FALSE;
				}

				if (bStarted)
				{
					// Task group has started sucessfully via WEB, set to active!
					LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Started sucessfully on WEB! Now ACTIVE");
					info.status = ACTIVE;
					m_execInfo.SetAt(tgInfo.execInfo.lTaskGroupId, info);

					if (info.tgStartup.bStopOnComplete)
					{
						m_bUseStopOnComplete = true;
						m_nStopOnCompleteRemaining++;
					}
	
					CString strTaskNumber;
					strTaskNumber.Format("Task Group %d started via the web",tgInfo.execInfo.lTaskGroupId+1);
					CECTestLog audit(0,(const char*)strTaskNumber.GetBuffer(0));
					m_pRepos->m_pTR->AddTestOutput(CECTestLog::OUTPUT_TYPE, audit.ToString().c_str());
					return bStarted;
				}
				else
				{
					//couldnt be started via WEB
					info.status = FAILED;
					m_execInfo.SetAt(tgInfo.execInfo.lTaskGroupId, info);
	
					CString strTaskNumber;
					strTaskNumber.Format("Task Group %d failed to start (Injector couldn't start Task Group)",tgInfo.execInfo.lTaskGroupId+1);
					CECTestLog audit(0,(const char*)strTaskNumber.GetBuffer(0));
					m_pRepos->m_pTR->AddTestOutput(CECTestLog::OUTPUT_TYPE, audit.ToString().c_str());
					LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Failed to start on WEB!");		
				}
			}
			return false;
		}
		
		// CONNECT OVER LAN
		//
		// If no delay has been set start the task group immediately by calling the
		// Injector Control object on the target machine.		
		
		try
		{
			LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Starting task group now on LAN");

			TaskGroupExecInfo EI = tgInfo.execInfo;
			bool bStarted = info.pIC->StartTaskGroup(EI);
			if (bStarted)
			{
				// Task group has started sucessfully, set to active!
				LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Started sucessfully on LAN! Now ACTIVE");
				info.status = ACTIVE;
				m_execInfo.SetAt(tgInfo.execInfo.lTaskGroupId, info);

				if (info.tgStartup.bStopOnComplete)
				{
					m_bUseStopOnComplete = true;
					m_nStopOnCompleteRemaining++;
				}

				CString strTaskNumber;
				strTaskNumber.Format("Task Group %d started",tgInfo.execInfo.lTaskGroupId+1);
				CECTestLog audit(0,(const char*)strTaskNumber.GetBuffer(0));
				m_pRepos->m_pTR->AddTestOutput(CECTestLog::OUTPUT_TYPE, audit.ToString().c_str());

				return bStarted;
			}
			else
			{
				info.status = FAILED;
				m_execInfo.SetAt(tgInfo.execInfo.lTaskGroupId, info);

				CString strTaskNumber;
				strTaskNumber.Format("Task Group %d failed to start (Injector couldn't start Task Group)",tgInfo.execInfo.lTaskGroupId+1);
				CECTestLog audit(0,(const char*)strTaskNumber.GetBuffer(0));
				m_pRepos->m_pTR->AddTestOutput(CECTestLog::OUTPUT_TYPE, audit.ToString().c_str());
			}

			LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Failed to start on LAN!");
		}
		catch(...)
		{
			LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_NONE, "Exception: StartTaskGroup - Immediate start on LAN");
		}
	}
	else
	{
		// Requires scheduling! Just add the TGExecInfo into the vector and let the
		// Scheduler thread start the task group

		LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Scheduling TaskGroup. Now DELAYED");
		info.status = DELAYED;
		m_nDelayedCount++;
		m_execInfo.SetAt(tgInfo.execInfo.lTaskGroupId, info);
		if (info.tgStartup.bStopOnComplete)
		{
			m_bUseStopOnComplete = true;
			m_nStopOnCompleteRemaining++;
		}
		return true;
	}

	return false;
}

TestExecuter_ptr TestManagerImpl::GetTaskGroupExecuter(long lTaskGroupId)
{
	try
	{
		LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Looking for IC responsible for this task group");
		TGExecInfo info;
		if(m_execInfo.Lookup(lTaskGroupId,info))
		{		
			if (info.tgStartup.execInfo.lTaskGroupId == lTaskGroupId)
			{
				if((info.pIC != NULL)&&(!(CORBA::is_nil(info.pIC))))
				{
					//on LAN
					LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Getting TestExecuter via LAN");
					return info.pIC->GetTaskGroupExecuter(lTaskGroupId);
				}
				else if((info.pWebIC != NULL)&&(!(CORBA::is_nil(info.pWebIC))))
				{
					//via WEB
					LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Getting TestExecuter via WEB");
					try
					{
						return false;//info.pWebIC->GetTaskGroupExecuter(info.tgStartup.strHost,lTaskGroupId);
					}
					catch(...)
					{
						LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Exception: GetTaskGroupExecuter via web");
					}
				}
			}
		}
	}
	catch(...)
	{
		LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_NONE, "Exception: GetTaskGroupExecuter");
	}

	return TestExecuter::_nil();
}

CORBA::Boolean TestManagerImpl::StopTaskGroup(long lTaskGroupId)
{
	try
	{
		LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "StopTaskGroup: Looking for IC responsible for this task group");

		TGExecInfo info;
		if(m_execInfo.Lookup(lTaskGroupId,info))
		{
			if (info.tgStartup.execInfo.lTaskGroupId == lTaskGroupId)
			{
				if (info.status == ACTIVE)
				{
					if((info.pIC!=NULL)&&(!(CORBA::is_nil(info.pIC))))
					{
						// it's a local pIC so attempt on LAN
						LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "calling StopTaskGroup on LAN Injector control");
					
						if(info.pIC->StopTaskGroup(lTaskGroupId))
						{
							info.status = STOPPING;
							m_execInfo[lTaskGroupId] = info;
							return true;
						}
						else
						{
							//it's not stopping
							return false;
						}
					}
					else if((info.pWebIC!=NULL)&&(!(CORBA::is_nil(info.pWebIC))))
					{
						// try and use XML relay to call StopTaskGroup()
						LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "calling StopTaskGroup on WEB Injector control");
							
						try
						{
							if(info.pWebIC->StopTaskGroup(info.tgStartup.strHost,lTaskGroupId))
							{
								//it is stoppping
								info.status = STOPPING;
								m_execInfo[lTaskGroupId] = info;
								return true;
							}
							else
							{
								//it's not stopping
								return false;
							}
						}
						catch(...)
						{
							//exception couldn't find XML relay?
							return false;
						}
					}
				}
				else if (info.status == DELAYED)
				{
					info.status = STOPPED;
					m_execInfo[lTaskGroupId] = info;
					m_nDelayedCount--;
					if (m_execInfo[lTaskGroupId].tgStartup.bStopOnComplete && m_bUseStopOnComplete)
					{
						m_nStopOnCompleteRemaining--;
						// Make sure that it does not get handled twice!
						m_execInfo[lTaskGroupId].tgStartup.bStopOnComplete = FALSE;
			
						if (m_nStopOnCompleteRemaining == 0)
						{
							LOG_MESSAGE(TRCCAT_SHUTDOWN, TRCLEV_MEDIUM, "All TasksGroup with StopOnComplete have finished. Calling StopTest");
							StopTest(false, 30000);
						}
					}
					return true;
				}
			}
		}
	}
	catch(...)
	{
		LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Exception: StopTaskGroup");
	}
	return false;
}

CORBA::Boolean TestManagerImpl::StopTest(CORBA::Boolean bForce, long lTimeout)
{
	LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "In StopTest");
	if((unsigned int)m_execInfo.GetCount()==0)
	{
		LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Number of enabled Task Groups is zero");
		blocking_semaphore.unlock();
		return false;
	}

	try
	{
		POSITION pos = m_execInfo.GetStartPosition();
		long    i;
		TestManagerImpl::TGExecInfo info;

		while (pos != NULL)
		{
			m_execInfo.GetNextAssoc( pos, i, info );
		
			if((info.pIC != NULL)&&(!(CORBA::is_nil(info.pIC))))
			{
				// assumes it always works! always returns false!
				info.pIC->StopTest(false,0);
				if (info.status == ACTIVE)
				{
					info.status = STOPPING;
					m_execInfo[i] = info;
				}
				else if (info.status == DELAYED)
				{
					info.status = STOPPED;
					m_execInfo[i] = info;
					m_nDelayedCount--;
				}
			}
			else if((info.pWebIC != NULL)&&(!(CORBA::is_nil(info.pWebIC))))
			{
				try
				{
					// assumes it always works! always returns false!
					info.pWebIC->StopTest(info.tgStartup.strHost,false,0);
					if (info.status == ACTIVE)
					{
						info.status = STOPPING;
						m_execInfo[i] = info;
					}
					else if (info.status == DELAYED)
					{
						info.status = STOPPED;
						m_execInfo[i] = info;
						m_nDelayedCount--;
					}
				}
				catch(...)
				{
					//couldnt get XML relay dmn from the nameserver so failed on web
				}
			}
		}
		if(m_injectorsInUse.size()==0)
		{
			LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Total number of ICs for test is zero");
			blocking_semaphore.unlock();
		}
	}
	catch(...)
	{
		LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_NONE, "Exception: StopTest");
	}	
	return false;
}

CORBA::Boolean TestManagerImpl::GetStatsSnapshot(TESnapList& TestSnapList)
{
	CTLockGuard<CTMutex> lock(m_InjListMutex);
	try
	{
	    for (int i = 0; i < (int)m_injectorsInUse.size(); i++)
		{
	        InjectorHost inj = m_injectorsInUse[i];

			if(inj.bOnLocalLAN==TRUE)
			{
				//on LAN
				LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "GetStatsSnapshot: on LAN");
				TESnapList InjectorSnapList;
				InjectorStatus_ptr temp = InjectorStatus::_narrow(inj.pIS);

				boolean retsts = temp->GetStatsSnapshot(InjectorSnapList);
				int curLen = TestSnapList.length();
				TestSnapList.length(curLen + InjectorSnapList.length());
		
				for (int snap = 0; snap < (int)InjectorSnapList.length(); snap++)
				{
					TestSnapList[curLen + snap] = InjectorSnapList[snap];
				}
			}
			else if(inj.bOnWEB==TRUE)
			{
				//on WEB
				LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "GetStatsSnapshot: on WEB");
				TESnapList InjectorSnapList;
				
				boolean retsts = inj.pWebIS->GetStatsSnapshot(inj.strHostName.c_str(),InjectorSnapList);
				int curLen = TestSnapList.length();
				TestSnapList.length(curLen + InjectorSnapList.length());
		
				for (int snap = 0; snap < (int)InjectorSnapList.length(); snap++)
				{
					TestSnapList[curLen + snap] = InjectorSnapList[snap];
				}
			}
			else
			{
				//not on lan or web
				LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "GetStatsSnapshot:No connection to IS on web or LAN");
			}
	    }
	}
	catch(...)
	{
		LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_NONE, "Exception: GetStatsSnapshot");
	}
    return true;
}

long TestManagerImpl::RefreshTestVUs()
{
	long nTotalVUs = 0;
	try
	{
		CTLockGuard<CTMutex> lock(m_InjListMutex);
		for (int i = 0; i < (int)m_injectorsInUse.size(); i++)
		{
			InjectorHost inj = m_injectorsInUse[i];
			if(inj.bOnLocalLAN==TRUE)
			{
				// on LAN
				LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "RefreshTestVUs: on LAN");
				nTotalVUs += inj.pIS->GetInjectorVUs(m_lCachedTotalVUs);
			}
			else if(inj.bOnWEB==TRUE)
			{
				//on WEB
				LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "RefreshTestVUs: on WEB");
				nTotalVUs += inj.pWebIS->GetInjectorVUs(inj.strHostName.c_str(),m_lCachedTotalVUs);
			}
			else
			{
				//not on lan or web
				LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "RefreshTestVUs:No connection to IS on web or LAN");
			}
		}
        m_lCachedTotalVUs = nTotalVUs;  // Cache the new value.
	}
	catch(...)
	{
		LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_NONE, "Exception: RefreshTestVUs");
	}

    return m_lCachedTotalVUs;
}

long TestManagerImpl::GetTestVUs()
{
	return m_lCachedTotalVUs;   // Just return the cached value (no locking needed under Windows for a 32-bit value).
}

void TestManagerImpl::OnTaskGroupStopped(long lTaskGroupId, TaskGroupStopReason& eReason)
{
    char strMsg[64];
    sprintf(strMsg, "TaskGroup %d has stopped", lTaskGroupId);
	LOG_MESSAGE(TRCCAT_SHUTDOWN, TRCLEV_MEDIUM, strMsg);

	TestManagerImpl::TGExecInfo info;
	if(m_execInfo.Lookup(lTaskGroupId,info))
	{
    
		if (m_execInfo[lTaskGroupId].tgStartup.execInfo.lTaskGroupId == lTaskGroupId)
		{
			if(eReason == tgstop_stopped)
			{
				m_execInfo[lTaskGroupId].status = STOPPED;
				CString strTaskNumber;
				strTaskNumber.Format("Task Group %d stopped",lTaskGroupId+1);
				CECTestLog audit(0,(const char*)strTaskNumber.GetBuffer(0));
				m_pRepos->m_pTR->AddTestOutput(CECTestLog::OUTPUT_TYPE, audit.ToString().c_str());
			}
			else if (eReason == tgstop_completed)
			{
				m_execInfo[lTaskGroupId].status = COMPLETED;
				CString strTaskNumber;
				strTaskNumber.Format("Task Group %d completed",lTaskGroupId+1);
				CECTestLog audit(0,(const char*)strTaskNumber.GetBuffer(0));
				m_pRepos->m_pTR->AddTestOutput(CECTestLog::OUTPUT_TYPE, audit.ToString().c_str());
			}

			if (m_execInfo[lTaskGroupId].tgStartup.bStopOnComplete && m_bUseStopOnComplete)
			{
				m_nStopOnCompleteRemaining--;
				// Make sure that it does not get handled twice!
				m_execInfo[lTaskGroupId].tgStartup.bStopOnComplete = FALSE;
			}
		}
	}

	if (m_bUseStopOnComplete && m_nStopOnCompleteRemaining == 0)
	{
		LOG_MESSAGE(TRCCAT_SHUTDOWN, TRCLEV_MEDIUM, "All TasksGroup with StopOnComplete have finished. Calling StopTest");
		StopTest(false, 30000);
	}
}

void TestManagerImpl::OnInjectorStopped(const char* strInjectorHost)
{
	string strMsg = "Injector has completed on host machine ";
	strMsg += strInjectorHost;
	LOG_MESSAGE(TRCCAT_SHUTDOWN, TRCLEV_MEDIUM, strMsg);

	CTLockGuard<CTMutex> lock(m_InjListMutex);
	for (unsigned int i = 0; i<m_injectorsInUse.size(); i++)
	{
		CString strOne;
		CString strTwo;

		//what if the strings are an IP address?
		CECInetInfo internetInfo;
		strOne=string(internetInfo.GetIPAddressAsString(m_injectorsInUse[i].strHostName)).c_str();
		strTwo=string(internetInfo.GetIPAddressAsString(string(strInjectorHost))).c_str();
		
		if (strOne == strTwo)
		{
			m_injectorsInUse[i].bComplete = true;
			LOG_MESSAGE(TRCCAT_SHUTDOWN, TRCLEV_MEDIUM, "Release Injector Status");
			CORBA::release(m_injectorsInUse[i].pIS);
			//break;
			//KM: incase more than one occurence of each Inj
			//if activate but not started never stops!!!
		}
	}

	BOOL bAllComplete = true;
	for (i = 0; i<m_injectorsInUse.size(); i++)
	{
		if (!m_injectorsInUse[i].bComplete)
		{
			bAllComplete = false;
			LOG_MESSAGE(TRCCAT_SHUTDOWN, TRCLEV_MEDIUM, string("Injector ")+m_injectorsInUse[i].strHostName+" NOT COMPLETE");
		}
		else
		{
			LOG_MESSAGE(TRCCAT_SHUTDOWN, TRCLEV_MEDIUM, string("Injector ")+m_injectorsInUse[i].strHostName+" COMPLETE");
		}
	}

	LOG_MESSAGE(TRCCAT_SHUTDOWN, TRCLEV_MEDIUM, "Checking if all injectors complete");
	if(bAllComplete && m_nDelayedCount == 0)
	{
		long nCurFiles = m_pRepos->GetNumResultsFiles();
		CECInetInfo inet;
		while(m_injectorsInUse.size()>0)
		{
			LOG_MESSAGE(TRCCAT_SHUTDOWN, TRCLEV_MEDIUM, string("Transfer Results Files from injector ")+m_injectorsInUse[0].strHostName);
			BOOL bUseWeb = false;

			try
			{
				LOG_MESSAGE(TRCCAT_SHUTDOWN, TRCLEV_MEDIUM, string("Attempt Transfer On LAN"));
			    string strICReg=inet.GetIPAddressAsString(m_injectorsInUse[0].strHostName);
	
				CORBA::Object_ptr icObj = m_pServer->FindObjectWithName(InjectorControl::CLASS_NAME,strICReg.c_str());
				if (!CORBA::is_nil(icObj))
				{
					//on local LAN
					InjectorControl_var pIC = InjectorControl::_narrow(icObj);
					long nFiles = pIC->TransferResults(m_strTestName.c_str(), inet.GetHostName().c_str(), m_pRepos->GetRunFolderName().c_str());
					nCurFiles+=nFiles;
				}
				else
				{
				}
			}
			catch(...)
			{
				LOG_MESSAGE(TRCCAT_SHUTDOWN, TRCLEV_NONE, string("Exception: Transfer files on LAN from ")+m_injectorsInUse[0].strHostName);
				bUseWeb = true;
			}

			if(bUseWeb)
			{
				try
				{
					LOG_MESSAGE(TRCCAT_SHUTDOWN, TRCLEV_MEDIUM, string("Attempt Transfer On WEB"));
					//via the web
					LOG_MESSAGE(TRCCAT_SHUTDOWN, TRCLEV_NONE,"TM: OnInjectorStopped Transfer files via the web");
					XMLRelayInjectorControl_ptr pWebIC = NULL;
					CORBA::Object_var obj=m_pServer->FindObjectWithName(XMLRelayInjectorControl::CLASS_NAME, m_strXMLreg.c_str()); 
					pWebIC = XMLRelayInjectorControl::_narrow(obj);
					long nFiles = pWebIC->TransferResults(m_injectorsInUse[0].strHostName.c_str(), m_strTestName.c_str(), inet.GetHostName().c_str(), m_pRepos->GetRunFolderName().c_str());
					nCurFiles+=nFiles;

				}
				catch(...)
				{
					LOG_MESSAGE(TRCCAT_SHUTDOWN, TRCLEV_NONE, string("Exception: Transfer files on WEB from ")+m_injectorsInUse[0].strHostName);
				}
			}
			m_injectorsInUse.erase(m_injectorsInUse.begin());
		}

		int nRetry = 0;
		while(m_pRepos->GetNumResultsFiles() < nCurFiles && nRetry<10)
		{
			char strMsg[64];
		    sprintf(strMsg, "Waiting for %d results files", nCurFiles-m_pRepos->GetNumResultsFiles());

			LOG_MESSAGE(TRCCAT_SHUTDOWN, TRCLEV_MEDIUM, string(strMsg));
			::Sleep(1000);
			nRetry++;
		}
		LOG_MESSAGE(TRCCAT_SHUTDOWN, TRCLEV_MEDIUM, "All complete. Shutting down");
		blocking_semaphore.unlock();
	}

}

XMLRelayInjectorControl_ptr TestManagerImpl::ConnectToInjectorViaWeb(const char* strHost)
{
	XMLRelayInjectorControl_ptr pWebIC = NULL;

	try
	{
		// go through the map containing the TGExecInfo of task groups already started
		// for a task group on the required machine and return it's InjectorControl object.
		LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Looking for XML IC locally");

		POSITION pos = m_execInfo.GetStartPosition();
		long    i;
		TestManagerImpl::TGExecInfo info;

		while (pos != NULL)
		{
			m_execInfo.GetNextAssoc( pos, i, info );
			if (string(info.tgStartup.strHost) == string(strHost))
			{
				LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Found existing local XML IC");
				pWebIC = info.pWebIC;
				break; //just use the first one found
			}
		}
	
		if(pWebIC == NULL)
		{
			// No task groups running on this machine so narrow the XML relay for the host and call to activate it!
			LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "No reference found, attempting to connect to local XML IC");
		    CECInetInfo inet;
	
			CORBA::Object_var obj=m_pServer->FindObjectWithName(XMLRelayInjectorControl::CLASS_NAME, m_strXMLreg.c_str()); 

			if (!CORBA::is_nil(obj))
			{
				LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Found local IC for web. Narrowing...");
				pWebIC = XMLRelayInjectorControl::_narrow(obj);
			}
			else
			{
				LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Cannot find a local XML IC InjectorControl for web!!");
				return XMLRelayInjectorControl::_nil();
			}
		}

		
		// Make sure the IC is activated for use by this test manager instance...
		if (!pWebIC->IsActivated(strHost,m_strReg.c_str()))
		{
			// ... if not attempt to activate it
			LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Activating foreign IC via web..");
			CECInetInfo inet;
			bool bActivated = pWebIC->Activate(strHost,m_strReg.c_str(), inet.GetHostName().c_str(), m_strTestRunString.c_str(), m_strTestName.c_str());
			if (!bActivated)
			{
				LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Foreign Injector control couldn't be activated via web!!");
			}
			else
			{
			    CECInetInfo inet;
			    string strISReg=inet.GetIPAddressAsString(strHost);
				InjectorHost inj;
				inj.bOnLocalLAN =false; 
				inj.bOnWEB =false; 
				inj.bNarrowedHTTP = true; //the IS XML relay can cope!
				inj.strHostName = strHost;
				inj.bComplete = false;
				
				inj.strClassName="";
				inj.strRegName="";
				inj.pIS = NULL;	//no local injector status available


				CORBA::Object_ptr isObj = m_pServer->FindObjectWithName(XMLRelayInjectorStatus::CLASS_NAME, m_strXMLreg.c_str());
				if (!CORBA::is_nil(isObj))
				{
					LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Found local IS for web. Narrowing...");
					inj.pWebIS = XMLRelayInjectorStatus::_narrow(isObj);

					if (!CORBA::is_nil(inj.pWebIS))
					{
						//it's a valid pointer to the IS XML Relay
						inj.bOnWEB=true;

						LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "XML Relay IS Narrow success");
						//start monitoring
						try
						{
							CString mess;
							mess.Format("IS Start Monitoring via the web (Sample rate set: %d milliseconds)",m_lSampleRate);
							CString strHTTPDataFileName;
							strHTTPDataFileName.Format("%s_%s.dat",strHTTPStatsFile,strISReg);

							inj.pWebIS->StartMonitoring(inj.strHostName.c_str(),strHTTPDataFileName,m_lSampleRate);
							LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, (char*)mess.GetBuffer(0));
						}
						catch(...)
						{
							LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Exception: IS Start monitoring via WEB");
						}
					}
					else
					{
						// couldnt be narrowed
						LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Couldnt narrow to XML IS object");
						inj.pWebIS = XMLRelayInjectorStatus::_nil();
						inj.bOnWEB = false;
						
					}
				}
				else
				{
					// not found on nameservice
					LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Failed to find Injector Status Object via WEB");
					inj.pWebIS = XMLRelayInjectorStatus::_nil();
					inj.bOnWEB = false;
				}
	
				CTLockGuard<CTMutex> lock(m_InjListMutex);
				m_injectorsInUse.push_back(inj);;
			}
		}
	}
	catch(...)
	{
		LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_NONE, "exception - Cannot find an InjectorControl on this host via the web!!");
		return XMLRelayInjectorControl::_nil();
	}

	return XMLRelayInjectorControl::_duplicate(pWebIC);
}


InjectorControl_ptr TestManagerImpl::ConnectToInjector(const char* strHost)
{
	//	return InjectorControl::_nil();
	//MAKE ALL RUN VIA WEB

	InjectorControl_ptr pIC = NULL;
	try
	{
		// go through the map containing the TGExecInfo of task groups already started
		// for a task group on the required machine and return it's InjectorControl object.
		LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Looking for existing reference to IC on required host via LAN.");

		POSITION pos = m_execInfo.GetStartPosition();
		long    i;
		TestManagerImpl::TGExecInfo info;

		while (pos != NULL)
		{
			m_execInfo.GetNextAssoc( pos, i, info );
			if (string(info.tgStartup.strHost) == string(strHost))
			{
				LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Found IC on required host via LAN.");
				pIC = info.pIC;
				break; //just use the first one found
			}
		}
	
		if(pIC == NULL)
		{
			// No task groups running on this machine so narrow the InjectorControl and activate it!
			LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "No IC reference found. Attempting to connect via LAN...");
		    CECInetInfo inet;
		    string strICReg=inet.GetIPAddressAsString(strHost);
	
			CORBA::Object_ptr icObj = m_pServer->FindObjectWithName(InjectorControl::CLASS_NAME,strICReg.c_str());
			if (!CORBA::is_nil(icObj))
			{
				LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Found IC on LAN. Narrowing...");
				pIC = InjectorControl::_narrow(icObj);
			}
			else
			{
				LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Cannot find an InjectorControl on this host via LAN!!");
				return InjectorControl::_nil();
			}
		}
	
		// Make sure the IC is activated for use by this test manager instance...
		if (!pIC->IsActivated(m_strReg.c_str()))
		{
			// ... if not attempt to activate it
			CECInetInfo inet;
			LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Activating IC on LAN..");
			bool bActivated = pIC->Activate(m_strReg.c_str(), inet.GetHostName().c_str(), m_strTestRunString.c_str(), m_strTestName.c_str());
			if (!bActivated)
			{
				LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Injector control is already in use on LAN!!");
			}
			else
			{
			    CECInetInfo inet;
			    string strISReg=inet.GetIPAddressAsString(strHost);
				InjectorHost inj;
				inj.bOnLocalLAN = true ;	//its local on the LAN
				inj.bOnWEB = false;
				inj.pWebIS = NULL;		
				inj.strHostName = strHost;
				inj.bComplete = false;
				inj.bNarrowedHTTP = false;
				inj.strClassName="";
				inj.strRegName="";
				


				CORBA::Object_ptr isObj = m_pServer->FindObjectWithName(InjectorStatus::CLASS_NAME,strISReg.c_str());
				if (!CORBA::is_nil(isObj))
				{
					LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Found IS. Narrowing for HTTP then original...");
					inj.pIS = HTTP_InjectorStatus::_narrow(isObj);
					if (CORBA::is_nil(inj.pIS))
					{
						LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "HTTP-IS Narrow failed");
							
						inj.pIS = InjectorStatus::_narrow(isObj);
						if (CORBA::is_nil(inj.pIS))
						{
							LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "IS Narrow failed");
						}
						else
						{
							LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "IS Narrow success");	
						}
					}
					else
					{
						LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "HTTP-IS Narrow success");

						//store these so the http calls can check it
						//and re narrow if it is possible

						inj.bNarrowedHTTP=true;	
						inj.strClassName=InjectorStatus::CLASS_NAME;
						inj.strRegName=strISReg.c_str();

						//start the monitoring intervals

						try
						{
							HTTP_InjectorStatus_ptr HTTPpIS;
							HTTPpIS = HTTP_InjectorStatus::_narrow(isObj);
							if (!(CORBA::is_nil(HTTPpIS)))
							{	
								CString mess;
								mess.Format("HTTP-IS Start Monitoring (Sample rate set: %d milliseconds)",m_lSampleRate);
								CString strHTTPDataFileName;
								strHTTPDataFileName.Format("%s_%s.dat",strHTTPStatsFile,strISReg);

								HTTPpIS->StartMonitoring(strHTTPDataFileName,m_lSampleRate);
								LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, (char*)mess.GetBuffer(0));
							}
						}
						catch(...)
						{
							LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Exception: HTTP-IS Start monitoring");
						}
					}
				}
				else
				{
					LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Failed to find Injector Status Object on LAN");
					inj.pIS = InjectorStatus::_nil();
					inj.bOnLocalLAN = false;
				}

				CTLockGuard<CTMutex> lock(m_InjListMutex);
				m_injectorsInUse.push_back(inj);;
			}
		}
	}
	catch(...)
	{
		LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_NONE, "Cannot find an InjectorControl on this host on LAN!!");
		return InjectorControl::_nil();
	}

	return InjectorControl::_duplicate(pIC);
}

char* TestManagerImpl::TestName()
{
	return CORBA::string_dup(m_strTestName.c_str());
}

TGExecStatus TestManagerImpl::GetTaskGroupStatus(long lTaskGroupId)
{
	TGExecInfo info;
	if(m_execInfo.Lookup(lTaskGroupId,info))
	{
		if (info.tgStartup.execInfo.lTaskGroupId == lTaskGroupId)
		{
			return info.status;
		}
	}

	return INACTIVE;
}

CORBA::Boolean TestManagerImpl::GetTaskSnapshot(const TaskIdList& taskIDs, TaskSnap2List& taskSnapshots)
{
	bool bRet = false;
	// Build a list of injectors where the specified task groups are being run
	CStringArray arrInj;
	for (int i=0; i<taskIDs.length(); i++)
	{
		TGExecInfo info;
		BOOL bLookup = m_execInfo.Lookup(taskIDs[i].lTaskGroupId, info);
		if (bLookup && info.tgStartup.execInfo.tgtType == tgt_script)
		{
			CString strInj = info.tgStartup.strHost;
			bool bAdd = true;
			for (int j = 0; j<arrInj.GetSize(); j++)
			{
				if (arrInj.GetAt(j) == strInj)
				{
					bAdd = false;
					break;
				}
			}
			if (bAdd)
			{
				arrInj.Add(strInj);
			}
		}

	}

	// Find the InjectorStatus Object on the machine running this task group
	// and forward the call.
	CTLockGuard<CTMutex> lock(m_InjListMutex);
	try
	{
	    for (i = 0; i < (int)m_injectorsInUse.size(); i++)
		{
	        InjectorHost inj = m_injectorsInUse[i];
			for (int j = 0; j<arrInj.GetSize(); j++)
			{
				if(CString(inj.strHostName.c_str()) == arrInj.GetAt(j))
				{
					if(inj.bOnLocalLAN==TRUE)
					{
						LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_HIGH, "Call IS->GetTaskSnapshot on local LAN");
						TaskSnap2List vuSnaps;
						bRet = inj.pIS-> GetTaskSnapshot(taskIDs, vuSnaps);
						if (bRet)
						{
							char strMsg[64];
						    sprintf(strMsg, "Got %d more snapshots", vuSnaps.length());
							LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_HIGH, strMsg);
							int curLen = taskSnapshots.length();
							taskSnapshots.length(curLen + vuSnaps.length());
		
							for (int snap = 0; snap < (int)vuSnaps.length(); snap++)
							{
								taskSnapshots[curLen + snap] = vuSnaps[snap];
							}
						}
					}
					else if(inj.bOnWEB==TRUE)
					{
						//it's on the web
						LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_HIGH, "Call IS->GetTaskSnapshot via WEB");
						TaskSnap2List vuSnaps;
						bRet = inj.pWebIS->GetTaskSnapshot(inj.strHostName.c_str(),taskIDs, vuSnaps);
						if (bRet)
						{
							char strMsg[64];
						    sprintf(strMsg, "Got %d more snapshots", vuSnaps.length());
							LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_HIGH, strMsg);
							int curLen = taskSnapshots.length();
							taskSnapshots.length(curLen + vuSnaps.length());
		
							for (int snap = 0; snap < (int)vuSnaps.length(); snap++)
							{
								taskSnapshots[curLen + snap] = vuSnaps[snap];
							}
						}
					}
					else
					{
						//no connection on web or lan so you wont get any stats from IS
						LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_HIGH, "Call IS->GetTaskSnapshot - No connection to an IS on LAN or WEB");
					}
				}
			}
	    }
	}
	catch(...)
	{
		LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_NONE, "Exception: GetTaskSnapshot");
	}

	return bRet;
}


