/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// XML_IC_TaskExecInfo.cpp: implementation of the CXML_IC_TaskExecInfo class.
//
//////////////////////////////////////////////////////////////////////

#include "XML_IC_TaskExecInfo.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CXML_IC_TaskExecInfo::CXML_IC_TaskExecInfo()
{
	m_bSet = false;
}

CXML_IC_TaskExecInfo::CXML_IC_TaskExecInfo(TaskExecInfo teInfo)
{
	SetValue(teInfo);
}

CXML_IC_TaskExecInfo::~CXML_IC_TaskExecInfo()
{

}

void CXML_IC_TaskExecInfo::SetValue(TaskExecInfo teInfo)
{
	m_teInfo = teInfo;
	m_bSet = true;
}

TaskExecInfo CXML_IC_TaskExecInfo::GetValue()
{
	return m_teInfo;
}

string CXML_IC_TaskExecInfo::GetType()
{
	return "TaskExecInfo";
}

bool CXML_IC_TaskExecInfo::ToXML(string& strXML)
{
	if (!m_bSet)
		return false;

	strXML += "<value>";
	strXML += "<TaskExecInfo>";
	strXML += AddMember("strScriptName",&CXMLString(string(m_teInfo.strScriptName)));
	strXML += AddMember("bIsDuration",&CXMLBool(m_teInfo.bIsDuration));
	strXML += AddMember("lDuration",&CXMLLong(m_teInfo.lDuration));
	strXML += AddMember("lIterations",&CXMLLong(m_teInfo.lIterations));
	strXML += AddMember("bIsFixedDelay",&CXMLBool(m_teInfo.bIsFixedDelay));
	strXML += AddMember("lFixedDelay",&CXMLLong(m_teInfo.lFixedDelay));
	strXML += AddMember("lMinDelay",&CXMLLong(m_teInfo.lMinDelay));
	strXML += AddMember("lMaxDelay",&CXMLLong(m_teInfo.lMaxDelay));
	strXML += "</TaskExecInfo>";
	strXML += "</value>";

	return true;
}

bool CXML_IC_TaskExecInfo::FromXML(XmlParse<char>::XMLTree::NodeElement* pNode)
{
    using XmlParse<char>::XMLTree::NodeChild;
    using XmlParse<char>::XMLTree::NodeElement;
    using XmlParse<char>::XMLTree::NodeText;
	using XmlParse<char>::XmlString;

    XmlParse<char>::XMLTree::Children::const_iterator pChild;
	for(pChild = pNode->m_children.begin();
		pChild != pNode->m_children.end();
		++pChild)
	{
		XmlParse<char>::XMLTree::NodeElement* pMember = NULL;
		pMember = dynamic_cast<XmlParse<char>::XMLTree::NodeElement*>(*pChild);

		const NodeElement* pName = NULL;
		pName = pMember->GetChildElementByName("name");
		string strName = pName->GetFirstChildText()->m_strValue;

		const NodeElement* pValue = NULL;
		pValue = pMember->GetChildElementByName("value");
		XmlParse<char>::XMLTree::NodeElement* pType = NULL;
		pType = dynamic_cast<XmlParse<char>::XMLTree::NodeElement*>(*pValue->FirstChildElement());

		if (strName == "strScriptName")
		{
			CXMLString* pVal = new CXMLString();
			pVal->FromXML(pType);
			m_teInfo.strScriptName=pVal->GetValue().c_str();
		}
		else if (strName == "bIsDuration")
		{
			CXMLBool* pVal = new CXMLBool();
			pVal->FromXML(pType);
			m_teInfo.bIsDuration=pVal->GetValue();
		}
		else if (strName == "lDuration")
		{
			CXMLLong* pVal = new CXMLLong();
			pVal->FromXML(pType);
			m_teInfo.lDuration=pVal->GetValue();
		}
		else if (strName == "lIterations")
		{
			CXMLLong* pVal = new CXMLLong();
			pVal->FromXML(pType);
			m_teInfo.lIterations=pVal->GetValue();
		}
		else if (strName == "bIsFixedDelay")
		{
			CXMLBool* pVal = new CXMLBool();
			pVal->FromXML(pType);
			m_teInfo.bIsFixedDelay=pVal->GetValue();
		}
		else if (strName == "lFixedDelay")
		{
			CXMLLong* pVal = new CXMLLong();
			pVal->FromXML(pType);
			m_teInfo.lFixedDelay=pVal->GetValue();
		}
		else if (strName == "lMinDelay")
		{
			CXMLLong* pVal = new CXMLLong();
			pVal->FromXML(pType);
			m_teInfo.lMinDelay=pVal->GetValue();
		}
		else if (strName == "lMaxDelay")
		{
			CXMLLong* pVal = new CXMLLong();
			pVal->FromXML(pType);
			m_teInfo.lMaxDelay=pVal->GetValue();
		}
	}

	return true;
}

string CXML_IC_TaskExecInfo::AddMember(string name, CXMLParameter* param)
{
	string strXML = "";
	strXML += "<member>";
	strXML += "<name>"+name+"</name>";
	string strTmp;
	param->ToXML(strTmp);
	strXML += strTmp;
	strXML += "</member>";
	return strXML;
}
