/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// XML_IS_TestExecuterSnapshot.cpp: implementation of the CXML_IS_TestExecuterSnapshot class.
//
//////////////////////////////////////////////////////////////////////

#include "XML_IS_TestExecuterSnapshot.h"
#include "XML_IS_TaskGroupSnap.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CXML_IS_TestExecuterSnapshot::CXML_IS_TestExecuterSnapshot()
{
	m_bSet = false;
}

CXML_IS_TestExecuterSnapshot::CXML_IS_TestExecuterSnapshot(TestExecuterSnapshot teSnap)
{
	SetValue(teSnap);
}

CXML_IS_TestExecuterSnapshot::~CXML_IS_TestExecuterSnapshot()
{

}

void CXML_IS_TestExecuterSnapshot::SetValue(TestExecuterSnapshot teSnap)
{
	m_teSnap = teSnap;
	m_bSet = true;
}

TestExecuterSnapshot CXML_IS_TestExecuterSnapshot::GetValue()
{
	return m_teSnap;
}

string CXML_IS_TestExecuterSnapshot::GetType()
{
	return "TestExecuterSnapshot";
}

bool CXML_IS_TestExecuterSnapshot::ToXML(string& strXML)
{
	if (!m_bSet)
		return false;

	strXML += "<value>";
	strXML += "<TestExecuterSnapshot>";
	strXML += AddMember("strTestExecuter",&CXMLString(string(m_teSnap.strTestExecuter)));

	CXMLArray arrTasks;
	for (int i = 0; i<m_teSnap.seqTaskGroupSnaps.length(); i++)
	{
		TaskGroupSnap snap = m_teSnap.seqTaskGroupSnaps[i];
		CXML_IS_TaskGroupSnap* tSnap = new CXML_IS_TaskGroupSnap(snap);
		arrTasks.AddElement(tSnap);
	}
	strXML += AddMember("seqTaskGroupSnaps",&arrTasks);

	strXML += AddMember("lNumberOfActiveVUs",&CXMLLong(m_teSnap.lNumberOfActiveVUs));
	strXML += "</TestExecuterSnapshot>";
	strXML += "</value>";

	return true;
}

bool CXML_IS_TestExecuterSnapshot::FromXML(XmlParse<char>::XMLTree::NodeElement* pNode)
{
    using XmlParse<char>::XMLTree::NodeChild;
    using XmlParse<char>::XMLTree::NodeElement;
    using XmlParse<char>::XMLTree::NodeText;
	using XmlParse<char>::XmlString;

    XmlParse<char>::XMLTree::Children::const_iterator pChild;
	for(pChild = pNode->m_children.begin();
		pChild != pNode->m_children.end();
		++pChild)
	{
		XmlParse<char>::XMLTree::NodeElement* pMember = NULL;
		pMember = dynamic_cast<XmlParse<char>::XMLTree::NodeElement*>(*pChild);

		const NodeElement* pName = NULL;
		pName = pMember->GetChildElementByName("name");
		string strName = pName->GetFirstChildText()->m_strValue;

		const NodeElement* pValue = NULL;
		pValue = pMember->GetChildElementByName("value");
		XmlParse<char>::XMLTree::NodeElement* pType = NULL;
		pType = dynamic_cast<XmlParse<char>::XMLTree::NodeElement*>(*pValue->FirstChildElement());

		if (strName == "strTestExecuter")
		{
			CXMLString* pVal = new CXMLString();
			pVal->FromXML(pType);
			m_teSnap.strTestExecuter=pVal->GetValue().c_str();
		}
		else if (strName == "seqTaskGroupSnaps")
		{
			CXMLArray* pVal = new CXMLArray();
			pVal->FromXML(pType);
			m_teSnap.seqTaskGroupSnaps.length(pVal->GetSize());
			for (int i =0; i<pVal->GetSize(); i++)
			{
				CXML_IS_TaskGroupSnap* pSnap = (CXML_IS_TaskGroupSnap*)pVal->GetAt(i);
				m_teSnap.seqTaskGroupSnaps[i] = pSnap->GetValue();
			}
		}
		else if (strName == "lNumberOfActiveVUs")
		{
			CXMLLong* pVal = new CXMLLong();
			pVal->FromXML(pType);
			m_teSnap.lNumberOfActiveVUs=pVal->GetValue();
		}

	}

	return true;
}

string CXML_IS_TestExecuterSnapshot::AddMember(string name, CXMLParameter* param)
{
	string strXML = "";
	strXML += "<member>";
	strXML += "<name>"+name+"</name>";
	string strTmp;
	param->ToXML(strTmp);
	strXML += strTmp;
	strXML += "</member>";
	return strXML;
}
