/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// TimeServiceHandler.cpp: implementation of the CTimeServiceHandler class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ArchMgrHandler.h"
#include "CECInetInfo.h"
#include "ArchMgr.h"
#include "..\include\Logger.h"
#include "OstaTraceDef.h"

extern CLogger* g_Logger;
extern CCallManager* g_CallManager;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CArchMgrHandler::CArchMgrHandler(CORBAServer* pServer)
{
	m_pCorba = pServer;
}

CArchMgrHandler::~CArchMgrHandler()
{

}

CXMLResponse* CArchMgrHandler::ProcessMethodCall(CXMLRequest* request)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "AM: ProcessMethodCall "+request->m_strFullMethodName);
	if (request->m_strMethod == "FileHostProperties")
	{
		if (request->GetNumParams() == 1)
		{
			if (request->GetParam(0)->GetType() == "string")
			{
				CXMLString* pString = (CXMLString*)request->GetParam(0);
				return FileHostProperties(pString->GetValue());
			}
			else
			{
				CXMLResponse* response = new CXMLResponse();
				response->SetError(CXMLResponse::INVALID_PARAM);
				return response;
			}
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 2)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 2)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}

	}
	else if (request->m_strMethod == "StartOSTADaemon")
	{
		if (request->GetNumParams() == 1)
		{
			if (request->GetParam(0)->GetType() == "string")
			{
				CXMLString* pString = (CXMLString*)request->GetParam(0);
				return StartOSTADaemon(pString->GetValue());
			}
			else
			{
				CXMLResponse* response = new CXMLResponse();
				response->SetError(CXMLResponse::INVALID_PARAM);
				return response;
			}
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 1)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 1)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}

	}
	else if (request->m_strMethod == "StopOSTADaemon")
	{
		if (request->GetNumParams() == 1)
		{
			if (request->GetParam(0)->GetType() == "string")
			{
				CXMLString* pString = (CXMLString*)request->GetParam(0);
				return StopOSTADaemon(pString->GetValue());
			}
			else
			{
				CXMLResponse* response = new CXMLResponse();
				response->SetError(CXMLResponse::INVALID_PARAM);
				return response;
			}
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 1)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 1)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}

	}
	else if (request->m_strMethod == "RestartOSTADaemon")
	{
		if (request->GetNumParams() == 1)
		{
			if (request->GetParam(0)->GetType() == "string")
			{
				CXMLString* pString = (CXMLString*)request->GetParam(0);
				return RestartOSTADaemon(pString->GetValue());
			}
			else
			{
				CXMLResponse* response = new CXMLResponse();
				response->SetError(CXMLResponse::INVALID_PARAM);
				return response;
			}
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 1)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 1)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}

	}
	else if (request->m_strMethod == "Stop")
	{
		if (request->GetNumParams() == 1)
		{
			if (request->GetParam(0)->GetType() == "string")
			{
				CXMLString* pString = (CXMLString*)request->GetParam(0);
				return Stop(pString->GetValue());
			}
			else
			{
				CXMLResponse* response = new CXMLResponse();
				response->SetError(CXMLResponse::INVALID_PARAM);
				return response;
			}
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 1)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 1)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}

	}
	else if (request->m_strMethod == "IsDmnRunning")
	{
		if (request->GetNumParams() == 1)
		{
			if (request->GetParam(0)->GetType() == "string")
			{
				CXMLString* pString = (CXMLString*)request->GetParam(0);
				return IsDmnRunning(pString->GetValue());
			}
			else
			{
				CXMLResponse* response = new CXMLResponse();
				response->SetError(CXMLResponse::INVALID_PARAM);
				return response;
			}
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 1)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 1)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}

	}

	g_Logger->WriteLogMsg(TRCLEV_LOW, "AM: ProcessMethodCall: Error: Method Not Found");
	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::FUNC_NOT_FOUND);
	return response;
}

CXMLResponse* CArchMgrHandler::FileHostProperties(string strTargetHost)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "AM: FileHostProperties");
	try
	{
		// Narrow the time service corba object on the target machine
	    CECInetInfo inet;
	    string strIP = inet.GetIPAddressAsString(strTargetHost);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "AM: FileHostProperties: Narrowing ArchMgr on "+strIP);
		CORBA::Object_var obj = m_pCorba->FindObjectWithName(ArchMgr::CLASS_NAME,strIP.c_str());
		ArchMgr_var AM = ArchMgr::_narrow(obj);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "AM: FileHostProperties: Calling FileHostProperties");
		string strRet = AM->FileHostProperties();
		g_Logger->WriteLogMsg(TRCLEV_HIGH, strRet);


		CXMLResponse* response = new CXMLResponse();
		response->m_params.Add(new CXMLBase64(strRet));
		return response;
	}
	catch(...)
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "AM: FileHostProperties: Exception calling FileHostProperties on ArchMgr");
	}


	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;
}

CXMLResponse* CArchMgrHandler::StartOSTADaemon(string strTargetHost)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "AM: StartOSTADaemon");
	try
	{
		// Narrow the time service corba object on the target machine
	    CECInetInfo inet;
	    string strIP = inet.GetIPAddressAsString(strTargetHost);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "AM: StartOSTADaemon: Narrowing ArchMgr on "+strIP);
		CORBA::Object_var obj = m_pCorba->FindObjectWithName(ArchMgr::CLASS_NAME,strIP.c_str());
		ArchMgr_var AM = ArchMgr::_narrow(obj);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "AM: StartOSTADaemon: Calling StartOSTADaemon");
		bool bRet = AM->StartOSTADaemon();

		CXMLResponse* response = new CXMLResponse();
		response->m_params.Add(new CXMLBool(bRet));
		return response;
	}
	catch(...)
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "AM: StartOSTADaemon: Exception calling StartOSTADaemon on ArchMgr");
	}


	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;
}

CXMLResponse* CArchMgrHandler::StopOSTADaemon(string strTargetHost)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "AM: StopOSTADaemon");
	try
	{
		// Narrow the time service corba object on the target machine
	    CECInetInfo inet;
	    string strIP = inet.GetIPAddressAsString(strTargetHost);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "AM: StopOSTADaemon: Narrowing ArchMgr on "+strIP);
		CORBA::Object_var obj = m_pCorba->FindObjectWithName(ArchMgr::CLASS_NAME,strIP.c_str());
		ArchMgr_var AM = ArchMgr::_narrow(obj);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "AM: StopOSTADaemon: Calling StopOSTADaemon");
		bool bRet = AM->StopOSTADaemon();

		CXMLResponse* response = new CXMLResponse();
		response->m_params.Add(new CXMLBool(bRet));
		return response;
	}
	catch(...)
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "AM: StopOSTADaemon: Exception calling StopOSTADaemon on ArchMgr");
	}


	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;
}

CXMLResponse* CArchMgrHandler::RestartOSTADaemon(string strTargetHost)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "AM: RestartOSTADaemon");
	try
	{
		// Narrow the time service corba object on the target machine
	    CECInetInfo inet;
	    string strIP = inet.GetIPAddressAsString(strTargetHost);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "AM: RestartOSTADaemon: Narrowing ArchMgr on "+strIP);
		CORBA::Object_var obj = m_pCorba->FindObjectWithName(ArchMgr::CLASS_NAME,strIP.c_str());
		ArchMgr_var AM = ArchMgr::_narrow(obj);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "AM: RestartOSTADaemon: Calling RestartOSTADaemon");
		bool bRet = AM->RestartOSTADaemon();

		CXMLResponse* response = new CXMLResponse();
		response->m_params.Add(new CXMLBool(bRet));
		return response;
	}
	catch(...)
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "AM: RestartOSTADaemon: Exception calling RestartOSTADaemon on ArchMgr");
	}


	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;
}

CXMLResponse* CArchMgrHandler::Stop(string strTargetHost)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "AM: Stop");
	try
	{
		// Narrow the time service corba object on the target machine
	    CECInetInfo inet;
	    string strIP = inet.GetIPAddressAsString(strTargetHost);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "AM: Stop: Narrowing ArchMgr on "+strIP);
		CORBA::Object_var obj = m_pCorba->FindObjectWithName(ArchMgr::CLASS_NAME,strIP.c_str());
		ArchMgr_var AM = ArchMgr::_narrow(obj);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "AM: Stop: Calling Stop");
		bool bRet = AM->Stop(true);

		CXMLResponse* response = new CXMLResponse();
		response->m_params.Add(new CXMLBool(bRet));
		return response;
	}
	catch(...)
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "AM: Stop: Exception calling Stop on ArchMgr");
	}


	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;
}

CXMLResponse* CArchMgrHandler::IsDmnRunning(string strTargetHost)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "AM: IsDmnRunning");
	try
	{
	    CECInetInfo inet;
	    string strIP = inet.GetIPAddressAsString(strTargetHost);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "AM: IsDmnRunning: Narrowing InjControl on "+strIP);
		CORBA::Object_ptr obj = m_pCorba->FindObjectWithName("InjectorControl",strIP.c_str());

		bool bRet = false;
		if (!CORBA::is_nil(obj))
			bRet = true;

		CXMLResponse* response = new CXMLResponse();
		response->m_params.Add(new CXMLBool(bRet));
		return response;
	}
	catch(...)
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "AM: Stop: Exception calling Stop on ArchMgr");
	}


	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;
}
