/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// TimeServiceHandler.cpp: implementation of the CTimeServiceHandler class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "TimeServiceHandler.h"
#include "CECInetInfo.h"
#include "TimeService.h"
#include "..\include\Logger.h"
#include "OstaTraceDef.h"

extern CLogger* g_Logger;
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CTimeServiceHandler::CTimeServiceHandler(CORBAServer* pServer)
{
	m_pCorba = pServer;
}

CTimeServiceHandler::~CTimeServiceHandler()
{

}

CXMLResponse* CTimeServiceHandler::ProcessMethodCall(CXMLRequest* request)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "TS: ProcessMethodCall "+request->m_strFullMethodName);
	if (request->m_strMethod == "GetTime")
	{
		if (request->GetNumParams() == 1)
		{
			if (request->GetParam(0)->GetType() == "string")
			{
				CXMLString* pString = (CXMLString*)request->GetParam(0);
				return GetTime(pString->GetValue());
			}
			else
			{
				CXMLResponse* response = new CXMLResponse();
				response->SetError(CXMLResponse::INVALID_PARAM);
				return response;
			}
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 1)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 1)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}

	}

	g_Logger->WriteLogMsg(TRCLEV_LOW, "TS: ProcessMethodCall: Error: Method Not Found");
	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::FUNC_NOT_FOUND);
	return response;
}

CXMLResponse* CTimeServiceHandler::GetTime(string strTargetMachine)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "TS: GetTime");
	try
	{
		// Narrow the time service corba object on the target machine
	    CECInetInfo inet;
	    string strIP = inet.GetIPAddressAsString(strTargetMachine);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "TS: GetTime: Narrowing TimeService on "+strIP);
		CORBA::Object_var obj = m_pCorba->FindObjectWithName(TimeService::CLASS_NAME,strIP.c_str());
		TimeService_var TS = TimeService::_narrow(obj);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "TS: GetTime: Calling GetTime");
		long lTime = TS->GetTime();

		CXMLResponse* response = new CXMLResponse();
		response->m_params.Add(new CXMLLong(lTime));
		return response;
	}
	catch(...)
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "TS: GetTime: Exception calling GetTime on TimeService");
	}


	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;
}