/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// InjectorStatusHandler.cpp: implementation of the CInjectorStatusHandler class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "InjectorStatusHandler.h"
#include "InjectorStatus.h"
#include "HTTPInjectorStatus.h"
#include "CECInetInfo.h"
#include "XML_IS_TaskGroupSnap.h"
#include "XML_IS_TaskSnap.h"
#include "XML_IS_TaskId.h"
#include "XML_IS_TaskSnap2.h"
#include "XML_IS_VUSnap.h"
#include "XML_IS_TestExecuterSnapshot.h"
#include "XML_IS_HTTPExecuterSnap.h"
#include "..\include\Logger.h"
#include "OstaTraceDef.h"

extern CLogger* g_Logger;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CInjectorStatusHandler::CInjectorStatusHandler(CORBAServer* pServer)
{
	m_pCorba = pServer;
}

CInjectorStatusHandler::~CInjectorStatusHandler()
{

}

CXMLResponse* CInjectorStatusHandler::ProcessMethodCall(CXMLRequest* request)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "IS: ProcessMethodCall "+request->m_strFullMethodName);

	if (request->m_strMethod == "GetStatsSnapshot")
	{
		if (request->GetNumParams() == 1)
		{
			CXMLString* pParam1 = (CXMLString*)request->GetParam(0);
			return GetStatsSnapshot(pParam1->GetValue());
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 1)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 1)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}
	}
	else if (request->m_strMethod == "GetInjectorVUs")
	{
		if (request->GetNumParams() == 2)
		{
			CXMLString* pParam1 = (CXMLString*)request->GetParam(0);
			CXMLLong* pParam2 = (CXMLLong*)request->GetParam(1);
			return GetInjectorVUs(pParam1->GetValue(), pParam2->GetValue());
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 2)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 2)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}
	}
	else if (request->m_strMethod == "GetTaskSnapshot")
	{
		if (request->GetNumParams() == 2)
		{
			CXMLString* pParam1 = (CXMLString*)request->GetParam(0);
			//return GetTaskSnapshot(pParam1->GetValue());
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 2)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 2)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}
	}
	else if (request->m_strMethod == "HTTP_GetBothStatsSnapshots")
	{
		if (request->GetNumParams() == 1)
		{
			CXMLString* pParam1 = (CXMLString*)request->GetParam(0);
			return HTTP_GetBothStatsSnapshots(pParam1->GetValue());
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 1)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 1)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}
	}
	else if (request->m_strMethod == "StartMonitoring")
	{
		if (request->GetNumParams() == 3)
		{
			CXMLString* pParam1 = (CXMLString*)request->GetParam(0);
			CXMLString* pParam2 = (CXMLString*)request->GetParam(1);
			CXMLLong* pParam3 = (CXMLLong*)request->GetParam(2);
			return StartMonitoring(pParam1->GetValue(),pParam2->GetValue(),pParam3->GetValue());
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 3)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 3)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}
	}

	g_Logger->WriteLogMsg(TRCLEV_LOW, "IS: ProcessMethodCall: Error: Method Not Found");
	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::FUNC_NOT_FOUND);
	return response;
}


CXMLResponse* CInjectorStatusHandler::GetStatsSnapshot(string strTargetMachine)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "IS: GetStatsSnapshot");
	try
	{
		// Narrow the is service corba object on the target machine
	    CECInetInfo inet;
	    string strIP = inet.GetIPAddressAsString(strTargetMachine);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "IS: GetStatsSnapshot: Narrowing InjectorStatus on "+strIP);
		CORBA::Object_var obj = m_pCorba->FindObjectWithName(InjectorStatus::CLASS_NAME,strIP.c_str());
		InjectorStatus_var IS = InjectorStatus::_narrow(obj);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "IS: GetStatsSnapshot: calling GetStatsSnapshot");

		TESnapList InjectorSnapList;
		bool bValid = IS->GetStatsSnapshot(InjectorSnapList);

		CXMLResponse* response = new CXMLResponse();
		response->m_params.Add(new CXMLBool(bValid));
		if(bValid==true)
		{
			//add the extra data to it
			long lNumExecuters = InjectorSnapList.length();
			response->m_params.Add(new CXMLLong(lNumExecuters));
			for(long a=0 ; a<lNumExecuters ; a++)
			{
				response->m_params.Add(new CXML_IS_TestExecuterSnapshot(InjectorSnapList[a]));
			}
		}
		return response;
	}
	catch(...)
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "IS: GetStatsSnapshot: Exception calling GetStatsSnapshot on InjectorStatus");
	}

	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;
}

CXMLResponse* CInjectorStatusHandler::GetInjectorVUs(string strTargetMachine, long lPreviousTestVUCount)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "IS: GetInjectorVUs");
	try
	{
		// Narrow the is service corba object on the target machine
	    CECInetInfo inet;
	    string strIP = inet.GetIPAddressAsString(strTargetMachine);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "IS: GetStatsSnapshot: Narrowing InjectorStatus on "+strIP);
		CORBA::Object_var obj = m_pCorba->FindObjectWithName(InjectorStatus::CLASS_NAME,strIP.c_str());
		InjectorStatus_var IS = InjectorStatus::_narrow(obj);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "IS: GetInjectorVUs: calling GetInjectorVUs");

		long lVUs = IS->GetInjectorVUs(lPreviousTestVUCount);
		CXMLResponse* response = new CXMLResponse();
		response->m_params.Add(new CXMLLong(lVUs));
		return response;
	}
	catch(...)
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "IS: GetInjectorVUs: Exception calling GetInjectorVUs on InjectorStatus");
	}

	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;
}

CXMLResponse* CInjectorStatusHandler::GetTaskSnapshot(string strTargetMachine, const TaskIdList& taskIDs)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "IS: GetTaskSnapshot");
	try
	{
		// Narrow the is service corba object on the target machine
	    CECInetInfo inet;
	    string strIP = inet.GetIPAddressAsString(strTargetMachine);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "IS: GetStatsSnapshot: Narrowing InjectorStatus on "+strIP);
		CORBA::Object_var obj = m_pCorba->FindObjectWithName(InjectorStatus::CLASS_NAME,strIP.c_str());
		InjectorStatus_var IS = InjectorStatus::_narrow(obj);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "IS: GetTaskSnapshot: calling GetTaskSnapshot");

		TaskSnap2List taskSnapshots;
		bool bValid =IS->GetTaskSnapshot(taskIDs,taskSnapshots);

		CXMLResponse* response = new CXMLResponse();
		response->m_params.Add(new CXMLBool(bValid));

		if(bValid==true)
		{
			//add the extra data to it
			long lNumTaskSnap2s = taskSnapshots.length();
			response->m_params.Add(new CXMLLong(lNumTaskSnap2s));
			for(long b=0 ; b<lNumTaskSnap2s ; b++)
			{
				response->m_params.Add(new CXML_IS_TaskSnap2(taskSnapshots[b]));
			}
		}
		return response;
	}
	catch(...)
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "IS: GetTaskSnapshot: Exception calling GetTaskSnapshot on InjectorStatus");
	}

	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;
}

CXMLResponse* CInjectorStatusHandler::HTTP_GetBothStatsSnapshots(string strTargetMachine)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "IS: HTTP_GetBothStatsSnapshots");
	try
	{
		// Narrow the is service corba object on the target machine
	    CECInetInfo inet;
	    string strIP = inet.GetIPAddressAsString(strTargetMachine);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "IS: HTTP_GetBothStatsSnapshots: Narrowing HTTP InjectorStatus on "+strIP);
		CORBA::Object_var obj = m_pCorba->FindObjectWithName(HTTP_InjectorStatus::CLASS_NAME,strIP.c_str());
		HTTP_InjectorStatus_var HTTPIS = HTTP_InjectorStatus::_narrow(obj);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "HTTPIS: HTTP_GetBothStatsSnapshots: calling HTTP_GetBothStatsSnapshots");

		HTTP_TestExecuterSnapshot_var result;
		TESnapList_var result2;

		bool bValid = HTTPIS->HTTP_GetBothStatsSnapshots(result,result2);

		CXMLResponse* response = new CXMLResponse();
		response->m_params.Add(new CXMLBool(bValid));

		if(bValid==true)
		{
			//the http snap - always one at present
			long lNumHTTPStats =1;
			response->m_params.Add(new CXMLLong(lNumHTTPStats));
			for(long b=0 ; b<lNumHTTPStats ; b++)
			{
				response->m_params.Add(new CXML_IS_HTTPExecuterSnap(result));
			}	

			//the list of other stats
			long lNumStats = result2->length();
			response->m_params.Add(new CXMLLong(lNumStats));
			for(long a=0 ; a<lNumStats ; a++)
			{
				response->m_params.Add(new CXML_IS_TestExecuterSnapshot( result2->operator [] (a)));
			}	
		
		}
		return response;
	}
	catch(...)
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "HTTPIS: HTTP_GetBothStatsSnapshots: Exception calling HTTP_GetBothStatsSnapshots on InjectorStatus");
	}

	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;
}

CXMLResponse* CInjectorStatusHandler::StartMonitoring(string strTargetMachine, string strFilename,long lInterval)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "HTTPIS: StartMonitoring");
	try
	{
		// Narrow the is service corba object on the target machine
	    CECInetInfo inet;
	    string strIP = inet.GetIPAddressAsString(strTargetMachine);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "HTTPIS: StartMonitoring: Narrowing InjectorStatus on "+strIP);
		CORBA::Object_var obj = m_pCorba->FindObjectWithName(HTTP_InjectorStatus::CLASS_NAME,strIP.c_str());
		HTTP_InjectorStatus_var HTTPIS = HTTP_InjectorStatus::_narrow(obj);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "HTTPIS: StartMonitoring: calling StartMonitoring");

		bool retVal= HTTPIS->StartMonitoring(strFilename.c_str(),lInterval);
		CXMLResponse* response = new CXMLResponse();
		response->m_params.Add(new CXMLBool(retVal));
		return response;
	}
	catch(...)
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "HTTPIS: StartMonitoring: Exception calling StartMonitoring on HTTP_InjectorStatus");
	}

	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;
}

