/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include "stdafx.h"
#include "XMLRelayTestManager_i.h"
#include <CECInetInfo.h>
#include "xmlrpc.h"
#include "..\include\Logger.h"
#include "OstaTraceDef.h"

#include <string>
#include <iostream>
#include <fstream>
using namespace std;

extern CLogger* g_Logger;

XMLRelayTestManagerImpl::XMLRelayTestManagerImpl(CORBAServer* corba)
{
    m_registered_name="WebRelay";
    m_corba=corba;

	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayTM: Registering XMLRelayTestManager interface");
	m_corba->ActivateObject(this);
    m_corba->NameObject( this->_this(),XMLRelayTestManager::CLASS_NAME,m_registered_name.c_str());

	m_RelayMap = XMLRelayMap::_nil();
}

XMLRelayTestManagerImpl::~XMLRelayTestManagerImpl()
{
    m_corba->UnNameObject( XMLRelayTestManager::CLASS_NAME,m_registered_name.c_str());
}

void XMLRelayTestManagerImpl::OnTaskGroupStopped(const char* strReposHost, const char* strManagerReg, CORBA::Long lTaskGroupId, TaskGroupStopReason& eReason)
{
	try
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayTM: OnTaskGroupStopped: Relaying OnTaskGroupStopped request for TestManager "+string(strManagerReg));
		if (CORBA::is_nil(m_RelayMap))
		{
			CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
			m_RelayMap = XMLRelayMap::_narrow(obj);
		}
	
		RelayDef def;	
		if (!m_RelayMap->GetRelayForAlias(strReposHost, def))
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayTM: OnTaskGroupStopped: No relay server defined for this host");
			return;
		}
		else
		{
			g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayTM: OnTaskGroupStopped: Relay server is "+string(def.strHost));
		}
	
		
		CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);
	
		CXMLRequest* req = new CXMLRequest();
		req->SetMethodName("TESTMANAGER.OnTaskGroupStopped");
		req->SetTransport(transport);
		req->AddParameter(new CXMLString(strManagerReg));
		req->AddParameter(new CXMLLong(lTaskGroupId));
		req->AddParameter(new CXMLInt((int)eReason));
		CXMLResponse* resp = req->SendRequest();
		if (resp->IsError())
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayTM: OnTaskGroupStopped: Relay Error:"+resp->PrintError());
			return;
		}
	
		delete req;
		delete resp;
	}
	catch(...)
	{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayTM: OnTaskGroupStopped: EXCEPTION");
	}
}

void XMLRelayTestManagerImpl::OnInjectorStopped(const char* strReposHost, const char* strManagerReg, const char* strInjectorHost)
{
	try
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayTM: OnInjectorStopped: Relaying OnTaskGroupStopped request for TestManager "+string(strManagerReg));
		if (CORBA::is_nil(m_RelayMap))
		{
			CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
			m_RelayMap = XMLRelayMap::_narrow(obj);
		}

		RelayDef def;	
		if (!m_RelayMap->GetRelayForAlias(strReposHost, def))
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayTM: OnInjectorStopped: No relay server defined for this host");
			return;
		}
		else
		{
			g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayTM: OnInjectorStopped: Relay server is "+string(def.strHost));
		}

	
		CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);
	
		CXMLRequest* req = new CXMLRequest();
		req->SetMethodName("TESTMANAGER.OnInjectorStopped");
		req->SetTransport(transport);
		req->AddParameter(new CXMLString(strManagerReg));
		req->AddParameter(new CXMLString(strInjectorHost));
		CXMLResponse* resp = req->SendRequest();
		if (resp->IsError())
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayTM: OnInjectorStopped: Relay Error:"+resp->PrintError());
			return;
		}

		delete req;
		delete resp;
	}
	catch(...)
	{
	}
}

CORBA::Long XMLRelayTestManagerImpl::GetTestVUs(const char* strReposHost, const char* strManagerReg)
{
	long lTestVUs = 0;

	try
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayTM: GetTestVUs: Relaying GetTestVUs request for TestManager "+string(strManagerReg));
		if (CORBA::is_nil(m_RelayMap))
		{
			CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
			m_RelayMap = XMLRelayMap::_narrow(obj);
		}

		RelayDef def;	
		if (!m_RelayMap->GetRelayForAlias(strReposHost, def))
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayTM: GetTestVUs: No relay server defined for this host");
			return -1;
		}
		else
		{
			g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayTM: GetTestVUs: Relay server is "+string(def.strHost));
		}

	
		CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

		CXMLRequest* req = new CXMLRequest();
		req->SetMethodName("TESTMANAGER.GetTestVUs");
		req->SetTransport(transport);
		req->AddParameter(new CXMLString(strManagerReg));
		CXMLResponse* resp = req->SendRequest();

		if (resp->IsError())
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayTM: GetTestVUs: Relay Error:"+resp->PrintError());
			return -1;
		}

		if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "i4")
		{
			CXMLLong* pVal = (CXMLLong*)resp->m_params.GetAt(0);
			lTestVUs = pVal->GetValue();
		}

		delete req;
		delete resp;
	}
	catch(...)
	{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayTM: GetTestVUs: EXCEPTION");
	}

    return lTestVUs;
}
