-/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// VarFactoryHandler.cpp: implementation of the CVarFactoryHandler class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "VarFactoryHandler.h"
#include "CECInetInfo.h"
#include "..\CyrDaemon\CyrVDK011\VarFactory.h"
#include "..\include\Logger.h"
#include "OstaTraceDef.h"

extern CLogger* g_Logger;
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CVarFactoryHandler::CVarFactoryHandler(CORBAServer* pServer)
{
	m_pCorba = pServer;
	m_VariableList.RemoveAll();
}

CVarFactoryHandler::~CVarFactoryHandler()
{

}

CXMLResponse* CVarFactoryHandler::ProcessMethodCall(CXMLRequest* request)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "VF: ProcessMethodCall "+request->m_strFullMethodName);
	if (request->m_strMethod == "ClearAll")
	{
		if (request->GetNumParams() == 1)
		{
			if (request->GetParam(0)->GetType() == "string")
			{
				CXMLString* pString = (CXMLString*)request->GetParam(0);
				return ClearAll(pString->GetValue());
			}
			else
			{
				CXMLResponse* response = new CXMLResponse();
				response->SetError(CXMLResponse::INVALID_PARAM);
				return response;
			}
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 1)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 1)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}

	}
	else if (request->m_strMethod == "DeregisterScript")
	{
		if (request->GetNumParams() == 3)
		{
			if ((request->GetParam(0)->GetType() == "string")&&(request->GetParam(1)->GetType() == "string")&&(request->GetParam(2)->GetType() == "boolean"))
			{
				CXMLString* pString1 = (CXMLString*)request->GetParam(0);
				CXMLString* pString2 = (CXMLString*)request->GetParam(1);
				CXMLBool*   pKeepAlive = (CXMLBool*)request->GetParam(2);
				return DeregisterScript(pString1->GetValue(),pString2->GetValue(),pKeepAlive->GetValue());
			}
			else
			{
				CXMLResponse* response = new CXMLResponse();
				response->SetError(CXMLResponse::INVALID_PARAM);
				return response;
			}
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 3)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 3)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}

	}
	else if (request->m_strMethod == "RegisterScript")
	{
		if (request->GetNumParams() == 2)
		{
			if ((request->GetParam(0)->GetType() == "string")&&(request->GetParam(1)->GetType() == "string"))
			{
				CXMLString* pString1 = (CXMLString*)request->GetParam(0);
				CXMLString* pString2 = (CXMLString*)request->GetParam(1);
				return RegisterScript(pString1->GetValue(),pString2->GetValue());
			}
			else
			{
				CXMLResponse* response = new CXMLResponse();
				response->SetError(CXMLResponse::INVALID_PARAM);
				return response;
			}
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 2)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 2)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}

	}
	else if (request->m_strMethod == "CreateMutex")
	{
		if (request->GetNumParams() == 2)
		{
			if ((request->GetParam(0)->GetType() == "string")&&(request->GetParam(1)->GetType() == "string"))
			{
				CXMLString* pString1 = (CXMLString*)request->GetParam(0);
				CXMLString* pString2 = (CXMLString*)request->GetParam(1);
				return CreateMutex(pString1->GetValue(),pString2->GetValue());
			}
			else
			{
				CXMLResponse* response = new CXMLResponse();
				response->SetError(CXMLResponse::INVALID_PARAM);
				return response;
			}
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 2)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 2)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}

	}
	else if (request->m_strMethod == "CreateSimpleSemaphore")
	{
		if (request->GetNumParams() == 2)
		{
			if ((request->GetParam(0)->GetType() == "string")&&(request->GetParam(1)->GetType() == "string"))
			{
				CXMLString* pString1 = (CXMLString*)request->GetParam(0);
				CXMLString* pString2 = (CXMLString*)request->GetParam(1);
				return CreateSimpleSemaphore(pString1->GetValue(),pString2->GetValue());
			}
			else
			{
				CXMLResponse* response = new CXMLResponse();
				response->SetError(CXMLResponse::INVALID_PARAM);
				return response;
			}
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 2)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 2)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}

	}
	else if (request->m_strMethod == "GetVariable")
	{
		if (request->GetNumParams() == 10)
		{
			if ((request->GetParam(0)->GetType() == "string")&&(request->GetParam(1)->GetType() == "string")&&
				(request->GetParam(2)->GetType() == "i4")&&(request->GetParam(3)->GetType() == "string")&&
				(request->GetParam(4)->GetType() == "string")&&(request->GetParam(5)->GetType() == "string")&&
				(request->GetParam(6)->GetType() == "string")&&(request->GetParam(7)->GetType() == "i4")&&
				(request->GetParam(8)->GetType() == "i4")&&(request->GetParam(9)->GetType() == "string"))
			{
				CXMLString* pString1  = (CXMLString*)request->GetParam(0);
				CXMLString* pString2  = (CXMLString*)request->GetParam(1);
				CXMLLong*	pLong3    = (CXMLLong*)request->GetParam(2);
				CXMLString* pString4  = (CXMLString*)request->GetParam(3);
				CXMLString* pString5  = (CXMLString*)request->GetParam(4);
				CXMLString* pString6  = (CXMLString*)request->GetParam(5);
				CXMLString* pString7  = (CXMLString*)request->GetParam(6);
				CXMLLong*	pLong8    = (CXMLLong*)request->GetParam(7);
				CXMLLong*	pLong9    = (CXMLLong*)request->GetParam(8);
				CXMLString* pString10 = (CXMLString*)request->GetParam(9);

				return GetVariable(pString1->GetValue(),pString2->GetValue(),pLong3->GetValue(),pString4->GetValue(),pString5->GetValue(),
									pString6->GetValue(),pString7->GetValue(),pLong8->GetValue(),pLong9->GetValue(),pString10->GetValue());
			}
			else
			{
				CXMLResponse* response = new CXMLResponse();
				response->SetError(CXMLResponse::INVALID_PARAM);
				return response;
			}
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 10)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 10)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}

	}
	else if (request->m_strMethod == "TPMutex_unlock")
	{
		if (request->GetNumParams() == 3)
		{
			if ((request->GetParam(0)->GetType() == "string")&&(request->GetParam(1)->GetType() == "string")&&(request->GetParam(2)->GetType() == "i4"))
			{
				CXMLString* pString1 = (CXMLString*)request->GetParam(0);
				CXMLString* pString2 = (CXMLString*)request->GetParam(1);
				CXMLLong*	pLong3    = (CXMLLong*)request->GetParam(2);
				return TPMutex_unlock(pString1->GetValue(),pString2->GetValue(),pLong3->GetValue());
			}
			else
			{
				CXMLResponse* response = new CXMLResponse();
				response->SetError(CXMLResponse::INVALID_PARAM);
				return response;
			}
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 3)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 3)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}

	}
	else if (request->m_strMethod == "TPMutex_trylock")
	{
		if (request->GetNumParams() == 2)
		{
			if ((request->GetParam(0)->GetType() == "string")&&(request->GetParam(1)->GetType() == "string"))
			{
				CXMLString* pString1 = (CXMLString*)request->GetParam(0);
				CXMLString* pString2 = (CXMLString*)request->GetParam(1);
				return TPMutex_trylock(pString1->GetValue(),pString2->GetValue());
			}
			else
			{
				CXMLResponse* response = new CXMLResponse();
				response->SetError(CXMLResponse::INVALID_PARAM);
				return response;
			}
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 2)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 2)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}

	}
	else if (request->m_strMethod == "TPMutex_lock")
	{
		if (request->GetNumParams() == 2)
		{
			if ((request->GetParam(0)->GetType() == "string")&&(request->GetParam(1)->GetType() == "string"))
			{
				CXMLString* pString1 = (CXMLString*)request->GetParam(0);
				CXMLString* pString2 = (CXMLString*)request->GetParam(1);
				return TPMutex_lock(pString1->GetValue(),pString2->GetValue());
			}
			else
			{
				CXMLResponse* response = new CXMLResponse();
				response->SetError(CXMLResponse::INVALID_PARAM);
				return response;
			}
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 2)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 2)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}
	}
	else if (request->m_strMethod == "SimpleSemaphore_acquire")
	{
		if (request->GetNumParams() == 2)
		{
			if ((request->GetParam(0)->GetType() == "string")&&(request->GetParam(1)->GetType() == "string"))
			{
				CXMLString* pString1 = (CXMLString*)request->GetParam(0);
				CXMLString* pString2 = (CXMLString*)request->GetParam(1);
				return TPSemaphore_acquire(pString1->GetValue(),pString2->GetValue());
			}
			else
			{
				CXMLResponse* response = new CXMLResponse();
				response->SetError(CXMLResponse::INVALID_PARAM);
				return response;
			}
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 2)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 2)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}

	}
	else if (request->m_strMethod == "SimpleSemaphore_release")
	{
		if (request->GetNumParams() == 2)
		{
			if ((request->GetParam(0)->GetType() == "string")&&(request->GetParam(1)->GetType() == "string"))
			{
				CXMLString* pString1 = (CXMLString*)request->GetParam(0);
				CXMLString* pString2 = (CXMLString*)request->GetParam(1);
				return TPSemaphore_release(pString1->GetValue(),pString2->GetValue());
			}
			else
			{
				CXMLResponse* response = new CXMLResponse();
				response->SetError(CXMLResponse::INVALID_PARAM);
				return response;
			}
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 2)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 2)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}

	}
	else if (request->m_strMethod == "SimpleSemaphore_tryacquire")
	{
		if (request->GetNumParams() == 2)
		{
			if ((request->GetParam(0)->GetType() == "string")&&(request->GetParam(1)->GetType() == "string"))
			{
				CXMLString* pString1 = (CXMLString*)request->GetParam(0);
				CXMLString* pString2 = (CXMLString*)request->GetParam(1);
				return TPSemaphore_tryacquire(pString1->GetValue(),pString2->GetValue());
			}
			else
			{
				CXMLResponse* response = new CXMLResponse();
				response->SetError(CXMLResponse::INVALID_PARAM);
				return response;
			}
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 2)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 2)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}

	}
	else if (request->m_strMethod == "GlobalVariable_get_value")
	{
		if (request->GetNumParams() == 4)
		{
			if ((request->GetParam(0)->GetType() == "string")&&(request->GetParam(1)->GetType() == "string")&&(request->GetParam(2)->GetType() == "string")&&(request->GetParam(3)->GetType() == "i4"))
			{
				CXMLString* pString1 = (CXMLString*)request->GetParam(0);
				CXMLString* pString2 = (CXMLString*)request->GetParam(1);
				CXMLString* pString3 = (CXMLString*)request->GetParam(2);
				CXMLLong* pLong4 = (CXMLLong*)request->GetParam(3);
				return GlobalVariable_get_value(pString1->GetValue(),pString2->GetValue(),pString3->GetValue(),pLong4->GetValue());
			}
			else
			{
				CXMLResponse* response = new CXMLResponse();
				response->SetError(CXMLResponse::INVALID_PARAM);
				return response;
			}
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 4)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 4)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}

	}
	else if (request->m_strMethod == "GlobalVariable_set_value")
	{
		if (request->GetNumParams() == 5)
		{
			if ((request->GetParam(0)->GetType() == "string")&&(request->GetParam(1)->GetType() == "string")&&(request->GetParam(2)->GetType() == "string")&&(request->GetParam(3)->GetType() == "string")&&(request->GetParam(4)->GetType() == "i4"))
			{
				CXMLString* pString1 = (CXMLString*)request->GetParam(0);
				CXMLString* pString2 = (CXMLString*)request->GetParam(1);
				CXMLString* pString3 = (CXMLString*)request->GetParam(2);
				CXMLString* pString4 = (CXMLString*)request->GetParam(3);
				CXMLLong* pLong5 = (CXMLLong*)request->GetParam(4);
				return GlobalVariable_set_value(pString1->GetValue(),pString2->GetValue(),pString3->GetValue(),pString4->GetValue(),pLong5->GetValue());
			}
			else
			{
				CXMLResponse* response = new CXMLResponse();
				response->SetError(CXMLResponse::INVALID_PARAM);
				return response;
			}
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 5)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 5)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}

	}
	else if (request->m_strMethod == "GlobalVariable_next_value")
	{
		if (request->GetNumParams() == 3)
		{
			if ((request->GetParam(0)->GetType() == "string")&&(request->GetParam(1)->GetType() == "string")&&(request->GetParam(2)->GetType() == "i4"))
			{
				CXMLString* pString1 = (CXMLString*)request->GetParam(0);
				CXMLString* pString2 = (CXMLString*)request->GetParam(1);
				CXMLLong* pLong3 = (CXMLLong*)request->GetParam(2);
				return GlobalVariable_next_value(pString1->GetValue(),pString2->GetValue(),pLong3->GetValue());
			}
			else
			{
				CXMLResponse* response = new CXMLResponse();
				response->SetError(CXMLResponse::INVALID_PARAM);
				return response;
			}
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 3)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 3)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}

	}
	else if (request->m_strMethod == "GlobalVariable_generate_value")
	{
		if (request->GetNumParams() == 3)
		{
			if ((request->GetParam(0)->GetType() == "string")&&(request->GetParam(1)->GetType() == "string")&&(request->GetParam(2)->GetType() == "i4"))
			{
				CXMLString* pString1 = (CXMLString*)request->GetParam(0);
				CXMLString* pString2 = (CXMLString*)request->GetParam(1);
				CXMLLong* pLong3 = (CXMLLong*)request->GetParam(2);
				return GlobalVariable_generate_value(pString1->GetValue(),pString2->GetValue(),pLong3->GetValue());
			}
			else
			{
				CXMLResponse* response = new CXMLResponse();
				response->SetError(CXMLResponse::INVALID_PARAM);
				return response;
			}
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 3)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 3)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}

	}
	else if (request->m_strMethod == "GlobalVariable_reset_seed")
	{
		if (request->GetNumParams() == 3)
		{
			if ((request->GetParam(0)->GetType() == "string")&&(request->GetParam(1)->GetType() == "string")&&(request->GetParam(2)->GetType() == "i4"))
			{
				CXMLString* pString1 = (CXMLString*)request->GetParam(0);
				CXMLString* pString2 = (CXMLString*)request->GetParam(1);
				CXMLLong* pLong3 = (CXMLLong*)request->GetParam(2);
				return GlobalVariable_reset_seed(pString1->GetValue(),pString2->GetValue(),pLong3->GetValue());
			}
			else
			{
				CXMLResponse* response = new CXMLResponse();
				response->SetError(CXMLResponse::INVALID_PARAM);
				return response;
			}
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 3)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 3)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}

	}
	else if (request->m_strMethod == "DoesSimpleSemaphoreExist")
	{
		if (request->GetNumParams() == 2)
		{
			if ((request->GetParam(0)->GetType() == "string")&&(request->GetParam(1)->GetType() == "string"))
			{
				CXMLString* pString1 = (CXMLString*)request->GetParam(0);
				CXMLString* pString2 = (CXMLString*)request->GetParam(1);
				return DoesSimpleSemaphoreExist(pString1->GetValue(),pString2->GetValue());
			}
			else
			{
				CXMLResponse* response = new CXMLResponse();
				response->SetError(CXMLResponse::INVALID_PARAM);
				return response;
			}
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 2)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 2)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}

	}
	else if (request->m_strMethod == "DoesMutexExist")
	{
		if (request->GetNumParams() == 2)
		{
			if ((request->GetParam(0)->GetType() == "string")&&(request->GetParam(1)->GetType() == "string"))
			{
				CXMLString* pString1 = (CXMLString*)request->GetParam(0);
				CXMLString* pString2 = (CXMLString*)request->GetParam(1);
				return DoesMutexExist(pString1->GetValue(),pString2->GetValue());
			}
			else
			{
				CXMLResponse* response = new CXMLResponse();
				response->SetError(CXMLResponse::INVALID_PARAM);
				return response;
			}
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 2)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 2)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}

	}

	g_Logger->WriteLogMsg(TRCLEV_LOW, "VF: ProcessMethodCall: Error: Method Not Found");
	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::FUNC_NOT_FOUND);
	return response;
}

CXMLResponse* CVarFactoryHandler::ClearAll(string strTargetMachine)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "VF: ClearAll");
	try
	{
		// Narrow the Variable Factory corba object on the target machine
	    CECInetInfo inet;
	    string strIP = inet.GetIPAddressAsString(strTargetMachine);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "VF: ClearAll: Narrowing VarFactory on "+strIP);
		CORBA::Object_var obj = m_pCorba->FindObjectWithName(VarFactory::CLASS_NAME,strIP.c_str());
		
		VarFactory_var VF = VarFactory::_narrow(obj);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "VF: ClearAll: Calling ClearAll");
		
		VF->ClearAll();	

		CTLockGuard<CTMutex> lock(m_VarListMutex);
		m_VariableList.RemoveAll(); //empty the list too!!
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "VF: ClearAll: variable list is now empty");

		CXMLResponse* response = new CXMLResponse();
		response->m_params.Add(new CXMLBool(true));
		return response;
	}
	catch(...)
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "VF: ClearAll: Exception calling ClearAll on VariableFactory");
	}


	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;
}

CXMLResponse* CVarFactoryHandler::DeregisterScript(string strTargetMachine, string strScript, bool bKeepAlive)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "VF: DeregisterScript");
	try
	{
		// Narrow the Variable Factory corba object on the target machine
	    CECInetInfo inet;
	    string strIP = inet.GetIPAddressAsString(strTargetMachine);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "VF: DeregisterScript: Narrowing VarFactory on "+strIP);
		CORBA::Object_var obj = m_pCorba->FindObjectWithName(VarFactory::CLASS_NAME,strIP.c_str());
		
		VarFactory_var VF = VarFactory::_narrow(obj);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "VF: DeregisterScript: Calling RegisterScript");

		long lResponse = VF->DeregisterScript(strScript.c_str(),bKeepAlive);

		CXMLResponse* response = new CXMLResponse();
		response->m_params.Add(new CXMLLong(lResponse));
		return response;
	}
	catch(...)
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "VF: DeregisterScript: Exception calling DeregisterScript on VariableFactory");
	}

	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;
}

CXMLResponse* CVarFactoryHandler::RegisterScript(string strTargetMachine, string strScript)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "VF: RegisterScript");
	try
	{
		// Narrow the Variable Factory corba object on the target machine
	    CECInetInfo inet;
	    string strIP = inet.GetIPAddressAsString(strTargetMachine);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "VF: RegisterScript: Narrowing VarFactory on "+strIP);
		CORBA::Object_var obj = m_pCorba->FindObjectWithName(VarFactory::CLASS_NAME,strIP.c_str());
		
		VarFactory_var VF = VarFactory::_narrow(obj);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "VF: RegisterScript: Calling RegisterScript");

		long lResponse = VF->RegisterScript(strScript.c_str());

		CXMLResponse* response = new CXMLResponse();
		response->m_params.Add(new CXMLLong(lResponse));
		return response;
	}
	catch(...)
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "VF: RegisterScript: Exception calling RegisterScript on VariableFactory");
	}

	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;
}

CXMLResponse* CVarFactoryHandler::CreateMutex(string strTargetMachine, string strName)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "VF: CreateMutex");
	try
	{
		// Narrow the Variable Factory corba object on the target machine
	    CECInetInfo inet;
	    string strIP = inet.GetIPAddressAsString(strTargetMachine);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "VF: ClearAll: Narrowing VarFactory on "+strIP);
		CORBA::Object_var obj = m_pCorba->FindObjectWithName(VarFactory::CLASS_NAME,strIP.c_str());
		
		VarFactory_var VF = VarFactory::_narrow(obj);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "VF: CreateMutex: Calling CreateMutex");
		
		bool bResp =false;
		if(!CORBA::is_nil(VF->CreateMutex(strName.c_str())))
		{
			bResp=true;
		}

		CXMLResponse* response = new CXMLResponse();
		response->m_params.Add(new CXMLBool(bResp));
		return response;
	}
	catch(...)
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "VF: CreateMutex: Exception calling CreateMutex on VariableFactory");
	}

	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;
}

CXMLResponse* CVarFactoryHandler::CreateSimpleSemaphore(string strTargetMachine, string strName)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "VF: CreateSimpleSemaphore");
	try
	{
		// Narrow the Variable Factory corba object on the target machine
	    CECInetInfo inet;
	    string strIP = inet.GetIPAddressAsString(strTargetMachine);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "VF: CreateSimpleSemaphore: Narrowing VarFactory on "+strIP);
		CORBA::Object_var obj = m_pCorba->FindObjectWithName(VarFactory::CLASS_NAME,strIP.c_str());
		
		VarFactory_var VF = VarFactory::_narrow(obj);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "VF: CreateSimpleSemaphore: Calling CreateSimpleSemaphore");
		
		bool bResp=false;
		if(!CORBA::is_nil(VF->CreateSimpleSemaphore(strName.c_str())))
		{
			bResp=true;
		}

		CXMLResponse* response = new CXMLResponse();
		response->m_params.Add(new CXMLBool(bResp));
		return response;
	}
	catch(...)
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "VF: CreateSimpleSemaphore: Exception calling CreateSimpleSemaphore on VariableFactory");
	}

	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;
}

CXMLResponse* CVarFactoryHandler::GetVariable(string strTargetMachine, string strName, long lSize, string strFile, string strFileid,
				  string strDimensions, string strValues, long lOptions, long lSeed, string strScriptname)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "VF: GetVariable");

	try
	{
		CECInetInfo inet;
		CORBA::Object_var obj = m_pCorba->FindObjectWithName(VarFactory::CLASS_NAME,
        inet.GetIPAddressAsString(strTargetMachine).c_str());
		VarFactory_var VF=VarFactory::_narrow(obj.inout());
		if(!(CORBA::is_nil(VF)))
		{
			obj=VF->GetVariable(strName.c_str(),lSize,strFile.c_str(),strFileid.c_str(),strDimensions.c_str(),strValues.c_str(),lOptions,lSeed,strScriptname.c_str());
			
			HandlerVarList newItem;
			newItem.variable=GlobalVariable::_narrow(obj.inout());
			
			CTLockGuard<CTMutex> lock(m_VarListMutex);
			newItem.lVarId = m_VariableList.GetSize();
			m_VariableList.Add(newItem);
			CString message;
			message.Format("VF: GetVariable added item \"%s\" at position %d in variable list",strName.c_str(),newItem.lVarId);
			g_Logger->WriteLogMsg(TRCLEV_MEDIUM, (char*)message.GetBuffer(0));
				
			CXMLResponse* response = new CXMLResponse();
			response->m_params.Add(new CXMLLong(newItem.lVarId));
			return response;
		}
	}
	catch(...)
	{
	}


	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;
}

CXMLResponse* CVarFactoryHandler::TPMutex_unlock(string strTargetMachine, string strMutexName, long lMutexid)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "VF: TPMutex_unlock");
	try
    {
		CORBA::Object_var obj;
        obj=m_pCorba->FindObjectWithName(TPMutex::CLASS_NAME,strMutexName.c_str());
		if(!(CORBA::is_nil(obj))) 
		{
			TPMutex_var Mutex;
			Mutex=TPMutex::_narrow(obj);
			//now try and acquire it
			Mutex->unlock(lMutexid);
			CXMLResponse* response = new CXMLResponse();
			response->m_params.Add(new CXMLBool(true));
			return response;
		}
    }
    catch(...)
    {
		g_Logger->WriteLogMsg(TRCLEV_LOW, "VF: TPMutex_unlock: Exception calling TPMutex_unlock on VariableFactory");
	}

	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;
}

CXMLResponse* CVarFactoryHandler::TPMutex_trylock(string strTargetMachine, string strMutexName)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "VF: TPMutex_trylock");
	try
    {
		CORBA::Object_var obj;
        obj=m_pCorba->FindObjectWithName(TPMutex::CLASS_NAME,strMutexName.c_str());
		if(!(CORBA::is_nil(obj))) 
		{
			TPMutex_var Mutex;
			Mutex=TPMutex::_narrow(obj);
			//now try and acquire it
			long bRet = -1;
			bRet=Mutex->trylock();
			CXMLResponse* response = new CXMLResponse();
			response->m_params.Add(new CXMLLong(bRet));
			return response;
		}
    }
    catch(...)
    {
		g_Logger->WriteLogMsg(TRCLEV_LOW, "VF: TPMutex_trylock: Exception calling TPMutex_trylock on VariableFactory");
	}
	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;
}

CXMLResponse* CVarFactoryHandler::TPMutex_lock(string strTargetMachine, string strMutexName)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "VF: TPMutex_lock");
	try
    {
		CORBA::Object_var obj;
        obj=m_pCorba->FindObjectWithName(TPMutex::CLASS_NAME,strMutexName.c_str());
		if(!(CORBA::is_nil(obj))) 
		{
			TPMutex_var Mutex;
			Mutex=TPMutex::_narrow(obj);
			//now try and acquire it
			long bRet = -1;
			bRet=Mutex->lock();
			CXMLResponse* response = new CXMLResponse();
			response->m_params.Add(new CXMLLong(bRet));
			return response;
		}
    }
    catch(...)
    {
		g_Logger->WriteLogMsg(TRCLEV_LOW, "VF: TPMutex_lock: Exception calling TPMutex_lock on VariableFactory");
	}
	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;
}

CXMLResponse* CVarFactoryHandler::TPSemaphore_acquire(string strTargetMachine, string strSemName)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "VF: TPSemaphore_acquire");
	try
    {
		CORBA::Object_var obj;
        obj=m_pCorba->FindObjectWithName(TPSemaphore::CLASS_NAME,strSemName.c_str());
		if(!(CORBA::is_nil(obj))) 
		{
			TPSemaphore_var Semaphore;
			Semaphore=TPSemaphore::_narrow(obj);
			//now acquire it
			Semaphore->acquire();
			CXMLResponse* response = new CXMLResponse();
			response->m_params.Add(new CXMLBool(true));
			return response;
		}
    }
    catch(...)
    {
		g_Logger->WriteLogMsg(TRCLEV_LOW, "VF: TPSemaphore_acquire: Exception calling TPSemaphore_acquire on VariableFactory");
	}

	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;
}

CXMLResponse* CVarFactoryHandler::TPSemaphore_release(string strTargetMachine, string strSemName)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "VF: TPSemaphore_release");

	try
    {
		CORBA::Object_var obj;
        obj=m_pCorba->FindObjectWithName(TPSemaphore::CLASS_NAME,strSemName.c_str());
		if(!(CORBA::is_nil(obj))) 
		{
			TPSemaphore_var Semaphore;
			Semaphore=TPSemaphore::_narrow(obj);
			//now try and release it
			Semaphore->release();
			CXMLResponse* response = new CXMLResponse();
			response->m_params.Add(new CXMLBool(true));
			return response;
		}
    }
    catch(...)
    {
		g_Logger->WriteLogMsg(TRCLEV_LOW, "VF: DoesSimpleSemaphoreExist: Exception calling TPSemaphore_release on VariableFactory");
	}
	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;
}

CXMLResponse* CVarFactoryHandler::TPSemaphore_tryacquire(string strTargetMachine, string strSemName)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "VF: TPSemaphore_tryacquire");

	try
    {
		CORBA::Object_var obj;
        obj=m_pCorba->FindObjectWithName(TPSemaphore::CLASS_NAME,strSemName.c_str());
		if(!(CORBA::is_nil(obj))) 
		{
			TPSemaphore_var Semaphore;
			Semaphore=TPSemaphore::_narrow(obj);
			//now try and acquire it
			bool bRet = false;
			bRet=Semaphore->tryacquire();
			CXMLResponse* response = new CXMLResponse();
			response->m_params.Add(new CXMLBool(bRet));
			return response;
		}
    }
    catch(...)
    {
		g_Logger->WriteLogMsg(TRCLEV_LOW, "VF: DoesSimpleSemaphoreExist: Exception calling TPSemaphore_tryacquire on VariableFactory");
	}
	
	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;
}

CXMLResponse* CVarFactoryHandler::GlobalVariable_get_value(string strTargetMachine, string strVarName, string strExpression, long lVarId)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "VF: GlobalVariable_get_value");
	try
	{
		CTLockGuard<CTMutex> lock(m_VarListMutex);
		for(long p=0 ; p<m_VariableList.GetSize(); p++)
		{
			if(m_VariableList[p].lVarId==lVarId)
			{
				g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "VF:  found var in list");
				CORBA::String_var Cstr=m_VariableList[p].variable->get_value(strExpression.c_str());
				string retstr(Cstr);
				CXMLResponse* response = new CXMLResponse();
				response->m_params.Add(new CXMLString(retstr.c_str()));
				return response;
			}
		}
		g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "VF:  couldn't find var in list");
	}
	catch(...)
	{
	}

	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;
}
CXMLResponse* CVarFactoryHandler::GlobalVariable_set_value(string strTargetMachine, string strVarName, string strNewval, string strExpression, long lVarId)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "VF: GlobalVariable_set_value");
	try
	{
		CTLockGuard<CTMutex> lock(m_VarListMutex);
		for(long p=0 ; p<m_VariableList.GetSize(); p++)
		{
			if(m_VariableList[p].lVarId==lVarId)
			{
				g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "VF:  found var in list");
				m_VariableList[p].variable->set_value(strNewval.c_str(),strExpression.c_str());
				CXMLResponse* response = new CXMLResponse();
				response->m_params.Add(new CXMLLong(0));
				return response;
			}
		}
		g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "VF:  couldn't find var in list");
	}
	catch(...)
	{
	}

	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;
}

CXMLResponse* CVarFactoryHandler::GlobalVariable_next_value(string strTargetMachine, string strVarName, long lVarId)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "VF: GlobalVariable_next_value");
	try
	{
		CTLockGuard<CTMutex> lock(m_VarListMutex);
		for(long p=0 ; p<m_VariableList.GetSize(); p++)
		{
			if(m_VariableList[p].lVarId==lVarId)
			{
				g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "VF:  found var in list");
				m_VariableList[p].variable->next_value();
				CXMLResponse* response = new CXMLResponse();
				response->m_params.Add(new CXMLBool(true));
				return response;
			}
		}
		g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "VF:  couldn't find var in list");
	}
	catch(...)
	{
	}

	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;
}

CXMLResponse* CVarFactoryHandler::GlobalVariable_generate_value(string strTargetMachine, string strVarName, long lVarId)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "VF: GlobalVariable_generate_value");
	try
	{
		CTLockGuard<CTMutex> lock(m_VarListMutex);
		for(long p=0 ; p<m_VariableList.GetSize(); p++)
		{
			if(m_VariableList[p].lVarId==lVarId)
			{
				g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "VF:  found var in list");
				m_VariableList[p].variable->generate_value();
				CXMLResponse* response = new CXMLResponse();
				response->m_params.Add(new CXMLBool(true));
				return response;
			}
		}
		g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "VF:  couldn't find var in list");
	}
	catch(...)
	{
	}

	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;
}

CXMLResponse* CVarFactoryHandler::GlobalVariable_reset_seed(string strTargetMachine, string strVarName, long lVarId)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "VF: GlobalVariable_reset_seed");
	try
	{
		CTLockGuard<CTMutex> lock(m_VarListMutex);
		for(long p=0 ; p<m_VariableList.GetSize(); p++)
		{
			if(m_VariableList[p].lVarId==lVarId)
			{
				g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "VF:  found var in list");
				m_VariableList[p].variable->reset_seed();
				CXMLResponse* response = new CXMLResponse();
				response->m_params.Add(new CXMLBool(true));
				return response;
			}
		}
		g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "VF:  couldn't find var in list");
	}
	catch(...)
	{
	}

	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;
}

CXMLResponse* CVarFactoryHandler::DoesSimpleSemaphoreExist(string strTargetMachine, string strVarName)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "VF: DoesSimpleSemaphoreExist");

	try
    {
		bool bResp=false;
		if(!CORBA::is_nil(m_pCorba->FindObjectWithName(TPSemaphore::CLASS_NAME,strVarName.c_str())))
		{
			bResp=true;
		}

		CXMLResponse* response = new CXMLResponse();
		response->m_params.Add(new CXMLBool(bResp));
		return response;
    }
    catch(...)
    {
		g_Logger->WriteLogMsg(TRCLEV_LOW, "VF: DoesSimpleSemaphoreExist: Exception calling DoesSimpleSemaphoreExist on VariableFactory - assume not registered");
	}
	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;
}

CXMLResponse* CVarFactoryHandler::DoesMutexExist(string strTargetMachine, string strVarName)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "VF: DoesMutexExist");

	try
    {
		bool bResp=false;
		if(!CORBA::is_nil(m_pCorba->FindObjectWithName(TPMutex::CLASS_NAME,strVarName.c_str())))
		{
			bResp=true;
		}

		CXMLResponse* response = new CXMLResponse();
		response->m_params.Add(new CXMLBool(bResp));
		return response;
    }
    catch(...)
    {
		g_Logger->WriteLogMsg(TRCLEV_LOW, "VF: DoesSimpleSemaphoreExist: Exception calling DoesMutexExist on VariableFactory - assume not registered");
	}
	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;
}
