/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include "stdafx.h"
#include "XMLRelayVarFactory_i.h"
#include <CECInetInfo.h>
#include "xmlrpc.h"
#include <string>
#include <iostream>
#include <fstream>
using namespace std;
#include "..\include\Logger.h"
#include "OstaTraceDef.h"

extern CLogger* g_Logger;

XMLRelayVarFactoryImpl::XMLRelayVarFactoryImpl(CORBAServer* corba)
{
    // Get IP address (with dots replaced by underscores) of daemon host.
    m_registered_name="WebRelay";
    m_corba=corba;

	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: Registering XMLRelayVarFactory interface");
	m_corba->ActivateObject(this);
    m_corba->NameObject( this->_this(),XMLRelayVarFactory::CLASS_NAME,m_registered_name.c_str());

	m_RelayMap = XMLRelayMap::_nil();
}

XMLRelayVarFactoryImpl::~XMLRelayVarFactoryImpl()
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: Unregistering XMLRelayVarFactory interface - destructor");
    m_corba->UnNameObject( XMLRelayVarFactory::CLASS_NAME,m_registered_name.c_str());
}

CORBA::Long  XMLRelayVarFactoryImpl::RegisterScript(const char* strTargetMachine,const char* strScriptName)
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: RegisterScript: Relaying RegisterScript request for host "+string(strTargetMachine));

	if (CORBA::is_nil(m_RelayMap))
	{
		CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
		m_RelayMap = XMLRelayMap::_narrow(obj);
	}

	RelayDef def;	
	if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: RegisterScript: No relay server defined for this host");
		return -1;
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayVF: RegisterScript: Relay server is "+string(def.strHost));
	}

	
	CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

	CXMLRequest* req = new CXMLRequest();
	req->SetMethodName("VARFACTORY.RegisterScript");
	req->SetTransport(transport);
	req->AddParameter(new CXMLString(strTargetMachine));
	req->AddParameter(new CXMLString(strScriptName));
	CXMLResponse* resp = req->SendRequest();
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: Relay Error: "+resp->PrintError());
		return -1;
	}

	long lReply =-1;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "i4")
	{
		CXMLLong* pLongVal = (CXMLLong*)resp->m_params.GetAt(0);
		lReply = pLongVal->GetValue();
	}

	delete req;
	delete resp;

    return lReply;
}

CORBA::Long  XMLRelayVarFactoryImpl::DeregisterScript(const char* strTargetMachine,const char* strScriptName, CORBA::Boolean bKeepAlive)
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: DeregisterScript: Relaying DeregisterScript request for host "+string(strTargetMachine));

	if (CORBA::is_nil(m_RelayMap))
	{
		CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
		m_RelayMap = XMLRelayMap::_narrow(obj);
	}

	RelayDef def;	
	if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: DeregisterScript: No relay server defined for this host");
		return -1;
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayVF: DeregisterScript: Relay server is "+string(def.strHost));
	}

	
	CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

	CXMLRequest* req = new CXMLRequest();
	req->SetMethodName("VARFACTORY.DeregisterScript");
	req->SetTransport(transport);
	req->AddParameter(new CXMLString(strTargetMachine));
	req->AddParameter(new CXMLString(strScriptName));
	req->AddParameter(new CXMLBool(bKeepAlive));
	CXMLResponse* resp = req->SendRequest();
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: Relay Error: "+resp->PrintError());
		return -1;
	}

	long lReply =-1;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "i4")
	{
		CXMLLong* pLongVal = (CXMLLong*)resp->m_params.GetAt(0);
		lReply = pLongVal->GetValue();
	}

	delete req;
	delete resp;

    return lReply;
}

CORBA::Boolean  XMLRelayVarFactoryImpl::ClearAll(const char* strTargetMachine)
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: ClearAll: Relaying ClearAll request for host "+string(strTargetMachine));

	if (CORBA::is_nil(m_RelayMap))
	{
		CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
		m_RelayMap = XMLRelayMap::_narrow(obj);
	}

	RelayDef def;	
	if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: ClearAll: No relay server defined for this host");
		return false;
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayVF: ClearAll: Relay server is "+string(def.strHost));
	}

	
	CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

	CXMLRequest* req = new CXMLRequest();
	req->SetMethodName("VARFACTORY.ClearAll");
	req->SetTransport(transport);
	req->AddParameter(new CXMLString(strTargetMachine));
	
	CXMLResponse* resp = req->SendRequest();
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: Relay Error: "+resp->PrintError());
		return false;
	}

	CORBA::Boolean bReply =false;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "boolean")
	{
		CXMLBool* pVal = (CXMLBool*)resp->m_params.GetAt(0);
		bReply = pVal->GetValue();
	}

	delete req;
	delete resp;

    return bReply;
}

CORBA::Boolean XMLRelayVarFactoryImpl::CreateMutex(const char* strTargetMachine,const char* strName)
{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: CreateMutex: Relaying CreateMutex request for host "+string(strTargetMachine));

	if (CORBA::is_nil(m_RelayMap))
	{
		CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
		m_RelayMap = XMLRelayMap::_narrow(obj);
	}

	RelayDef def;	
	if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: CreateMutex: No relay server defined for this host");
		return false;
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayVF: CreateMutex: Relay server is "+string(def.strHost));
	}

	
	CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

	CXMLRequest* req = new CXMLRequest();
	req->SetMethodName("VARFACTORY.CreateMutex");
	req->SetTransport(transport);
	req->AddParameter(new CXMLString(strTargetMachine));
	req->AddParameter(new CXMLString(strName));
	
	CXMLResponse* resp = req->SendRequest();
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: Relay Error: "+resp->PrintError());
		return false;
	}

	CORBA::Boolean bReply =false;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "boolean")
	{
		CXMLBool* pVal = (CXMLBool*)resp->m_params.GetAt(0);
		bReply = pVal->GetValue();
	}

	delete req;
	delete resp;

    return bReply;
}

CORBA::Boolean XMLRelayVarFactoryImpl::CreateSimpleSemaphore(const char* strTargetMachine,const char* strName)
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: CreateSimpleSemaphore: Relaying CreateSimpleSemaphore request for host "+string(strTargetMachine));

	if (CORBA::is_nil(m_RelayMap))
	{
		CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
		m_RelayMap = XMLRelayMap::_narrow(obj);
	}

	RelayDef def;	
	if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: CreateSimpleSemaphore: No relay server defined for this host");
		return false;
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayVF: CreateSimpleSemaphore: Relay server is "+string(def.strHost));
	}

	
	CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

	CXMLRequest* req = new CXMLRequest();
	req->SetMethodName("VARFACTORY.CreateSimpleSemaphore");
	req->SetTransport(transport);
	req->AddParameter(new CXMLString(strTargetMachine));
	req->AddParameter(new CXMLString(strName));
	
	CXMLResponse* resp = req->SendRequest();
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: Relay Error: "+resp->PrintError());
		return false;
	}

	CORBA::Boolean bReply =false;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "boolean")
	{
		CXMLBool* pVal = (CXMLBool*)resp->m_params.GetAt(0);
		bReply = pVal->GetValue();
	}

	delete req;
	delete resp;

    return bReply;
}

CORBA::Long XMLRelayVarFactoryImpl::GetVariable(const char* strTargetMachine, const char* strName, CORBA::Long size, const char* strFile, const char* strFileid,
					  const char* strDimensions, const char* strValues, CORBA::Long options, CORBA::Long seed, const char* strScriptname)
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: GetVariable: Relaying GetVariable request for host "+string(strTargetMachine));

	if (CORBA::is_nil(m_RelayMap))
	{
		CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
		m_RelayMap = XMLRelayMap::_narrow(obj);
	}

	RelayDef def;	
	if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: GetVariable: No relay server defined for this host");
		return -1;
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayVF: GetVariable: Relay server is "+string(def.strHost));
	}

	
	CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

	CXMLRequest* req = new CXMLRequest();
	req->SetMethodName("VARFACTORY.GetVariable");
	req->SetTransport(transport);
	req->AddParameter(new CXMLString(strTargetMachine));
	req->AddParameter(new CXMLString(strName));
	req->AddParameter(new CXMLLong(size));
	req->AddParameter(new CXMLString(strFile));
	req->AddParameter(new CXMLString(strFileid));
	req->AddParameter(new CXMLString(strDimensions));
	req->AddParameter(new CXMLString(strValues));
	req->AddParameter(new CXMLLong(options));
	req->AddParameter(new CXMLLong(seed));
	req->AddParameter(new CXMLString(strScriptname));
	CXMLResponse* resp = req->SendRequest();
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: Relay Error: "+resp->PrintError());
		return -1;
	}

	CORBA::Long lReply = -1;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "i4")
	{
		CXMLLong* pVal = (CXMLLong*)resp->m_params.GetAt(0);
		lReply = pVal->GetValue();
	}

	delete req;
	delete resp;

    return lReply;
}
	
CORBA::Long XMLRelayVarFactoryImpl::TPMutex_lock(const char* strTargetMachine, const char* mutexName)
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: TPMutex_lock: Relaying TPMutex_lock request for host "+string(strTargetMachine));

	if (CORBA::is_nil(m_RelayMap))
	{
		CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
		m_RelayMap = XMLRelayMap::_narrow(obj);
	}

	RelayDef def;	
	if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: TPMutex_lock: No relay server defined for this host");
		return -1;
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayVF: TPMutex_lock: Relay server is "+string(def.strHost));
	}

	CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

	CXMLRequest* req = new CXMLRequest();
	req->SetMethodName("VARFACTORY.TPMutex_lock");
	req->SetTransport(transport);
	req->AddParameter(new CXMLString(strTargetMachine));
	req->AddParameter(new CXMLString(mutexName));
	CXMLResponse* resp = req->SendRequest();
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: Relay Error: "+resp->PrintError());
		return -1;
	}

	long lReply =-1;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "i4")
	{
		CXMLLong* pLongVal = (CXMLLong*)resp->m_params.GetAt(0);
		lReply = pLongVal->GetValue();
	}

	delete req;
	delete resp;

    return lReply;
}

CORBA::Boolean XMLRelayVarFactoryImpl::TPMutex_unlock(const char* strTargetMachine, const char* mutexName, CORBA::Long mutexid)
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: TPMutex_unlock: Relaying TPMutex_unlock request for host "+string(strTargetMachine));

	if (CORBA::is_nil(m_RelayMap))
	{
		CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
		m_RelayMap = XMLRelayMap::_narrow(obj);
	}

	RelayDef def;	
	if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: TPMutex_unlock: No relay server defined for this host");
		return false;
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayVF: TPMutex_unlock: Relay server is "+string(def.strHost));
	}

	
	CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

	CXMLRequest* req = new CXMLRequest();
	req->SetMethodName("VARFACTORY.TPMutex_unlock");
	req->SetTransport(transport);
	req->AddParameter(new CXMLString(strTargetMachine));
	req->AddParameter(new CXMLString(mutexName));
	req->AddParameter(new CXMLLong(mutexid));
	
	CXMLResponse* resp = req->SendRequest();
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: Relay Error: "+resp->PrintError());
		return false;
	}

	CORBA::Boolean bReply =false;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "boolean")
	{
		CXMLBool* pVal = (CXMLBool*)resp->m_params.GetAt(0);
		bReply = pVal->GetValue();
	}

	delete req;
	delete resp;

    return bReply;
}

CORBA::Long XMLRelayVarFactoryImpl::TPMutex_trylock(const char* strTargetMachine, const char* mutexName)
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: TPMutex_trylock: Relaying TPMutex_trylock request for host "+string(strTargetMachine));

	if (CORBA::is_nil(m_RelayMap))
	{
		CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
		m_RelayMap = XMLRelayMap::_narrow(obj);
	}

	RelayDef def;	
	if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: TPMutex_trylock: No relay server defined for this host");
		return -1;
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayVF: TPMutex_trylock: Relay server is "+string(def.strHost));
	}

	CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

	CXMLRequest* req = new CXMLRequest();
	req->SetMethodName("VARFACTORY.TPMutex_trylock");
	req->SetTransport(transport);
	req->AddParameter(new CXMLString(strTargetMachine));
	req->AddParameter(new CXMLString(mutexName));
	CXMLResponse* resp = req->SendRequest();
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: Relay Error: "+resp->PrintError());
		return -1;
	}

	long lReply =-1;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "i4")
	{
		CXMLLong* pLongVal = (CXMLLong*)resp->m_params.GetAt(0);
		lReply = pLongVal->GetValue();
	}

	delete req;
	delete resp;

    return lReply;
}

CORBA::Boolean XMLRelayVarFactoryImpl::SimpleSemaphore_acquire(const char* strTargetMachine, const char* semName)
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: SimpleSemaphore_acquire: Relaying SimpleSemaphore_acquire request for host "+string(strTargetMachine));

	if (CORBA::is_nil(m_RelayMap))
	{
		CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
		m_RelayMap = XMLRelayMap::_narrow(obj);
	}

	RelayDef def;	
	if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: SimpleSemaphore_acquire: No relay server defined for this host");
		return false;
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayVF: SimpleSemaphore_acquire: Relay server is "+string(def.strHost));
	}

	
	CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

	CXMLRequest* req = new CXMLRequest();
	req->SetMethodName("VARFACTORY.SimpleSemaphore_acquire");
	req->SetTransport(transport);
	req->AddParameter(new CXMLString(strTargetMachine));
	req->AddParameter(new CXMLString(semName));
	
	CXMLResponse* resp = req->SendRequest();
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: Relay Error: "+resp->PrintError());
		return false;
	}

	CORBA::Boolean bReply =false;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "boolean")
	{
		CXMLBool* pVal = (CXMLBool*)resp->m_params.GetAt(0);
		bReply = pVal->GetValue();
	}

	delete req;
	delete resp;

    return bReply;
}

CORBA::Boolean XMLRelayVarFactoryImpl::SimpleSemaphore_release(const char* strTargetMachine, const char* semName)
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: SimpleSemaphore_release: Relaying SimpleSemaphore_release request for host "+string(strTargetMachine));

	if (CORBA::is_nil(m_RelayMap))
	{
		CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
		m_RelayMap = XMLRelayMap::_narrow(obj);
	}

	RelayDef def;	
	if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: SimpleSemaphore_release: No relay server defined for this host");
		return false;
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayVF: SimpleSemaphore_release: Relay server is "+string(def.strHost));
	}
	
	CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

	CXMLRequest* req = new CXMLRequest();
	req->SetMethodName("VARFACTORY.SimpleSemaphore_release");
	req->SetTransport(transport);
	req->AddParameter(new CXMLString(strTargetMachine));
	req->AddParameter(new CXMLString(semName));
	
	CXMLResponse* resp = req->SendRequest();
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: Relay Error: "+resp->PrintError());
		return false;
	}

	CORBA::Boolean bReply =false;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "boolean")
	{
		CXMLBool* pVal = (CXMLBool*)resp->m_params.GetAt(0);
		bReply = pVal->GetValue();
	}

	delete req;
	delete resp;

    return bReply;
}

CORBA::Boolean XMLRelayVarFactoryImpl::SimpleSemaphore_tryacquire(const char* strTargetMachine, const char* semName)
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: SimpleSemaphore_tryacquire: Relaying SimpleSemaphore_tryacquire request for host "+string(strTargetMachine));

	if (CORBA::is_nil(m_RelayMap))
	{
		CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
		m_RelayMap = XMLRelayMap::_narrow(obj);
	}

	RelayDef def;	
	if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: SimpleSemaphore_tryacquire: No relay server defined for this host");
		return false;
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayVF: SimpleSemaphore_tryacquire: Relay server is "+string(def.strHost));
	}

	
	CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

	CXMLRequest* req = new CXMLRequest();
	req->SetMethodName("VARFACTORY.SimpleSemaphore_tryacquire");
	req->SetTransport(transport);
	req->AddParameter(new CXMLString(strTargetMachine));
	req->AddParameter(new CXMLString(semName));
	
	CXMLResponse* resp = req->SendRequest();
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: Relay Error: "+resp->PrintError());
		return false;
	}

	CORBA::Boolean bReply =false;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "boolean")
	{
		CXMLBool* pVal = (CXMLBool*)resp->m_params.GetAt(0);
		bReply = pVal->GetValue();
	}

	delete req;
	delete resp;

    return bReply;
}

char* XMLRelayVarFactoryImpl::GlobalVariable_get_value(const char* strTargetMachine, const char* strVarName, const char* strExpression, long lVarId)
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: GlobalVariable_get_value: Relaying GlobalVariable_get_value request for host "+string(strTargetMachine));

	if (CORBA::is_nil(m_RelayMap))
	{
		CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
		m_RelayMap = XMLRelayMap::_narrow(obj);
	}

	RelayDef def;	
	if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: GlobalVariable_get_value: No relay server defined for this host");
		return "";
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayVF: GlobalVariable_get_value: Relay server is "+string(def.strHost));
	}

	CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

	CXMLRequest* req = new CXMLRequest();
	req->SetMethodName("VARFACTORY.GlobalVariable_get_value");
	req->SetTransport(transport);
	req->AddParameter(new CXMLString(strTargetMachine));
	req->AddParameter(new CXMLString(strVarName));
	req->AddParameter(new CXMLString(strExpression));
	req->AddParameter(new CXMLLong(lVarId));
	CXMLResponse* resp = req->SendRequest();
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: Relay Error: "+resp->PrintError());
		return "";
	}

	string strReply ="";
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "string")
	{
		CXMLString* pStrVal = (CXMLString*)resp->m_params.GetAt(0);
		strReply = pStrVal->GetValue();
	}

	delete req;
	delete resp;

    return (char*)(strReply.c_str());
}
CORBA::Long XMLRelayVarFactoryImpl::GlobalVariable_set_value(const char* strTargetMachine, const char* strVarName, const char* strNewval, const char* strExpression, long lVarId)
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: GlobalVariable_set_value: Relaying GlobalVariable_set_value request for host "+string(strTargetMachine));

	if (CORBA::is_nil(m_RelayMap))
	{
		CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
		m_RelayMap = XMLRelayMap::_narrow(obj);
	}

	RelayDef def;	
	if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: GlobalVariable_set_value: No relay server defined for this host");
		return -1;
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayVF: GlobalVariable_set_value: Relay server is "+string(def.strHost));
	}

	CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

	CXMLRequest* req = new CXMLRequest();
	req->SetMethodName("VARFACTORY.GlobalVariable_set_value");
	req->SetTransport(transport);
	req->AddParameter(new CXMLString(strTargetMachine));
	req->AddParameter(new CXMLString(strVarName));
	req->AddParameter(new CXMLString(strExpression));
	req->AddParameter(new CXMLLong(lVarId));
	CXMLResponse* resp = req->SendRequest();
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: Relay Error: "+resp->PrintError());
		return -1;
	}

	long lReply =-1;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "i4")
	{
		CXMLLong* pLongVal = (CXMLLong*)resp->m_params.GetAt(0);
		lReply = pLongVal->GetValue();
	}

	delete req;
	delete resp;

    return lReply;
}
CORBA::Boolean XMLRelayVarFactoryImpl::GlobalVariable_next_value(const char* strTargetMachine, const char* strVarName, long lVarId)
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: GlobalVariable_next_value: Relaying GlobalVariable_next_value request for host "+string(strTargetMachine));

	if (CORBA::is_nil(m_RelayMap))
	{
		CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
		m_RelayMap = XMLRelayMap::_narrow(obj);
	}

	RelayDef def;	
	if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: GlobalVariable_next_value: No relay server defined for this host");
		return false;
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayVF: GlobalVariable_next_value: Relay server is "+string(def.strHost));
	}

	
	CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

	CXMLRequest* req = new CXMLRequest();
	req->SetMethodName("VARFACTORY.GlobalVariable_next_value");
	req->SetTransport(transport);
	req->AddParameter(new CXMLString(strTargetMachine));
	req->AddParameter(new CXMLString(strVarName));
	req->AddParameter(new CXMLLong(lVarId));
	
	CXMLResponse* resp = req->SendRequest();
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: Relay Error: "+resp->PrintError());
		return false;
	}

	CORBA::Boolean bReply =false;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "boolean")
	{
		CXMLBool* pVal = (CXMLBool*)resp->m_params.GetAt(0);
		bReply = pVal->GetValue();
	}

	delete req;
	delete resp;

    return bReply;
}
CORBA::Boolean XMLRelayVarFactoryImpl::GlobalVariable_generate_value(const char* strTargetMachine, const char* strVarName, long lVarId)
{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: GlobalVariable_generate_value: Relaying GlobalVariable_generate_value request for host "+string(strTargetMachine));

	if (CORBA::is_nil(m_RelayMap))
	{
		CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
		m_RelayMap = XMLRelayMap::_narrow(obj);
	}

	RelayDef def;	
	if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: GlobalVariable_generate_value: No relay server defined for this host");
		return false;
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayVF: GlobalVariable_generate_value: Relay server is "+string(def.strHost));
	}

	
	CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

	CXMLRequest* req = new CXMLRequest();
	req->SetMethodName("VARFACTORY.GlobalVariable_generate_value");
	req->SetTransport(transport);
	req->AddParameter(new CXMLString(strTargetMachine));
	req->AddParameter(new CXMLString(strVarName));
	req->AddParameter(new CXMLLong(lVarId));
	
	CXMLResponse* resp = req->SendRequest();
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: Relay Error: "+resp->PrintError());
		return false;
	}

	CORBA::Boolean bReply =false;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "boolean")
	{
		CXMLBool* pVal = (CXMLBool*)resp->m_params.GetAt(0);
		bReply = pVal->GetValue();
	}

	delete req;
	delete resp;

    return bReply;
}
CORBA::Boolean XMLRelayVarFactoryImpl::GlobalVariable_reset_seed(const char* strTargetMachine, const char* strVarName, long lVarId)
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: GlobalVariable_reset_seed: Relaying GlobalVariable_reset_seed request for host "+string(strTargetMachine));

	if (CORBA::is_nil(m_RelayMap))
	{
		CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
		m_RelayMap = XMLRelayMap::_narrow(obj);
	}

	RelayDef def;	
	if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: GlobalVariable_reset_seed: No relay server defined for this host");
		return false;
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayVF: GlobalVariable_reset_seed: Relay server is "+string(def.strHost));
	}

	
	CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

	CXMLRequest* req = new CXMLRequest();
	req->SetMethodName("VARFACTORY.GlobalVariable_reset_seed");
	req->SetTransport(transport);
	req->AddParameter(new CXMLString(strTargetMachine));
	req->AddParameter(new CXMLString(strVarName));
	req->AddParameter(new CXMLLong(lVarId));
	
	CXMLResponse* resp = req->SendRequest();
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: Relay Error: "+resp->PrintError());
		return false;
	}

	CORBA::Boolean bReply =false;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "boolean")
	{
		CXMLBool* pVal = (CXMLBool*)resp->m_params.GetAt(0);
		bReply = pVal->GetValue();
	}

	delete req;
	delete resp;

    return bReply;
}

CORBA::Boolean XMLRelayVarFactoryImpl::DoesSimpleSemaphoreExist(const char* strTargetMachine, const char* strVarName)
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: DoesSimpleSemaphoreExist: Relaying DoesSimpleSemaphoreExist request for host "+string(strTargetMachine));

	if (CORBA::is_nil(m_RelayMap))
	{
		CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
		m_RelayMap = XMLRelayMap::_narrow(obj);
	}

	RelayDef def;	
	if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: DoesSimpleSemaphoreExist: No relay server defined for this host");
		return false;
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayVF: DoesSimpleSemaphoreExist: Relay server is "+string(def.strHost));
	}

	
	CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

	CXMLRequest* req = new CXMLRequest();
	req->SetMethodName("VARFACTORY.DoesSimpleSemaphoreExist");
	req->SetTransport(transport);
	req->AddParameter(new CXMLString(strTargetMachine));
	req->AddParameter(new CXMLString(strVarName));
	
	CXMLResponse* resp = req->SendRequest();
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: Relay Error: "+resp->PrintError());
		return false;
	}

	CORBA::Boolean bReply =false;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "boolean")
	{
		CXMLBool* pVal = (CXMLBool*)resp->m_params.GetAt(0);
		bReply = pVal->GetValue();
	}

	delete req;
	delete resp;

    return bReply;
}

CORBA::Boolean XMLRelayVarFactoryImpl::DoesMutexExist(const char* strTargetMachine, const char* strVarName)
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: DoesMutexExist: Relaying DoesMutexExist request for host "+string(strTargetMachine));

	if (CORBA::is_nil(m_RelayMap))
	{
		CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
		m_RelayMap = XMLRelayMap::_narrow(obj);
	}

	RelayDef def;	
	if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: DoesMutexExist: No relay server defined for this host");
		return false;
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayVF: DoesMutexExist: Relay server is "+string(def.strHost));
	}

	
	CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

	CXMLRequest* req = new CXMLRequest();
	req->SetMethodName("VARFACTORY.DoesMutexExist");
	req->SetTransport(transport);
	req->AddParameter(new CXMLString(strTargetMachine));
	req->AddParameter(new CXMLString(strVarName));
	
	CXMLResponse* resp = req->SendRequest();
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: Relay Error: "+resp->PrintError());
		return false;
	}

	CORBA::Boolean bReply =false;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "boolean")
	{
		CXMLBool* pVal = (CXMLBool*)resp->m_params.GetAt(0);
		bReply = pVal->GetValue();
	}

	delete req;
	delete resp;

    return bReply;
}
