/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include "stdafx.h"
#include "xmlpost.h"
#include "..\CallProcessor\XMLCallProcessor.h"
#include <string>
using namespace std;

bool bAnyLogging = true;			//create any log file
bool bIndividualLogging = false;	//start a new log file for each instance of the class

///////////////////////////////////////////////////////////////////////
// The one and only CWinApp object
// NOTE: You may remove this object if you alter your project to no
// longer use MFC in a DLL.

CWinApp theApp;

///////////////////////////////////////////////////////////////////////
// command-parsing map

BEGIN_PARSE_MAP(CXMLRPCExtension, CHttpServer)
	// TODO: insert your ON_PARSE_COMMAND() and 
	// ON_PARSE_COMMAND_PARAMS() here to hook up your commands.
	// For example:

	ON_PARSE_COMMAND(Default, CXMLRPCExtension, ITS_EMPTY)
	DEFAULT_PARSE_COMMAND(Default, CXMLRPCExtension)
END_PARSE_MAP(CXMLRPCExtension)


///////////////////////////////////////////////////////////////////////
// The one and only CXMLRPCExtension object

CXMLRPCExtension theExtension;


///////////////////////////////////////////////////////////////////////
// CXMLRPCExtension implementation

CXMLRPCExtension::CXMLRPCExtension()
{
	if(bAnyLogging)
	{
		CString strUniqueId = "";
		if(bIndividualLogging)
		{
			//each request is to be recorded in a different file
			strUniqueId.Format("%d",GetCurrentThreadId());
		}
		CString strFile;
		strFile="c:\\temp\\postlog"+ strUniqueId+".txt";

		if(m_LogFile.Open(strFile, CFile::modeCreate | CFile::modeWrite | CFile::shareDenyNone))
		{
		    m_LogFile.WriteString("created post log file in c:\\temp \r\n");
			m_LogFile.Flush();
		}
	    else
		{
		   bAnyLogging = false; //couldn't open a log file in c:\temp
		}
	}
 
	m_pServer = new CORBAServer( 0, NULL);

	if(bAnyLogging)
	{
		if(m_pServer)
		{
		    m_LogFile.WriteString("successfully created server\r\n");
			m_LogFile.Flush();
		}
		else
		{
		    m_LogFile.WriteString("failed to create server\r\n");
			m_LogFile.Flush();
		}
	}
}

CXMLRPCExtension::~CXMLRPCExtension()
{
	if(bAnyLogging)
	{
	   m_LogFile.WriteString("closing post log file\r\n");
	   m_LogFile.Flush();
	   m_LogFile.Close();
	}
	delete m_pServer;
}

BOOL CXMLRPCExtension::GetExtensionVersion(HSE_VERSION_INFO* pVer)
{
	// Call default implementation for initialization
	CHttpServer::GetExtensionVersion(pVer);

	// Load description string
	TCHAR sz[HSE_MAX_EXT_DLL_NAME_LEN+1];
	ISAPIVERIFY(::LoadString(AfxGetResourceHandle(),
			IDS_SERVER, sz, HSE_MAX_EXT_DLL_NAME_LEN));
	_tcscpy(pVer->lpszExtensionDesc, sz);
	return TRUE;
}

BOOL CXMLRPCExtension::TerminateExtension(DWORD dwFlags)
{
	// extension is being terminated
	//TODO: Clean up any per-instance resources
	return TRUE;
}

DWORD CXMLRPCExtension::HttpExtensionProc( EXTENSION_CONTROL_BLOCK *pECB )
{
	if(bAnyLogging)
	{
		m_LogFile.WriteString("In HttpExtensionProc\r\n");
		m_LogFile.Flush();
	}
	// get the data from the form 
	if( 0 == stricmp(pECB->lpszMethod, "post") )
	{
		if(bAnyLogging)
		{
			m_LogFile.WriteString("Got a POST\r\n");
			m_LogFile.Flush();
		}
		// Start processing of input information here. 
		CHAR* lpszTemp = (CHAR *)LocalAlloc( LPTR, pECB->cbTotalBytes); 
		if( NULL == lpszTemp ) 
			return HSE_STATUS_ERROR; 

		// do not always trust the compiler to zero the memory 
		memset(lpszTemp, '\0', pECB->cbTotalBytes); 
		strncpy(lpszTemp, (CHAR *)pECB->lpbData, pECB->cbAvailable ); 
		unsigned long cbQuery = pECB->cbTotalBytes - pECB->cbAvailable; 

		if( cbQuery > 0 ) 
		{ 
			pECB->ReadClient(pECB->ConnID, (LPVOID)(lpszTemp + pECB->cbAvailable), &cbQuery); 
		} 
	
		string strCall = lpszTemp;
		strCall.resize(pECB->cbTotalBytes);

		if(bAnyLogging)
		{
			m_LogFile.WriteString(strCall.c_str());
			m_LogFile.WriteString("***\r\n");		
			m_LogFile.Flush();

			m_LogFile.WriteString("Create CORBAServer\r\n");
			m_LogFile.Flush();
		}
		try
		{
			CORBA::Object_var obj = m_pServer->FindObjectWithName(XMLCallProcessor::CLASS_NAME, "WebRelay");
			
			if(bAnyLogging)
			{
				m_LogFile.WriteString("Narrow CallProcessor\r\n");
				m_LogFile.Flush();
		
			}
			
			XMLCallProcessor_ptr CP = XMLCallProcessor::_narrow(obj);
			if (CORBA::is_nil(CP))
			{
				if(bAnyLogging)
				{
					m_LogFile.WriteString("CallProcessor is NULL\r\n");
					m_LogFile.Flush();
				}
			}

			if(bAnyLogging)
			{
				m_LogFile.WriteString("Forward Call\r\n");
				m_LogFile.Flush();
			}

			string strReply = "Content-Type: text/xml\r\n\r\n";
			string strTmp = CP->ProcessCall(strCall.c_str());
			strReply += strTmp;
	
			DWORD size=strReply.length();
	
			if(bAnyLogging)
			{
				m_LogFile.WriteString("Send Reply\r\n");
				m_LogFile.Flush();
			}
			if ( !pECB->ServerSupportFunction( 
					pECB->ConnID,
		            HSE_REQ_SEND_RESPONSE_HEADER,
		             "200 OK",
		            &size,
		            (LPDWORD) strReply.c_str() ))
			{
				return HSE_STATUS_ERROR;
			}
		}
		catch(...)
		{
			if(bAnyLogging)
			{
				m_LogFile.WriteString("EXCEPTION\r\n");
				m_LogFile.Flush();
			}
		}
	}

	// ready to return to IIS
    pECB->dwHttpStatusCode=200; // 200 OK
    return HSE_STATUS_SUCCESS;
}
///////////////////////////////////////////////////////////////////////
// CXMLRPCExtension command handlers

void CXMLRPCExtension::Default(CHttpServerContext* pCtxt)
{
	StartContent(pCtxt);
	WriteTitle(pCtxt);

	*pCtxt << _T("This default message was produced by the Internet");
	*pCtxt << _T(" Server DLL Wizard. Edit your CXMLRPCExtension::Default()");
	*pCtxt << _T(" implementation to change it.\r\n");

	EndContent(pCtxt);
}

// Do not edit the following lines, which are needed by ClassWizard.
#if 0
BEGIN_MESSAGE_MAP(CXMLRPCExtension, CHttpServer)
	//{{AFX_MSG_MAP(CXMLRPCExtension)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()
#endif	// 0




///////////////////////////////////////////////////////////////////////
// If your extension will not use MFC, you'll need this code to make
// sure the extension objects can find the resource handle for the
// module.  If you convert your extension to not be dependent on MFC,
// remove the comments arounn the following AfxGetResourceHandle()
// and DllMain() functions, as well as the g_hInstance global.

/****

static HINSTANCE g_hInstance;

HINSTANCE AFXISAPI AfxGetResourceHandle()
{
	return g_hInstance;
}

BOOL WINAPI DllMain(HINSTANCE hInst, ULONG ulReason,
					LPVOID lpReserved)
{
	if (ulReason == DLL_PROCESS_ATTACH)
	{
		g_hInstance = hInst;
	}

	return TRUE;
}

****/
