/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// HTTPTransport.cpp: implementation of the CHTTPTransport class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "HTTPTransport.h"
#include "Base64Coder.h"
#include "HTTPSocket.h"

#include <iostream>
using namespace std;
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CHTTPTransport::CHTTPTransport()
{
	// Start Winsock
	WSADATA wsd;
	WSAStartup(0x0101, &wsd);

	m_strHost = "";
	m_strObject = "";
	m_nPort = 80;
	m_strUserName = "";
	m_strPassword = "";
	m_bUseSSL = false;
}

CHTTPTransport::CHTTPTransport(string strHost, string strObject, int nPort, string strUserName, string strPassword, bool bUseSSL)
{
	// Start Winsock
	WSADATA wsd;
	WSAStartup(0x0101, &wsd);

	m_strHost = strHost;
	m_strObject = strObject;
	m_nPort = nPort;
	m_strUserName = strUserName;
	m_strPassword = strPassword;
	m_bUseSSL = bUseSSL;
}

CHTTPTransport::CHTTPTransport(string strFullURL)
{
	// Start Winsock
	WSADATA wsd;
	WSAStartup(0x0101, &wsd);

	m_strHost = "";
	m_strObject = "";
	m_nPort = 80;
	m_strUserName = "";
	m_strPassword = "";
	m_bUseSSL = false;
	m_strFullURL = strFullURL;
	ParseURL(strFullURL);
}

CHTTPTransport::~CHTTPTransport()
{
	WSACleanup();
}

string CHTTPTransport::SendXMLCall(string strXML)
{
	string strHeaders = "";
	strHeaders += "POST "+m_strObject+" HTTP/1/0\r\n";
	strHeaders += "Host: "+m_strHost+"\r\n";
	// Authorization
	if (!m_strUserName.empty())
	{
		string strAuth = m_strUserName+":"+m_strPassword;

		Base64Coder		Coder;
		Coder.Encode(strAuth.c_str());
		// needs base64 encoding!!
		strHeaders += "\r\nAuthorization: "+CString(Coder.EncodedMessage());
	}

	strHeaders += "Content-Type: text/xml\r\n";
	char buf[16];
	sprintf(buf, "%d",strXML.length());
	strHeaders += "Content-length: "+string(buf)+"\r\n\r\n";

	string req = strHeaders + strXML;

	CHttpSocket sock;
	sock.Create();
	sock.Connect(m_strHost.c_str(),m_nPort);
	sock.Send((void*)req.c_str(), req.length());

	char sRequest[10000];
	LPSTR pszOverFlowBuffer = NULL;
	char* sRequestReturned = sRequest;
	if (sock.ReadResponse(sRequest, 10000, "\r\n\r\n", &pszOverFlowBuffer, 3000))
	{
		//Point to the overflow buffer if it is valid
		if (pszOverFlowBuffer)
			sRequestReturned = pszOverFlowBuffer;
	}

	sock.Close();

	string strResp = sRequestReturned;
	int nPos = strResp.find_first_of('<');
	strResp.erase(0,nPos);

	return strResp;
}

bool CHTTPTransport::GetRemoteFile(string strLocalPath)
{
	CWSocket sock;
	sock.Create();
	sock.Connect(m_strHost.c_str(),m_nPort);

	string strHeaders = "";
	strHeaders += "GET "+m_strObject+" HTTP/1/0\r\n";
	strHeaders += "Host: "+m_strHost+"\r\n\r\n";

	sock.Send((void*)strHeaders.c_str(), strHeaders.length());

	bool bFirstTime=true;

	char sRequest[10002];
	int nContentLength = 0;
	string strResponse = "";

	CFile localFile(strLocalPath.c_str(), CFile::modeCreate|CFile::modeWrite);
	
	try
	{
		int nBytes = 0;
		do
		{
			nBytes = sock.Receive(sRequest, 10000);
			strResponse += string(sRequest).substr(0, nBytes);
		} while(nBytes > 0);

		if(sock)
		{
			sock.Close();
		}

		if(bFirstTime)
		{
			//get the headers
			CStringArray arrHeaders;
			int nPos = strResponse.find_first_of('\r');
				
			while (nPos > 0)
			{
				string strLine = strResponse.substr(0,nPos);
				strResponse.erase(0, nPos+2);
				arrHeaders.Add(strLine.c_str());
				nPos = strResponse.find_first_of('\r');
			}

			// erase the dividing carriage return + line feed
			strResponse.erase(0,2);

			//get content length
			for (int i =0; i<arrHeaders.GetSize(); i++)
			{
				CString strHeader = arrHeaders.GetAt(i);
				if (strHeader.Left(16) == "Content-Length: ")
				{
					strHeader.Delete(0,16);
					nContentLength = atoi(strHeader);
					break;
				}
			}

			//write minus the headers
			localFile.Write(strResponse.c_str(), strResponse.length());
			bFirstTime=false;	//dont do the headers again!!
		}
		else
		{
			//now the normal reading in
			localFile.Write(strResponse.c_str(), strResponse.length());
		}

		
		//close the socket and file
		if(localFile)
		{
			localFile.Close();
		}

	}
	catch(...)
	{

		cout<<"EXCEPTION WRITING LOCAL FILE."<<endl;
		if(sock)
		{
			sock.Close();
		}
		if(localFile)
		{
			localFile.Close();
		}

	}

	return true;
}

void CHTTPTransport::ParseURL(string strURL)
{
	if (strURL.substr(0,7) != "http://")
	{
		cout<<"URL Parse Error!: "<<strURL.c_str()<<endl;;
		return ;
	}
	strURL.erase(0,7);
	int nPos = strURL.find_first_of('/');
	if(nPos > 0)
	{
		m_strHost = strURL.substr(0,nPos).c_str();
		m_strObject = strURL.substr(nPos).c_str();
	}
	else
	{
		m_strHost = strURL.c_str();
		m_strObject = "/";
	}

	int pos = m_strHost.find_first_of(':');
	if (pos > 0)
	{
		int pos = m_strHost.find_first_of(':');
		string strPort = m_strHost.substr(pos);
		m_nPort = atoi(strPort.c_str());
		m_strHost = m_strHost.substr(0,pos);
	}
	else
	{
		m_nPort = 80;
	}
}