/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// XMLRequest.cpp: implementation of the CXMLRequest class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "XMLParser.hpp"
#include "xmlrpc.h"

#include <iostream>
using namespace std;

extern CCallManager* g_pCallManager;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CXMLRequest::CXMLRequest()
{

}

CXMLRequest::CXMLRequest(string strXMLRequest)
{
	// Parse the XML and set up the member variables.
    XmlParse<char> parser(strXMLRequest.c_str());
    XmlParse<char>::XMLTree tree;
    parser.Parse(&tree);

    using XmlParse<char>::XMLTree::NodeChild;
    using XmlParse<char>::XMLTree::NodeElement;
    using XmlParse<char>::XMLTree::NodeText;
	using XmlParse<char>::XmlString;

	// Retrieve the root element (methodcall):
    NodeElement* pMethodCall = NULL;
	pMethodCall = dynamic_cast<NodeElement*>(*tree.m_root.m_children.begin());

    NodeElement* pMethodName = NULL;
	pMethodName = dynamic_cast<NodeElement*>(*pMethodCall->FirstChildElement());

	string strTmp = pMethodName->GetFirstChildText()->m_strValue;
	string strMethodName = strTmp.c_str();
	m_strFullMethodName = strMethodName;

	int nPos = strMethodName.find_first_of(".");
	m_strHandlerName = strMethodName.substr(0,nPos);
	strMethodName.erase(0,nPos+1);
	m_strMethod = strMethodName;

    const NodeElement* pParams = NULL;
	pParams = pMethodCall->GetChildElementByName("params");

	// pParamsElement should now point to the "params" child element.
	// Let's enumerate the "param" sons:
    XmlParse<char>::XMLTree::Children::const_iterator pChild;
	XmlParse<char>::XMLTree::NodeElement* pParam = NULL;
	for(pChild = pParams->m_children.begin();
		pChild != pParams->m_children.end();
		++pChild)
	{
		pParam = dynamic_cast<XmlParse<char>::XMLTree::NodeElement*>(*pChild);
		if(pParam && pParam->m_strValue == "param")
		{
			NodeElement* pValue = NULL;
			NodeElement* pType = NULL;
			pValue = dynamic_cast<NodeElement*>(*pParam->FirstChildElement());
			pType = dynamic_cast<NodeElement*>(*pValue->FirstChildElement());
    
			string strType = pType->m_strValue;

			if (strType == "string")
			{
				CXMLString* pStrXML = new CXMLString();
				pStrXML->FromXML(pType);
				m_params.Add(pStrXML);
			}
			else if (strType == "int")
			{
				CXMLInt* pIXML = new CXMLInt();
				pIXML->FromXML(pType);
				m_params.Add(pIXML);
			}
			else if (strType == "i4")
			{
				CXMLLong* pLXML = new CXMLLong();
				pLXML->FromXML(pType);
				m_params.Add(pLXML);
			}
			else if (strType == "boolean")
			{
				CXMLBool* pBXML = new CXMLBool();
				pBXML->FromXML(pType);
				m_params.Add(pBXML);
			}
			else if (strType == "double")
			{
				CXMLDouble* pDXML = new CXMLDouble();
				pDXML->FromXML(pType);
				m_params.Add(pDXML);
			}
			else if (strType == "array")
			{
				CXMLArray* pArrXML = new CXMLArray();
				pArrXML->FromXML(pType);
				m_params.Add(pArrXML);
			}
			else if (strType == "struct")
			{
				CXMLStruct* pColXML = new CXMLStruct();
				pColXML->FromXML(pType);
				m_params.Add(pColXML);
			}
			else if (strType == "base64")
			{
				CXMLBase64* b64XML = new CXMLBase64();
				b64XML->FromXML(pType);
				m_params.Add(b64XML);
			}
			else if (strType == "dateTime.iso8601")
			{
				CXMLDate* dateXML = new CXMLDate();
				dateXML->FromXML(pType);
				m_params.Add(dateXML);
			}
			else
			{
				CCustomDataProvider* pProvider = g_pCallManager->GetProviderForType(strType);
				if (pProvider != NULL)
				{
					CXMLParameter* pParam = pProvider->CreateInstance();
					pParam->SetCallManager(g_pCallManager);
					pParam->FromXML(pType);
					m_params.Add(pParam);
				}
				else
				{
					cout<<"Unable to find handler for custom type: "<<strType<<endl;
				}
			}

		}
	}

}

CXMLRequest::~CXMLRequest()
{
	while (m_params.GetSize() > 0)
	{
		delete m_params.GetAt(0);
		m_params.RemoveAt(0);
	}
}

void CXMLRequest::SetMethodName(string strMethod)
{
	m_strFullMethodName = strMethod;
}

void CXMLRequest::AddParameter(CXMLParameter* param)
{
	m_params.Add(param);
}

void CXMLRequest::SetTransport(CHTTPTransport& trans)
{
	m_transport = trans;
}

CXMLResponse* CXMLRequest::SendRequest()
{
	string buffer = "";
	buffer+="<?xml version=\"1.0\"?>\r\n";
	buffer+="<methodCall>\r\n";
	buffer+="<methodName>" + m_strFullMethodName + "</methodName>\r\n";
	buffer+="<params>\r\n";

	for (int i = 0; i < m_params.GetSize(); i++) 
	{
		string strParam;
		if (m_params.GetAt(i)->ToXML(strParam))
		{
			buffer+="<param>"+strParam+"</param>\r\n";
		}
	}

	buffer+="</params>\r\n";
	buffer+="</methodCall>";

	string strResp = m_transport.SendXMLCall(buffer);
	CXMLResponse* pResp = new CXMLResponse (strResp);
	return pResp;
}

int CXMLRequest::GetNumParams()
{
	return m_params.GetSize();
}

CXMLParameter* CXMLRequest::GetParam(int nParam)
{
	return m_params.GetAt(nParam);
}
