/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// XMLResponse.cpp: implementation of the CXMLResponse class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "xmlrpc.h"

#include <iostream>
using namespace std;

extern CCallManager* g_pCallManager;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CXMLResponse::CXMLResponse()
{
	m_bError = false;
}

CXMLResponse::CXMLResponse(string strXMLResponse)
{
	m_bError = false;
	m_strXMLResponse = strXMLResponse;

	try
	{
	    XmlParse<char> parser(m_strXMLResponse.c_str());
	    XmlParse<char>::XMLTree tree;
	    parser.Parse(&tree);

	    using XmlParse<char>::XMLTree::NodeChild;
	    using XmlParse<char>::XMLTree::NodeElement;
	    using XmlParse<char>::XMLTree::NodeText;
		using XmlParse<char>::XmlString;

		// Retrieve the root element (methodcall):
	    NodeElement* pMethodResponse = NULL;
		pMethodResponse = dynamic_cast<NodeElement*>(*tree.m_root.m_children.begin());

	    const NodeElement* pParamsElement = NULL;
		pParamsElement = pMethodResponse->GetChildElementByName("params");
		if (pParamsElement)
		{
			// pParamsElement should now point to the "params" child element.
			// Let's enumerate the "param" sons:

		    XmlParse<char>::XMLTree::Children::const_iterator pChild;
			XmlParse<char>::XMLTree::NodeElement* pParam = NULL;
			for(pChild = pParamsElement->m_children.begin();
				pChild != pParamsElement->m_children.end();
				++pChild)
			{
				pParam = dynamic_cast<XmlParse<char>::XMLTree::NodeElement*>(*pChild);
				if(pParam && pParam->m_strValue == "param")
				{
					XmlParse<char>::XMLTree::NodeElement* pValue = NULL;
					XmlParse<char>::XMLTree::NodeElement* pType = NULL;
					pValue = dynamic_cast<XmlParse<char>::XMLTree::NodeElement*>(*pParam->FirstChildElement());
					pType = dynamic_cast<XmlParse<char>::XMLTree::NodeElement*>(*pValue->FirstChildElement());
	
					string strType = pType->m_strValue;
	
					if (strType == "string")
					{
						CXMLString* pStrXML = new CXMLString();
						pStrXML->FromXML(pType);
						m_params.Add(pStrXML);
					}
					else if (strType == "int")
					{
						CXMLInt* pIXML = new CXMLInt();
						pIXML->FromXML(pType);
						m_params.Add(pIXML);
					}
					else if (strType == "i4")
					{
						CXMLLong* pLXML = new CXMLLong();
						pLXML->FromXML(pType);
						m_params.Add(pLXML);
					}
					else if (strType == "boolean")
					{
						CXMLBool* pBXML = new CXMLBool();
						pBXML->FromXML(pType);
						m_params.Add(pBXML);
					}
					else if (strType == "double")
					{
						CXMLDouble* pDXML = new CXMLDouble();
						pDXML->FromXML(pType);
						m_params.Add(pDXML);
					}
					else if (strType == "array")
					{
						CXMLArray* pArrXML = new CXMLArray();
						pArrXML->FromXML(pType);
						m_params.Add(pArrXML);
					}
					else if (strType == "struct")
					{
						CXMLStruct* pColXML = new CXMLStruct();
						pColXML->FromXML(pType);
						m_params.Add(pColXML);
					}
					else if (strType == "base64")
					{
						CXMLBase64* b64XML = new CXMLBase64();
						b64XML->FromXML(pType);
						m_params.Add(b64XML);
					}
					else if (strType == "dateTime.iso8601")
					{
						CXMLDate* dateXML = new CXMLDate();
						dateXML->FromXML(pType);
						m_params.Add(dateXML);
					}
					else
					{
						CCustomDataProvider* pProvider = g_pCallManager->GetProviderForType(strType);
						if (pProvider)
						{
							CXMLParameter* pParam = pProvider->CreateInstance();
							pParam->FromXML(pType);
							m_params.Add(pParam);
						}
						else
						{
							cout<<"Unable to find handler for custom type: "<<strType<<endl;
							SetError(UNRECOGNIZED_PARAM);
						}
					}
				}
			}
		}
		else
		{
		    const NodeElement* pFaultElement = NULL;
			pFaultElement = pMethodResponse->GetChildElementByName("fault");
			XmlParse<char>::XMLTree::NodeElement* pValue = NULL;
			XmlParse<char>::XMLTree::NodeElement* pStruct = NULL;
			pValue = dynamic_cast<XmlParse<char>::XMLTree::NodeElement*>(*pFaultElement->FirstChildElement());
			pStruct = dynamic_cast<XmlParse<char>::XMLTree::NodeElement*>(*pValue->FirstChildElement());

			CXMLStruct* pErrStruct = new CXMLStruct();
			pErrStruct->FromXML(pStruct);
			CXMLInt* pInt = (CXMLInt*)pErrStruct->GetValueForName("faultCode");
			if (pInt)
			{
				SetError(pInt->GetValue());
			}
		}
	}
	catch (...)
	{
		SetError(PARSE_FAILURE);
	}
}

CXMLResponse::~CXMLResponse()
{
	while (m_params.GetSize() > 0)
	{
		delete m_params.GetAt(0);
		m_params.RemoveAt(0);
	}
}


string CXMLResponse::ToXML()
{
	if (m_strXMLResponse!="")
		return m_strXMLResponse;

	if (IsError())
		return ErrorToXML();

	m_strXMLResponse = "";
	m_strXMLResponse += "<?xml version=\"1.0\"?>\r\n";
	m_strXMLResponse += "<methodResponse>\r\n";
	m_strXMLResponse += "<params>\r\n";

	for (int i=0; i<m_params.GetSize(); i++)
	{
		m_strXMLResponse += "<param>";
		string strParam;
		m_params.GetAt(i)->ToXML(strParam);
		m_strXMLResponse += strParam;
		m_strXMLResponse += "</param>\r\n";
	}

	m_strXMLResponse += "</params>\r\n";
	m_strXMLResponse += "</methodResponse>\r\n";

	return m_strXMLResponse;
}

string CXMLResponse::ErrorToXML()
{
	string strErrXml = "";
	strErrXml += "<?xml version=\"1.0\"?>\r\n";
	strErrXml += "<methodResponse>\r\n";
	strErrXml += "<fault>";
	strErrXml += "<value>";
	strErrXml += "<struct>";
	strErrXml += "<member>";
	strErrXml += "<name>faultCode</name>";
	strErrXml += "<value><int>";
	char buffer[20];
	itoa(m_errType, buffer, 10);
	strErrXml += buffer;
	strErrXml += "</int></value>";
	strErrXml += "</member>";
	strErrXml += "<member>";
	strErrXml += "<name>faultString</name>";
	strErrXml += "<value><string>";

	switch(m_errType)
	{
		case PARSE_FAILURE:
		{
			strErrXml += "Parse Failure";
			break;
		}
		case UNRECOGNIZED_PARAM:
		{
			strErrXml += "Unrecognized Parameter";
			break;
		}
		case INVALID_PARAM:
		{
			strErrXml += "Invalid Parameter";
			break;
		}
		case INSUFFICIENT_PARAMS:
		{
			strErrXml += "Insufiicient Parameters";
			break;
		}
		case TOO_MANY_PARAMS:
		{
			strErrXml += "Too Many Parameters";
			break;
		}
		case FUNC_NOT_FOUND:
		{
			strErrXml += "Function Not Found";
			break;
		}
		case UNKNOWN_ERROR:
		{
			strErrXml += "Unknown Error";
			break;
		}
	}

	strErrXml += "</string></value>";
	strErrXml += "</member>";
	strErrXml += "</struct>";
	strErrXml += "</value>";
	strErrXml += "</fault>\r\n";
	strErrXml += "</methodResponse>\r\n";

	return strErrXml;
}

string CXMLResponse::PrintError()
{
	string strErrMsg = "";
	strErrMsg += "ERROR:\n";
	strErrMsg += "Error Code = ";
	char buffer[20];
	itoa(m_errType, buffer, 10);
	strErrMsg += buffer;
	strErrMsg += "\n";
	strErrMsg += "Error Message = ";

	switch(m_errType)
	{
		case PARSE_FAILURE:
		{
			strErrMsg += "Parse Failure";
			break;
		}
		case UNRECOGNIZED_PARAM:
		{
			strErrMsg += "Unrecognized Parameter";
			break;
		}
		case INVALID_PARAM:
		{
			strErrMsg += "Invalid Parameter";
			break;
		}
		case INSUFFICIENT_PARAMS:
		{
			strErrMsg += "Insufiicient Parameters";
			break;
		}
		case TOO_MANY_PARAMS:
		{
			strErrMsg += "Too Many Parameters";
			break;
		}
		case FUNC_NOT_FOUND:
		{
			strErrMsg += "Function Not Found";
			break;
		}
	}
	strErrMsg += "\n";

	return strErrMsg;
}
