/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/*===========================================================================*/
/*                                       */
/*               **** C O M M O N ****                   */
/*                                       */
/*===========================================================================*/
/*                                       */
/*          Module Name: PSL_MACROS                  */
/*          ============                         */
/*                                       */
/* Description:                                  */
/* ============                                  */
/*                                       */
/*  This module contains the fundamental macros used by C programs that  */
/*  might change from one environment to another.  This includes macros  */
/*  that will allow the program to locate other header files.        */
/*                                       */
/*===========================================================================*/



/*   Create some more predefined variables */

#ifdef VAX
#define VMS 1
#define PSL_LITTLE_ENDIAN   1
typedef short PSL_WORD_T;
typedef long  PSL_LONGWORD_T;
typedef unsigned short PSL_U_WORD_T;
typedef unsigned long  PSL_U_LONGWORD_T;
#endif

#ifdef __ALPHA
#define ALPHA   1
#define VMS 1
#define PSL_LITTLE_ENDIAN   1
typedef short PSL_WORD_T;
typedef long  PSL_LONGWORD_T;
typedef unsigned short PSL_U_WORD_T;
typedef unsigned long  PSL_U_LONGWORD_T;
#endif

#ifdef sun
#define SUN 1
#define UNIX    1
#define SVR4    1
#define PSL_BIG_ENDIAN  1
typedef short PSL_WORD_T;
typedef long  PSL_LONGWORD_T;
typedef unsigned short PSL_U_WORD_T;
typedef unsigned long  PSL_U_LONGWORD_T;
#define SELECT_BIT_TYPE fd_set
#endif

/*
 * The idea behind PSL_LONGWORD etc was to create a type which is of a fixed
 * size of each platform - its doesn't work. If you change the definition
 * below from long to int (4 bytes) you'll get thousands of compilation
 * errors - why? Because sdl generates PSL_LONGWORD which is effectively an int
 * but fields in structures are passed by address and used to store ptrs which
 * are 8 bytes in OSF
 */
#ifdef __osf__
#define OSF 1
#define UNIX    1
#define PSL_LITTLE_ENDIAN   1
typedef short PSL_WORD_T;
typedef long  PSL_LONGWORD_T;
typedef unsigned short PSL_U_WORD_T;
typedef unsigned long  PSL_U_LONGWORD_T;
#define SELECT_BIT_TYPE fd_set
#endif

#ifdef HPV9
#define hpux    1
#define HP  1
#define HP9AND10 1
#define UNIX    1
#define PSL_BIG_ENDIAN  1
typedef short PSL_WORD_T;
typedef long  PSL_LONGWORD_T;
typedef unsigned short PSL_U_WORD_T;
typedef unsigned long  PSL_U_LONGWORD_T;
#define PSL_HAS_SUN_LEN_FIELD 1
#define SELECT_BIT_TYPE int
#endif

#ifdef HPV10
#define hpux    1
#define HP  1
#define HP9AND10 1
#define UNIX    1
#define PSL_BIG_ENDIAN  1
typedef short PSL_WORD_T;
typedef long  PSL_LONGWORD_T;
typedef unsigned short PSL_U_WORD_T;
typedef unsigned long  PSL_U_LONGWORD_T;
#define PSL_HAS_SUN_LEN_FIELD 1
#define SELECT_BIT_TYPE int
#endif

#ifdef HPV1020
#define hpux    1
#define HP  1
#define UNIX    1
#define PSL_BIG_ENDIAN  1
typedef short PSL_WORD_T;
typedef long  PSL_LONGWORD_T;
typedef unsigned short PSL_U_WORD_T;
typedef unsigned long  PSL_U_LONGWORD_T;
#define SELECT_BIT_TYPE fd_set
#endif

#ifdef _AIX
#define AIX 1
#define IBM 1
#define UNIX    1
#define PSL_BIG_ENDIAN  1
typedef short PSL_WORD_T;
typedef long  PSL_LONGWORD_T;
typedef unsigned short PSL_U_WORD_T;
typedef unsigned long  PSL_U_LONGWORD_T;
#define PSL_HAS_SUN_LEN_FIELD 1
#endif

#ifdef NCRV202
#define NCR 1
#define UNIX    1
#define SVR4    1
#define PSL_LITTLE_ENDIAN   1
typedef short PSL_WORD_T;
typedef long  PSL_LONGWORD_T;
typedef unsigned short PSL_U_WORD_T;
typedef unsigned long  PSL_U_LONGWORD_T;
#define SELECT_BIT_TYPE fd_set
#endif

#ifdef NCRV203
#define NCR 1
#define UNIX    1
#define SVR4    1
#define PSL_LITTLE_ENDIAN   1
typedef short PSL_WORD_T;
typedef long  PSL_LONGWORD_T;
typedef unsigned short PSL_U_WORD_T;
typedef unsigned long  PSL_U_LONGWORD_T;
#define SELECT_BIT_TYPE fd_set
#endif

#ifdef _WIN32
#define NTI 1
#define WIN32   1
#define PSL_LITTLE_ENDIAN   1
typedef short PSL_WORD_T;
typedef long  PSL_LONGWORD_T;
typedef unsigned short PSL_U_WORD_T;
typedef unsigned long  PSL_U_LONGWORD_T;
#endif

#ifdef _WIN32
#define NTALPHA 1
#define WIN32   1
#define PSL_LITTLE_ENDIAN   1
#define HAVE_STRING_H   1
typedef short PSL_WORD_T;
typedef long  PSL_LONGWORD_T;
typedef unsigned short PSL_U_WORD_T;
typedef unsigned long  PSL_U_LONGWORD_T;
#endif


/*     Define symbol used in PSL_CNV routines */
#ifdef PSL_BIG_ENDIAN
#define PSLRECFMT_LOCAL 2
#else
#define PSLRECFMT_LOCAL 1
#endif

/*    Define a Macro to reference Common Data Files */

#define psl_common_file(file)   <file>

/*    Define a Macro to reference VTEST Data Files */

#define psl_vtest_file(file)    <file>

/*    Define a Macro to reference UTIL Data Files */

#define psl_util_file(file) <file>

/*    Define a Macro to reference DISTRIB (Distributor) Data Files */

#define psl_distrib_file(file)  <file>

/*    Define a Macro to reference VIDEO Data Files */

#define psl_video_file(file)    <file>

/*    Define a Macro to reference VTIMER Data Files */

#define psl_vtimer_file(file)   <file>


/*    Define a Macro to reference SYSTEL Data Files */

#define psl_systel_file(file)   <file>


/*    Define a Macro to test a routine status:
 *
 *    ((x >> 30) & 0X3) extracts the '2 bit' severity from the
 *    supplied windows message number. The severity is obtained
 *    from bits 30 and 31, which are interpreted as follows:
 *
 *    0  Success
 *    1  Informational
 *    2  Warning
 *    3  Error
 *
 *    the psl_test_status condition is TRUE if the severity of the
 *    supplied windows message number is 'Success or Informational',
 *    otherwise it is FALSE.
 */
#define psl_test_status(x)  (((x >> 30) & 0X3) <= 1)

/*    Define the JMP_ADDR Macro */
/*    Note that it is a Function Prototype (in PSL_RTNDEF) for VAX/VMS */

#ifndef VAX
#define JMP_ADDR &
#endif


/*    Define a Macro to clear the various File I/O related error numbers */

#ifdef VMS
#define PSL_CLEARERR()  errno = 0, vaxc$errno = 0
#else
#define PSL_CLEARERR()  errno = 0
#endif

/*    Define the SETJMP Macro to establish an exception handler if on VMS */

#ifdef VMS
#pragma nostandard
#define SETJMP(jmp_env) setjmp(jmp_env); VAXC$ESTABLISH (PSL_EXCEPT)
#pragma standard
#else
#define SETJMP(jmp_env) setjmp(jmp_env)
#endif

/*    Define the PSL_SETJMP Macro to establish an exception handler if on VMS */

#ifdef VMS
#pragma nostandard
#define PSL_SETJMP(jmp_env) (VAXC$ESTABLISH (PSL_EXCEPT), setjmp(jmp_env))
#pragma standard
#else
#define PSL_SETJMP(jmp_env) setjmp(jmp_env)
#endif

/*    Define a Boolean type */

#define PFALSE  0
#define PTRUE   1
typedef unsigned long   Boolean;

/*
 *    Define macros for toupper and tolower. These usually exist in ctype.h
 *    but are not public macros and therefore non standard.
 */
#define psl_toupper(c)  ((c) >= 'a' && (c) <= 'z' ? (c) & 0x5F:(c))
#define psl_tolower(c)  ((c) >= 'A' && (c) <= 'Z' ? (c) | 0x20:(c))

/*
 *    If this is a SUN then define the SEMUNO_LIMIT_PROBLEM. The problem is
 *    that the X-server on the sun consumes the SEM_UNDO structures.
 *    When there are no undos left operations requiring semaphore undos
 *    fail and this causes the ctm_server to fail to start and lgr to fail
 *    to obtain license slots.
 */
#ifdef SUN
#   define SEMUNDO_LIMIT_PROBLEM
#endif

/*
 *  Define ssize_t as int for the NCR which is missing this typedef.
 */
#ifdef NCR
#   define ssize_t int
#endif

/*
 *  When building against Open Interface on the HP we get a compilation
 *  message warning of the redefinition of the macro _HPUX_SOURCE in
 *  the Open Interface header file MCHPUB.H.
 *  In order to try to get around this (assuming that Neuron Data will
 *  not be fixing the problem in the short term) we are going to undefine
 *  any existing value of the symbol and redefine it without a value so
 *  that the compiler should not object.
 */
#ifdef HP
#ifdef _HPUX_SOURCE
#undef _HPUX_SOURCE
#define _HPUX_SOURCE
#endif /* _HPUX_SOURCE */
#endif /* HP */

/*
 *  Define VMS_MSG_FORMAT for those platforms we want to use VMS message
 *  prefixes in diagnostics output.
 */
#ifdef VMS
#   define VMS_MSG_FORMAT 1
#endif
