#ifndef PSL__TRMDEF_LOADED
#define PSL__TRMDEF_LOADED 1
/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/*===========================================================================*/
/*                                       */
/*               **** C O M M O N ****                   */
/*                                       */
/*===========================================================================*/
/*                                       */
/*          Module Name: TRMDEF                  */
/*          ============                         */
/*                                       */
/* Description:                                  */
/* ============                                  */
/*                                       */
/*  Define various symbolic values required for use with the         */
/*  Terminal Handling routines.                      */
/*                                       */
/*===========================================================================*/

/*** MODULE TRMDEF ***/

/*    This Include File defines symbolic values for parameters */
/*    passed to certain TRM (and SCR) routines. */

/*    Symbolic Values for I/O Subfunctions.  These are specified on a */
/*    call to TRM_INPUT to determine the type of input required. */

#define            IOS_UPPER 0X1    /* Convert to Upper Case */
#define            IOS_NOECHO 0X2   /* No Echo */
#define            IOS_ENFTRM 0X4   /* Enforced Termination */
#define            IOS_PRGTY 0X8    /* Purge Type-Ahead */
#define            IOS_EDIT 0X10    /* Perform Line Editing */
#define            IOS_SWCURS 0X20  /* Switch Cursor On/Off */
/* during Input operation */
#define            IOS_RETDEL 0X40  /* Return <DELETE>  */
#define            IOS_NOREFR 0X80  /* Omit SCR_REFRESH */

/*    Symbolic Values for Colour.  These are specified on calls to */
/*    routines such as SCR_OUTPUT and SCR_DRWLIN. */

/*    NOTE: Some of these symbols are defined for */
/*     Programming convenience ONLY.  Various parts of the */
/*     Terminal Handling Routines rely on the specific values */
/*     given and these MUST NOT be changed. */

#define            TCOL_MIN 1   /* Minimum Legal Value  */
#define            TCOL_BLACK 1 /* Black */
#define            TCOL_RED 2   /* Red */
#define            TCOL_GREEN 3 /* Green */
#define            TCOL_YELLOW 4    /* Yellow */
#define            TCOL_BLUE 5  /* Blue */
#define            TCOL_MAGENTA 6   /* Magenta */
#define            TCOL_CYAN 7  /* Cyan */
#define            TCOL_WHITE 8 /* White */
#define            TCOL_CREAM 9 /* Cream */
#define            TCOL_ORANGE 10   /* Orange */
#define            TCOL_LIME 11 /* Lime */
#define            TCOL_TURQUOISE 12    /* Turquoise */
#define            TCOL_LIGHTBLUE 13    /* Lightblue */
#define            TCOL_VIOLET 14   /* Violet */
#define            TCOL_MAUVE 15    /* Mauve */
#define            TCOL_GREY 16 /* Grey */
#define            TCOL_MAX 16  /* Maximum Legal Value  */

/*    Colour Types */

#define            TCTP_SCREEN 0    /* Screen */
#define            TCTP_BORDER 1    /* Border */

/*    Graphics Line Types */

#define            TGPH_HORIZ 0 /* Horizontal       */
#define            TGPH_VERT 1  /* Vertical     */

/*    Cursor Types */

#define            TCUR_NONE 0  /* No Cursor */
#define            TCUR_STATBLK 1   /* Static Block */
#define            TCUR_BLNKBLK 2   /* Blinking Block */
#define            TCUR_STATUND 3   /* Static Underscore */
#define            TCUR_BLNKUND 4   /* Blinking Underscore */

/*   Scrolling types - one of these is specified on a call to */
/*   TRM/SCR_SCROLL to indicate what sort of scrolling is required */

#define            TSCR_UP 1    /* Scroll Up */
#define            TSCR_DOWN 2  /* Scroll Down */
#define            TSCR_LEFT 3  /* Scroll Left */
#define            TSCR_RIGHT 4 /* Scroll Right */

/*    Symbolic Values for Character Attributes. */

#define            TCAT_MONO_INV 0X1    /* Inverse Video for Mono Term  */
#define            TCAT_COLR_INV 0X2    /* Inverse Video for Colr Term  */
#define            TCAT_INV 0X3 /* Inverse Video for Both   */
#define            TCAT_MONO_INT 0X4    /* Half Intensity for Mono Term */
#define            TCAT_COLR_INT 0X8    /* Half Intensity for Colr Term */
#define            TCAT_INT 0XC /* Half Intensity for Both  */
#define            TCAT_BLK 0X10    /* Blinking */
#define            TCAT_UND 0X20    /* Underlined */
#define            TCAT_DWD 0X40    /* Double width if set */
#define            TCAT_DHT 0X80    /* Double height if set */

#define            TCAT_NIO 0X4000  /* Don't actually do the I/O*//*    */
#define            TCAT_SAV 0X8000  /* Save the Output if set*//*   */



/*    Default Colours and Attributes for Errors, (Help) Messages and */
/*    Broadcast Messages */

#define            TDEF_HLP_TRMATTW 0   /* Help Attr */
#define            TDEF_HLP_COLCHRB TCOL_YELLOW /* Help Char Colr */
#define            TDEF_HLP_COLBCKB TCOL_BLACK  /* Help Cell Colr */
#define            TDEF_ERR_TRMATTW TCAT_MONO_INV   /* Error Attr */
#define            TDEF_ERR_COLCHRB TCOL_BLACK  /* Error Char Colr */
#define            TDEF_ERR_COLBCKB TCOL_RED    /* Error Cell Colr */
#define            TDEF_BRD_TRMATTW 0   /* Broadcast Attr */
#define            TDEF_BRD_COLCHRB TCOL_YELLOW /* Broadcast Char Colr */
#define            TDEF_BRD_COLBCKB TCOL_BLACK  /* Broadcast Cell Colr */

/*    Symbolic Values for TRM_CLEAR. */

#define            TCLR_V_TYPE 0    /* Bits 0 - 2 are the ... */
#define            TCLR_S_TYPE 3    /* ... Clear Type */
#define            TCLR_SCN_ALL 1   /* Clear the Whole Screen */
#define            TCLR_SCN_RST 2   /* Clear Screen from Current/Specified */
#define            TCLR_LIN_ALL 3   /* Clear a Whole Line */
#define            TCLR_LIN_RST 4   /* Clear Line from Current/Specified */
#define            TCLR_RCT 5   /* Clear a Rectangle */
#define            TCLR_BOX 6   /* Clear a Box */
#define            TCLR_MAX 6   /* Maximum Clear Type */
#define            TCLR_GPH 0X20    /* Clear Graphics Table as well */
#define            TCLR_NIO 0X40    /* Do not do I/O to terminal */
#define            TCLR_SAV 0X80    /* Clear Screen Image as well */

/*    Symbolic Values for TRM_RESET_LINES/TRM_OUTPUT_LINE. */

#define            TLIN_MSG 0X1 /* Message */
#define            TLIN_ERR 0X2 /* Error */
#define            TLIN_BRD 0X4 /* Broadcast */
#define            TLIN_AHL 0X8 /* Auto-Help */
#define            TLIN_ONLY 0X80   /* Do only if necessary */

/*    Symbolic values for TRM_GET_DETAILS item codes */

#define            TRMDET_ATT 1 /* Does trm support attributes? */
#define            TRMDET_SCR 2 /* Does trm support vertical scrolling? */
#define            TRMDET_COL 3 /* Does trm support Colour? */
#define            TRMDET_132 4 /* Does trm support 132 columns? */
#define            TRMDET_WID 5 /* Return terminal width */
#define            TRMDET_DEP 6 /* Return terminal depth */
#define            TRMDET_ANS 7 /* Is trm ANSI-compatible?*/
#define            TRMDET_SCR_LST 8 /* Return Screen Image Listhead */
#define            TRMDET_SCR_WRK 9 /* Return Work Screen Image Address */
#define            TRMDET_SCR_LIV 10    /* Return Live Screen Image Address */
#define            TRMDET_DHT_LIN 11    /* Is Double Height a Line Attribute? */
#define            TRMDET_DWD_LIN 12    /* Is Double Width a Line Attribute? */
#define            TRMDET_DHT_DWD 13    /* Does Double Height need Double Width? */
#define            TRMDET_RCT 14    /* Does trm support Rectangular Operations? */
#define            TRMDET_CHR_SET 15    /* Retrieve Current Character Set */
#define            TRMDET_CHR_TBL 16    /* Retrieve Character Set Table */
#define            TRMDET_OSC 17    /* Are Old Scroll Image Mechanisms Active? */
#define            TRMDET_FBK 18    /* Is Feedback in operation?*/
#define            TRMDET_ASZ 19    /* Supports Autosizing*/
#define            TRMDET_MAX 19    /* Maximum supported value */





/*    Symbolic values for TRM_INIT flags */

#define            TFLG_ASZ 0X1 /* Auto-Size terminal */
#define            TFLG_OSC 0X2 /* Use old Screen Image Support */
#define            TFLG_CHAN 0X4    /* Channel already assigned */





/*    Symbolic Values for TRM_FKEY_HELP */

#define            TFKH_PT1 0X1 /* Do Part 1 only */

#define            TFKH_HARD_KEYS 0X2   /* Display hard fn keys */
#define            TFKH_NORM_KEYS 0X4   /* Display normal fn keys */





/*    Symbolic Values for TRM_RESET_TERMINAL */

#define            TRST_WID132 0X1  /* Set Width to 132 */
#define            TRST_CLEAR 0X2   /* Clear the Screen */
#define            TRST_POSCUR 0X4  /* Position Cursor at (-1,1) */





/*    Symbolic names for some maximum values */

#define            TSIZ_MAXCOL 255  /* Maximum Terminal Width */
#define            TSIZ_MAXLIN 255  /* Maximum Terminal Width */
#define            TSIZ_MAXINP 132  /* Maximum Input QIO Length */
#define            TSIZ_FKYPAD 10   /* Padding for Fkeys    */




/*  End of C Include File */

#endif  /* PSL__TRMDEF_LOADED */
