/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// ArchitectureManager.cpp : implementation file
//

///////////////////////////////////////////////////////////////////////////////
// OpenSTA includes.
#include "stdafx.h"
#include "commander.h"
#include "solutions.h"              // Registry key symbols
#include "ArchitectureManager.h"
#include "ArchMgrStopNameServer.h"
#include "Mainfrm.h"
#include "ArchMgr.h"
#include "XMLRelayMap.h"
#include "XMLRelayArchMgr.h"

///////////////////////////////////////////////////////////////////////////////
// Standard includes.
#include "winsock.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// Architecture Manager (zero-based) Tab index values.
#define AMTAB_SYSTEM        0
#define AMTAB_HOST          1
#define AMTAB_REPOSITORY    2

// Architecture Manager (zero-based) Tab Image index values.
#define AMTABIMG_SYSTEM     1
#define AMTABIMG_HOST       0
#define AMTABIMG_REPOSITORY 2

/////////////////////////////////////////////////////////////////////////////
// CArchitectureManager dialog


CArchitectureManager::CArchitectureManager(CWnd* pParent /*=NULL*/)
	: CDialog(CArchitectureManager::IDD, pParent)
{
	//{{AFX_DATA_INIT(CArchitectureManager)
	//}}AFX_DATA_INIT
    m_pArchMgrHost = NULL;
    m_pArchMgrSystem = NULL;
    m_pArchMgrRepository = NULL;
	m_strOpenSTAHome = getenv("OPENSTA_HOME");

	m_pCorba = new CORBAServer(0,NULL);
}

void CArchitectureManager::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CArchitectureManager)
	DDX_Control(pDX, IDC_ARCH_MGR_TAB, m_ArchMgrTab);
	DDX_Control(pDX, IDC_OSTA_HOST_TREE, m_OstaHostTree);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CArchitectureManager, CDialog)
	//{{AFX_MSG_MAP(CArchitectureManager)
	ON_WM_SIZE()
	ON_WM_SHOWWINDOW()
	ON_COMMAND(ID_REFRESH, OnRefresh)
	ON_NOTIFY(TVN_SELCHANGED, IDC_OSTA_HOST_TREE, OnSelchangedOstaHostTree)
	ON_NOTIFY(TCN_SELCHANGE, IDC_ARCH_MGR_TAB, OnSelchangeArchMgrTab)
	ON_NOTIFY(NM_RCLICK, IDC_OSTA_HOST_TREE, OnRclickOstaHostTree)
	ON_WM_CONTEXTMENU()
	ON_COMMAND(ID_STARTNAMESERVER, OnStartnameserver)
	ON_COMMAND(ID_STOPNAMESERVER, OnStopnameserver)
	ON_COMMAND(ID_SHUTDOWN, OnShutdown)
	ON_UPDATE_COMMAND_UI(ID_STARTNAMESERVER, OnUpdateStartnameserver)
	ON_UPDATE_COMMAND_UI(ID_STOPNAMESERVER, OnUpdateStopnameserver)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CArchitectureManager message handlers

BOOL CArchitectureManager::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	// TODO: Add extra initialization here
    HICON hicon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
    SetIcon(hicon, false);

    m_ArchMgrImages.Create(IDB_ARCHMGR_IMAGES, 16, 0, RGB(0,128,128));
    m_OstaHostTree.SetImageList(&m_ArchMgrImages, TVSIL_NORMAL);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CArchitectureManager::OnSize(UINT nType, int cx, int cy) 
{
	CDialog::OnSize(nType, cx, cy);
	
	// TODO: Add your message handler code here

    CRect myrect;

    // Resize the Host Tree Control Window:
    //   Adjust height with main window.
    if (m_OstaHostTree.GetSafeHwnd())
    {
        m_OstaHostTree.GetWindowRect(myrect);
        ScreenToClient(myrect);
        myrect.NormalizeRect();
        m_OstaHostTree.MoveWindow(myrect.left ,myrect.top, myrect.Width(), (cy - (myrect.top * 2)));
    }

    // Resize the Data Display Tab Control Window:
    //   Adjust Width with main window.
    //   Adjust height with main window.
    if (m_ArchMgrTab.GetSafeHwnd())
    {
        m_ArchMgrTab.GetWindowRect(myrect);
        ScreenToClient(myrect);
        myrect.NormalizeRect();
        m_ArchMgrTab.MoveWindow(myrect.left ,myrect.top, (cx - myrect.left), (cy - (myrect.top * 2)));
    }

    // Resize the Tab Dialog Windows:
    //   Adjust to fit parent Tab control display area.
    m_ArchMgrTab.GetWindowRect(myrect);
    myrect.OffsetRect(-myrect.left, -myrect.top);
    m_ArchMgrTab.AdjustRect(FALSE, myrect);
    if (m_pArchMgrHost != NULL) m_pArchMgrHost->MoveWindow(myrect);
    if (m_pArchMgrSystem != NULL) m_pArchMgrSystem->MoveWindow(myrect);
    if (m_pArchMgrRepository != NULL) m_pArchMgrRepository->MoveWindow(myrect);
}

void CArchitectureManager::Scan() 
{
    // Initialize Windows sockets - must call WSACleanup() when done.
	WORD      VersionRequested;
	WSADATA   stWSAData;
	VersionRequested = MAKEWORD(1, 1);
    if (WSAStartup (VersionRequested, &stWSAData) != 0) 
    {
        AfxMessageBox ("WSAStartup failed!");
        return;
    }

    // Get host name of machine we are running on.
    char hostname[256];
    gethostname(hostname,256);
    struct hostent* host=gethostbyname(hostname);

    // Get and save IP address and name of machine we are running on.
    if(host)
    {
        in_addr* addr=((struct in_addr*)((host->h_addr_list)[0]));
        m_strCmdrIPAddrDots = inet_ntoa(*addr);
        m_strCmdrIPAddrUnderscores = m_strCmdrIPAddrDots;
        m_strCmdrIPAddrUnderscores.Replace('.','_');
        m_strCmdrHostName = hostname;
        m_strCmdrHostName.MakeUpper();
    }

    // Scan Name Service for OpenSTA hosts.
    NSScan();

    // Cleanup windows sockets.
    WSACleanup();

    return;
}

void CArchitectureManager::NSScan()
{
    // Get the Repository Path from the ...OpenSTA\Repository\Path key in the Registry.
    // This is used to locate the Architecture Manager Host data files.
    CString KeyName = CURRENT_PROJECT_KEY;
    CString ValueName = "Path";
    CString strArchMgrPath = GetStringValue(HKEY_LOCAL_MACHINE ,KeyName ,ValueName);
    strArchMgrPath += "Repository\\ArchMgr\\";

    // Clear Repository Hosts list.
    m_RepHosts.RemoveAll();

    // Get the Repository Host name from the ServerAddress key in the Registry.
    KeyName = CURRENT_PROJECT_KEY;
    ValueName = "ServerAddress";
    OstaHost RepHost;
    RepHost.strName = GetStringValue(HKEY_LOCAL_MACHINE ,KeyName ,ValueName);
    struct hostent *h = gethostbyname(RepHost.strName.GetBuffer(0)) ;
    if(h)
    {
        RepHost.strName = CString(h->h_name);
        in_addr* addr = ((struct in_addr*)((h->h_addr_list)[0]));
        RepHost.strIPAddrDots = CString(inet_ntoa(*addr));
        RepHost.strIPAddrUnderscores = RepHost.strIPAddrDots;
        RepHost.strIPAddrUnderscores.Replace('.','_');
        RepHost.strHostFileName.Format("%sArchMgr_%s.txt", strArchMgrPath,
            RepHost.strName);
    }
    RepHost.strName.MakeUpper();
	RepHost.bIsWeb = false;
    if (RepHost.strName.IsEmpty()) RepHost.strName = "<REPOSITORY_HOST>";

    // Add hosts (actually ArchMgr objects) registered with the Naming Service to the list.
	try
	{
		CORBAServer::NameList_t ArchMgrList=m_pCorba->GetListOfNames(ArchMgr::CLASS_NAME);

        // Add Repository Host name (if registered) to tree.
        OstaHost emptyHost;
        if (ArchMgrList.size() != 0) AddHost(RepHost, emptyHost);

		for (std::vector<std::string>::size_type iter=0;iter<ArchMgrList.size();iter++)  
		{
            // Get IP address from ArchMgr object name (IP address with undrescores not dots).
			CORBA::Object_var obj;
			obj=m_pCorba->FindObjectWithName(ArchMgr::CLASS_NAME, ArchMgrList[iter].Id.c_str());
		    ArchMgr_var AM=ArchMgr::_narrow(obj.inout());
            OstaHost thisHost;
            thisHost.strIPAddrUnderscores = ArchMgrList[iter].Id.c_str();
            thisHost.strIPAddrDots = thisHost.strIPAddrUnderscores;
            thisHost.strIPAddrDots.Replace('_','.');
            thisHost.strHostFileName.Format("%sArchMgr_%s.txt", strArchMgrPath,
                thisHost.strName);
			thisHost.bIsWeb = false;

            // Get host name from IP address.
            unsigned long ulInetAddr = inet_addr(thisHost.strIPAddrDots.GetBuffer(0));
            struct hostent *h = gethostbyaddr((char *)&ulInetAddr, sizeof(unsigned long), AF_INET) ;
            if (h != NULL && h->h_name != NULL) thisHost.strName = CString(h->h_name);
            thisHost.strName.MakeUpper();

            // Add host name to tree, unless it's the Repository Host.
            if ((thisHost.strIPAddrDots.CompareNoCase(RepHost.strIPAddrDots) != 0)) AddHost(RepHost,thisHost);
		}
	}
	catch(...)
	{
	}

    // Add hosts registered in the WebRelay
	try
	{
		// Get a list of host machines defined in the relay map (if it is running).
		CORBA::Object_ptr rmObj = m_pCorba->FindObjectWithName(XMLRelayMap::CLASS_NAME, "WebRelay");
		XMLRelayMap_var RM = XMLRelayMap::_narrow(rmObj);
		RelayList rList;
		RM->GetRelayList(rList);

		for (int i =0; i<rList.length(); i++)
		{
			RelayDef def = rList[i];

            OstaHost thisHost;
			thisHost.strName = def.strAlias;
			thisHost.bIsWeb = true;

            AddHost(RepHost,thisHost);
		}
		
	}
	catch(...)
	{
	}
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   Add a Repository Host to the list of Repository Hosts, unless it
//          already exists. If a registered host is specified, add it to the
//          registered host list.
//
// @parm    Repository Host to add, unless it already exists.
// @parm    Registered OpenSTA host to add. No name = no host to add.
//
void CArchitectureManager::AddHost(OstaHost repHost, OstaHost ostaHost)
{
    // Look for the Repository Host in the Repository Host list.
	POSITION repPos = m_RepHosts.GetHeadPosition();
    POSITION oldRepPos;
    bool namfnd = false;
	while(repPos)
	{
        oldRepPos = repPos;
		RepHost &rHost = m_RepHosts.GetNext(repPos);
        if (rHost.strName.CompareNoCase(repHost.strName) == 0)
        {
            repPos = oldRepPos;
            namfnd = true;
            break;
        }
    }

    // If the Repository Host was not found, add it to the list.
    if (!namfnd)
    {
        RepHost thisRH;
        thisRH.strName = repHost.strName;
        thisRH.strIPAddrUnderscores = repHost.strIPAddrUnderscores;
        thisRH.strIPAddrDots = repHost.strIPAddrDots;
        thisRH.strHostFileName = repHost.strHostFileName;
        repPos = m_RepHosts.AddTail(thisRH);
    }

    // if there is something to add to the Registered Hosts list, add it now.
    if (!ostaHost.strName.IsEmpty())
    {
        m_RepHosts.GetAt(repPos).ostaHosts.AddTail(ostaHost);
    }
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   Get Host entry from Repository or registered Hosts list.
//
// @parm    Name of Host.
// @parm    OpenSTA host details.
//
void CArchitectureManager::GetHost(CString inHostName, OstaHost &outOstaHost)
{
    // If no host selected, just return immediately.
    if (inHostName.CompareNoCase("Network") == 0) return;

    // Look for the Host in the Repository Host list.
	POSITION repPos = m_RepHosts.GetHeadPosition();
    POSITION oldRepPos;
	while(repPos)
	{
        oldRepPos = repPos;
		RepHost &rHost = m_RepHosts.GetNext(repPos);
        // If Host found in Repository Host list, return details.
        if (rHost.strName.CompareNoCase(inHostName) == 0)
        {
            outOstaHost.strName = rHost.strName;
            outOstaHost.strIPAddrUnderscores = rHost.strIPAddrUnderscores;
            outOstaHost.strIPAddrDots = rHost.strIPAddrDots;
            outOstaHost.strHostFileName = rHost.strHostFileName;
            outOstaHost.bIsWeb = rHost.bIsWeb;
            return;
        }
        // Else look for Host in registered Host list.
        else
        {
            POSITION ostaPos = rHost.ostaHosts.GetHeadPosition();
            POSITION oldOstaPos;
	        while(ostaPos)
	        {
                oldOstaPos = ostaPos;
		        OstaHost thisOstaHost = m_RepHosts.GetAt(repPos).ostaHosts.GetNext(ostaPos);

                if (thisOstaHost.strName.CompareNoCase(inHostName) == 0)
                {
                    outOstaHost.strName = thisOstaHost.strName;
                    outOstaHost.strIPAddrUnderscores = thisOstaHost.strIPAddrUnderscores;
                    outOstaHost.strIPAddrDots = thisOstaHost.strIPAddrDots;
                    outOstaHost.strHostFileName = thisOstaHost.strHostFileName;
                    outOstaHost.bIsWeb = thisOstaHost.bIsWeb;
                    return;
                }
            }
        }
    }
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   Get the string value of ANY key value in the Registry.
//
CString CArchitectureManager::GetStringValue(HKEY hKeyParent, LPCTSTR lpszKeyName, LPCTSTR lpszField)
{
    HKEY    hRegKey;
    char    szKeyValueData[256];
    DWORD   dwKeyLen = sizeof(szKeyValueData);

    //
    // Assume that the context has not been set so that we have to query the registry to obtain the status
    //
    CString szValue="";

    //
    // First lets open the key
    //
    if (RegOpenKeyEx (hKeyParent, lpszKeyName, 0, KEY_READ, &hRegKey) == ERROR_SUCCESS)
    {

        //
        // And query the Requested Field
        //
        if (RegQueryValueEx(hRegKey, lpszField, 0, NULL, (unsigned char *) &szKeyValueData, &dwKeyLen) == ERROR_SUCCESS)
        {
            szValue = szKeyValueData;
        }

        // then close the key again
        RegCloseKey(hRegKey);
    }
    return szValue;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   Refresh the data in the Architecture Manager dialog.
//
void CArchitectureManager::RefreshArchMgr()
{
    // Scan the for OpenSTA hosts.
    Scan();

    // Clear tree, ready for refresh.
    m_OstaHostTree.DeleteAllItems();

    // Add the root to the host tree.
    HTREEITEM hRoot = m_OstaHostTree.InsertItem("Network",0,0);

    // Now add the hosts.
	POSITION repPos = m_RepHosts.GetHeadPosition();
	while(repPos)
	{
		RepHost &rHost = m_RepHosts.GetNext(repPos);
		HTREEITEM hItem = m_OstaHostTree.InsertItem(rHost.strName,1,1,hRoot);
	    POSITION ostaPos = rHost.ostaHosts.GetHeadPosition();
	    while(ostaPos)
	    {
		    OstaHost &ostaHost = rHost.ostaHosts.GetNext(ostaPos);
            m_OstaHostTree.InsertItem(ostaHost.strName,2,2,hItem);
        }
        m_OstaHostTree.Expand(hItem,TVE_EXPAND);
    }
    m_OstaHostTree.Expand(hRoot,TVE_EXPAND);
}

void CArchitectureManager::OnShowWindow(BOOL bShow, UINT nStatus) 
{
	CDialog::OnShowWindow(bShow, nStatus);
	
	// TODO: Add your message handler code here
    // Refresh the Architecture Manager data.
    RefreshArchMgr();	
}

BOOL CArchitectureManager::PreTranslateMessage(MSG* pMsg) 
{
	HACCEL hAccel = ((CMainFrame*)AfxGetApp()->m_pMainWnd)->GetAccelTable();
	if(!(hAccel && ::TranslateAccelerator(AfxGetApp()->m_pMainWnd->m_hWnd, hAccel,pMsg)))
		return CDialog::PreTranslateMessage(pMsg);
	else
		return TRUE;
}

void CArchitectureManager::OnRefresh() 
{
    RefreshArchMgr();	
}

void CArchitectureManager::OnSelchangedOstaHostTree(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;
	// TODO: Add your control notification handler code here

    // Set up the return value.
    *pResult = 0;

    // Get the currently selected Tree item; return if none selected.
	HTREEITEM hSelected = m_OstaHostTree.GetSelectedItem();
    if (hSelected == NULL) return;

    // Get host details for currently selected Tree item.
    CString strSelected = m_OstaHostTree.GetItemText(hSelected);
    GetHost(strSelected, m_selOstaHost);

    // Clear Informational Tab Display.            
    m_ArchMgrTab.DeleteAllItems();

    CRect myrect;

    // Size the Host Tree Control Window:
    //   Adjust height with main window.
    if (m_OstaHostTree.GetSafeHwnd())
    {
        m_OstaHostTree.GetWindowRect(myrect);
        ScreenToClient(myrect);
        myrect.NormalizeRect();
        m_OstaHostTree.MoveWindow(myrect);
    }

    // Size the Data Display Tab Control Window:
    //   Adjust Width with main window.
    //   Adjust height with main window.
    if (m_ArchMgrTab.GetSafeHwnd())
    {
        m_ArchMgrTab.GetWindowRect(myrect);
        ScreenToClient(myrect);
        myrect.NormalizeRect();
        m_ArchMgrTab.MoveWindow(myrect);
    }

    // If the currently selected Tree item is the root, display "Architecture Manager"
    // in the Window Title and hide all the tab dialogs.
    if (strSelected.Compare("Network") == 0)
    {
        SetWindowText("Architecture Manager");
        if (m_pArchMgrHost != NULL) m_pArchMgrHost->ShowWindow(SW_HIDE);
        if (m_pArchMgrSystem != NULL) m_pArchMgrSystem->ShowWindow(SW_HIDE);
        if (m_pArchMgrRepository != NULL) m_pArchMgrRepository->ShowWindow(SW_HIDE);
        return;
    }

    // Drop through here if the currently selected Tree item corresponds to a Host.
    // Display the host name in the Window Title as "hostname - Architecture Manager"
    // and the host details in the Tab display.
    SetWindowText(strSelected + " - Architecture Manager");

    // Set up Informational Tab Display.
    TCITEM tcItem;      // Tab item

    // Associate images with Tabs.
    m_ArchMgrTabImages.Create(IDB_ARCHMGR_TAB_IMAGES, 16, 0, RGB(0,128,128));
    m_ArchMgrTab.SetImageList(&m_ArchMgrTabImages);

    // Get data for selected host.
	string strHostInfo = "";
	if (!m_selOstaHost.bIsWeb)
	{
	    ArchMgr_var AM;
	    CORBA::Object_var obj;
	    obj=m_pCorba->FindObjectWithName(ArchMgr::CLASS_NAME,
	        m_selOstaHost.strIPAddrUnderscores.GetBuffer(0)); 
	    AM=ArchMgr::_narrow(obj.inout());
	    strHostInfo = AM->FileHostProperties();

	}
	else
	{
	    CORBA::Object_var obj;
	    obj=m_pCorba->FindObjectWithName(XMLRelayArchMgr::CLASS_NAME, "WebRelay");
		XMLRelayArchMgr_var XMLAM = XMLRelayArchMgr::_narrow(obj);
		strHostInfo = XMLAM->FileHostProperties(m_selOstaHost.strName.GetBuffer(0));
	}

	if (strHostInfo.length() == 0)
	{
		AfxMessageBox("Unable to retrieve host properties.");
		return;
	}

    CString KeyName = CURRENT_PROJECT_KEY;
    CString ValueName = "Path";
    CString strArchMgrPath = GetStringValue(HKEY_LOCAL_MACHINE ,KeyName ,ValueName);
    strArchMgrPath += "Repository\\ArchMgr\\";
	
	CString strLocalFileName = strArchMgrPath + "ArchMgr_"+m_selOstaHost.strName+".txt";
	CFile hostFile(strLocalFileName, CFile::modeWrite|CFile::modeCreate);
	hostFile.Write(strHostInfo.c_str(), strHostInfo.length());
	hostFile.Close();

	m_selOstaHost.strHostFileName = strLocalFileName;

    // Insert System tab and set up System dialog.
    tcItem.mask = (TCIF_TEXT | TCIF_IMAGE);
    tcItem.pszText = _T("System");
    tcItem.iImage = AMTABIMG_SYSTEM;
    m_ArchMgrTab.InsertItem(AMTAB_SYSTEM, &tcItem);

    // Get Tab control rectangle details; to be used in sizing overlaid dialogs.
    m_ArchMgrTab.GetWindowRect(myrect);
    myrect.OffsetRect(-myrect.left, -myrect.top);
    m_ArchMgrTab.AdjustRect(FALSE, myrect); // AdjustRect must follow InsertItem

    if (m_pArchMgrSystem == NULL)
    {
        m_pArchMgrSystem = new CArchMgrSystem (this);
    }

    // Create and display the System Tab dialog, as necessary.
    if (!m_pArchMgrSystem->GetSafeHwnd())
    {
        m_pArchMgrSystem->Create(IDD_ARCHMGR_SYSTEM, &m_ArchMgrTab);
        m_pArchMgrSystem->MoveWindow(myrect);
    }

    if (m_ArchMgrTab.GetCurSel() == AMTAB_SYSTEM)
    {
        m_pArchMgrSystem->UpdateDialog();
        m_pArchMgrSystem->ShowWindow(SW_SHOW);
    }
    else
    {
        m_pArchMgrSystem->ShowWindow(SW_HIDE);
    }

    // Insert Host tab and set up Host dialog.
    tcItem.mask = (TCIF_TEXT | TCIF_IMAGE);
    tcItem.pszText = _T("Host");
    tcItem.iImage = AMTABIMG_HOST;
    m_ArchMgrTab.InsertItem(AMTAB_HOST, &tcItem);
    m_ArchMgrTab.AdjustRect(FALSE, myrect);

    if (m_pArchMgrHost == NULL)
    {
        m_pArchMgrHost = new CArchMgrHost (this);
    }

    // Create and display the Host Tab dialog, as necessary.
    if (!m_pArchMgrHost->GetSafeHwnd())
    {
        m_pArchMgrHost->Create(IDD_ARCHMGR_HOST, &m_ArchMgrTab);
        m_pArchMgrHost->MoveWindow(myrect);
    }

    if (m_ArchMgrTab.GetCurSel() == AMTAB_HOST)
    {
        m_pArchMgrHost->UpdateDialog();
        m_pArchMgrHost->ShowWindow(SW_SHOW);
    }
    else
    {
        m_pArchMgrHost->ShowWindow(SW_HIDE);
    }

    // If this is the Repository Host.
    if (m_selOstaHost.strIPAddrDots.CompareNoCase(m_strCmdrIPAddrDots) == 0)
    {
        // Insert Repository tab and set up Repository dialog.
        tcItem.mask = (TCIF_TEXT | TCIF_IMAGE);
        tcItem.pszText = _T("Repository");
        tcItem.iImage = AMTABIMG_REPOSITORY;
        m_ArchMgrTab.InsertItem(AMTAB_REPOSITORY, &tcItem);

        if (m_pArchMgrRepository == NULL)
        {
            m_pArchMgrRepository = new CArchMgrRepository (this);
        }

        // Create and display the Repository Tab dialog, as necessary.
        if (!m_pArchMgrRepository->GetSafeHwnd())
        {
            m_pArchMgrRepository->Create(IDD_ARCHMGR_REPOSITORY, &m_ArchMgrTab);
            m_pArchMgrRepository->MoveWindow(myrect);
        }

        if (m_ArchMgrTab.GetCurSel() == AMTAB_REPOSITORY)
        {
            m_pArchMgrRepository->UpdateDialog();
            m_pArchMgrRepository->ShowWindow(SW_SHOW);
        }
        else
        {
            m_pArchMgrRepository->ShowWindow(SW_HIDE);
        }
    }
    // Else not a Repository Host; hide Repository Window.
    else
    {
        if (m_pArchMgrRepository != NULL)
        {
            m_pArchMgrRepository->ShowWindow(SW_HIDE);
        }
    }
}

void CArchitectureManager::OnSelchangeArchMgrTab(NMHDR* pNMHDR, LRESULT* pResult) 
{
	// TODO: Add your control notification handler code here
	
    // Update and display new tab.
    int curtab;
    curtab = m_ArchMgrTab.GetCurSel();

    if (curtab == AMTAB_SYSTEM)
    {
       m_pArchMgrSystem->ShowWindow(SW_SHOW);
    }
    else
    {
        m_pArchMgrSystem->ShowWindow(SW_HIDE);
    }

    if (curtab == AMTAB_HOST)
    {
       m_pArchMgrHost->ShowWindow(SW_SHOW);
    }
    else
    {
        m_pArchMgrHost->ShowWindow(SW_HIDE);
    }

    if (curtab == AMTAB_REPOSITORY)
    {
       m_pArchMgrRepository->ShowWindow(SW_SHOW);
    }
    else if (m_pArchMgrRepository != NULL)
    {
        m_pArchMgrRepository->ShowWindow(SW_HIDE);
    }
    
    *pResult = 0;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   Create a process to execute a specified command and return the
//          processes PID and handle.
//
// @rdesc
//
// @parm    Command to execute.
// @parm    Returned process handle.
// @parm    Returned process PID.
//
int CArchitectureManager::StartProcess(CString& strCommand ,HANDLE* pReturnHandle ,DWORD* pReturnPid)
{
    STARTUPINFO StartInfo = {sizeof(StartInfo)};

    STARTUPINFO si;
    ZeroMemory(&si,sizeof(STARTUPINFO));
    si.cb = sizeof(STARTUPINFO);
    si.dwFlags = STARTF_USESTDHANDLES|STARTF_USESHOWWINDOW;
    si.hStdOutput = NULL;
    si.hStdInput  = NULL;
    si.hStdError  = NULL;
    si.wShowWindow = SW_HIDE;

    // Change the current directory to the server directory.
    CString strServerDir;
    strServerDir.Format ("%sServer" , m_strOpenSTAHome);
    ::SetCurrentDirectory(strServerDir.GetBuffer(0));

    // Create the process to execute the command
    PROCESS_INFORMATION pi;
    if (!CreateProcess(NULL,
                       strCommand.GetBuffer(0), // command line
                       NULL,                    // process security attributes
                       NULL,                    // primary thread security attributes
                       TRUE,                    // handles are inherited
                       CREATE_NEW_CONSOLE,      // creation flags
                       NULL,                    // use parent's environment
                       strServerDir.GetBuffer(0), // Current directory
                       &si,                     // STARTUPINFO pointer
                       &pi))                    // receives PROCESS_INFORMATION
    {
        CString strErrorBuffer;
        strErrorBuffer.Format ("Error invoking the image %s" ,strCommand);
        LPTSTR lpMsgBuf;

        FormatMessage (FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
                       NULL,
                       GetLastError(),
                       MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
                       (LPTSTR) &lpMsgBuf,
                       0,
                       NULL);
        AfxMessageBox (lpMsgBuf);
        // Free the buffer.
        LocalFree(lpMsgBuf);
        return -1;
    }

    // Save the process details.
    *pReturnHandle = pi.hProcess;
    *pReturnPid = pi.dwProcessId;
    return 0;
}

void CArchitectureManager::OnRclickOstaHostTree(NMHDR* pNMHDR, LRESULT* pResult) 
{
	// TODO: Add your control notification handler code here
	SendMessage(WM_CONTEXTMENU, (WPARAM) m_hWnd, GetMessagePos());
	
	*pResult = 1;
}

void CArchitectureManager::OnContextMenu(CWnd* pWnd, CPoint point) 
{
    // Get the currently selected Tree item.
	HTREEITEM hSelected = m_OstaHostTree.GetSelectedItem();
    if (hSelected == NULL) return;

    // Get name of currently selected Tree item.
    CString strSelected = m_OstaHostTree.GetItemText(hSelected);
	GetHost(strSelected, m_selOstaHost);

    // If the currently selected Tree item is the root, nothing to do.
    if (strSelected.Compare("Network") == 0) return;

    if (point.x == -1 && point.y == -1)
    {
        //keystroke invocation
        CRect rect;
        GetClientRect(rect);
        ClientToScreen(rect);

        point = rect.TopLeft();
        point.Offset(5, 5);
    }

	ScreenToClient(&point);

	CMenu menu;
	menu.LoadMenu(IDR_ARCHMGR_TREE_MENU);

	if (!m_selOstaHost.bIsWeb)
	{
	    // Use the Injector Control object on the Naming Service to determine
	    // whether the OpenSTA Daemon is running on the selected host. If the
	    // Injector Control IS present, "Stop Name Sever" should be grayed,
	    // otherwise "Start Name Server" should be grayed.
	    try
	    {
	        CORBA::Object_var obj;
	        obj=m_pCorba->FindObjectWithName(InjectorControl::CLASS_NAME, m_selOstaHost.strIPAddrUnderscores); 
	        menu.EnableMenuItem(ID_STARTNAMESERVER,MF_GRAYED);
	    }
	    catch (...)
	    {
	        menu.EnableMenuItem(ID_STOPNAMESERVER,MF_GRAYED);
	    }
	}
	else
	{
	    try
	    {
			CORBA::Object_var obj;
			obj=m_pCorba->FindObjectWithName(XMLRelayArchMgr::CLASS_NAME, "WebRelay");
			XMLRelayArchMgr_var XMLAM = XMLRelayArchMgr::_narrow(obj);
			bool bRunning = XMLAM->IsDmnRunning(m_selOstaHost.strName.GetBuffer(0));
			if( bRunning)
		        menu.EnableMenuItem(ID_STARTNAMESERVER,MF_GRAYED);
			else
		        menu.EnableMenuItem(ID_STOPNAMESERVER,MF_GRAYED);
	    }
	    catch (...)
	    {
	        menu.EnableMenuItem(ID_STOPNAMESERVER,MF_GRAYED);
	    }

	}
	
    ClientToScreen(&point);
    menu.GetSubMenu(0)->TrackPopupMenu(TPM_LEFTALIGN|TPM_LEFTBUTTON|TPM_RIGHTBUTTON,point.x+5,point.y+5,this);
}

void CArchitectureManager::OnStartnameserver() 
{
	AfxMessageBox("Start Name Sever selected");

    // Check the configuration on this machine and allow the user to change it/set it up.
    //**if (!CheckConfiguration()) return;

    // Start the OpenSTA Daemon process on the selected host.
    try
    {
        CORBA::Object_var obj;
		if (!m_selOstaHost.bIsWeb)
		{
	        obj=m_pCorba->FindObjectWithName(ArchMgr::CLASS_NAME, m_selOstaHost.strIPAddrUnderscores); 
	        ArchMgr_var AM=ArchMgr::_narrow(obj.inout());
	        AM->StartOSTADaemon();
	        //**m_bOSTADaemonRunning = true;
		}
		else
		{
			obj=m_pCorba->FindObjectWithName(XMLRelayArchMgr::CLASS_NAME, "WebRelay");
			XMLRelayArchMgr_var XMLAM = XMLRelayArchMgr::_narrow(obj);
	        XMLAM->StartOSTADaemon(m_selOstaHost.strName.GetBuffer(0));
		}
    }
    catch (...)
    {
    }

    // Display dialog indicating that the Name Server has started.
    //**AfxMessageBox(IDS_SERVER_UP);
    AfxMessageBox("Name Server Started on " + m_selOstaHost.strName);
}

void CArchitectureManager::OnStopnameserver() 
{
    if (AfxMessageBox (IDS_CONFIRM_STOP ,MB_YESNO) == IDYES)
    {
        //**m_Stopping = TRUE;
        CArchMgrStopNameServer StopNameServerDlg;
	    StopNameServerDlg.Create(IDD_ARCHMGR_STOPNAMESERVER, &m_ArchMgrTab);
        StopNameServerDlg.SetWindowText("Stopping Name Server on " + m_selOstaHost.strName);
        StopOSTADaemon();
        StopNameServerDlg.DestroyWindow();
        //**AfxMessageBox (IDS_SERVER_DOWN);
        //**m_TrayIcon.SetIcon(IDI_INACTIVE);
        //**if (m_bNotify) KillTimer(WMU_PROCESS_CHECK);
    }

    AfxMessageBox("Name Server Stopped on " + m_selOstaHost.strName);
}

///////////////////////////////////////////////////////////////////////////////
// @func    This function stops the OpenSTA Daemon.
//
void CArchitectureManager::StopOSTADaemon()
{
    try
    {
        CORBA::Object_var obj;
		if (!m_selOstaHost.bIsWeb)
		{
	        obj=m_pCorba->FindObjectWithName(ArchMgr::CLASS_NAME, m_selOstaHost.strIPAddrUnderscores); 
	        ArchMgr_var AM=ArchMgr::_narrow(obj.inout());
	        AM->StopOSTADaemon();
	        //**m_bOSTADaemonRunning = false;
		}
		else
		{
			obj=m_pCorba->FindObjectWithName(XMLRelayArchMgr::CLASS_NAME, "WebRelay");
			XMLRelayArchMgr_var XMLAM = XMLRelayArchMgr::_narrow(obj);
	        XMLAM->StopOSTADaemon(m_selOstaHost.strName.GetBuffer(0));
		}
    }
    catch (...)
    {
    }

    // And delete any old log files. As we are exiting normally they are not required.
    // If the OpenSTA Daemon has 'fallen over' then this point will not be reached and
    // the log files will remain in the Server directory.
    //**DeleteUsedLogs ();
}

void CArchitectureManager::OnShutdown() 
{
	// TODO: Add your command handler code here
	AfxMessageBox("Shutdown selected");	
}

void CArchitectureManager::OnUpdateStartnameserver(CCmdUI* pCmdUI) 
{
    // "Start Name Sever" is only available if the OpenSTA Daemon process is NOT running.
    //**pCmdUI->Enable (false);	
}

void CArchitectureManager::OnUpdateStopnameserver(CCmdUI* pCmdUI) 
{
    // "Stop Name Sever" is only available if the OpenSTA Daemon process IS running.
    //**pCmdUI->Enable (true);	
}
