/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// Commander.cpp : Defines the class behaviors for the application.
//

#include "stdafx.h"
#include "Commander.h"
#include <process.h>


#include "MainFrm.h"
#include "CommanderDoc.h"
#include "CommanderView.h"
#include "CECAboutDlg.h"
#include "RegFunctions.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CCommanderApp
CLIPFORMAT g_ClipFormat = RegisterClipboardFormat(strClipboardFormat);

BEGIN_MESSAGE_MAP(CCommanderApp, CWinApp)
	//{{AFX_MSG_MAP(CCommanderApp)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG_MAP
	// Standard file based document commands
	ON_COMMAND(ID_FILE_NEW, CWinApp::OnFileNew)
	ON_COMMAND(ID_FILE_OPEN, CWinApp::OnFileOpen)
	// Standard print setup command
//	ON_COMMAND(ID_FILE_PRINT_SETUP, CWinApp::OnFilePrintSetup)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCommanderApp construction

CCommanderApp::CCommanderApp():
	m_strClassName("OpenSTA Commander Window Class")
{
	m_bSetProcId = false;
}

/////////////////////////////////////////////////////////////////////////////
// The one and only CCommanderApp object

CCommanderApp theApp;

/////////////////////////////////////////////////////////////////////////////
// CCommanderApp initialization

BOOL CCommanderApp::InitInstance()
{
	//Check if there is another active commander
	//If there is show it and quit
	if(!FirstInstance())
		return false;

	// Initialize OLE libraries
	if (!AfxOleInit())
	{
		AfxMessageBox(IDP_OLE_INIT_FAILED);
		return FALSE;
	}

	AfxOleGetMessageFilter()->EnableNotRespondingDialog(FALSE); 
	AfxOleGetMessageFilter()->EnableBusyDialog(FALSE); 

	AfxEnableControlContainer();

	// Standard initialization
	// If you are not using these features and wish to reduce the size
	//  of your final executable, you should remove from the following
	//  the specific initialization routines you do not need.

#ifdef _AFXDLL
	Enable3dControls();			// Call this when using MFC in a shared DLL
#else
	Enable3dControlsStatic();	// Call this when linking to MFC statically
#endif

	// Change the registry key under which our settings are stored.
	// TODO: You should modify this string to be something appropriate
	// such as the name of your company or organization.
	SetRegistryKey(_T("OpenSTA"));

	LoadStdProfileSettings();  // Load standard INI file options (including MRU)

    WNDCLASS wndcls;
    memset(&wndcls, 0, sizeof(WNDCLASS));   // start with NULL defaults
    wndcls.style = CS_DBLCLKS | CS_HREDRAW | CS_VREDRAW;
    wndcls.lpfnWndProc = ::DefWindowProc;
    wndcls.hInstance = AfxGetInstanceHandle();
    wndcls.hIcon = LoadIcon(IDR_MAINFRAME); // or load a different icon
    wndcls.hCursor = LoadCursor( IDC_ARROW );
    wndcls.hbrBackground = (HBRUSH) (COLOR_WINDOW + 1);
    wndcls.lpszClassName = m_strClassName ;
    if(!AfxRegisterClass(&wndcls))
    {
        TRACE("Class Registration Failed\n");
        return FALSE;
    }

	//If there is a commander process left over without
	//a window, put it out of its misery
	KillExistingInstance();

	//Save the proc id, for killexistingInstance
	DWORD dwProcId = _getpid();
	HKEY hKey;
	RegCreateKeyEx(HKEY_LOCAL_MACHINE,"SOFTWARE\\Opensta\\Processes",0,NULL,REG_OPTION_VOLATILE,KEY_WRITE,NULL,&hKey,NULL);
	RegSetValueEx(hKey,"Commander",0,REG_DWORD,(LPBYTE)&dwProcId,sizeof(DWORD));
	RegCloseKey(hKey);
	m_bSetProcId = true;

	//Clear any file locks not cleared from previous instances
	CMap <CString,LPCTSTR,DWORD,DWORD> mapFileToState;
	GetAllRepositoryFileFlags(mapFileToState);
	CString strFile;
	DWORD dwState;
	POSITION pos = mapFileToState.GetStartPosition();
	while(pos)
	{
		mapFileToState.GetNextAssoc(pos,strFile,dwState);
		if(dwState & RFF_LOCKED)
			SetRepositoryFileFlag(strFile,dwState & ~RFF_LOCKED);
	}

	// Register the application's document templates.  Document templates
	//  serve as the connection between documents, frame windows and views.

	CSingleDocTemplate* pDocTemplate;
	pDocTemplate = new CSingleDocTemplate(
		IDR_MAINFRAME,
		RUNTIME_CLASS(CCommanderDoc),
		RUNTIME_CLASS(CMainFrame),       // main SDI frame window
		RUNTIME_CLASS(CCommanderView));
	pDocTemplate->SetContainerInfo(IDR_CNTR_INPLACE);
	AddDocTemplate(pDocTemplate);

	// Parse command line for standard shell commands, DDE, file open
	CCommandLineInfo cmdInfo;
	ParseCommandLine(cmdInfo);

	// Dispatch commands specified on the command line
	if (!ProcessShellCommand(cmdInfo))
		return FALSE;

	// The one and only window has been initialized, so show and update it.
	m_pMainWnd->ShowWindow(SW_SHOW);
	m_pMainWnd->UpdateWindow();

	return TRUE;
}

// App command to run the dialog
void CCommanderApp::OnAppAbout()
{
	CECAboutDlg aboutDlg;
	aboutDlg.DoModal();
}

/////////////////////////////////////////////////////////////////////////////
// CCommanderApp message handlers

void CCommanderApp::KillExistingInstance()
{
	HKEY hKey;
	DWORD dwResult = RegOpenKeyEx(HKEY_LOCAL_MACHINE,"SOFTWARE\\Opensta\\Processes",0,KEY_READ,&hKey);
	if(dwResult != ERROR_SUCCESS)
		return;

	DWORD dwProcId;
	DWORD dwDataSize = sizeof(DWORD);
	
	dwResult = RegQueryValueEx(hKey,"Commander",0,NULL,(LPBYTE)&dwProcId,&dwDataSize);
	if(dwResult == ERROR_SUCCESS)
	{
		HANDLE hProcess = OpenProcess(PROCESS_ALL_ACCESS,TRUE,dwProcId);
		if(!hProcess || !TerminateProcess(hProcess, 0))
		{
			TRACE("Failed to kill existing commander (%d, %d, %d)\n",dwProcId,hProcess,GetLastError());
		}
	}

	RegCloseKey(hKey);
}

int CCommanderApp::ExitInstance() 
{
	if(m_bSetProcId)
	{
		HKEY hKey;
		RegOpenKeyEx(HKEY_LOCAL_MACHINE,"SOFTWARE\\Opensta\\Processes",0,KEY_WRITE,&hKey);
		RegDeleteValue(hKey,"Commander");
		RegCloseKey(hKey);
	}

    ::UnregisterClass(m_strClassName, AfxGetInstanceHandle());

	return CWinApp::ExitInstance();
}

BOOL CCommanderApp::FirstInstance()
{
    CWnd *pWndPrev, *pWndChild;
    
    // Determine if another window with our class name exists...
    if (pWndPrev = CWnd::FindWindow( m_strClassName, NULL ))
    {
        // if so, does it have any popups?
        pWndChild = pWndPrev->GetLastActivePopup();

        // If iconic, restore the main window
        if (pWndPrev->IsIconic())
            pWndPrev->ShowWindow(SW_RESTORE);

        // Bring the main window or its popup to
        // the foreground
        pWndChild->SetForegroundWindow();

        // and we are done activating the previous one.
        return FALSE;
    }
    // First instance. Proceed as normal.
    else
        return TRUE;
}
