/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
// TraceSetDlg.cpp : implementation file
//

#include "stdafx.h"
#include "commander.h"
#include "TraceSetDlg.h"
#include "RegFunctions.h"
#include "BaseUIHelpID.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTraceSetDlg dialog


CTraceSetDlg::CTraceSetDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CTraceSetDlg::IDD, pParent),
	strNoTracing("None"),
	strLowTracing("Low"),
	strMedTracing("Medium"),
	strHighTracing("High")
{
	CECGridCtrl::GridCtrlInit(AfxGetInstanceHandle());
	//{{AFX_DATA_INIT(CTraceSetDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


void CTraceSetDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CTraceSetDlg)
	DDX_Control(pDX, IDC_LIST, m_ctlList);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CTraceSetDlg, CDialog)
	//{{AFX_MSG_MAP(CTraceSetDlg)
	ON_WM_HELPINFO()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTraceSetDlg message handlers

BOOL CTraceSetDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	//Setup up the columns in the grid
	m_ctlList.SetColumnCount(2);
	m_ctlList.SetFixedRowCount(1);

	m_ctlList.SetItemText(0,0,"Target Name");
	m_ctlList.SetItemText(0,1,"Trace Level");
	m_ctlList.AutoSizeColumns();

	CStringArray arstrTargets;
	GetTargetList(arstrTargets);
	
	CMap <CString,LPCTSTR,int,int> mapLevels;
	for(int nTarget=0;nTarget<arstrTargets.GetSize();nTarget++)
		mapLevels.SetAt(arstrTargets[nTarget],enNone);
	
	LoadTraceSettings(mapLevels);

	CStringArray arstrLevels;
	arstrLevels.Add(strNoTracing);
	arstrLevels.Add(strLowTracing);
	arstrLevels.Add(strMedTracing);
	arstrLevels.Add(strHighTracing);
		
	CString strTarget;
	int nLevel;
	POSITION pos = mapLevels.GetStartPosition();
	while(pos)
	{
		mapLevels.GetNextAssoc(pos,strTarget,nLevel);

		int nRow = m_ctlList.InsertRow(strTarget);
		m_ctlList.CreateDropListCell(nRow,1,arstrLevels);
		
		DWORD dwState = m_ctlList.GetItemState(nRow,0);
		m_ctlList.SetItemState(nRow,0,dwState | GVIS_READONLY);

		switch(nLevel)
		{
		case enLow:
			m_ctlList.SetItemText(nRow,1,strLowTracing);
		break;
		case enMedium:
			m_ctlList.SetItemText(nRow,1,strMedTracing);
		break;
		case enHigh:
			m_ctlList.SetItemText(nRow,1,strHighTracing);
		break;
		case enNone:
		default:
			m_ctlList.SetItemText(nRow,1,strNoTracing);
		break;
		}
	}

	m_ctlList.ExpandColumnsToFit();

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CTraceSetDlg::GetTargetList(CStringArray& arstrTargets)
{
	arstrTargets.Add("Test Manager");
	arstrTargets.Add("General");
		

	//Get list of executers from registry
	HKEY hKeyScripts;
	if(RegOpenKeyEx(HKEY_LOCAL_MACHINE,"SOFTWARE\\OpenSTA\\Plugins\\",0,KEY_READ,&hKeyScripts)!=ERROR_SUCCESS)
		return;
	
	DWORD dwTypeNdx=0;
	char szType[256];
	DWORD dwTypeSize = 256;
	while(RegEnumKeyEx(hKeyScripts,dwTypeNdx,szType,&dwTypeSize,NULL,NULL,NULL,NULL) == ERROR_SUCCESS)
	{
		dwTypeSize = 256;
		dwTypeNdx++;
		if(szType == strTestType)
			continue;

		HKEY hKeyType;
		if(RegOpenKeyEx(HKEY_LOCAL_MACHINE,CString("SOFTWARE\\OpenSTA\\Plugins\\")+szType,0,KEY_READ,&hKeyType) != ERROR_SUCCESS)
			continue;

		DWORD dwNameNdx=0;
		char szName[256];
		DWORD dwNameSize = 256;
		while(RegEnumKeyEx(hKeyType,dwNameNdx,szName,&dwNameSize,NULL,NULL,NULL,NULL) == ERROR_SUCCESS)
		{
			dwNameSize = 256;
			dwNameNdx++;

			HKEY hKeyPlugin;
			if(RegOpenKeyEx(hKeyType,szName,NULL,KEY_READ,&hKeyPlugin)==ERROR_SUCCESS)
			{
				char szExtension[256];
				DWORD dwExtensionSize = 256;
				DWORD dwType;
				if(RegQueryValueEx(hKeyPlugin,"Extension",NULL,&dwType,(LPBYTE)szExtension,&dwExtensionSize)==ERROR_SUCCESS)
				{
					char *pszToken = strtok(szExtension,";");
					if(pszToken)
						arstrTargets.Add(pszToken);	
				}
				RegCloseKey(hKeyPlugin);
			}
		}
		RegCloseKey(hKeyType);
	}
	RegCloseKey(hKeyScripts);
}

void CTraceSetDlg::LoadTraceSettings(CMap <CString,LPCTSTR,int,int>& mapLevels)
{
	CString strReposPath = GetRepositoryPath();
	strReposPath += strTraceSettingsFile;
	try
	{
		CStdioFile f(strReposPath,CFile::modeRead|CFile::shareDenyNone);
		
		CString strLine;
		while(f.ReadString(strLine))
		{
			int nCommaPos = strLine.ReverseFind(',');
			CString strTarget = strLine.Left(nCommaPos);
			int nLevel = atoi(strLine.Mid(nCommaPos+1));
			
			//Only set the level if it exists in current list
			int nOldLevel;
			if(mapLevels.Lookup(strTarget,nOldLevel))
				mapLevels.SetAt(strTarget,nLevel);
		}
		f.Close();
	}
	catch(...)
	{

	}
}

void CTraceSetDlg::SaveTraceSettings(CMap <CString,LPCTSTR,int,int>& mapLevels)
{
	CString strReposPath = GetRepositoryPath();
	strReposPath += strTraceSettingsFile;

	try
	{
		CStdioFile f(strReposPath,CFile::modeCreate|CFile::modeWrite);

		CString strTarget,strLine;
		int nLevel;
		POSITION pos = mapLevels.GetStartPosition();
		while(pos)
		{
			mapLevels.GetNextAssoc(pos,strTarget,nLevel);
			
			strLine.Format("%s,%d\n",strTarget,nLevel);
			f.WriteString(strLine);
		}
		f.Close();
	}
	catch(...)
	{

	}
}

void CTraceSetDlg::OnOK() 
{
	CMap <CString,LPCTSTR,int,int> mapLevels;

	int nNumRows = m_ctlList.GetRowCount();
	for(int nRow=1;nRow<nNumRows;nRow++)
	{
		CString strTgt = m_ctlList.GetItemText(nRow,0);
		CString strLvl = m_ctlList.GetItemText(nRow,1);

		int nLevel=0;
		if(strLvl == strNoTracing)
			nLevel = enNone;
		else
		if(strLvl == strLowTracing)
			nLevel = enLow;
		else
		if(strLvl == strMedTracing)
			nLevel = enMedium;
		else
		if(strLvl == strHighTracing)
			nLevel = enHigh;
		
		mapLevels.SetAt(strTgt,nLevel);
	}
	
	SaveTraceSettings(mapLevels);	
	
	CDialog::OnOK();
}

BOOL CTraceSetDlg::OnHelpInfo(HELPINFO* pHelpInfo) 
{
	pHelpInfo->dwContextId = HIDD_TRACESETTINGS;
	
	return CDialog::OnHelpInfo(pHelpInfo);
}
