/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// TestFile.cpp: implementation of the CTestFile class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "TestFile.h"
#include "RegFunctions.h"
#include "TaskGroupDefData.h"
#include "ScriptDefData.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CTestFile::CTestFile():
	m_nStepByStepTaskGroup(-1),
	m_bAdvancedStepByStep(false)
{
}

CTestFile::~CTestFile()
{
	ClearArrays();
}

void CTestFile::Serialize(CArchive& ar)
{
	const int nCurrentVersion = 3;
	if(ar.IsStoring())
	{
		ar.Write(&nCurrentVersion,sizeof(int));		//write a version number to file so it is ease to change the structure later

		ar.WriteString(m_strFilePath + "\n");

		int nNumGroups = m_arGroups.GetSize();
		ar.Write(&nNumGroups,sizeof(int));
				
		for(int nGroup=0;nGroup<nNumGroups;nGroup++)
		{
			TaskGroup *pGroup = m_arGroups[nGroup];
			
			ar.WriteString(pGroup->strHost + "\n");
			ar.WriteString(pGroup->strName + "\n");

			ar.Write(&(pGroup->dwDataSize),sizeof(DWORD));
			ar.Write(pGroup->pData,pGroup->dwDataSize);

			int nNumTasks = pGroup->m_arTasks.GetSize();
			ar.Write(&nNumTasks,sizeof(int));
			
			for(int nTask=0;nTask<nNumTasks;nTask++)
			{
				Task *pTask = pGroup->m_arTasks[nTask];

				ar.Write(&(pTask->nType),sizeof(int));
				ar.WriteString(pTask->strName + "\n");
				ar.WriteString(pTask->strPath + "\n");
				ar.WriteString(pTask->strTarget + "\n");
				ar.Write(&(pTask->dwDataSize),sizeof(DWORD));
				ar.Write(pTask->pData,pTask->dwDataSize);
			}
		}
		
		ar.WriteString(m_strDescription + "\n");

		ar.Write(&m_nStepByStepTaskGroup,sizeof(int));
		ar.Write(&m_bAdvancedStepByStep,sizeof(BOOL));
	}
	else
	{
		ClearArrays();

		int nVersion;
		if(!ar.Read(&nVersion,sizeof(int)))
			return;

		if((nVersion >= 0)&&(nVersion <= nCurrentVersion))
		{
			ar.ReadString(m_strFilePath);

			int nNumGroups;
			ar.Read(&nNumGroups,sizeof(int));

			m_arGroups.SetSize(nNumGroups);
			
			for(int nGroup=0;nGroup<nNumGroups;nGroup++)
			{
				TaskGroup *pGroup = new TaskGroup;
				m_arGroups[nGroup] = pGroup;
				
				ar.ReadString(pGroup->strHost);

				ar.ReadString(pGroup->strName);

				ar.Read(&(pGroup->dwDataSize),sizeof(DWORD));
				if(pGroup->dwDataSize)
				{
					pGroup->pData = new BYTE[pGroup->dwDataSize];
					ar.Read(pGroup->pData,pGroup->dwDataSize);
				}
				else
					pGroup->pData = NULL;
				
				int nNumTasks;
				ar.Read(&nNumTasks,sizeof(int));	
		
				pGroup->m_arTasks.SetSize(nNumTasks);

				for(int nTask=0;nTask<nNumTasks;nTask++)
				{
					Task *pTask = new Task;
					pGroup->m_arTasks[nTask] = pTask;
						
					ar.Read(&(pTask->nType),sizeof(int));
					ar.ReadString(pTask->strName);
					ar.ReadString(pTask->strPath);
					ar.ReadString(pTask->strTarget);
					ar.Read(&(pTask->dwDataSize),sizeof(DWORD));
					if(pTask->dwDataSize)
					{
						pTask->pData = new BYTE[pTask->dwDataSize];
						ar.Read(pTask->pData,pTask->dwDataSize);
					}
					else
						pTask->pData = NULL;
				}
			}
		}
		
		if((nVersion >= 1) && (nVersion <=nCurrentVersion))
			ar.ReadString(m_strDescription);

		if((nVersion >= 2) && (nVersion <=nCurrentVersion))
			ar.Read(&m_nStepByStepTaskGroup,sizeof(int));

		if((nVersion >= 3) && (nVersion <=nCurrentVersion))
			ar.Read(&m_bAdvancedStepByStep,sizeof(BOOL));
	}
}

void CTestFile::ClearArrays()
{
	//Clear arrays
	for(int nGroup=0;nGroup<m_arGroups.GetSize();nGroup++)
	{
		for(int nTask=0;nTask<m_arGroups[nGroup]->m_arTasks.GetSize();nTask++)
		{
			delete []m_arGroups[nGroup]->m_arTasks[nTask]->pData;
			delete m_arGroups[nGroup]->m_arTasks[nTask];
		}
		m_arGroups[nGroup]->m_arTasks.RemoveAll();

		delete []m_arGroups[nGroup]->pData;
		delete m_arGroups[nGroup];
	}
	m_arGroups.RemoveAll();
}

int CTestFile::AddNewTaskGroup()
{
	//Get Test name
	int nSlashPos = m_strFilePath.ReverseFind('\\');
	CString strTestName( m_strFilePath.Mid(nSlashPos+1));
	int nDotPos = strTestName.ReverseFind('.');
	strTestName = strTestName.Left(nDotPos);

	TaskGroup *pGroup = new TaskGroup;
	pGroup->strHost = "localhost";
	pGroup->strName.Empty();//.Format("%s_%d",strTestName,m_arGroups.GetSize()+1);
	pGroup->pData = NULL;
	pGroup->dwDataSize = 0;
	return m_arGroups.Add(pGroup);
}

BOOL CTestFile::SetTaskGroupHost(int nGroup,const CString& strHost)
{
	m_arGroups[nGroup]->strHost = strHost;
	return true;
}

int CTestFile::AddNewTask(int nGroup,TaskType nType,const CString& strName,const CString& strPath,const CString& strTarget)
{
	Task *pTask = new Task;
	pTask->nType = nType;
	pTask->strName = strName;
	pTask->strPath = GetRelativePath(strPath);
	pTask->strTarget = strTarget;
	pTask->dwDataSize = 0;
	pTask->pData = NULL;

	return m_arGroups[nGroup]->m_arTasks.Add(pTask);
}

BOOL CTestFile::SetTaskData(int nGroup,int nTask,LPBYTE pData,DWORD dwDataSize)
{
	Task *pTask = m_arGroups[nGroup]->m_arTasks[nTask];
	pTask->dwDataSize = dwDataSize;
	pTask->pData = pData;
	
	return true;
}

CString CTestFile::GetTaskPath(int nGroup,int nTask)
{
	return GetRepositoryPath() + m_arGroups[nGroup]->m_arTasks[nTask]->strPath;
}

CTestFile::TaskType CTestFile::GetTaskType(int nGroup,int nTask)
{
	return m_arGroups[nGroup]->m_arTasks[nTask]->nType;
}

CString CTestFile::GetTaskName(int nGroup,int nTask)
{
	return m_arGroups[nGroup]->m_arTasks[nTask]->strName;
}

BOOL CTestFile::GetTaskData(int nGroup,int nTask,LPBYTE* pData,DWORD& dwBufSize)
{
	*pData = m_arGroups[nGroup]->m_arTasks[nTask]->pData;
	dwBufSize = m_arGroups[nGroup]->m_arTasks[nTask]->dwDataSize;

	return true;
}

BOOL CTestFile::SetTask(int nGroup,int nTask,TaskType nType,const CString& strName,const CString& strPath,const CString& strTarget)
{
	Task *pTask = m_arGroups[nGroup]->m_arTasks[nTask];
	
	pTask->nType = nType;
	pTask->strName = strName;
	pTask->strPath = GetRelativePath(strPath);
	pTask->strTarget = strTarget;
	pTask->dwDataSize = 0;
	pTask->pData = NULL;

	return TRUE;
}

int CTestFile::GetNumTasks(int nGroup)
{
	return m_arGroups[nGroup]->m_arTasks.GetSize();
}

CString CTestFile::GetTaskGroupHost(int nGroup)
{
	return m_arGroups[nGroup]->strHost;
}

int CTestFile::GetNumTaskGroups()
{
	return m_arGroups.GetSize();
}

CString CTestFile::GetTaskTargetType(int nGroup,int nTask)
{
	return m_arGroups[nGroup]->m_arTasks[nTask]->strTarget;
}

void CTestFile::DeleteItem(int nGroup,int nTask)
{
	delete []m_arGroups[nGroup]->m_arTasks[nTask]->pData;
	m_arGroups[nGroup]->m_arTasks.RemoveAt(nTask);
	
	if(!m_arGroups[nGroup]->m_arTasks.GetSize())
		m_arGroups.RemoveAt(nGroup);
}

void CTestFile::DeleteTaskGroup(int nGroup)
{	
	TaskGroup* pGroup = m_arGroups[nGroup];

	CArray <Task*,Task*>& arTasks = pGroup->m_arTasks;
	for(int nTask=0;nTask<arTasks.GetSize();nTask++)
		delete [](arTasks[nTask]->pData);
	arTasks.RemoveAll();	
	
	delete [](pGroup->pData);
	m_arGroups.RemoveAt(nGroup);
	delete pGroup;
}

void CTestFile::Clear()
{
	m_strFilePath.Empty();
	ClearArrays();
}

BOOL CTestFile::GetTaskGroupData(int nGroup,LPBYTE* pData,DWORD &dwBufSize)
{
	*pData = m_arGroups[nGroup]->pData;
	dwBufSize = m_arGroups[nGroup]->dwDataSize;

	return true;
}

BOOL CTestFile::SetTaskGroupData(int nGroup,LPBYTE pData,DWORD dwDataSize)
{
	m_arGroups[nGroup]->dwDataSize = dwDataSize;
	m_arGroups[nGroup]->pData = pData;

	return true;
}

CString CTestFile::GetTestFilePath()
{
	return GetRepositoryPath() + m_strFilePath;
}

void CTestFile::SetTestFilePath(const CString& strPath)
{
	m_strFilePath = GetRelativePath(strPath);

	int nSlashPos = m_strFilePath.ReverseFind('\\');
	m_strTestName = m_strFilePath.Mid(nSlashPos+1);
	int nDotPos = m_strTestName.ReverseFind('.');
	m_strTestName = m_strTestName.Left(nDotPos);
}

CString CTestFile::GetRelativePath(const CString& strPath)
{
	CString strRelPath(strPath);
	CString strReposPath = GetRepositoryPath();
	int nReposPathLen = strReposPath.GetLength();
	if(strPath.Left(strReposPath.GetLength()).CompareNoCase(strReposPath)==0)
		strRelPath.Delete(0,nReposPathLen);
	
	return strRelPath;
}

BOOL CTestFile::SetTaskGroupName(int nGroup, const CString& strName)
{
	m_arGroups[nGroup]->strName = strName;
	return true;
}

CString CTestFile::GetTaskGroupName(int nGroup)
{
	if(m_arGroups[nGroup]->strName.IsEmpty())
	{
		CString str;
		str.Format("%s_%d",m_strTestName,nGroup+1);
		return str;
	}

	return m_arGroups[nGroup]->strName;
}

CString CTestFile::GetDescription()
{
	return m_strDescription;
}

void CTestFile::SetDescription(const CString& strDescription)
{
	m_strDescription = strDescription;
}

void CTestFile::GetSummary(CString& strSummary)
{
	strSummary.Empty();

	CString strTemp;
	int nGroup = GetStepByStepTaskGroup();
	BOOL bDebug = (nGroup != -1);

	if(bDebug)
		strSummary += "Single step test for task group " + GetTaskGroupName(nGroup) + "\n";
	
	strSummary += m_strDescription + "\n";
	strSummary += "\n";

	for(int nGrp=0;nGrp<m_arGroups.GetSize();nGrp++)
	{
		TaskGroup *pGroup = m_arGroups[nGrp];
	
		if(bDebug && (nGrp != nGroup) && (pGroup->m_arTasks[0]->nType == enScript))
			continue;

		CTaskGroupDefData taskGroupData;
		taskGroupData.LoadFromBuffer(pGroup->pData);

		strSummary += "Task Group:\n";
		strSummary += "\tName:\t" + GetTaskGroupName(nGrp) + "\n";
		strSummary += "\tHost:\t" + pGroup->strHost + "\n";

		strSummary += "\n";
		strSummary += "\tStart Task Group:";

		if(taskGroupData.GetIsDelayed()==TRUE)
		{
			strSummary += "\tDelayed:\n";
		}
		else if (taskGroupData.GetIsScheduled()==TRUE)
		{
			strSummary += "\tScheduled:\n";
		}
		else
		{
			strSummary += "\tImmediately:\n";
		}

		strSummary += "\tStop Task Group:";
		if(taskGroupData.GetbIsDuration()==TRUE)
		{
			if(taskGroupData.GetTaskDuration()>-1)
			{
				strSummary += "\tAfter Fixed Time:\n";
			}
			else
			{
				strSummary += "\tManually:\n";
			}
		}
		else
		{
			strSummary += "\tOn Completion:\n";
		}
		

		for(int nTask=0;nTask<pGroup->m_arTasks.GetSize();nTask++)
		{
			Task *pTask = pGroup->m_arTasks[nTask];
			switch(pTask->nType)
			{
			case enProfile:
				strSummary += "\n";
				strSummary += "\tCollector:\n";
				strSummary += "\t\tName:\t" + pTask->strName + "\n";
			break;
			case enScript:
				if(!nTask)
				{
					strSummary+="\n";
					strTemp.Format("\tTotal Virtual Users:\t%d\n",taskGroupData.GetNumVUs());
					strSummary += strTemp;

					if(taskGroupData.GetUseAdvancedVUInfo()==TRUE)
					{
						strSummary+="\tIntroduce Virtual Users in batches:\tYes\n";

						strTemp.Format("\tInterval Between Batches:\t\t%d seconds\n",taskGroupData.GetInterBatchDelay());
						strSummary += strTemp;

						strTemp.Format("\tNumber of Virtual Users Per Batch:\t%d\n",taskGroupData.GetVUsPerBatch());
						strSummary += strTemp;

						strTemp.Format("\tBatch Ramp Up Time:\t\t%d seconds\n",taskGroupData.GetBatchRampUpTime());
						strSummary += strTemp;
					}
					else
					{
						strSummary+="\tIntroduce Virtual Users in batches:\tNo\n";
					}
					strSummary += "\n";
				}
				
				strSummary += "\tScript:\n";
				strSummary += "\t\tName:\t" + pTask->strName + "\n";
								
				CScriptDefData scriptData;
				scriptData.LoadFromBuffer(pTask->pData);
		
				strTemp.Format("\t\tIterations:\t%d\n",scriptData.GetNumIterations());
				strSummary += strTemp;
				
				if(scriptData.GetUseFixedIterDelay())
				{
					strTemp.Format("\t\tFixed Iteration Delay:\t%d seconds\n",scriptData.GetFixedIterDelay());
					strSummary += strTemp;
				}
				else
				{
					strTemp.Format("\t\tMin Iteration Delay:\t%d\n",scriptData.GetMinIterDelay());
					strSummary += strTemp;
					strTemp.Format("\t\tMax Iteration Delay:\t%d\n",scriptData.GetMaxIterDelay());
					strSummary += strTemp;
				}
				if(scriptData.GetIsDuration()==TRUE)
				{
					strTemp.Format("\t\tTask termination:\t After Fixed Time:\t%d seconds\n",scriptData.GetDuration());
					strSummary += strTemp;
				}
				else
				{
					strTemp.Format("\t\tTask termination:\t On Completion:\t%d iterations\n",scriptData.GetNumIterations());
					strSummary += strTemp;
				}
			break;
			}
			strSummary += "\n";
		}
	}
}

LPBYTE CTestFile::WriteTaskGroupToBuffer(int nGroup,DWORD &dwDataSize)
{
	TaskGroup *pGroup = m_arGroups[nGroup];

	//Calculate sizeof of Task group
	dwDataSize=0;
	dwDataSize += pGroup->strHost.GetLength()+1;	//Group Host
	dwDataSize += pGroup->strName.GetLength()+1;	//Group name	
	dwDataSize += sizeof(DWORD) +  pGroup->dwDataSize;	//Group Data
	dwDataSize += sizeof(int);					//Num tasks

	int nNumTasks = pGroup->m_arTasks.GetSize();
	for(int nTask=0;nTask<nNumTasks;nTask++)
	{
		Task *pTask = pGroup->m_arTasks[nTask];
	
		dwDataSize += sizeof(int);		//Task Type
		dwDataSize += pTask->strName.GetLength() + 1; //Task Name
		dwDataSize += pTask->strPath.GetLength() + 1; //Task path
		dwDataSize += pTask->strTarget.GetLength() + 1; //Task Target
		dwDataSize += sizeof(DWORD) + pTask->dwDataSize; //Task Data
	}

	LPBYTE pData = new BYTE[dwDataSize];
	
	LPBYTE pDataPos = pData;
	
	strcpy((char*)pDataPos,pGroup->strHost);
	pDataPos += pGroup->strHost.GetLength()+1;

	strcpy((char*)pDataPos,pGroup->strName);
	pDataPos += pGroup->strName.GetLength()+1;

	memcpy(pDataPos,&(pGroup->dwDataSize),sizeof(DWORD));
	pDataPos += sizeof(DWORD);
	
	memcpy(pDataPos,pGroup->pData,pGroup->dwDataSize);
	pDataPos += pGroup->dwDataSize;

	memcpy(pDataPos,&nNumTasks,sizeof(int));
	pDataPos += sizeof(int);
	
	for(nTask=0;nTask<nNumTasks;nTask++)
	{
		Task *pTask = pGroup->m_arTasks[nTask];

		memcpy(pDataPos,&(pTask->nType),sizeof(int));
		pDataPos += sizeof(int);

		strcpy((char*)pDataPos,pTask->strName);
		pDataPos += pTask->strName.GetLength()+1;
		
		strcpy((char*)pDataPos,pTask->strPath);
		pDataPos += pTask->strPath.GetLength()+1;

		strcpy((char*)pDataPos,pTask->strTarget);
		pDataPos += pTask->strTarget.GetLength()+1;

		memcpy(pDataPos,&(pTask->dwDataSize),sizeof(DWORD));
		pDataPos += sizeof(DWORD);
		
		memcpy(pDataPos,pTask->pData,pTask->dwDataSize);
		pDataPos += pTask->dwDataSize;
	}

	return pData;	
}

void CTestFile::AddTaskGroupFromBuffer(LPBYTE pTaskGroup)
{
	LPBYTE pDataPos = pTaskGroup;
	
	TaskGroup *pGroup = new TaskGroup;
	m_arGroups.Add(pGroup);
	
	pGroup->strHost = (char*)pDataPos;
	pDataPos += pGroup->strHost.GetLength()+1;

	pGroup->strName.Empty();// = ;
	pDataPos += strlen((char*)pDataPos) + 1;

	memcpy(&(pGroup->dwDataSize),pDataPos,sizeof(DWORD));
	pDataPos += sizeof(DWORD);
	
	if(pGroup->dwDataSize)
	{
		pGroup->pData = new BYTE[pGroup->dwDataSize];
		memcpy(pGroup->pData,pDataPos,pGroup->dwDataSize);
	}
	else
		pGroup->pData=NULL;

	pDataPos += pGroup->dwDataSize;

	int nNumTasks =0;
	memcpy(&nNumTasks,pDataPos,sizeof(int));
	pDataPos += sizeof(int);
	
	for(int nTask=0;nTask<nNumTasks;nTask++)
	{
		Task *pTask = new Task;
		pGroup->m_arTasks.Add(pTask);

		memcpy(&(pTask->nType),pDataPos,sizeof(int));
		pDataPos += sizeof(int);

		pTask->strName = (char*)pDataPos;
		pDataPos += pTask->strName.GetLength()+1;
		
		pTask->strPath = (char*)pDataPos;
		pDataPos += pTask->strPath.GetLength()+1;

		pTask->strTarget = (char*)pDataPos;
		pDataPos += pTask->strTarget.GetLength()+1;

		memcpy(&(pTask->dwDataSize),pDataPos,sizeof(DWORD));
		pDataPos += sizeof(DWORD);
		
		if(pTask->dwDataSize)
		{
			pTask->pData = new BYTE[pTask->dwDataSize];
			memcpy(pTask->pData,pDataPos,pTask->dwDataSize);
		}
		else
			pTask->pData=NULL;

		pDataPos += pTask->dwDataSize;
	}
}

//Set task group to -1 to disable debug mode
void CTestFile::SetStepByStepMode(int nTaskGroup)
{
	m_nStepByStepTaskGroup = nTaskGroup;
}	

int CTestFile::GetStepByStepTaskGroup()
{
	return m_nStepByStepTaskGroup;
}

BOOL CTestFile::IsAdvancedStepByStepMode()
{
	return m_bAdvancedStepByStep;
}

void CTestFile::SetAdvancedStepByStepMode(BOOL bEnable)
{
	m_bAdvancedStepByStep = bEnable;
}
