/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// RelayMapDlg.cpp : implementation file
//

#include "stdafx.h"
#include "relaycfg.h"
#include "RelayMapDlg.h"
#include "EditRelayDef.h"
#include "Mainfrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CRelayMapDlg dialog


CRelayMapDlg::CRelayMapDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CRelayMapDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CRelayMapDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT

	m_bModified = false;
}


void CRelayMapDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CRelayMapDlg)
	DDX_Control(pDX, IDC_DEL_ROW, m_ctlDel);
	DDX_Control(pDX, IDC_ADD_ROW, m_ctlAdd);
	//}}AFX_DATA_MAP
    DDX_Control(pDX, IDC_RELAY_MAP, m_ctlRelayMap);
}


BEGIN_MESSAGE_MAP(CRelayMapDlg, CDialog)
	//{{AFX_MSG_MAP(CRelayMapDlg)
	ON_BN_CLICKED(IDC_ADD_ROW, OnAddRow)
	ON_BN_CLICKED(IDC_DEL_ROW, OnDelRow)
	//}}AFX_MSG_MAP
	ON_NOTIFY(NM_DBLCLK,IDC_RELAY_MAP,OnDblClkRelayMap)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CRelayMapDlg message handlers

BOOL CRelayMapDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	m_ctlAdd.LoadBitmaps(IDB_ADD);
	m_ctlAdd.SizeToContent();
	m_ctlDel.LoadBitmaps(IDB_DEL);
	m_ctlDel.SizeToContent();

    const int nNumColumns = 7;
    CString headers = "Alias\tHost\tExtension Path\tPort\tUsername\tPassword\tUse SSL";
	CRect r;
	m_ctlRelayMap.GetWindowRect(r);

    m_ctlRelayMap.SetColumnCount(nNumColumns);
    m_ctlRelayMap.SetRowCount(1);
    m_ctlRelayMap.SetFixedRowCount();
	m_ctlRelayMap.SetColumnWidth(0,(r.Width()/7)-1);
	m_ctlRelayMap.SetColumnWidth(1,(r.Width()/7)-1);
	m_ctlRelayMap.SetColumnWidth(2,(r.Width()/7)-1);
	m_ctlRelayMap.SetColumnWidth(3,(r.Width()/7)-1);
	m_ctlRelayMap.SetColumnWidth(4,(r.Width()/7)-1);
	m_ctlRelayMap.SetColumnWidth(5,(r.Width()/7)-1);
	m_ctlRelayMap.SetColumnWidth(6,(r.Width()/7)-1);
	
    m_ctlRelayMap.PasteTextFromBuffer(0,0, headers);
	
	LoadRelayMap();

	UpdateMap();

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CRelayMapDlg::LoadRelayMap()
{
	CString strPath = getenv("OPENSTA_HOME");
	strPath += "Server\\relaymap.dat";

	CFileStatus fs;
	if (!CFile::GetStatus(strPath, fs))
		return;

	CStdioFile f(strPath, CFile::modeRead | CFile::shareDenyNone);

	CString strLine;
	while(f.ReadString(strLine))
	{
		RelayDef def;

		int nPos = strLine.Find(',');
		def.strAlias = strLine.Left(nPos);
		strLine.Delete(0, nPos+1);

		nPos = strLine.Find(',');
		def.strHost = strLine.Left(nPos);
		strLine.Delete(0, nPos+1);

		nPos = strLine.Find(',');
		def.strExtPath = strLine.Left(nPos);
		strLine.Delete(0, nPos+1);

		nPos = strLine.Find(',');
		def.strPort = strLine.Left(nPos);
		strLine.Delete(0, nPos+1);

		nPos = strLine.Find(',');
		def.strUser = strLine.Left(nPos);
		strLine.Delete(0, nPos+1);

		nPos = strLine.Find(',');
		def.strPass = strLine.Left(nPos);
		strLine.Delete(0, nPos+1);

		nPos = strLine.Find(',');
		def.bUseSSL = atoi(strLine.Left(nPos));

		m_arrDefs.Add(def);
	}
}

void CRelayMapDlg::UpdateMap()
{
	// Delete All Rows (apart from headers)
	for (int i = 1; i<m_ctlRelayMap.GetRowCount(); i++)
	{
		m_ctlRelayMap.DeleteRow(i);
	}

    m_ctlRelayMap.SetRowCount(m_arrDefs.GetSize()+1);

	int nRow = 1;
	for (i = 0; i<m_arrDefs.GetSize(); i++)
	{
		RelayDef def = m_arrDefs.GetAt(i);
		m_ctlRelayMap.SetItemText(nRow, 0, def.strAlias);
		m_ctlRelayMap.SetItemText(nRow, 1, def.strHost);
		m_ctlRelayMap.SetItemText(nRow, 2, def.strExtPath);
		m_ctlRelayMap.SetItemText(nRow, 3, def.strPort);
		m_ctlRelayMap.SetItemText(nRow, 4, def.strUser);

		CString strPass = "";
		for (int z=0;z<def.strPass.GetLength(); z++)
			strPass += "*";

		m_ctlRelayMap.SetItemText(nRow, 5, strPass);

		m_ctlRelayMap.CreateCheckCell(nRow, 6);
		m_ctlRelayMap.SetCheckBoxCellState(nRow, 6, def.bUseSSL);
	
		nRow++;
	}

	m_ctlRelayMap.SetEditable(FALSE);

	m_ctlRelayMap.Invalidate();
}

void CRelayMapDlg::OnAddRow() 
{
	CEditRelayDef dlg;
	if(dlg.DoModal()==IDOK)
	{
		RelayDef def;
		def.strAlias = dlg.m_strAlias;
		def.strHost = dlg.m_strIPAddress;
		def.strExtPath = dlg.m_strExtPath;
		def.strPort = dlg.m_strPort;
		def.strUser = dlg.m_strUser;
		def.strPass = dlg.m_strPass;
		def.bUseSSL = dlg.m_bUseSSL;

		m_arrDefs.Add(def);

		UpdateMap();
		m_bModified = true;
	}	
}

void CRelayMapDlg::OnDelRow() 
{
	CCellRange range = m_ctlRelayMap.GetSelectedCellRange();
	if (!range.IsValid() || range.GetMaxRow() != range.GetMinRow())
	{
		AfxMessageBox("Please select a single row to delete.");
		return;
	}

	int nRow = range.GetMinRow();
	m_arrDefs.RemoveAt(nRow-1);
	UpdateMap();
	m_bModified = true;
}

void CRelayMapDlg::OnDblClkRelayMap(NMHDR* pNMHDR,LRESULT *pResult)
{
	NM_GRIDVIEW *pGridView = (NM_GRIDVIEW*)pNMHDR;

	CEditRelayDef dlg;

	dlg.m_strAlias = m_ctlRelayMap.GetItemText(pGridView->iRow,0);
	dlg.m_strIPAddress = m_ctlRelayMap.GetItemText(pGridView->iRow,1);
	dlg.m_strExtPath = m_ctlRelayMap.GetItemText(pGridView->iRow,2);
	dlg.m_strPort = m_ctlRelayMap.GetItemText(pGridView->iRow,3);
	dlg.m_strUser = m_ctlRelayMap.GetItemText(pGridView->iRow,4);
	dlg.m_strPass = m_ctlRelayMap.GetItemText(pGridView->iRow,5);
	dlg.m_bUseSSL = m_ctlRelayMap.GetCheckBoxCellState(pGridView->iRow,6);


	if(dlg.DoModal()==IDOK)
	{
		m_arrDefs[pGridView->iRow-1].strAlias = dlg.m_strAlias;
		m_arrDefs[pGridView->iRow-1].strHost = dlg.m_strIPAddress;
		m_arrDefs[pGridView->iRow-1].strExtPath = dlg.m_strExtPath;
		m_arrDefs[pGridView->iRow-1].strPort = dlg.m_strPort;
		m_arrDefs[pGridView->iRow-1].strUser = dlg.m_strUser;
		m_arrDefs[pGridView->iRow-1].strPass = dlg.m_strPass;
		m_arrDefs[pGridView->iRow-1].bUseSSL = dlg.m_bUseSSL;

		UpdateMap();

		m_bModified = true;
	}

	*pResult=0;
}

void CRelayMapDlg::OnOK() 
{
	if (!m_bModified)
	{
		CDialog::OnOK();
		return;
	}

	CString strPath = getenv("OPENSTA_HOME");
	strPath += "Server\\relaymap.dat";

	CStdioFile f(strPath, CFile::modeCreate | CFile::modeWrite);

	for (int i =0; i<m_arrDefs.GetSize(); i++)
	{
		RelayDef def = m_arrDefs.GetAt(i);
		CString strLine;
		strLine.Format("%s,%s,%s,%s,%s,%s,%d,\r\n", def.strAlias, def.strHost, 
			def.strExtPath,	def.strPort, def.strUser, def.strPass, def.bUseSSL);

		f.WriteString(strLine);
	}
	

	CMainFrame* pMainFrame = (CMainFrame*)AfxGetMainWnd();
	if(pMainFrame->m_bRunning)
	{
		int nRet = AfxMessageBox("The Relay Daemon needs to be restarted for\nthe changes to take effect.\nRestart now?", MB_YESNO);
		if (nRet == IDYES)
		{
			pMainFrame->StopDmn();
			pMainFrame->StartDmn();
		}
	}

	CDialog::OnOK();
}
