/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
#include "StepByStep_Commander_i.h"
#include <CECInetInfo.h>

StepByStepCommanderImpl::StepByStepCommanderImpl():
	m_corba(NULL)
{
}

StepByStepCommanderImpl::~StepByStepCommanderImpl()
{
    // De-register the interface.
	try
	{
		if(m_corba)
			m_corba->UnNameObject(StepByStepCommander::CLASS_NAME, m_registered_name.c_str());
	}
	catch(...)
	{
	}
}

void StepByStepCommanderImpl::Initialise(CORBAServer* corba)
{
	m_corba = corba;
	char * pOpenStaHome = getenv("OPENSTA_HOME") ;
    if(!pOpenStaHome)
        return;
	if(!m_corba)
		return;

    string strLocalPath = pOpenStaHome;

    // Get IP address (with dots replaced by underscores) of daemon host.
    CECInetInfo inet;
    m_registered_name=inet.GetIPAddressAsString();
    PortableServer::ObjectId* p = m_corba->ActivateObject(this);    
    m_corba->NameObject(this->_this(), StepByStepCommander::CLASS_NAME, m_registered_name.c_str());

	// Connect to the StepByStepControl
	CORBA::Object_ptr obj;
	obj = m_corba->FindObjectWithName(StepByStepControl::CLASS_NAME, m_registered_name.c_str());
	m_stepCtrl = StepByStepControl::_narrow(obj);
}

CORBA::Boolean StepByStepCommanderImpl::IsStepByStep()
{
	return TRUE;
}
 
void StepByStepCommanderImpl::NextRequest(const char* ScriptName, CORBA::Long lScriptLineNumber, CORBA::Long lTaskGroupID, CORBA::Long lVUId, CORBA::Long lConId, const char* strRequest)
{
}

void StepByStepCommanderImpl::RequestComplete(CORBA::Long lTaskGroupID, CORBA::Long lVUId, CORBA::Long lConId, CORBA::Boolean bIsBinaryData, const char* strHeaders, const char* strBody, CORBA::Long lBodyLength)
{
}

BOOL StepByStepCommanderImpl::Next(long lTaskGroupID, long lVUId)
{
	if(!m_stepCtrl)
		return FALSE;

	return m_stepCtrl->Next(lTaskGroupID, lVUId);
}

void StepByStepCommanderImpl::SetCommander(BOOL bAttach /*=TRUE*/)
{
	if(!m_stepCtrl)
		return;

	if(bAttach)
		m_stepCtrl->SetCommander(this->_this());
	else
		m_stepCtrl->SetCommander(StepByStepCommander::_nil());
}

