/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// TestCtrl.cpp : implementation file
//

#include "stdafx.h"
#include "TestCtrl.h"
#include "OstaDefs.h"
#include "TaskGroupDefData.h"
#include "ConfigurationDlg.h"
#include "RegFunctions.h"
#include "CECGridCtrl.h"
#include "CORBAInfo.h"
#include "InjectorControl.h"
#include <ctime.hpp>
#include "PropertiesBar.h"
#include "PluginTypes.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define IDT_REFRESH_LIST 1
const int MINUTE = 60;
const int HOUR = 60*MINUTE;
const int DAY = 24*HOUR;
const int CELL_MARGIN_SIZE=3;
const int BUTTON_CELL_MARGIN_SIZE=1;

extern CPropertiesBar* m_pPropertiesBar;
extern CCORBAInfo* m_pCorbaInfo;
extern HINSTANCE m_hInstance;
extern CDocument* m_pDocument;
extern HWND m_hWndNotify;

UINT GetUpdateTestCtrl(LPVOID pParam)
{
	CTestCtrl* pWnd = (CTestCtrl*) pParam;
	try
	{
		if (!m_pCorbaInfo)
			return 0;

		do
		{
			for (int i = 0; i<pWnd->m_TaskGroupStatuses.GetSize(); i++)
			{
				TaskGroupTuple tgt = pWnd->m_TaskGroupStatuses.GetAt(i);
				TGExecStatus current=m_pCorbaInfo->GetTaskGroupStatus(tgt.TaskGroupId,pWnd->GetStrReg());
				tgt.TaskGroupStatus=current;
				pWnd->m_TaskGroupStatuses.SetAt(i, tgt);
			}
		}
		while(WaitForSingleObject(pWnd->m_hCloseThread,2000) == WAIT_TIMEOUT);
	}
	catch(...)
	{
	}
	return 0;
}

/////////////////////////////////////////////////////////////////////////////
// CTestCtrl

CTestCtrl::CTestCtrl():
	m_nMaxTasks(200),
	m_nMaxGroups(200),
	m_pTestFile(NULL),
	m_colMissing(RGB(255,0,0)),
	m_colNormal(GetSysColor(COLOR_WINDOW)),
	m_pDoc(NULL),
	m_ptContextClick(0,0)
{
	m_hCloseThread = CreateEvent(NULL,FALSE,FALSE,NULL);
	m_ClipFormat = RegisterClipboardFormat(strClipboardFormat);
	m_strReg="";
	m_pUpdateThreadctrl = NULL;
	EnableTitleTips(FALSE);	//dont use titletips
}

CTestCtrl::~CTestCtrl()
{
	if (m_pUpdateThreadctrl)
	{
		SetEvent(m_hCloseThread);
		WaitForSingleObject(m_pUpdateThreadctrl->m_hThread,5000);
	}

	CloseHandle(m_hCloseThread);
}


BEGIN_MESSAGE_MAP(CTestCtrl, CECGridCtrl)
	//{{AFX_MSG_MAP(CTestCtrl)
	ON_WM_LBUTTONDOWN()
	ON_WM_TIMER()
	ON_WM_CONTEXTMENU()
	ON_COMMAND(ID_DUPLICATETASKGROUP, OnDuplicatetaskgroup)
	ON_COMMAND(ID_SHOW_HOST, OnShowHost)
	ON_COMMAND(ID_SHOW_SCHEDULING, OnShowScheduling)
	ON_COMMAND(ID_SHOW_STATUS, OnShowStatus)
	ON_COMMAND(ID_SHOW_TASKGROUP, OnShowTaskgroup)
	ON_COMMAND(ID_SHOW_VUS, OnShowVus)
	ON_WM_DESTROY()
	ON_COMMAND(ID_DELETESELECTION, OnDeleteselection)
	ON_COMMAND(ID_DELETETASKGROUP, OnDeleteTaskGroup)
	ON_COMMAND(ID_SINGLESTEP_TASKGROUP, OnSinglestepTaskgroup)
	//}}AFX_MSG_MAP
	ON_MESSAGE(WMU_REFRESH_LIST,OnRefreshList)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////

// CTestCtrl message handlers
BOOL CTestCtrl::HitTest(CPoint pt,int& nRow,int& nCol)
{
	CCellID cell = GetCellFromPt(pt);
	if(cell.IsValid())
	{
		nRow = cell.row;
		nCol = cell.col;
		return true;
	}
	return false;
}

void CTestCtrl::PreSubclassWindow() 
{
	CECGridCtrl::PreSubclassWindow();

	m_DropTarget.Revoke();	//don't let grid handle drag and drop
	EnableDragAndDrop(FALSE);

	m_ListImages.Create(IDB_CONFIG_IMAGES,16,1,RGB(192,192,192));
	SetImageList(&m_ListImages);

	Refresh();
	SetTimer(IDT_REFRESH_LIST,2000,NULL);
}

void CTestCtrl::OnLButtonDown(UINT nFlags, CPoint point) 
{
	int nRow,nCol;
	if(HitTest(point,nRow,nCol))
	{
		if(nRow && (nCol == enNameCol) && (m_pTestFile->GetNumTaskGroups()>nRow-1))
		{
			DWORD dwDataSize=0;
			LPBYTE pData = m_pTestFile->WriteTaskGroupToBuffer(nRow-1,dwDataSize);

			int nDataSize = ClipboardData::GetSize() + dwDataSize;
		
			HGLOBAL hData = GlobalAlloc(GMEM_FIXED|GMEM_ZEROINIT,nDataSize);
			ClipboardData *pClipData = (ClipboardData*)GlobalLock(hData);
			pClipData->nDataType = ClipboardData::enTaskGroup;
			memcpy(pClipData->GetData(),pData,dwDataSize);
			GlobalUnlock(hData);

			delete []pData;

			COleDataSource DataSource;
			DataSource.CacheGlobalData(m_ClipFormat,hData);
			DROPEFFECT DE = DataSource.DoDragDrop();
			if(DE == DROPEFFECT_NONE)
			{
				CECGridCtrl::OnLButtonDown(nFlags, point);
				CECGridCtrl::OnLButtonUp(nFlags, point);
			}
			return;
		}

	}
	
	CECGridCtrl::OnLButtonDown(nFlags, point);
}

void CTestCtrl::SetupList()
{
	//Setup up the columns in the grid
	SetColumnCount(m_nMaxTasks+enFirstTaskCol);
	SetFixedRowCount();

	CWinApp *pApp = AfxGetApp();

	SetItemText(0,enScheduleCol,"Start");
	if(!pApp->GetProfileInt("Enabled Cols","Start",1))
	{
		SetColumnWidth(enScheduleCol,0);
	//	AutoSizeColumn(enScheduleCol);
	}

	SetItemText(0,enNameCol,"Task Group");
	if(!pApp->GetProfileInt("Enabled Cols","Task Group",1))
		SetColumnWidth(enNameCol,0);
//	AutoSizeColumn(enNameCol);

	SetItemText(0,enStartStopStatusCol,"Status");
	if(!pApp->GetProfileInt("Enabled Cols","Status",1))
		SetColumnWidth(enStartStopStatusCol,0);
//	AutoSizeColumn(enStartStopStatusCol);

	SetItemText(0,enHostCol, "Host");
	if(!pApp->GetProfileInt("Enabled Cols","Host",1))
		SetColumnWidth(enHostCol,0);
//	AutoSizeColumn(enHostCol);
	SetItemText(0,enVUCol, "VUs");
	if(!pApp->GetProfileInt("Enabled Cols","VUs",1))
		SetColumnWidth(enVUCol,0);
//	AutoSizeColumn(enVUCol);

	
	CString strHeader;
	for(int nCol=0;nCol<m_nMaxTasks;nCol++)
	{
		strHeader.Format("Task %d",nCol+1);
		SetItemText(0,nCol+enFirstTaskCol, strHeader);
//		AutoSizeColumn(nCol+enFirstTaskCol);
	}
	SetRowCount(m_nMaxGroups+1);

	for(nCol=0;nCol<enFirstTaskCol+m_nMaxTasks;nCol++)
	{
		for(int nRow=1;nRow<=m_nMaxGroups;nRow++)
		{
			DWORD dwState = GetItemState(nRow,nCol);
			SetItemState(nRow,nCol,dwState | GVIS_READONLY);
		}
	}

}

void CTestCtrl::Refresh()
{
	
	CCellID currentCell=GetFocusCell();
	CCellRange cellRange = GetSelectedCellRange();

	SetRedraw(FALSE);
	DeleteAllItems();
	SetupList();
	LoadTestFile();
	if(currentCell.row>-1&&currentCell.col>-1&&currentCell.col<m_nMaxTasks+enFirstTaskCol&&currentCell.row<m_nMaxGroups+1)
	{
		SetFocusCell(currentCell.row,currentCell.col);
		SetSelectedRange(cellRange);
	}
	SetRedraw(TRUE);
}

void CTestCtrl::LoadTestFile()
{
	int highestTask=0;

	if(!m_pTestFile)
		return;

	CConfigurationDlg* pointerDlg = (CConfigurationDlg*)GetParent();

	CString strTestPath(m_pTestFile->GetTestFilePath());
	int nPos = strTestPath.ReverseFind('\\');
	if(nPos!=-1)
	{
		m_strTestFolder = strTestPath.Left(nPos+1);
		m_strTestName = strTestPath.Mid(nPos+1);
	}
	else
		m_strTestName = strTestPath;

	nPos = m_strTestName.ReverseFind('.');
	if(nPos != -1)
		m_strTestName = m_strTestName.Left(nPos);

	CFileStatus fs;
	int nNumGroups = m_pTestFile->GetNumTaskGroups();

	m_TaskGroupStatuses.RemoveAll();
	//m_TaskGroupStatuses.SetSize(nNumGroups,1);	//default number of task groups, grow 1 at a time
	//then use m_TaskGroupStatuses.SetAt(nGroup,newitem);//in add 6 lines below or array will add to numgroups so end up with 2X correct size

	for(int nGroup=0;nGroup<nNumGroups;nGroup++)
	{
		TaskGroupTuple newItem;
		newItem.TaskGroupId=nGroup;
		newItem.TaskGroupStatus=INACTIVE;//GetTGStatus(nGroup);
		m_TaskGroupStatuses.Add(newItem);

		DisplayScheduleInfo(nGroup+1);
		DisplayDisableButton(nGroup+1);

		DisplayStartStopButton(nGroup+1);
		CString strHost = m_pTestFile->GetTaskGroupHost(nGroup);
		SetItemText(nGroup+1,enHostCol, strHost );
		SetItemImage(nGroup+1,enHostCol,enHostImage);

		int nNumTasks = m_pTestFile->GetNumTasks(nGroup);

		if(nNumTasks>highestTask)
		{
			highestTask=nNumTasks;
		}

		if(nNumTasks)
		{
			switch(m_pTestFile->GetTaskType(nGroup,0))
			{
			case CTestFile::enScript:
			{
				SetItemImage(nGroup+1,enVUCol,enVUImage);

				LPBYTE pData;
				DWORD dwBufSize;
				m_pTestFile->GetTaskGroupData(nGroup,&pData,dwBufSize);
				CTaskGroupDefData TGData;
				TGData.LoadFromBuffer(pData);
				
				CString strNumVUs;
				strNumVUs.Format("%d",TGData.GetNumVUs());
				SetItemText(nGroup+1,enVUCol,strNumVUs);

			}
			break;
			case CTestFile::enProfile:
			{
				SetItemBkColour(nGroup+1,enVUCol,GetSysColor(COLOR_BTNFACE));
				for(int nCol=enFirstTaskCol+1;nCol<enFirstTaskCol+m_nMaxTasks;nCol++)
				{
					SetItemBkColour(nGroup+1,nCol,GetSysColor(COLOR_BTNFACE));
				}

			}
			break;
			}
		}

		for(int nTask=0;nTask<nNumTasks;nTask++)
		{
			CString strName = m_pTestFile->GetTaskName(nGroup,nTask);
			SetItemText(nGroup+1,nTask+enFirstTaskCol, strName);
			
			CString strPath = m_pTestFile->GetTaskPath(nGroup,nTask);
			int nImage = RegisterFileExtension(strPath);
			SetItemImage(nGroup+1,nTask+enFirstTaskCol,nImage);

			if(!CFile::GetStatus(strPath,fs))
				SetItemBkColour(nGroup+1,nTask+enFirstTaskCol,m_colMissing);
			else
				SetItemBkColour(nGroup+1,nTask+enFirstTaskCol,m_colNormal);
		}
	}

	for(int autosizecounter=0; autosizecounter<highestTask+enFirstTaskCol;autosizecounter++)
	{
		AutoSizeColumn(autosizecounter);
	}
	return;
}

void CTestCtrl::OnTimer(UINT nIDEvent) 
{
	switch(nIDEvent)
	{
	case IDT_REFRESH_LIST:
	{
		if(m_bTestActive ||!m_pTestFile)
			return;

		CFileStatus fs;
		int nNumGroups = m_pTestFile->GetNumTaskGroups();
		for(int nGroup=0;nGroup<nNumGroups;nGroup++)
		{
			int nNumTasks = m_pTestFile->GetNumTasks(nGroup);
			for(int nTask=0;nTask<nNumTasks;nTask++)
			{
				CString strPath = m_pTestFile->GetTaskPath(nGroup,nTask);
				int nImage = RegisterFileExtension(strPath);
				SetItemImage(nGroup+1,nTask+enFirstTaskCol,nImage);
				
				COLORREF col = GetItemBkColour(nGroup+1,nTask+enFirstTaskCol);
				if(!CFile::GetStatus(strPath,fs))
				{
					if(col != m_colMissing)
					{
						SetItemBkColour(nGroup+1,nTask+enFirstTaskCol,m_colMissing);
						RedrawCell(nGroup+1,nTask+enFirstTaskCol);
					}
				}
				else
				{
					if(col != m_colNormal)
					{
						SetItemBkColour(nGroup+1,nTask+enFirstTaskCol,m_colNormal);
						RedrawCell(nGroup+1,nTask+enFirstTaskCol);
					}
				}
			}
		}
		return;
	}
	break;

	case IDT_UPDATE_STATUS:
		 {

			//update the start and stop buttons
			int nNumGroups = m_pTestFile->GetNumTaskGroups();
			for(int g=0;g<nNumGroups;g++)
				DisplayStartStopButton(g+1);

			if(m_bModified)
			{
				AutoSizeColumn(enStartStopStatusCol);
				Invalidate(FALSE);
			}
		 }
		break;
	}
	
	CECGridCtrl::OnTimer(nIDEvent);
}

CString CTestCtrl::GetScriptOutputPath()
{
	return m_strTestFolder;
}


void CTestCtrl::DeleteSelection()
{
	CConfigurationDlg* pDlg = (CConfigurationDlg*)GetParent();
	int nNumRowsDeleted=0;
	int nNumColsDeleted=0;
	BOOL DeleteItemFailure=FALSE;
	BOOL DeleteTGFailure=FALSE;

	CCellRange cellRange = GetSelectedCellRange();
	
	for(int nRow=cellRange.GetMinRow();nRow<=cellRange.GetMaxRow();nRow++)
	{
		int nGroup = nRow-1-nNumRowsDeleted;
		if(nGroup>=m_pTestFile->GetNumTaskGroups())
			break;

		BOOL taskgroupdeleted=FALSE;

		for(int nCol=cellRange.GetMinCol();nCol<=cellRange.GetMaxCol();nCol++)
		{
			int nTask = nCol-enFirstTaskCol-nNumColsDeleted;
			if(nCol == enScheduleCol || nCol==enHostCol || nCol==enStartStopStatusCol || nCol == enVUCol || (nTask>=m_pTestFile->GetNumTasks(nGroup)))
				continue;

			if((GetCell(nRow,nCol)->IsSelected())&&(taskgroupdeleted==FALSE))
			{
				if(nCol==enNameCol)
				{
					if(GetTestActive()==FALSE)
					{
						m_pTestFile->DeleteTaskGroup(nGroup);
						nNumRowsDeleted++;
						DeleteTGStatus(nRow-1);		//remove from array
						m_pPropertiesBar->EditNothing();	
						taskgroupdeleted=TRUE;
					}
					else
					{
						//cant delete a task group ever when test is active!
						DeleteTGFailure=TRUE;
					}
				}
				else
				{
					if(GetTestActive()==FALSE)
					{
						int nNumGroups = m_pTestFile->GetNumTaskGroups();
						m_pTestFile->DeleteItem(nGroup,nTask);
						if(nNumGroups > m_pTestFile->GetNumTaskGroups())
						{
							nNumRowsDeleted++;
							DeleteTGStatus(nRow-1);	//remove from array?
							taskgroupdeleted=TRUE;
						}
						nNumColsDeleted++;
						m_pPropertiesBar->EditNothing();	
					}
					else
					{
						if(m_pTestFile->GetNumTasks(nGroup)<2)
						{
							//all task groups must have one task or they would be deleted
							DeleteTGFailure=TRUE;
						}
						else 
						{
							TGExecStatus statusnow=GetTGStatus(nGroup);
							if(statusnow==INACTIVE||statusnow==STOPPED||statusnow==FAILED||statusnow==COMPLETED)
							{
								//TG is INACTIVE so all nTask>0 can be deleted
								int nNumGroups = m_pTestFile->GetNumTaskGroups();
								m_pTestFile->DeleteItem(nGroup,nTask);
								if(nNumGroups > m_pTestFile->GetNumTaskGroups())//these 3 lines cant be called ever
								{
									nNumRowsDeleted++;
								}
								nNumColsDeleted++;
								m_pPropertiesBar->EditNothing();
							}
							else
							{
								// TG is ACTIVE or SCHEDULED cant delete from here
								DeleteItemFailure=TRUE;
							}
						}
					}
				}
			}
		}
		nNumColsDeleted=0;
	}

	if(DeleteTGFailure)
	{
		AfxMessageBox("Cannot delete a Task Group when running a test");
	}
	else if(DeleteItemFailure)
	{
		AfxMessageBox("Cannot delete tasks in an Active or Scheduled Task Group");
	}
	
	Refresh();
	Invalidate();
	m_pDoc->SetModifiedFlag();
}


 void CTestCtrl::DeleteItem(int nGroup,int nTask)
 {
 	if(m_pTestFile->GetNumTaskGroups() >= nGroup ||
 		m_pTestFile->GetNumTasks(nGroup) >= nTask)
 		return;
 
 	m_pTestFile->DeleteItem(nGroup,nTask);
 
 	//Refresh();//why refresh here too because delete selection() will refresh
 	m_pDoc->SetModifiedFlag();
 }

DROPEFFECT CTestCtrl::OnDragOver(COleDataObject* pDataObject, DWORD dwKeyState, CPoint point)
{
	if(!pDataObject->IsDataAvailable(m_ClipFormat))
		return DROPEFFECT_NONE;

	int nRow,nCol;
	if(!HitTest(point,nRow,nCol))
		return DROPEFFECT_NONE;

	//Can't drop on header row
	if(!nRow)
		return DROPEFFECT_NONE;

	int nGroup = nRow-1;
	int nTask = nCol-enFirstTaskCol;

	HGLOBAL hClipData = pDataObject->GetGlobalData(m_ClipFormat);
	ClipboardData *pClipData = (ClipboardData*)GlobalLock(hClipData);

	int nType = pClipData->nDataType;
	switch(nType)
	{
	case ClipboardData::enTaskGroup:
	{
		GlobalUnlock(hClipData);

		//Can drop on a new line only
		if(m_pTestFile->GetNumTaskGroups()<=nGroup)
			return DROPEFFECT_COPY;
		return DROPEFFECT_NONE;
	}
	case ClipboardData::enScript:
	case ClipboardData::enProfile:
	{
		CString strItem = (char*)pClipData->GetData();
		GlobalUnlock(hClipData);

		//Dropping on the hosts column or VUs column or name column or start/stop column
		if((nCol==enHostCol) || (nCol==enVUCol) || (nCol == enNameCol) ||(nCol==enStartStopStatusCol)||(nCol==enScheduleCol))
			return DROPEFFECT_NONE;

		//Disallow if the file does not exist or is a directory
		CFileStatus fs;
		if(!CFile::GetStatus(strItem,fs) || (fs.m_attribute & CFile::Attribute::directory))
			return DROPEFFECT_NONE;

		//Dropping on a new line
		if(m_pTestFile->GetNumTaskGroups()<=nGroup)	
			return DROPEFFECT_COPY;
		//Only one item in a group with a profile
		if(nTask>0 && m_pTestFile->GetNumTasks(nGroup) &&
			(m_pTestFile->GetTaskType(nGroup,0)==CTestFile::enProfile))
			return DROPEFFECT_NONE;

		//Disallow drop if dragging profile anywhere but column 1
		if((nType==ClipboardData::enProfile)&&nTask)
			return DROPEFFECT_NONE;

		//Disallow if adding scripts to an active task group
		if(GetTestActive()==TRUE)
		{
			TGExecStatus currentStatus=GetTGStatus(nGroup);
			if((currentStatus==ACTIVE)||(currentStatus==DELAYED)||(currentStatus==STOPPING))
			{
				return DROPEFFECT_NONE;
			}
		}

		//Can only have scripts of the same type in a group
		CString strType = m_pTestFile->GetTaskTargetType(nGroup,0);

		int nDotPos = strItem.ReverseFind('.');
		if(strItem.Mid(nDotPos+1).CompareNoCase(strType)!=0)
			return DROPEFFECT_NONE;
	
		return DROPEFFECT_COPY;
	}
	default:
		GlobalUnlock(hClipData);
		return DROPEFFECT_NONE;
	}
}


BOOL CTestCtrl::OnDrop(COleDataObject* pDataObject, DROPEFFECT dropEffect, CPoint point)
{
	if(pDataObject->IsDataAvailable(m_ClipFormat))
	{
		int nRow,nCol;
		if(!HitTest(point,nRow,nCol))
			return false;

		HGLOBAL hClipData = pDataObject->GetGlobalData(m_ClipFormat);
		ClipboardData *pClipData = (ClipboardData*)GlobalLock(hClipData);

		switch(pClipData->nDataType)
		{
		case ClipboardData::enTaskGroup:
		{
			LPBYTE pTaskGroup = pClipData->GetData();
			m_pTestFile->AddTaskGroupFromBuffer(pTaskGroup);

			Refresh();
			m_pDoc->SetModifiedFlag();
		}
		break;
		case ClipboardData::enScript:
		{
			CString strItem = (char*)pClipData->GetData();
			InsertScript(nRow,nCol,strItem);
			m_pDoc->SetModifiedFlag();
		}
		break;
		case ClipboardData::enProfile:
		{
			CString strItem = (char*)pClipData->GetData();
			InsertProfile(nRow,nCol,strItem);
			m_pDoc->SetModifiedFlag();
		}
		break;
		}
					
		GlobalUnlock(hClipData);

		return true;
	}
	return false;
}

void CTestCtrl::InsertScript(int nItem,int nSubItem,const CString& strItemPath)
{

	CConfigurationDlg* pDlg = (CConfigurationDlg*)GetParent();

	int nGroup = nItem-1;
	int nTask = nSubItem-enFirstTaskCol;
	if(nGroup >= m_pTestFile->GetNumTaskGroups())
	{
		nGroup = m_pTestFile->AddNewTaskGroup();

		////////////////////////////////////////////////////////////////////
		//save the task group data
		DWORD  dwBufSize=0;	
		LPBYTE pData = NULL;
		//load
		m_pTestFile->GetTaskGroupData(nGroup,&pData,dwBufSize);
		CTaskGroupDefData data;
		data.LoadFromBuffer(pData);
		//set the initial data
		data.SetBatchRampUpTime(1);
		data.SetbIsDuration(FALSE);
		data.SetInterBatchDelay(1);
		data.SetIsScheduled(FALSE);
		data.SetNumVUPerInterval(1);
		data.SetNumVUs(1);
		data.SetScheduledTime("+0 00:00:00");
		data.SetStartDelay(1);
		data.SetStartInterval(0);
		data.SetStatLevel('L');
		data.SetStopOnComplete(TRUE);
		data.SetTaskDuration(-1);
		if(m_bTestActive==TRUE)
		{
			data.SetTaskGroupEnabled(FALSE);
		}
		else
		{
			data.SetTaskGroupEnabled(TRUE);
		}
		data.SetTaskIterations(1);
		data.SetTimerStatLevel('L');
		data.SetUseAdvancedVUInfo(FALSE);
		data.SetUsePageTimers(FALSE);
		data.SetVUsPerBatch(1);
		data.SetIsDelayed(FALSE);
		//save
		delete []pData;
		data.SaveToBuffer(NULL,dwBufSize);
		pData = new BYTE[dwBufSize];
		data.SaveToBuffer(pData,dwBufSize);
		m_pTestFile->SetTaskGroupData(nGroup,pData,dwBufSize);
		//////////////////////////////////////////////////////////////

		TaskGroupTuple newItem;
		newItem.TaskGroupId=nGroup;
		newItem.TaskGroupStatus=INACTIVE;
		m_TaskGroupStatuses.Add(newItem);

		DisplayScheduleInfo(nGroup+1);
		AutoSizeColumn(enScheduleCol);

		DisplayDisableButton(nGroup+1);
		AutoSizeColumn(enNameCol);

		DisplayStartStopButton(nGroup+1);
		AutoSizeColumn(enStartStopStatusCol);

		SetItemText(nGroup+1,enHostCol,m_pTestFile->GetTaskGroupHost(nGroup));
		SetItemImage(nGroup+1,enHostCol,enHostImage);
		AutoSizeColumn(enHostCol);

		//Insert VUs
		SetItemImage(nGroup+1,enVUCol,enVUImage);
		SetItemText(nGroup+1,enVUCol,"1");
		AutoSizeColumn(enVUCol);
	}

	int nImage = RegisterFileExtension(strItemPath);

	int nPos = strItemPath.ReverseFind('\\');
	CString strName = (nPos == -1) ? strItemPath : strItemPath.Mid(nPos+1);
		
	nPos = strName.ReverseFind('.');
	strName = strName.Left(nPos);
	
	if(m_pTestFile->GetNumTasks(nGroup)<=nTask)
	{
		nTask = m_pTestFile->AddNewTask(nGroup,CTestFile::enScript,strName,strItemPath,GetTargetType(strItemPath));
		
		//load and save script data
		DWORD dwBufSizes1;
		LPBYTE pDatas1;
		m_pTestFile->GetTaskData(nGroup,nTask,&pDatas1,dwBufSizes1);
		CScriptDefData sData1;
		sData1.SetDuration(1);
		sData1.SetFixedIterDelay(0);
		sData1.SetIsDuration(FALSE);
		sData1.SetMaxIterDelay(1);
		sData1.SetMinIterDelay(0);
		sData1.SetNumIterations(1);
		sData1.SetUseFixedIterDelay(TRUE);
		sData1.SaveToBuffer(NULL,dwBufSizes1);
		//create a buffer and save data to it
		pDatas1 = new BYTE[dwBufSizes1];
		sData1.SaveToBuffer(pDatas1,dwBufSizes1);
		//set the data in the test file
		m_pTestFile->SetTaskData(nGroup,nTask,pDatas1,dwBufSizes1);
	//	AfxMessageBox("saved script task with default values");
	}
	else
	{
		CString strMsg;
		strMsg = "Are you sure you want to overwrite \"" + m_pTestFile->GetTaskName(nGroup,nTask) + "\"?";
		if(AfxMessageBox(strMsg,MB_YESNO)==IDNO)
			return;
		m_pTestFile->SetTask(nGroup,nTask,CTestFile::enScript,strName,strItemPath,GetTargetType(strItemPath));
	
		//load and save script data
		DWORD dwBufSizes2;
		LPBYTE pDatas2;
		m_pTestFile->GetTaskData(nGroup,nTask,&pDatas2,dwBufSizes2);
		CScriptDefData sData2;
		sData2.SetDuration(1);
		sData2.SetFixedIterDelay(0);
		sData2.SetIsDuration(FALSE);
		sData2.SetMaxIterDelay(1);
		sData2.SetMinIterDelay(0);
		sData2.SetNumIterations(1);
		sData2.SetUseFixedIterDelay(TRUE);
		sData2.SaveToBuffer(NULL,dwBufSizes2);
		//create a buffer and save data to it
		pDatas2 = new BYTE[dwBufSizes2];
		sData2.SaveToBuffer(pDatas2,dwBufSizes2);
		//set the data in the test file
		m_pTestFile->SetTaskData(nGroup,nTask,pDatas2,dwBufSizes2);
		//AfxMessageBox("saved script task with default values");
	}

	SetItemText(nGroup+1,nTask+enFirstTaskCol,strName);
	SetItemImage(nGroup+1,nTask+enFirstTaskCol,nImage);
	AutoSizeColumn(nTask+enFirstTaskCol);
	Invalidate();
	

	// initialise the property window, otherwise the property window is vulnerable
	// As if a user had single clicked in nRow, nCol

	SetFocusCell(nGroup+1,nTask+enFirstTaskCol);
	SetSelectedRange(nGroup+1,nTask+enFirstTaskCol,nGroup+1,nTask+enFirstTaskCol);
	pDlg->SetFocus();
	m_pPropertiesBar->Edit(nGroup+1,nTask+enFirstTaskCol);

	if(!m_pPropertiesBar->EditScriptTask(nGroup,nTask))
	{
		//Delete everything we've added
		DeleteItem(nGroup,nTask);
	}
}

void CTestCtrl::InsertProfile(int nItem,int nSubItem,const CString& strItemPath)
{
	CConfigurationDlg* pDlg = (CConfigurationDlg*)GetParent();

	//Only one task allowed in a profiles group
	int nImage = RegisterFileExtension(strItemPath);

	int nPos = strItemPath.ReverseFind('\\');
	CString strName = (nPos == -1) ? strItemPath : strItemPath.Mid(nPos+1);
		
	nPos = strName.ReverseFind('.');
	strName = strName.Left(nPos);
	
	int nGroup = nItem-1;
	int nTask = nSubItem-enFirstTaskCol;
	if(nGroup >= m_pTestFile->GetNumTaskGroups())
	{
		nGroup = m_pTestFile->AddNewTaskGroup();

		////////////////////////////////////////////////////////////////////
		//save the task group data
		DWORD  dwBufSize=0;
		LPBYTE pData = NULL;
		//load
		m_pTestFile->GetTaskGroupData(nGroup,&pData,dwBufSize);
		CTaskGroupDefData data;
		data.LoadFromBuffer(pData);
		//set the initial data
		data.SetBatchRampUpTime(1);
		data.SetbIsDuration(TRUE);
		data.SetInterBatchDelay(1);
		data.SetIsScheduled(FALSE);
		data.SetNumVUPerInterval(1);
		data.SetNumVUs(1);
		data.SetScheduledTime("+0 00:00:00");
		data.SetStartDelay(1);
		data.SetStartInterval(0);
		data.SetStatLevel('L');
		data.SetStopOnComplete(FALSE);
		data.SetTaskDuration(-1);
		if(m_bTestActive==TRUE)
		{
			data.SetTaskGroupEnabled(FALSE);
		}
		else
		{
			data.SetTaskGroupEnabled(TRUE);
		}
		data.SetTaskIterations(1);
		data.SetTimerStatLevel('L');
		data.SetUseAdvancedVUInfo(FALSE);
		data.SetUsePageTimers(FALSE);
		data.SetVUsPerBatch(1);
		data.SetIsDelayed(FALSE);
		//save
		delete []pData;
		data.SaveToBuffer(NULL,dwBufSize);
		pData = new BYTE[dwBufSize];
		data.SaveToBuffer(pData,dwBufSize);
		m_pTestFile->SetTaskGroupData(nGroup,pData,dwBufSize);
		//AfxMessageBox("Saved to test file for profile - this task group data");
		//////////////////////////////////////////////////////////////

		TaskGroupTuple newItem;
		newItem.TaskGroupId=nGroup;
		newItem.TaskGroupStatus=INACTIVE;
		m_TaskGroupStatuses.Add(newItem);

		DisplayScheduleInfo(nGroup+1);
		AutoSizeColumn(enScheduleCol);

		DisplayDisableButton(nGroup+1);
		AutoSizeColumn(enNameCol);

		DisplayStartStopButton(nGroup+1);
		AutoSizeColumn(enStartStopStatusCol);

		SetItemText(nGroup+1,enHostCol,m_pTestFile->GetTaskGroupHost(nGroup));
		SetItemImage(nGroup+1,enHostCol,enHostImage);
		AutoSizeColumn(enHostCol);

		nTask = m_pTestFile->AddNewTask(nGroup,CTestFile::enProfile,strName,strItemPath,GetTargetType(strItemPath));

		//Disable other columns
		SetItemBkColour(nGroup+1,enVUCol,GetSysColor(COLOR_BTNFACE));
		for(int nCol=enFirstTaskCol+1;nCol<enFirstTaskCol+m_nMaxTasks;nCol++)
		{
			SetItemBkColour(nGroup+1,nCol,GetSysColor(COLOR_BTNFACE));
		}
	}
	else
	{
		CString strMsg;
		strMsg = "Are you sure you want to overwrite \"" + m_pTestFile->GetTaskName(nGroup,nTask) + "\"?";
		if(AfxMessageBox(strMsg,MB_YESNO)==IDNO)
			return;
		m_pTestFile->SetTask(nGroup,nTask,CTestFile::enProfile,strName,strItemPath,GetTargetType(strItemPath));
	}

	SetItemText(nGroup+1,nTask+enFirstTaskCol,strName);
	SetItemImage(nGroup+1,nTask+enFirstTaskCol,nImage);
	AutoSizeColumn(nTask+enFirstTaskCol);
	Invalidate();
	//why invalidate here
	
	SetFocusCell(nGroup+1,nTask+enFirstTaskCol);
	SetSelectedRange(nGroup+1,nTask+enFirstTaskCol,nGroup+1,nTask+enFirstTaskCol);
	pDlg->SetFocus();

	m_pPropertiesBar->Edit(nGroup+1,nTask+enFirstTaskCol);

	if(!m_pPropertiesBar->EditProfileTask(nGroup))
	{
		//Delete everything we've added
		DeleteItem(nGroup,nTask);
	}
}


int CTestCtrl::RegisterFileExtension(const CString& strFile)
{
	int nPos = strFile.ReverseFind('.');
	if(nPos == -1)
		return -1;

	CString strExtension(strFile.Mid(nPos));

	int nIndex=0;
	if(m_mapExtToImage.Lookup(strExtension,nIndex))
		return nIndex;

	SHFILEINFO shfi;
	SHGetFileInfo( strFile, 0, &shfi, sizeof( SHFILEINFO ), 
               SHGFI_ICON | SHGFI_SHELLICONSIZE | SHGFI_SMALLICON);

	if(!shfi.hIcon || !(nIndex = (WORD)m_ListImages.Add(shfi.hIcon)))
		return -1;

	DestroyIcon(shfi.hIcon);

	m_mapExtToImage.SetAt(strExtension,nIndex);	
	return nIndex;
}

CString CTestCtrl::GetTargetType(const CString& strPath)
{
	int nPos = strPath.ReverseFind('.');
	return strPath.Mid(nPos+1);
}

void CTestCtrl::SetDocument(CDocument* pDoc)
{
	m_pDoc = pDoc;
}

void CTestCtrl::SetTestFile(CTestFile *pTestFile)
{
	m_pTestFile = pTestFile;
}

void CTestCtrl::OnContextMenu(CWnd* pWnd, CPoint point) 
{
	m_ptContextClick=point;

	CPoint pt(point);
	ScreenToClient(&pt);
	int nRow,nCol;
	
	if(!HitTest(pt,nRow,nCol))
		return;

	//This block was added by the Pop-up Menu component
    if (point.x == -1 && point.y == -1)
    {
        //keystroke invocation
        CRect rect;
        GetClientRect(rect);
        ClientToScreen(rect);

        point = rect.TopLeft();
        point.Offset(5, 5);
    }
	AfxSetResourceHandle(m_hInstance);

    CMenu menu;

	if(!nRow)
	{
	
		//Do show/hide columns men
		menu.LoadMenu(IDR_SHOWHIDE_COLS);

		menu.CheckMenuItem(ID_SHOW_TASKGROUP,MF_BYCOMMAND|(GetColumnWidth(enNameCol) ? MF_CHECKED:MF_UNCHECKED));
		menu.CheckMenuItem(ID_SHOW_SCHEDULING,MF_BYCOMMAND|(GetColumnWidth(enScheduleCol) ? MF_CHECKED:MF_UNCHECKED));
		menu.CheckMenuItem(ID_SHOW_STATUS,MF_BYCOMMAND|(GetColumnWidth(enStartStopStatusCol) ? MF_CHECKED:MF_UNCHECKED));
		menu.CheckMenuItem(ID_SHOW_HOST,MF_BYCOMMAND|(GetColumnWidth(enHostCol) ? MF_CHECKED:MF_UNCHECKED));
		menu.CheckMenuItem(ID_SHOW_VUS,MF_BYCOMMAND|(GetColumnWidth(enVUCol) ? MF_CHECKED:MF_UNCHECKED));

		CMenu* pPopup = menu.GetSubMenu(0);
		ASSERT(pPopup != NULL);

		pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y,this);

		return;
	}

	if(m_pTestFile->GetNumTaskGroups()<=nRow-1)
		return;

    VERIFY(menu.LoadMenu(IDR_CONFIG_CONTEXT_MENU));
	AfxSetResourceHandle(AfxGetInstanceHandle());

	if(m_bTestActive || (m_pTestFile->GetTaskType(nRow-1,0) == CTestFile::enProfile))
		menu.RemoveMenu(ID_SINGLESTEP_TASKGROUP,MF_BYCOMMAND);

    CMenu* pPopup = menu.GetSubMenu(0);
    ASSERT(pPopup != NULL);

    pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y,this);

}

void CTestCtrl::OnDuplicatetaskgroup() 
{
	CPoint pt(m_ptContextClick);
	ScreenToClient(&pt);
	int nRow,nCol;
	
	if(!HitTest(pt,nRow,nCol) || !nRow)
		return;

	if(m_pTestFile->GetNumTaskGroups()<=nRow-1)
		return;

	DWORD dwDataSize=0;
	LPBYTE pTG = m_pTestFile->WriteTaskGroupToBuffer(nRow-1,dwDataSize);
	m_pTestFile->AddTaskGroupFromBuffer(pTG);
	delete []pTG; 

	Refresh();
	m_pDoc->SetModifiedFlag();
}

BOOL CTestCtrl::DisplayStartStopButton(int nRow)
{
	CString strButtonText;
	CString strTheTaskStatus;
	ConfigImages nChosenImage;
	int nButtonWidth;

	DWORD  dwBufSize=0;
	LPBYTE pLoadData = NULL;
	m_pTestFile->GetTaskGroupData(nRow-1,&pLoadData,dwBufSize);

	CTaskGroupDefData data;
	data.LoadFromBuffer(pLoadData);

	if(!data.GetTaskGroupEnabled())
	{
		//task is not enabled 
		strButtonText="";
		strTheTaskStatus="Disabled";
		nChosenImage=enDISABLEDImage;
	}
	else 
	{
		if(m_bTestActive==FALSE)
		{
			//task is inactive so no button 
			strButtonText="";
			strTheTaskStatus="Enabled";
			nChosenImage=enDISABLEDImage;
		}
		else
		{
			switch (GetTGStatus(nRow-1))
			{
				case ACTIVE:
				{
					//AfxMessageBox("task active");
					strButtonText="Stop";
					strTheTaskStatus="Running";
					nChosenImage=enACTIVEImage;
					break;	
				}

				case INACTIVE:
				{
					//	AfxMessageBox("task inactive");
					strButtonText="Start";
					strTheTaskStatus="Waiting enabled";
					nChosenImage=enINACTIVEImage;
					break;
				}

				case DELAYED:
				{
					//	AfxMessageBox("task scheduled");
					strButtonText="Stop";
					strTheTaskStatus="Scheduled";
					nChosenImage=enSCHEDULEDImage;	
					break;
				}

				case STOPPED:
				{
					//	AfxMessageBox("task stopped");
					strButtonText="Start";
					strTheTaskStatus="Stopped";
					nChosenImage=enINACTIVEImage;
					break;				
				}
				case COMPLETED:
				{
					//	AfxMessageBox("task completed");
					strButtonText="Start";
					strTheTaskStatus="Completed";
					nChosenImage=enINACTIVEImage;
					break;				
				}
				case FAILED:
				{
					//	AfxMessageBox("task failed");
					strButtonText="Start";
					strTheTaskStatus="Failed";
					nChosenImage=enINACTIVEImage;
					break;				
				}
				case STOPPING:
				{
					//	AfxMessageBox("task stopping");
					strButtonText="";//no button
					strTheTaskStatus="Stopping";
					nChosenImage=enACTIVEImage;
					break;				
				}
				default:
				{
					strTheTaskStatus="get task group status error";
					//AfxMessageBox("task error");
					break;
				}
			}
		}
	}

	CGridCellBase* pCell=NULL;

	if(!strButtonText.IsEmpty())
	{
		nButtonWidth=40;//=(int)(strlen(strButtonText)*9);//assuming a nice standard gui font
		CreateButtonCell(nRow,enStartStopStatusCol,strButtonText,CGridBtnCellBase::CTL_ALIGN_RIGHT,nButtonWidth,FALSE,BUTTON_CELL_MARGIN_SIZE);
		pCell = GetCell(nRow, enStartStopStatusCol);
	}
	else
	{
		//if there is a button cell destroy it
		SetCellType(nRow,enStartStopStatusCol,RUNTIME_CLASS(CGridCell));
		pCell = GetCell(nRow, enStartStopStatusCol);
		pCell->SetMargin(CELL_MARGIN_SIZE);
	}

	//these should relate to the cell not the button
	pCell->SetText(strTheTaskStatus);
	//each image must have its own button database
	pCell->SetImage(nChosenImage);
	
	m_bAllowDraw = false;
	SetCellEnabled(nRow,enStartStopStatusCol,FALSE);
	m_bAllowDraw = true;
	//Invalidate(); 

	return true;
}
 
void CTestCtrl::StartStopButtonPressed(int nRow)
{
	CConfigurationDlg* pDlg = (CConfigurationDlg*)GetParent();
	if(pDlg->m_bTestActive==FALSE)
	{
		AfxMessageBox("start stop button pressed when test is inactive\nNo buttons should be displayed when a test is inactive");
		return;
	}

	BOOL success=FALSE;
	switch(GetTGStatus(nRow-1))
	{
		case INACTIVE:
		{
			//start an inactive
			success=StartTG(nRow-1);
		}
		break;

		case STOPPED:
		{
			//start a stopped
			success=StartTG(nRow-1);
		}
		break;
		case ACTIVE:
		{
			//stop a started
			success=StopTG(nRow-1);
		}
		break;
		case DELAYED:
		{
			//stop a scheduled
			success=StopTG(nRow-1);
			if(success==TRUE)
			{
				//save the changes to the task group data
				DWORD  dwBufSize3=0;
				LPBYTE pData3 = NULL;
				//load
				m_pTestFile->GetTaskGroupData(nRow-1,&pData3,dwBufSize3);
				CTaskGroupDefData data3;
				data3.LoadFromBuffer(pData3);
				//set scheduling and delay data false in the task group
				data3.SetIsScheduled(FALSE);
				data3.SetIsDelayed(FALSE);
				//save
				delete []pData3;
				data3.SaveToBuffer(NULL,dwBufSize3);
				pData3 = new BYTE[dwBufSize3];
				data3.SaveToBuffer(pData3,dwBufSize3);
				m_pTestFile->SetTaskGroupData(nRow-1,pData3,dwBufSize3);
				m_pDoc->SetModifiedFlag();
				DisplayScheduleInfo(nRow);
				//DisplayDisableButton(nRow)
				//AutoSizeColumn(enNameCol)
				AutoSizeColumn(enScheduleCol);
			}	
		}
		break;
		case FAILED:
		{
				//start a failed try it again
			success=StartTG(nRow-1);
		}
		break;
		case COMPLETED:
		{
			//start a completed
			success=StartTG(nRow-1);
		}
		break;
		case STOPPING:
		{
			//dont do anything, this should never happen as no button is displayed
			success=false;
		}
		break;
	}

	//could it be done successfully?
    if(success==TRUE)
	{
		//update the test window
		DisplayStartStopButton(nRow);
		AutoSizeColumn(enStartStopStatusCol);
		Invalidate();
		m_pDoc->SetModifiedFlag();
		//Update the property window too
		m_pPropertiesBar->EditStartStop(nRow-1);
	}
	else
	{
		// the task could not be stopped or started!
		AfxMessageBox("The task could not be stopped or started");
	}

	return;
}


BOOL CTestCtrl::StartTG(int nGroup)
{
	LPBYTE pData;
	DWORD dwBufSize;
	TaskGroupExecInfo execInfo;

	m_pTestFile->GetTaskGroupData(nGroup,&pData,dwBufSize);
	CTaskGroupDefData TGData;
	TGData.LoadFromBuffer(pData);
		
	TaskGroupStartupInfo tgStartup;
	tgStartup.strHost = m_pTestFile->GetTaskGroupHost(nGroup);
	if (TGData.GetIsScheduled())
	{
		tgStartup.lStartTime = ConvertToTime(TGData.GetScheduledTime());
	}
	else if(TGData.GetIsDelayed())
	{
		int nDelay = TGData.GetStartDelay();
		CTTime tm;
		tm.now();
		tgStartup.lStartTime = tm.getSeconds() + nDelay;
	}
	else
	{
		tgStartup.lStartTime = 0;
	}
	tgStartup.bStopOnComplete = (TGData.GetStopOnComplete()? true:false);
	if (m_pTestFile->GetTaskType(nGroup,0) == CTestFile::enProfile)
	{
		execInfo.tgtType = tgt_profile;
	}
	else
	{
		execInfo.tgtType = tgt_script;
	}
	execInfo.bIsDuration = (TGData.GetbIsDuration()? true:false);
	execInfo.lTaskGroupDuration = TGData.GetTaskDuration();
	execInfo.lTaskGroupIterations = TGData.GetTaskIterations();
	execInfo.lNumVUs = TGData.GetNumVUs(); 
	
	if(TGData.GetUseAdvancedVUInfo())
	{
		execInfo.lInterBatchDelay=TGData.GetInterBatchDelay();    
		execInfo.lVUsPerBatch=TGData.GetVUsPerBatch();        
		execInfo.lBatchRampUpTime=TGData.GetBatchRampUpTime();
	}
	else
	{
		execInfo.lInterBatchDelay=0;
		execInfo.lVUsPerBatch=TGData.GetNumVUs();        
		execInfo.lBatchRampUpTime=0;
	}

    execInfo.bUsePageTimers=(TGData.GetUsePageTimers()? true:false);

	switch (TGData.GetStatLevel())
	{
		case 'N':
		{
			execInfo.lStatsLevel=stalev_none;
			break;
		}
		case 'L':
		{
			execInfo.lStatsLevel=stalev_low;
			break;
		}
		case 'H':
		{
			execInfo.lStatsLevel=stalev_high;
			break;
		}
		default:
		{
			execInfo.lStatsLevel=stalev_none;
		}
			
	}

	execInfo.seqTasks.length(m_pTestFile->GetNumTasks(nGroup));
	for (int t =0; t<m_pTestFile->GetNumTasks(nGroup); t++)
	{
		LPBYTE pTaskData = NULL;
		DWORD dwTaskBufSize;
		m_pTestFile->GetTaskData(nGroup,t, &pTaskData, dwTaskBufSize);

		TaskExecInfo task;
		task.strScriptName = m_pTestFile->GetTaskName(nGroup,t);
		if (pTaskData != NULL)
		{
			CScriptDefData TaskData;
			TaskData.LoadFromBuffer(pTaskData);	

			task.bIsDuration = (TaskData.GetIsDuration()? true:false);
			task.lDuration = TaskData.GetDuration();    
			task.lIterations = TaskData.GetNumIterations();
			task.bIsFixedDelay = (TaskData.GetUseFixedIterDelay()? true:false);
			task.lFixedDelay = TaskData.GetFixedIterDelay();
			task.lMinDelay = TaskData.GetMinIterDelay();	
			task.lMaxDelay = TaskData.GetMaxIterDelay();
		}
		else
		{
			task.bIsDuration = 0;
			task.lDuration = 0;    
			task.lIterations = 0;
			task.bIsFixedDelay = 0;
			task.lFixedDelay = 0;
			task.lMinDelay = 0;	
			task.lMaxDelay = 0;
		}
		execInfo.seqTasks[t] = task; 
	}
	
	execInfo.strTarget = m_pTestFile->GetTaskTargetType(nGroup,0);
	execInfo.lTaskGroupId = nGroup;
	tgStartup.execInfo = execInfo;

	//corba info class
	return m_pCorbaInfo->StartTaskGroup(tgStartup,GetStrReg());
}

BOOL CTestCtrl::StopTG(int nGroup)
{
	//corba info class
	return m_pCorbaInfo->StopTaskGroup(nGroup,GetStrReg());
}

long CTestCtrl::ConvertToTime(CString strScheduledTime)
{
	//Parse string: format "+days hours:minutes:seconds"
	strScheduledTime.Remove('+');

	char* token;
	token = strtok( strScheduledTime.GetBuffer(0), " ");
	int nOffsetDays = atoi(token);

	token = strtok(NULL, ":");
	int nOffsetHours = atoi(token);

	token = strtok(NULL, ":");
	int nOffsetMins = atoi(token);

	token = strtok( NULL, "\0");
	int nOffsetSecs = atoi(token);

	// No delay so return start time as 0 (now!)
	if (nOffsetDays == 0 && nOffsetHours == 0 && nOffsetMins == 0 && nOffsetSecs == 0)
		return 0;

	// Get the time in seconds for today's date with no time
	CTTime tm;
	tm.now();
	long lTime = tm.getSeconds();

	string strTime = tm.Format("%H:%M:%S");
    char * buf = new char[strTime.size() + 1] ;
    strcpy(buf, strTime.c_str());
	token = strtok( buf, ":");
	lTime -= (atoi(token) * HOUR);
	token = strtok( NULL, ":");
	lTime -= (atoi(token) * MINUTE);
	token = strtok( NULL, "\0");

	lTime -= atoi(token);

	// Add the time offsets
	lTime += nOffsetDays * DAY;
	lTime += nOffsetHours * HOUR;
	lTime += nOffsetMins * MINUTE;
	lTime += nOffsetSecs;

	return lTime;
}

LRESULT CTestCtrl::OnRefreshList(WPARAM,LPARAM lParam)
{
	int nRow = LOWORD(lParam);
	int nCol = HIWORD(lParam);

	switch(nCol)
	{
	case CTestCtrl::enNameCol:
		DisplayStartStopButton(nRow);
		AutoSizeColumn(enStartStopStatusCol);
	break;
	case CTestCtrl::enHostCol:
		SetItemText(nRow,nCol,m_pTestFile->GetTaskGroupHost(nRow-1));
	break;
	case CTestCtrl::enVUCol:
	{
		LPBYTE pData;
		DWORD dwBufLen;
		m_pTestFile->GetTaskGroupData(nRow-1,&pData,dwBufLen);

		CTaskGroupDefData data;
		data.LoadFromBuffer(pData);

		CString str;
		str.Format("%d",data.GetNumVUs());
		SetItemText(nRow,nCol,str);
	}
	break;
	case CTestCtrl::enScheduleCol:
	{
		DisplayScheduleInfo(nRow);
	}
	break;
	default:
		m_pDoc->SetModifiedFlag();	
		return 0;
	break;
	}

	RedrawCell(nRow,nCol);
	AutoSizeColumn(nCol);
	Invalidate();

	m_pDoc->SetModifiedFlag();	
	return 0;
}

CString CTestCtrl::GetStrReg()
{
	return m_strReg;
}

BOOL CTestCtrl::GetTestActive()
{
	return m_bTestActive;
}

void CTestCtrl::DisableButtonPressed(int nRow)
{
	BOOL success=FALSE;

	DWORD  dwBufSize=0;
	LPBYTE pData = NULL;
	m_pTestFile->GetTaskGroupData(nRow-1,&pData,dwBufSize);
	CTaskGroupDefData data;
	data.LoadFromBuffer(pData);

	BOOL current=data.GetTaskGroupEnabled();
	if(current==FALSE)
	{
		//if its disabled then ok
		success=TRUE;
	}
	else if(GetTestActive()==FALSE)
	{
		//if its stopped then ok
		success=TRUE;
	}
	else
	{
		//if its an active test then only certain states can be disabled

		TGExecStatus status=GetTGStatus(nRow-1);
		if(status==INACTIVE||status==STOPPED||status==FAILED||status==COMPLETED)
		{
			success=TRUE;
		}
	}

	if(success)
	{
		BOOL newStatus;
		if(current==FALSE)
		{
			newStatus=TRUE;
		}
		else
		{
			newStatus=FALSE;
		}

		data.SetTaskGroupEnabled(newStatus);
		//save
		delete []pData;
		
		data.SaveToBuffer(NULL,dwBufSize);
		pData = new BYTE[dwBufSize];
		data.SaveToBuffer(pData,dwBufSize);
		m_pTestFile->SetTaskGroupData(nRow-1,pData,dwBufSize);
	
		SetCheckBoxCellState(nRow,enNameCol,newStatus);
	
		AutoSizeColumn(enNameCol);
		RedrawCell(nRow,enNameCol);
		DisplayStartStopButton(nRow);
		AutoSizeColumn(enStartStopStatusCol);
		Invalidate();
		m_pDoc->SetModifiedFlag();
	}
	else
	{
		//dont save and set it back to what it was
		SetCheckBoxCellState(nRow,enNameCol,current);
		RedrawCell(nRow,enNameCol);
		AfxMessageBox("You cannot disable an active or scheduled Task Group\nIn order to disable the Task Group stop it first");
	}
	return;
}

BOOL CTestCtrl::DisplayDisableButton(int nRow)
{
	DWORD  dwBufSize=0;
	LPBYTE pData = NULL;
	//load
	m_pTestFile->GetTaskGroupData(nRow-1,&pData,dwBufSize);
	CTaskGroupDefData data;
	data.LoadFromBuffer(pData);
	CString strName = m_pTestFile->GetTaskGroupName(nRow-1);

	CreateCheckCell(nRow,enNameCol,CGridBtnCellBase::CTL_ALIGN_LEFT,20,FALSE,BUTTON_CELL_MARGIN_SIZE);
	SetCheckBoxCellState(nRow,enNameCol,data.GetTaskGroupEnabled(),FALSE);
	SetItemText(nRow,enNameCol,strName);
	//SetItemImage(nRow,enNameCol,enNameImage);
	
	return true;
}

BOOL CTestCtrl::DisplayScheduleInfo(int nRow)
{
	DWORD  dwBufSize=0;
	LPBYTE pData = NULL;
	//load
	m_pTestFile->GetTaskGroupData(nRow-1,&pData,dwBufSize);
	CTaskGroupDefData data;
	data.LoadFromBuffer(pData);

	CString temp;

	if(data.GetIsScheduled())
	{
		temp="S "+data.GetScheduledTime();
		SetItemText(nRow,enScheduleCol,temp);
	}
	else if(data.GetIsDelayed())
	{
		temp="D "+ConvertToTime(data.GetStartDelay());
		SetItemText(nRow,enScheduleCol,temp);
	}
	else
	{
		SetItemText(nRow,enScheduleCol,"Immediate");
	}

	SetItemImage(nRow,enScheduleCol,enWatchImage);
	return true;
}


BOOL CTestCtrl::SetTestActive(CString StrReg)
{
	m_pUpdateThreadctrl = AfxBeginThread(GetUpdateTestCtrl,this);
	m_bTestActive=TRUE;
	m_strReg=StrReg;
	SetTimer(IDT_UPDATE_STATUS,2000,NULL);
	return true;
}

BOOL CTestCtrl::SetTestInactive()
{
	if (m_pUpdateThreadctrl)
	{
		SetEvent(m_hCloseThread);
		m_pUpdateThreadctrl = NULL;
	}

	m_bTestActive=FALSE;
	m_strReg="";
	KillTimer(IDT_UPDATE_STATUS);
	return false;
}


TGExecStatus CTestCtrl::GetTGStatus(int nTaskGroup)
{

	for(int a=0; a<m_TaskGroupStatuses.GetSize();a++)
	{	
		if(m_TaskGroupStatuses[a].TaskGroupId==nTaskGroup)
		{
			return m_TaskGroupStatuses[a].TaskGroupStatus;
		}
	}
	return INACTIVE;//the default
}

BOOL CTestCtrl::DeleteTGStatus(int nTaskGroup)
{

	for(int a=0; a<m_TaskGroupStatuses.GetSize();a++)
		{	
		if(m_TaskGroupStatuses[a].TaskGroupId==nTaskGroup)
		{
			m_TaskGroupStatuses.RemoveAt(a,1);
			//free the memory
			return true;
		}
	}
	return false;
}

void CTestCtrl::OnShowControl()
{
	AfxMessageBox("remove onshowcontrol in test ctrl.cpp");
	return;
}

void CTestCtrl::OnShowEnabled()
{
	AfxMessageBox("remove onshowenabled in ctestctrl");
	return;
}

void CTestCtrl::OnShowHost()
{
	ToggleColumn(enHostCol);
}

void CTestCtrl::OnShowScheduling()
{
	ToggleColumn(enScheduleCol);
}

void CTestCtrl::OnShowStatus()
{
	ToggleColumn(enStartStopStatusCol);
}

void CTestCtrl::OnShowTaskgroup()
{
	ToggleColumn(enNameCol);

}
void CTestCtrl::OnShowVus()
{
	ToggleColumn(enVUCol);
}

void CTestCtrl::ToggleColumn(int nCol)
{
	if(GetColumnWidth(nCol))
		SetColumnWidth(nCol,0);
	else
{	
		SetColumnWidth(nCol,10);
		AutoSizeColumn(nCol);
}	

	SaveColumnState();
	Invalidate();

}
void CTestCtrl::OnDestroy()
{
	CECGridCtrl::OnDestroy();

	SaveColumnState();
}


void CTestCtrl::SaveColumnState()
{
	CWinApp *pApp = AfxGetApp();
	pApp->WriteProfileInt("Enabled Cols","Start",(GetColumnWidth(enScheduleCol) ? TRUE:FALSE));
	pApp->WriteProfileInt("Enabled Cols","Task Group",(GetColumnWidth(enNameCol) ? TRUE:FALSE));
	pApp->WriteProfileInt("Enabled Cols","Status",(GetColumnWidth(enStartStopStatusCol) ? TRUE:FALSE));
	pApp->WriteProfileInt("Enabled Cols","Host",(GetColumnWidth(enHostCol) ? TRUE:FALSE));
	pApp->WriteProfileInt("Enabled Cols","VUs",(GetColumnWidth(enVUCol) ? TRUE:FALSE));
}

void CTestCtrl::OnDeleteselection()
{
	DeleteSelection();
}



void CTestCtrl::OnDeleteTaskGroup()
{
	CPoint pt(m_ptContextClick);
	ScreenToClient(&pt);
	int nRow,nCol;
	
	if(!HitTest(pt,nRow,nCol) || !nRow)
		return;

	if(m_pTestFile->GetNumTaskGroups()<=nRow-1)
		return;

	//force the selection to the name column then delete selection
	SetFocusCell(nRow,enNameCol);
	SetSelectedRange(nRow,enNameCol,nRow,enNameCol);
	SetFocus();
	DeleteSelection();
}

CString CTestCtrl::ConvertToTime(long seconds)
{
	CString answer;
	int remainder;
	div_t temp;
	int h=0;
	int m=0;
	int s=0;
	int days=0;

	temp=div(seconds,3600*24);
	days=temp.quot;
	remainder=temp.rem;

	temp=div(remainder,3600);
	h=temp.quot;
	remainder=temp.rem;

	temp=div(remainder,60);
	m=temp.quot;
	s=temp.rem;

	answer.Format("+%d +%02d:%02d:%02d",days,h,m,s);
	return answer;
}

void CTestCtrl::OutputTestFileAsText()
{
	//called when F6 pressed
	CStdioFile file;
	BOOL createdfile=FALSE;

	//set directory	m_strTestFolder
	CString strPath = getenv("OPENSTA_HOME");
    strPath += "\\Engines\\testfileastext.txt";
	if(!file.Open(strPath,CFile::modeCreate|CFile::modeWrite))
		return;

	CString str;
	file.WriteString("* General info from the test file *\n");
	file.WriteString("Test File Path\t\""+m_pTestFile->GetTestFilePath()+"\"\n");
	file.WriteString("Test Description\t\"" + m_pTestFile->GetDescription()+ "\"\n");
	str.Format("%d",m_pTestFile->GetNumTaskGroups());
	file.WriteString("Number of Task groups\t" + str + "\n");

	for(int nCount=0;nCount<m_pTestFile->GetNumTaskGroups();nCount++)
	{
		//load the task group data
		DWORD  dwBufSize=0;
		LPBYTE pData = NULL;
		//load
		m_pTestFile->GetTaskGroupData(nCount,&pData,dwBufSize);
		CTaskGroupDefData data;
		data.LoadFromBuffer(pData);
	
		//get the initial data
		str.Format("\n* Task group %d *\n",nCount);
		file.WriteString(str);
		file.WriteString("Task Group name\t\t" +m_pTestFile->GetTaskGroupName(nCount)+ "\n");
		file.WriteString("Hostname\t\t\""+m_pTestFile->GetTaskGroupHost(nCount)+"\"\n");
		
		str.Format("%d",data.GetBatchRampUpTime());
		file.WriteString("ramp up time\t\t" + str + "\n");
		if(data.GetbIsDuration()? true:false)
			file.WriteString("is duration\t\tTRUE\n");
		else	
			file.WriteString("is duration\t\tFALSE\n");

		str.Format("%d",data.GetInterBatchDelay());
		file.WriteString("Inter batch delay\t" +str + "\n");
		
		if(data.GetIsScheduled()? true:false)
			file.WriteString("is scheduled\t\tTRUE\n");
		else
			file.WriteString("is scheduled\t\tFALSE\n");

		str.Format("%d",data.GetNumVUPerInterval());
		file.WriteString("VU per interval\t\t"+str+"\n");
		
		str.Format("%d",data.GetNumVUs());
		file.WriteString("Number VUs \t\t" +str+"\n");
		
		file.WriteString("Scheduled time\t\t\"" +data.GetScheduledTime()+"\"\n");
		file.WriteString("Start delay \t\t"+ str+"\n");
		str.Format("%d",data.GetStartInterval());
		file.WriteString("Start interval \t\t"+str+"\n");
		
		switch(data.GetStatLevel())
		{
		case'L':
			file.WriteString("Stat Level\t\tL\n");
		break;
		case 'H':
			file.WriteString("Stat Level\t\tH\n");
		break;
		case 'N':
			file.WriteString("Stat Level\t\tN\n");
		break;
		default:
			file.WriteString("Stat Level\t\tError\n");
		break;	
		}
		
		if(data.GetStopOnComplete())
			file.WriteString("stop on complete\tTRUE\n");
		else
			file.WriteString("stop on complete\tFALSE\n");
		
		str.Format("%d",data.GetTaskDuration());
		file.WriteString("Task duration \t\t"+str+"\n");
		
		if(data.GetTaskGroupEnabled())
			file.WriteString("task group enabled\tTRUE\n");
		else
			file.WriteString("task group enabled\tFALSE\n");

		str.Format("%d",data.GetTaskIterations());
		file.WriteString("Task iterations \t"+str+"\n");
		
		switch(data.GetTimerStatLevel())
		{
		case'L':
			file.WriteString("Timer Stat Level\tL\n");
		break;
		case 'H':
			file.WriteString("Timer Stat Level\tH\n");
		break;
		case 'N':
			file.WriteString("Timer Stat Level\tN\n");
		break;
		default:
			file.WriteString("Timer Stat Level\tError\n");
		break;	
		}
		
		if(data.GetUseAdvancedVUInfo())
			file.WriteString("use advanced VU info\tTRUE\n");
		else
			file.WriteString("use advanced VU info\tFALSE\n");

		if(data.GetUsePageTimers())
			file.WriteString("use page timers\t\tTRUE\n");
		else
			file.WriteString("use page timers\t\tFALSE\n");
		
		str.Format("%d",data.GetVUsPerBatch());
		file.WriteString("VU per batch\t\t"+str+"\n");
		
		if(data.GetIsDelayed())
			file.WriteString("is delayed\t\tTRUE\n");
		else
			file.WriteString("is delayed\t\tFALSE\n");
		
		switch(m_pTestFile->GetTaskType(nCount,0))
		{
		case CTestFile::enScript:
		{
			file.WriteString("\nTask Type SCRIPT\n");
			int nNumTasks=m_pTestFile->GetNumTasks(nCount);
			str.Format("%d",nNumTasks);
			file.WriteString("Number of Tasks = "+str+"\n");

			for(int nTask=0;nTask<nNumTasks;nTask++)
			{
				str.Format("%d",nTask);
				file.WriteString("\nTask id                 "+str+"\n");

				CString strPath = m_pTestFile->GetTaskPath(nCount,nTask);
				file.WriteString("Script Path               "+strPath+"\n");
				CFileStatus fs;
				if(!CFile::GetStatus(strPath,fs))
				{
					//if the script has been deleted!
					file.WriteString("The .HTP file for this script has been deleted\n");
				}
				else
				{
					DWORD dwBufSize=0;
					LPBYTE pLoadData = NULL;
					CScriptDefData sData;
					m_pTestFile->GetTaskData(nCount,nTask,&pLoadData,dwBufSize);
					sData.LoadFromBuffer(pLoadData);

					if(sData.GetIsDuration())
						file.WriteString("Fixed iteration delay     TRUE\n");
					else
						file.WriteString("is duration           FALSE\n");

					str.Format("%d",sData.GetFixedIterDelay());
					file.WriteString("Fixed iteration delay     "+str+"\n");
					str.Format("%d",sData.GetMinIterDelay());
					file.WriteString("Min iteration delay       "+str+"\n");
					str.Format("%d",sData.GetMaxIterDelay());
					file.WriteString("Max iteration delay       "+str+"\n");
					str.Format("%d",sData.GetNumIterations());
					file.WriteString("Num iterations            "+str+"\n");
					str.Format("%d",sData.GetDuration());
					file.WriteString("Duration                  "+str+"\n");
					if(sData.GetUseFixedIterDelay())
						file.WriteString("Min iteration	delay    TRUE\n");
					else
						file.WriteString("Use fixed iteration	 FALSE\n");
				}
			}
		}
		break;
		case CTestFile::enProfile:
			file.WriteString("\nTask Type PROFILE\n");
		break;
		default:
			file.WriteString("\nTask Type UNKNOWN\n");
		break;
		}
	}
	file.Close();
}

void CTestCtrl::OnSinglestepTaskgroup() 
{
	CPoint pt(m_ptContextClick);
	ScreenToClient(&pt);
	int nRow,nCol;
	
	if(!HitTest(pt,nRow,nCol) || !nRow)
		return;

	if(m_pTestFile->GetNumTaskGroups()<=nRow-1)
		return;

	m_pDocument->SetModifiedFlag();

	::SendMessage(m_hWndNotify,WMU_SETSTEPBYSTEPMODE,nRow-1,1);
}
