/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
// VUsDlg.cpp : implementation file
//

#include "stdafx.h"
#include "VUsDlg.h"
#include "TestFile.h"
#include "TestCtrl.h"
#include "BaseUIHelpID.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CVUsDlg dialog


CVUsDlg::CVUsDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CVUsDlg::IDD, pParent),
	m_pTestFile(NULL),
	m_nTaskGroup(-1),
	m_hwndNotify(NULL)
{
	//{{AFX_DATA_INIT(CVUsDlg)
	m_bUseAdvancedOptions = FALSE;
	m_bGenPageTimers = FALSE;
	m_nInterBatchDelay = 0;
	m_nNumVUs = 1;
	m_nStartInterval = 0;
	m_nVUPerBatch = 1;
	m_nVUsPerInterval = 0;
	m_nBatchRampTime = 0;
	m_WarningString = _T("");
	//}}AFX_DATA_INIT
}


void CVUsDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CVUsDlg)
	//DDX_Control(pDX, IDC_START_DELAY, m_ctlStartDelay);
	DDX_Control(pDX, IDC_VU_PER_BATCH_SPIN, m_ctlVUPerBatchSpin);
	DDX_Control(pDX, IDC_NUM_VUS_SPIN, m_ctlNumVusSpin);
	DDX_Control(pDX, IDC_LOGGING_LEVEL, m_ctlLoggingLevel);
	DDX_Control(pDX, IDC_BATCHDELAY_SPIN, m_ctlBatchDelaySpin);
	DDX_Control(pDX, IDC_BATCH_RAMP_TIME_SPIN, m_ctlBatchRampSpin);
	DDX_Check(pDX, IDC_ADVANCED_OPTIONS, m_bUseAdvancedOptions);
	DDX_Check(pDX, IDC_GEN_PAGE_TIMERS, m_bGenPageTimers);
	DDX_Text(pDX, IDC_INTER_BATCHDELAY, m_nInterBatchDelay);
	DDX_Text(pDX, IDC_NUM_VUS, m_nNumVUs);
	DDX_Text(pDX, IDC_VU_PER_BATCH, m_nVUPerBatch);
	DDX_Text(pDX, IDC_BATCH_RAMP_TIME, m_nBatchRampTime);
	DDX_Text(pDX, IDC_VU_NO_EDIT, m_WarningString);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CVUsDlg, CDialog)
	//{{AFX_MSG_MAP(CVUsDlg)
	ON_BN_CLICKED(IDC_ADVANCED_OPTIONS, OnAdvancedOptions)
	ON_CBN_SELCHANGE(IDC_LOGGING_LEVEL, OnSelchangeLoggingLevel)
	ON_EN_CHANGE(IDC_NUM_VUS, OnChangeNumVus)
	ON_BN_CLICKED(IDC_GEN_PAGE_TIMERS, OnGenPageTimers)
	ON_EN_CHANGE(IDC_INTER_BATCHDELAY, OnChangeInterBatchdelay)
	ON_EN_CHANGE(IDC_VU_PER_BATCH, OnChangeVuPerBatch)
	ON_EN_CHANGE(IDC_BATCH_RAMP_TIME, OnChangeBatchRampTime)
	ON_WM_HELPINFO()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CVUsDlg message handlers
void CVUsDlg::SetTestFile(CTestFile* pTestFile,int nGroup,HWND hwndNotify)
{
	m_pTestFile = pTestFile;
	m_nTaskGroup = nGroup;
	m_hwndNotify = hwndNotify;
	
	DWORD dwBufSize=0;
	LPBYTE pLoadData = NULL;
	m_pTestFile->GetTaskGroupData(nGroup,&pLoadData,dwBufSize);

	m_TGData.LoadFromBuffer(pLoadData);

	m_nNumVUs = m_TGData.GetNumVUs();
	m_nStartInterval = m_TGData.GetStartInterval();
	m_nVUsPerInterval = m_TGData.GetNumVUPerInterval();
	CTimeSpan TS(m_TGData.GetStartDelay());
	m_ctlStartDelay.SetHours(TS.GetHours());
	m_ctlStartDelay.SetMinutes(TS.GetMinutes());
	m_ctlStartDelay.SetSeconds(TS.GetSeconds());

	m_ctlLoggingLevel.ResetContent();
	m_ctlLoggingLevel.AddString("None");
	m_ctlLoggingLevel.AddString("Low");
	m_ctlLoggingLevel.AddString("High");

	switch(m_TGData.GetStatLevel())
	{
	case 'N':
		m_ctlLoggingLevel.SetCurSel(0);
	break;
	case 'H':
		m_ctlLoggingLevel.SetCurSel(2);
	break;
	default:
		m_ctlLoggingLevel.SetCurSel(1);
	break;
	}

	m_bGenPageTimers = m_TGData.GetUsePageTimers();

	m_bUseAdvancedOptions = m_TGData.GetUseAdvancedVUInfo();
	
	UpdateData(FALSE);

	OnAdvancedOptions();	

    const int nNumAccel = 3;
    UDACCEL arAccel[nNumAccel];
    arAccel[0].nInc = 1;
    arAccel[0].nSec = 2;
    arAccel[1].nInc = 10;
    arAccel[1].nSec = 3;
    arAccel[2].nInc = 100;
    arAccel[2].nSec = 4;

	m_ctlBatchDelaySpin.SetAccel(nNumAccel,arAccel);
	m_ctlBatchRampSpin.SetAccel(nNumAccel,arAccel);
	m_ctlNumVusSpin.SetAccel(nNumAccel,arAccel);
	m_ctlVUPerBatchSpin.SetAccel(nNumAccel,arAccel);

	
	m_ctlNumVusSpin.SetRange(1,UD_MAXVAL);
	m_ctlBatchRampSpin.SetRange(0,UD_MAXVAL);
	m_ctlBatchDelaySpin.SetRange(0,UD_MAXVAL);
	m_ctlVUPerBatchSpin.SetRange(1,UD_MAXVAL);	
}

void CVUsDlg::OnAdvancedOptions() 
{
	UpdateData();
	if(m_bUseAdvancedOptions)
	{
		m_nInterBatchDelay = m_TGData.GetInterBatchDelay();
		m_nVUPerBatch = m_TGData.GetVUsPerBatch();
		m_nBatchRampTime = m_TGData.GetBatchRampUpTime();
		UpdateData(FALSE);	//redraw dialog with the new values
	}

	GetDlgItem(IDC_ADVANCED_FRAME)->ShowWindow(m_bUseAdvancedOptions ? SW_SHOW:SW_HIDE);
	GetDlgItem(IDC_BATCHDELAY_CAPTION)->ShowWindow(m_bUseAdvancedOptions ? SW_SHOW:SW_HIDE);
	GetDlgItem(IDC_VU_PER_BATCH_CAPTION)->ShowWindow(m_bUseAdvancedOptions ? SW_SHOW:SW_HIDE);
	GetDlgItem(IDC_BATCHRAMP_CAPTION)->ShowWindow(m_bUseAdvancedOptions ? SW_SHOW:SW_HIDE);
	GetDlgItem(IDC_INTER_BATCHDELAY)->ShowWindow(m_bUseAdvancedOptions ? SW_SHOW:SW_HIDE);
	GetDlgItem(IDC_VU_PER_BATCH)->ShowWindow(m_bUseAdvancedOptions ? SW_SHOW:SW_HIDE);
	GetDlgItem(IDC_BATCH_RAMP_TIME)->ShowWindow(m_bUseAdvancedOptions ? SW_SHOW:SW_HIDE);
	GetDlgItem(IDC_BATCHDELAY_SPIN)->ShowWindow(m_bUseAdvancedOptions ? SW_SHOW:SW_HIDE);
	GetDlgItem(IDC_VU_PER_BATCH_SPIN)->ShowWindow(m_bUseAdvancedOptions ? SW_SHOW:SW_HIDE);
	GetDlgItem(IDC_BATCH_RAMP_TIME_SPIN)->ShowWindow(m_bUseAdvancedOptions ? SW_SHOW:SW_HIDE);

	if(m_bUseAdvancedOptions != m_TGData.GetUseAdvancedVUInfo())
	{
		m_TGData.SetUseAdvancedVUInfo(m_bUseAdvancedOptions);
		SaveDataToTestFile();
	}
}

void CVUsDlg::OnSelchangeLoggingLevel() 
{
	int nCurSel = m_ctlLoggingLevel.GetCurSel();
	
	CString strItem;
	m_ctlLoggingLevel.GetLBText(nCurSel,strItem);
	m_TGData.SetStatLevel(strItem[0]);
	m_TGData.SetTimerStatLevel(strItem[0]);
	SaveDataToTestFile();
}

void CVUsDlg::SaveDataToTestFile()
{
	if(m_nTaskGroup == -1)
		return;

	//AfxMessageBox("save data to test file 1");
	//Delete the old script data from the testfile
	DWORD dwBufSize;
	LPBYTE pData;
	m_pTestFile->GetTaskGroupData(m_nTaskGroup,&pData,dwBufSize);
	delete []pData;

	//Get size of buffer required
	m_TGData.SaveToBuffer(NULL,dwBufSize);

	//create a buffer and save data to it
	pData = new BYTE[dwBufSize];
	m_TGData.SaveToBuffer(pData,dwBufSize);

	//set the data in the test file
	m_pTestFile->SetTaskGroupData(m_nTaskGroup,pData,dwBufSize);

	//Tell test ctrl data has changed
	::SendMessage(m_hwndNotify,WMU_REFRESH_LIST,0,MAKELPARAM(m_nTaskGroup+1,CTestCtrl::enVUCol));
//	AfxMessageBox("save data to test file 2");
}

void CVUsDlg::OnChangeNumVus() 
{
	if(m_nTaskGroup == -1)
		return;

	UpdateData();
	if(m_nNumVUs < 1)
	{
		m_nNumVUs=1;
		m_TGData.SetVUsPerBatch(m_nNumVUs);
		UpdateData(FALSE);
	}

	if(m_nNumVUs < m_TGData.GetVUsPerBatch())
	{
		m_TGData.SetVUsPerBatch(m_nNumVUs);
		m_nVUPerBatch = m_nNumVUs;
		UpdateData(FALSE);
	}

	if(m_nNumVUs != m_TGData.GetNumVUs())
	{
		m_TGData.SetNumVUs(m_nNumVUs);
		SaveDataToTestFile();
	}
}


void CVUsDlg::OnGenPageTimers() 
{
	if(m_nTaskGroup == -1)
		return;

	UpdateData();
	if(m_bGenPageTimers != m_TGData.GetUsePageTimers())
	{
		m_TGData.SetUsePageTimers(m_bGenPageTimers);
		SaveDataToTestFile();
	}
}

void CVUsDlg::OnChangeInterBatchdelay() 
{
	if(m_nTaskGroup == -1)
		return;

	UpdateData();
	if(m_nInterBatchDelay != m_TGData.GetInterBatchDelay())
	{
		m_TGData.SetInterBatchDelay(m_nInterBatchDelay);
		SaveDataToTestFile();
	}
}

void CVUsDlg::OnChangeVuPerBatch() 
{
	if(m_nTaskGroup == -1)
		return;

	UpdateData();

	if(m_nVUPerBatch <1)
	{
		m_nVUPerBatch =1;
		UpdateData(FALSE);
	}

	if(m_nVUPerBatch >= m_TGData.GetNumVUs())
	{
		m_nVUPerBatch = m_TGData.GetNumVUs();
		UpdateData(FALSE);
	}

	if(m_nVUPerBatch != m_TGData.GetVUsPerBatch())
	{
		m_TGData.SetVUsPerBatch(m_nVUPerBatch);
		SaveDataToTestFile();
	}
}

void CVUsDlg::OnChangeBatchRampTime() 
{
	if(m_nTaskGroup == -1)
		return;

	UpdateData();
	if(m_nBatchRampTime != m_TGData.GetBatchRampUpTime())
	{
		m_TGData.SetBatchRampUpTime(m_nBatchRampTime);
		SaveDataToTestFile();
	}
}
/*
void CVUsDlg::OnChangeStartDelay() 
{
	if(m_nTaskGroup == -1)
		return;

	int nHours = m_ctlStartDelay.GetHours();
	int nMins = m_ctlStartDelay.GetMinutes();
	int nSecs = m_ctlStartDelay.GetSeconds();
	
	CTimeSpan TS(0,nHours,nMins,nSecs);
	long lDelay = TS.GetTotalSeconds();

	if(lDelay != m_TGData.GetStartDelay())
	{
		m_TGData.SetStartDelay(lDelay);
		SaveDataToTestFile();
	}

  }
*/



BOOL CVUsDlg::SetWindowEnabled()
{	
	//get the cwnd of this dialog
	if(IsWindowEnabled()==FALSE)
	{
		m_WarningString="";
		//These fields cannot be edited because the Task group is in use, if a test is running disable the Task group before attempting to edit it";
		GetDlgItem(IDC_NUM_VUS)->EnableWindow(FALSE);
		GetDlgItem(IDC_NUM_VUS_SPIN)->EnableWindow(FALSE);
		//GetDlgItem(IDC_START_DELAY)->EnableWindow(FALSE);
		GetDlgItem(IDC_LOGGING_LEVEL)->EnableWindow(FALSE);
		GetDlgItem(IDC_GEN_PAGE_TIMERS)->EnableWindow(FALSE);
		GetDlgItem(IDC_ADVANCED_OPTIONS)->EnableWindow(FALSE);
		GetDlgItem(IDC_INTER_BATCHDELAY)->EnableWindow(FALSE);
		GetDlgItem(IDC_BATCHDELAY_SPIN)->EnableWindow(FALSE);
		GetDlgItem(IDC_VU_PER_BATCH)->EnableWindow(FALSE);
		GetDlgItem(IDC_VU_PER_BATCH_SPIN)->EnableWindow(FALSE);
		GetDlgItem(IDC_BATCH_RAMP_TIME)->EnableWindow(FALSE);
		GetDlgItem(IDC_BATCH_RAMP_TIME_SPIN)->EnableWindow(FALSE);
		//m_ctlStartDelay.EnableWindow(FALSE);
	}
	else
	{
		m_WarningString="";//These fields can be edited";
		GetDlgItem(IDC_NUM_VUS)->EnableWindow(TRUE);
		GetDlgItem(IDC_NUM_VUS_SPIN)->EnableWindow(TRUE);
	//	GetDlgItem(IDC_START_DELAY)->EnableWindow(TRUE);
		GetDlgItem(IDC_LOGGING_LEVEL)->EnableWindow(TRUE);
		GetDlgItem(IDC_GEN_PAGE_TIMERS)->EnableWindow(TRUE);
		GetDlgItem(IDC_ADVANCED_OPTIONS)->EnableWindow(TRUE);
		GetDlgItem(IDC_INTER_BATCHDELAY)->EnableWindow(TRUE);
		GetDlgItem(IDC_BATCHDELAY_SPIN)->EnableWindow(TRUE);
		GetDlgItem(IDC_VU_PER_BATCH)->EnableWindow(TRUE);
		GetDlgItem(IDC_VU_PER_BATCH_SPIN)->EnableWindow(TRUE);
		GetDlgItem(IDC_BATCH_RAMP_TIME)->EnableWindow(TRUE);
		GetDlgItem(IDC_BATCH_RAMP_TIME_SPIN)->EnableWindow(TRUE);
		//m_ctlStartDelay.EnableWindow(TRUE);
	}
	UpdateData(FALSE);
	return true;
}

BOOL CVUsDlg::OnHelpInfo(HELPINFO* pHelpInfo) 
{
	pHelpInfo->dwContextId = HIDD_VUS_DLG;
	return CDialog::OnHelpInfo(pHelpInfo);
}
