// StepByStepProxy.cpp: implementation of the CStepByStepProxy class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "StepByStepProxy.h"
#include "StepByStepDlg.h"
#include "TestFile.h"
#include "DebugComms.h"


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

extern CTestFile *m_pTestFile;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CStepByStepProxy::CStepByStepProxy(CStepByStepDlg* pDialog):
	m_pDialog(pDialog),
	m_hPipe(INVALID_HANDLE_VALUE),
	m_pServerInfoThread(NULL)
{
	m_DetachEvt = CreateEvent(NULL,TRUE,FALSE,NULL);
}

CStepByStepProxy::~CStepByStepProxy()
{
	Detach();
	CloseHandle(m_DetachEvt);
}

BOOL CStepByStepProxy::StartDebugger(BOOL bRun)
{
	if(Attach())
		return true;

	CString strTestName = m_pTestFile->GetTestName();
	PROCESS_INFORMATION pi;
	CString strPath = getenv("OPENSTA_HOME");
	strPath += "\\BaseUI\\TestDebugger.exe";
	CString strCmdLine;
	strCmdLine.Format("%s -T %s",strPath,strTestName);
	
//	if(m_pTestFile->IsAdvancedStepByStepMode())
		strCmdLine += " -Advanced";

	if(bRun)
		strCmdLine += " -Run";


//#define _TRACE_DEBUGGER_

#ifdef _TRACE_DEBUGGER_
	strCmdLine += " -trace";
#endif

	STARTUPINFO si;
    ZeroMemory( &si, sizeof(si) );
    si.cb = sizeof(si);
    
//#define _TRACE_DEBUGGER_

	if(!CreateProcess(strPath,strCmdLine.GetBuffer(0),NULL,NULL,FALSE,
#ifdef _TRACE_DEBUGGER_
			0,
#else
			DETACHED_PROCESS,
#endif
			NULL,NULL,&si,&pi))
	{
		CString str;
		str.Format("Error (%d): Failed to start debugger.",GetLastError());
		AfxMessageBox(str);
		return false;
	}

	CString strPipename = "\\\\.\\pipe\\" + strTestName;
	int nLoops = 30;
	while(nLoops && !WaitNamedPipe(strPipename,5000))
	{
		if(GetLastError() != ERROR_FILE_NOT_FOUND)
			return false;

		Sleep(200);	
		nLoops--;
	}

	return Attach();
}

BOOL CStepByStepProxy::Attach()
{
	ResetEvent(m_DetachEvt);

	if(m_hPipe != INVALID_HANDLE_VALUE)
		return true;

	if(!m_pTestFile)
		return false;

	CString strTestName = m_pTestFile->GetTestName();
	CString strPipename = "\\\\.\\pipe\\" + strTestName;
	m_hPipe = CreateFile (strPipename,              // Pipe name.
                           GENERIC_WRITE          // Generic access, read/write.
                           | GENERIC_READ,
                           FILE_SHARE_READ        // Share both read and write.
                           | FILE_SHARE_WRITE ,
                           NULL,                  // No security.
                           OPEN_EXISTING,         // Fail if not existing.
                           FILE_FLAG_OVERLAPPED,  // Use overlap.
                           NULL);                 // No template.

	DWORD dwErr = GetLastError();

	if((m_hPipe != INVALID_HANDLE_VALUE) && !m_pServerInfoThread)
		m_pServerInfoThread = AfxBeginThread(&ServerInfoThread,this);
	
	return (m_hPipe != INVALID_HANDLE_VALUE);
}

void CStepByStepProxy::Detach()
{
	SetEvent(m_DetachEvt);

	if(m_pServerInfoThread)
		WaitForSingleObject(m_pServerInfoThread->m_hThread,INFINITE);

	m_pServerInfoThread = NULL;
	CloseHandle(m_hPipe);
	m_hPipe = INVALID_HANDLE_VALUE;
}

BOOL CStepByStepProxy::StepSingleUser(int nVUId)
{
	DWORD dwBufSize = sizeof(int)*2;
	LPBYTE pBuffer = new BYTE[dwBufSize];
	LPBYTE pBufPos = pBuffer;

	int nCommand = enStepSingleUser;
	memcpy(pBufPos,&nCommand,sizeof(int));	pBufPos += sizeof(int);
	memcpy(pBufPos,&nVUId,sizeof(int));	pBufPos += sizeof(int);

	BOOL bResult = WriteToPipe(pBuffer,dwBufSize);
	delete [] pBuffer;
	
	return bResult;
}

BOOL CStepByStepProxy::StepSingleEvent()
{
	int nCommand = enStepSingleEvent;
	return WriteToPipe((LPBYTE)&nCommand,sizeof(int));
}

BOOL CStepByStepProxy::Run()
{
	int nCommand = enRun;
	return WriteToPipe((LPBYTE)&nCommand,sizeof(int));
}

BOOL CStepByStepProxy::Break()
{
	int nCommand = enBreak;
	return WriteToPipe((LPBYTE)&nCommand,sizeof(int));
}

BOOL CStepByStepProxy::SetBreakPoint(const CString& strScript,int nLineNumber,BOOL bSet)
{
	int nScrNameLen = strScript.GetLength()+1;
	DWORD dwBufSize = sizeof(int)*3+nScrNameLen+sizeof(BOOL);
	LPBYTE pBuffer = new BYTE[dwBufSize];
	LPBYTE pBufPos = pBuffer;

	int nCommand = enSetBreakPoint;
	memcpy(pBufPos,&nCommand,sizeof(int));				pBufPos += sizeof(int);
	memcpy(pBufPos,&nScrNameLen,sizeof(int));			pBufPos += sizeof(int);
	memcpy(pBufPos,(LPCTSTR)strScript,nScrNameLen);		pBufPos += nScrNameLen;
	memcpy(pBufPos,&nLineNumber,sizeof(int));			pBufPos += sizeof(int);
	memcpy(pBufPos,&bSet,sizeof(BOOL));					pBufPos += sizeof(BOOL);
	
	BOOL bResult = WriteToPipe(pBuffer,dwBufSize);
	delete [] pBuffer;
	
	return bResult;
}	

BOOL CStepByStepProxy::RequestState()
{
	int nCommand = enGetState;
	return WriteToPipe((LPBYTE)&nCommand,sizeof(int));
}

UINT CStepByStepProxy::ServerInfoThread(LPVOID pParam)
{
	CStepByStepProxy *pComms = (CStepByStepProxy*)pParam;

	int nType;
	while(1)
	{
		if(!pComms->ReadFromPipe((LPBYTE)&nType,sizeof(int)))
			break;
		switch(nType)
		{
		case enNextRequest:
		{
			DWORD dwRemBytes=0;
			if(!pComms->ReadFromPipe((LPBYTE)&dwRemBytes,sizeof(DWORD)))
				return -1;

			LPBYTE pBuf = new BYTE[dwRemBytes];
			LPBYTE pBufPos=pBuf;
			if(!pComms->ReadFromPipe(pBuf,dwRemBytes))
				return -1;

			CNextRequestData *pNRData = new CNextRequestData;
			pNRData->LoadFromBuffer(pBuf);
			delete []pBuf;

			pComms->m_pDialog->SendMessage(WMU_NEXTREQUEST,0,(LPARAM)pNRData);
		}
		break;
		case enRequestComplete:
		{
			DWORD dwRemBytes=0;
			if(!pComms->ReadFromPipe((LPBYTE)&dwRemBytes,sizeof(DWORD)))
				return -1;

			LPBYTE pBuf = new BYTE[dwRemBytes];
			LPBYTE pBufPos=pBuf;
			if(!pComms->ReadFromPipe(pBuf,dwRemBytes))
				return -1;

			CRequestCompleteData *pRCData = new CRequestCompleteData;
			pRCData->LoadFromBuffer(pBuf);
			delete []pBuf;

			pComms->m_pDialog->SendMessage(WMU_REQUESTCOMPLETE,0,(LPARAM)pRCData);
		}
		break;
		case enReadyToStep:
			pComms->m_pDialog->SendMessage(WMU_READYTOSTEP);
		break;
		case enRunning:
			pComms->m_pDialog->SendMessage(WMU_RUNNING);
		break;
		case enRequestStepped:
			int nBufSize = sizeof(int)*2;
			LPBYTE pBuf = new BYTE[nBufSize];
			LPBYTE pBufPos = pBuf;
			if(!pComms->ReadFromPipe(pBuf,nBufSize))
				return -1;

			int nVUId,nConId;
			memcpy(&nVUId,pBufPos,sizeof(int));		pBufPos += sizeof(int);
			memcpy(&nConId,pBufPos,sizeof(int));	pBufPos += sizeof(int);

			delete []pBuf;
			pComms->m_pDialog->SendMessage(WMU_REQUESTSTEPPED,nVUId,nConId);
		break;
		}
	}

	return 0;
}

BOOL CStepByStepProxy::ReadFromPipe(LPBYTE pBuffer,DWORD dwBufSize)
{
	if(m_hPipe == INVALID_HANDLE_VALUE)
		return false;

	OVERLAPPED ol;
	memset(&ol,0,sizeof(ol));
	ol.hEvent = CreateEvent(NULL,TRUE,FALSE,NULL);

	DWORD dwBytesRead=0;
	BOOL bResult = ReadFile(m_hPipe,pBuffer,dwBufSize,&dwBytesRead,&ol);
	if(!bResult)
	{
		if(GetLastError() == ERROR_IO_PENDING)
		{
			const DWORD dwNumHandles = 2;
			HANDLE pHandles[dwNumHandles] = {ol.hEvent,m_DetachEvt};
			DWORD dwResult = WaitForMultipleObjects(dwNumHandles,pHandles,FALSE,INFINITE);

			bResult = ((dwResult == WAIT_OBJECT_0) &&
						GetOverlappedResult(m_hPipe,&ol,&dwBytesRead,FALSE) && 
						(dwBytesRead == dwBufSize));
		}
		else
			bResult = false;

		if(!bResult)
			CancelIo(m_hPipe);
	}
	CloseHandle(ol.hEvent);
	return bResult;
}

BOOL CStepByStepProxy::WriteToPipe(LPBYTE pBuffer, DWORD dwBufSize)
{
	CSingleLock lock(&m_CritSect);
	lock.Lock();

	if(m_hPipe == INVALID_HANDLE_VALUE)
		return false;

	OVERLAPPED ol;
	memset(&ol,0,sizeof(ol));
	ol.hEvent = CreateEvent(NULL,TRUE,FALSE,NULL);

	DWORD dwBytesWritten=0;
	BOOL bResult = WriteFile(m_hPipe,pBuffer,dwBufSize,&dwBytesWritten,&ol);
	if(!bResult)
	{
		if(GetLastError() == ERROR_IO_PENDING)
		{
			const DWORD dwNumHandles = 2;
			HANDLE pHandles[dwNumHandles] = {ol.hEvent,m_DetachEvt};

			DWORD dwResult = WaitForMultipleObjects(dwNumHandles,pHandles,FALSE,INFINITE);

			bResult = ((dwResult == WAIT_OBJECT_0) && 
						GetOverlappedResult(m_hPipe,&ol,&dwBytesWritten,FALSE) && 
						(dwBytesWritten == dwBufSize));
		}
		else
			bResult = false;
		
		if(!bResult)
			CancelIo(m_hPipe);
	}

	CloseHandle(ol.hEvent);
	return bResult;
}

BOOL CStepByStepProxy::Abort()
{
	int nCommand = enAbort;
	return WriteToPipe((LPBYTE)&nCommand,sizeof(int));
}