/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// TestRunBar.cpp : implementation file
//

#include "stdafx.h"
#include "TestRunBar.h"
#include "RegFunctions.h"
#include "PluginTypes.h"
#include "TestResultsPlugin.h"
#include "resource.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTestRunBar
extern HINSTANCE m_hInstance;
extern CPluginManager* m_pPluginManager;

const CString CTestRunBar::m_strAuditLogName("Test Audit Log");
const CString CTestRunBar::m_strHistoryLogName("Test History Log");
const CString CTestRunBar::m_strReportLogName("Test Report Log");
const CString CTestRunBar::m_strSummaryName("Test Configuration");
const CString CTestRunBar::m_strErrorLogName("Test Error Log");
const CString CTestRunBar::m_strHTTPDataName("Test Summary Snapshots");


#define IDC_TESTRUN_TREE 1050
CTestRunBar::CTestRunBar(CWnd* pParent /*=NULL*/):
	m_hwndNotify(NULL)
{
}

BEGIN_MESSAGE_MAP(CTestRunBar, CSizingControlBarG)
	//{{AFX_MSG_MAP(CTestRunBar)
	ON_WM_CREATE()
	ON_WM_SIZE()
	ON_NOTIFY(TVN_ITEMEXPANDING, IDC_TESTRUN_TREE, OnItemexpandingTestrunTree)
	ON_NOTIFY(TVN_SELCHANGED, IDC_TESTRUN_TREE, OnSelchangedTestrunTree)
	ON_NOTIFY(NM_CLICK, IDC_TESTRUN_TREE, OnClickTestrunTree)
	ON_NOTIFY(NM_RCLICK, IDC_TESTRUN_TREE, OnRClickTestrunTree)
	ON_WM_DESTROY()
	ON_WM_CONTEXTMENU()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTestRunBar message handlers

int CTestRunBar::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CSizingControlBarG::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	AfxSetResourceHandle(m_hInstance);
	m_ImageList.Create(IDB_TRTREE_IMAGES,16,0,RGB(0,128,128));
	AfxSetResourceHandle(AfxGetInstanceHandle());

	m_ctlTestRunTree.Create(WS_CHILD|WS_VISIBLE|WS_BORDER|TVS_HASLINES|TVS_LINESATROOT|TVS_HASBUTTONS,CRect(0,0,0,0),this,IDC_TESTRUN_TREE);
	m_ctlTestRunTree.SetImageList(&m_ImageList,TVSIL_STATE);
	m_ctlTestRunTree.SetImageList(&m_ImageList,TVSIL_NORMAL);
	
	m_ImageList.SetOverlayImage(1,1);
	m_ImageList.SetOverlayImage(2,2);

	return 0;
}

void CTestRunBar::SetTestFolder(const CString& strFolder)
{
	m_strTestFolder = strFolder;
	if(m_strTestFolder.Right(1) != "\\")
		m_strTestFolder += "\\";
}

void CTestRunBar::Refresh()
{
	if(m_strTestFolder.IsEmpty())
		return;

	CMap <CString,LPCTSTR,UINT,UINT> mapItemToState;
	
	//Obtain the current list of runs
	HTREEITEM hRun = m_ctlTestRunTree.GetRootItem();
	while(hRun)
	{
		SaveBranchState(hRun,"",mapItemToState);
		hRun = m_ctlTestRunTree.GetNextSiblingItem(hRun);
	}

	m_ctlTestRunTree.DeleteAllItems();

	CStringArray arstrDirs;
	CFileFind FF;
	BOOL bContinue=FF.FindFile(m_strTestFolder + "\?\?-\?\?-\?\?\?\? \?\?-\?\?-\?\?.00?");
	while(bContinue)
	{
		bContinue = FF.FindNextFile();
		if(FF.IsDirectory() && !FF.IsDots() && !FF.IsHidden())
			arstrDirs.Add(FF.GetFileName());
	}

	int nNumDirs = arstrDirs.GetSize();
	qsort(arstrDirs.GetData(),nNumDirs,sizeof(char*),&CompareTestRunName);

	for(int nDir=0;nDir<arstrDirs.GetSize();nDir++)
	{
		TVINSERTSTRUCT tvInsert;
		tvInsert.hParent = NULL;
		tvInsert.hInsertAfter = TVI_LAST;

		tvInsert.item.mask = TVIF_CHILDREN|TVIF_TEXT|TVIF_IMAGE|TVIF_SELECTEDIMAGE;
		tvInsert.item.cChildren = 1;
		tvInsert.item.pszText = arstrDirs[nDir].GetBuffer(0);
		tvInsert.item.cchTextMax = arstrDirs[nDir].GetLength();
		tvInsert.item.iImage = enRunFolder;
		tvInsert.item.iSelectedImage = enRunFolder;

		CString strTestFile(m_strTestFolder + arstrDirs[nDir] + "\\TestConf.dat");
		CFile fTest;
		if(fTest.Open(strTestFile,CFile::modeRead))
		{
			CArchive ar(&fTest,CArchive::load);
			CTestFile f;
			f.Serialize(ar);
			int nGroup = f.GetStepByStepTaskGroup();
			if(nGroup!=-1)
			{
				tvInsert.item.iImage = enStepByStepRun;
				tvInsert.item.iSelectedImage = enStepByStepRun;
			}
			ar.Close();
			fTest.Close();
		}

		m_ctlTestRunTree.InsertItem(&tvInsert);
	}

	hRun = m_ctlTestRunTree.GetRootItem();
	while(hRun)
	{
		RestoreBranchState(hRun,"",mapItemToState);
		hRun = m_ctlTestRunTree.GetNextSiblingItem(hRun);
	}
}

int CTestRunBar::CompareTestRunName( const void *arg1, const void *arg2 )
{
	char *a = *(char**)arg1;
	char *b = *(char**)arg2;

	//Format of run name is "DD-MM-YYYY HH-MM-SS.XXX"	
	//						 01234567890123456789012
	//						 0         1         2

	const int nNumPositions = 17;
	BYTE byTestOrder[nNumPositions] = {6,7,8,9,3,4,0,1,11,12,14,15,17,18,20,21,22};
									//{Y,Y,Y,Y,M,M,S,S,H ,H , M, M, S, S, X, X, X}
	
	for(int nPos=0;nPos<nNumPositions;nPos++)
	{
		BYTE &byPos = byTestOrder[nPos];

		if(a[byPos] > b[byPos])
			return -1;
		else
		if(a[byPos] < b[byPos])
			return 1;
	}
	return 0;
}

void CTestRunBar::SaveBranchState(HTREEITEM hBranch,const CString& strFolder,CMap <CString,LPCTSTR,UINT,UINT>& mapItemToState)
{
	CString strRun = strFolder + m_ctlTestRunTree.GetItemText(hBranch);
	UINT uiState = m_ctlTestRunTree.GetItemState(hBranch,0xffffffff);
	mapItemToState.SetAt(strRun,uiState);

	if(m_ctlTestRunTree.ItemHasChildren(hBranch))
	{
		HTREEITEM hNextItem;
		HTREEITEM hChildItem = m_ctlTestRunTree.GetChildItem(hBranch);
		while (hChildItem != NULL)
		{
			hNextItem = m_ctlTestRunTree.GetNextItem(hChildItem, TVGN_NEXT);
			SaveBranchState(hChildItem,strRun,mapItemToState);
			hChildItem = hNextItem;
		}
	}
}

void CTestRunBar::RestoreBranchState(HTREEITEM hBranch,const CString& strFolder,CMap <CString,LPCTSTR,UINT,UINT>& mapItemToState)
{
	UINT uiState;
	CString strRun = strFolder + m_ctlTestRunTree.GetItemText(hBranch);
	if(mapItemToState.Lookup(strRun,uiState))
	{
		if(uiState & TVIS_EXPANDED)
			m_ctlTestRunTree.Expand(hBranch,TVE_EXPAND);
		if(uiState & TVIS_SELECTED)
			m_ctlTestRunTree.Select(hBranch,TVGN_CARET);

		m_ctlTestRunTree.SetItemState(hBranch,uiState,TVIS_OVERLAYMASK);
	}
		
	if(m_ctlTestRunTree.ItemHasChildren(hBranch))
	{
		HTREEITEM hNextItem;
		HTREEITEM hChildItem = m_ctlTestRunTree.GetChildItem(hBranch);
		while (hChildItem != NULL)
		{
			hNextItem = m_ctlTestRunTree.GetNextItem(hChildItem, TVGN_NEXT);
			RestoreBranchState(hChildItem,strRun,mapItemToState);
			hChildItem = hNextItem ;
		}
	}
}

void CTestRunBar::OnSize(UINT nType, int cx, int cy) 
{
	CSizingControlBarG::OnSize(nType, cx, cy);
	
	if(m_ctlTestRunTree.GetSafeHwnd())
		m_ctlTestRunTree.MoveWindow(0,0,cx,cy);
}

void CTestRunBar::OnItemexpandingTestrunTree(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;
	
	HTREEITEM hParent = m_ctlTestRunTree.GetParentItem(pNMTreeView->itemNew.hItem);
	
	if((hParent==NULL) && (m_ctlTestRunTree.GetChildItem(pNMTreeView->itemNew.hItem)==NULL))
	{
		CString strRun = m_strTestFolder +  m_ctlTestRunTree.GetItemText(pNMTreeView->itemNew.hItem);

		CFileStatus fs;
		if(!CFile::GetStatus(strRun + "\\.",fs))
		{
			m_ctlTestRunTree.DeleteItem(pNMTreeView->itemNew.hItem);
			return;
		}

		//Load the selected run
		CArray <GraphType*,GraphType*> arGraphs;

		GraphType *pGT;
		if(CFile::GetStatus(strRun + "\\" + strSummaryFile,fs))
		{
			pGT = new GraphType;
			pGT->GI.strName = m_strSummaryName;
			pGT->GI.bFolder = false;
			arGraphs.Add(pGT);
		}
	
		if(CFile::GetStatus(strRun + "\\TestLog.txt",fs))
		{
			pGT = new GraphType;
			pGT->GI.strName  = m_strAuditLogName;
			pGT->GI.bFolder = false;
			arGraphs.Add(pGT);
		}

		if(CFile::GetStatus(strRun + "\\ErrLog.txt",fs))
		{
			pGT = new GraphType;
			pGT->GI.strName  = m_strErrorLogName;
			pGT->GI.bFolder = false;
			arGraphs.Add(pGT);
		}

		//any file with "HTTPDataFile_" in the name are acceptable
		CFileFind FF;
		BOOL bContinue=FF.FindFile(strRun + "\\*.*");
		
		char* match = "HTTPDataFile_";
		while(bContinue)
		{
			bContinue = FF.FindNextFile();
			CString filename=FF.GetFileName();

			if(strncmp(match,filename,sizeof(match))==0)
			{
				pGT = new GraphType;
				pGT->GI.strName  = m_strHTTPDataName;
				pGT->GI.bFolder = false;
				arGraphs.Add(pGT);
				bContinue=false;
			}
		}
		FF.Close();

		if(CFile::GetStatus(strRun + "\\TestRep.txt",fs))
		{
			pGT = new GraphType;
			pGT->GI.strName  = m_strReportLogName;
			pGT->GI.bFolder = false;
			arGraphs.Add(pGT);
		}

		if(CFile::GetStatus(strRun + "\\TestHis.txt",fs))
		{
			pGT = new GraphType;
			pGT->GI.strName  = m_strHistoryLogName;
			pGT->GI.bFolder = false;
			arGraphs.Add(pGT);
		}
	
		CStringArray arResultsPlugIns;
		m_pPluginManager->ListPluginsByType(TEST_RESULTS_PLUGIN, arResultsPlugIns);
		
		for (int nPlugIn = 0; nPlugIn<arResultsPlugIns.GetSize();nPlugIn++)
		{
			CTestResultsPlugin plug(arResultsPlugIns[nPlugIn], m_pPluginManager);
			plug.LoadRun(strRun);

			//Get the graphs which are now available
			GraphInfoArray arTypes;
			plug.GetAvailableGraphs(arTypes);

			for(int nNdx=0;nNdx<arTypes.GetSize();nNdx++)
			{
				pGT = new GraphType;
				pGT->strPlugin = arResultsPlugIns[nPlugIn];
				pGT->GI = arTypes[nNdx];
				arGraphs.Add(pGT);
			}
		}

		for(int nNdx=0;nNdx<arGraphs.GetSize();nNdx++)
		{
			HTREEITEM hParent = pNMTreeView->itemNew.hItem;
			
			CString strTitle = arGraphs[nNdx]->GI.strName;
			char *szTok = strtok(strTitle.GetBuffer(0),"\\");
			while(szTok)
			{
				CString strItem(szTok);
				szTok = strtok(NULL,"\\");

				HTREEITEM hItem = FindTreeItem(strItem,hParent);
				if(hItem)
					hParent = hItem;
				else
				if(szTok || arGraphs[nNdx]->GI.bFolder)
					hParent = m_ctlTestRunTree.InsertItem(strItem,0,0,hParent);
				else
					hParent = m_ctlTestRunTree.InsertItem(strItem,1,1,hParent);
			}
			m_ctlTestRunTree.SetItemData(hParent,(DWORD)arGraphs[nNdx]);
		}
	}

	*pResult = 0;
}

void CTestRunBar::OnSelchangedTestrunTree(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;

	*pResult = 0;
}

void CTestRunBar::OnRClickTestrunTree(NMHDR* pNMHDR, LRESULT* pResult) 
{
	SendMessage(WM_CONTEXTMENU, (WPARAM) m_hWnd, GetMessagePos());
	
	*pResult = 1;
}

void CTestRunBar::OnClickTestrunTree(NMHDR* pNMHDR, LRESULT* pResult) 
{
	*pResult = 0;

	CPoint ptCur;
	GetCursorPos(&ptCur);
	m_ctlTestRunTree.ScreenToClient(&ptCur);

	UINT uiFlags;
	HTREEITEM hHitItem = m_ctlTestRunTree.HitTest(ptCur,&uiFlags);
	if(!hHitItem || !(uiFlags & TVHT_ONITEM))
		return;

	GraphType *pGraph = (GraphType*)m_ctlTestRunTree.GetItemData(hHitItem);
	if(!pGraph || pGraph->GI.bFolder)
		return;

	//Check that run still exists
	HTREEITEM hParent = hHitItem;
	while(1)
	{
		HTREEITEM hItem = m_ctlTestRunTree.GetParentItem(hParent);
		if(hItem)
			hParent = hItem;
		else
			break;
	}

	CString strRun = m_strTestFolder + m_ctlTestRunTree.GetItemText(hParent)+"\\";
	CFileStatus fs;
	if(!CFile::GetStatus(strRun + "\\.",fs))
	{
		m_ctlTestRunTree.DeleteItem(hParent);
		return;
	}

	//Crude locks stops a second click occuring while
	//first is being processed
	static bool bLocked = false;
	if(bLocked)
		return;
	bLocked = true;

	UINT uiState = m_ctlTestRunTree.GetItemState(hHitItem,TVIS_OVERLAYMASK);
	BOOL bChecked = (uiState & INDEXTOOVERLAYMASK(2));
	if(bChecked)
		m_ctlTestRunTree.SetItemState(hHitItem,INDEXTOOVERLAYMASK(1),TVIS_OVERLAYMASK);
	else
		m_ctlTestRunTree.SetItemState(hHitItem,INDEXTOOVERLAYMASK(2),TVIS_OVERLAYMASK);

	NM_TESTRUN *pTestRun = new NM_TESTRUN;
	pTestRun->strGraph = pGraph->GI.strName;
	pTestRun->strPlugin = pGraph->strPlugin;
	pTestRun->strTestRun = strRun;

	::SendMessage(m_hwndNotify,WMU_GRAPH_SELCHANGED,(WPARAM)pTestRun,!bChecked);
	delete pTestRun;

	bLocked = false;
}

void CTestRunBar::SetCheck(const CString& strRun,const CString& strGraph,bool bCheck)
{
	HTREEITEM hParent = m_ctlTestRunTree.GetRootItem();
	
	while(hParent)
	{
		if(m_strTestFolder + m_ctlTestRunTree.GetItemText(hParent) +"\\"== strRun)
		{
			DWORD dwData =0;
			CString strTitle(strGraph);
			HTREEITEM hSearch=NULL;
			char *szTok = strtok(strTitle.GetBuffer(0),"\\");
			while(szTok)
			{
				hSearch = m_ctlTestRunTree.GetChildItem(hParent);
				while(hSearch)
				{
					if(m_ctlTestRunTree.GetItemText(hSearch) == szTok)
					{
						dwData = m_ctlTestRunTree.GetItemData(hSearch);				
						hParent = hSearch;
						break;
					}
					
					hSearch = m_ctlTestRunTree.GetNextItem(hSearch,TVGN_NEXT);
				}
				
				szTok = strtok(NULL,"\\");
			};
			
			if(dwData)
			{
				UINT uiState = m_ctlTestRunTree.GetItemState(hSearch,TVIS_OVERLAYMASK);
				m_ctlTestRunTree.SetItemState(hSearch,INDEXTOOVERLAYMASK(bCheck ? 2: 1),TVIS_OVERLAYMASK);
			}

			return;
		}
		hParent = m_ctlTestRunTree.GetNextSiblingItem(hParent);
	};
}

void CTestRunBar::OnDestroy() 
{
	CRect rc;
	GetWindowRect(&rc);
	CString str;
	str.Format("%d,%d,%d,%d",rc.left,rc.top,rc.right,rc.bottom);
	AfxGetApp()->WriteProfileString("Results","Test Run Dlg Pos",str);

	CSizingControlBarG::OnDestroy();
}

HTREEITEM CTestRunBar::FindTreeItem(const CString& strName,HTREEITEM hParent)
{
	HTREEITEM hSearch=m_ctlTestRunTree.GetChildItem(hParent);

	while(hSearch)
	{
		if(m_ctlTestRunTree.GetItemText(hSearch) == strName)
			return hSearch;
					
		hSearch = m_ctlTestRunTree.GetNextItem(hSearch,TVGN_NEXT);
	}
	return NULL;
}

void CTestRunBar::OnContextMenu(CWnd* pWnd, CPoint point) 
{
	//This block was added by the Pop-up Menu component
    if (point.x == -1 && point.y == -1)
    {
        //keystroke invocation
        CRect rect;
        GetClientRect(rect);
        ClientToScreen(rect);

        point = rect.TopLeft();
        point.Offset(5, 5);
    }

	CPoint ptTree(point);
	m_ctlTestRunTree.ScreenToClient(&ptTree);

	UINT uiFlags;
	HTREEITEM hHitItem = m_ctlTestRunTree.HitTest(ptTree,&uiFlags);
	if(!hHitItem || !(uiFlags & TVHT_ONITEM))
		return;

	GraphType *pGraph = (GraphType*)m_ctlTestRunTree.GetItemData(hHitItem);
	if(!pGraph)
		return;

    CMenu menu;
    VERIFY(menu.LoadMenu(IDR_TESTRUN_POPUP));

    CMenu* pPopup = menu.GetSubMenu(0);

	for(int n=1;n<=pGraph->GI.arstrMenuItems.GetSize();n++)
		pPopup->InsertMenu(n,MF_BYPOSITION,n,pGraph->GI.arstrMenuItems[n-1]);
	pPopup->RemoveMenu(0,MF_BYPOSITION);

	ASSERT(pPopup != NULL);

	BOOL bResult = TrackPopupMenuEx(pPopup->m_hMenu,TPM_LEFTALIGN|TPM_TOPALIGN|TPM_NONOTIFY|TPM_RETURNCMD, 
		point.x, point.y,m_hWnd,NULL);

	if(!bResult)
		return;
	
	//Perform menu command
	HTREEITEM hParent = hHitItem;
	while(1)
	{
		HTREEITEM hPrev = hParent;
		hParent = m_ctlTestRunTree.GetParentItem(hParent);
		if(!hParent)
		{
			hParent = hPrev;
			break;
		}
	}

	CString strRun = m_strTestFolder + m_ctlTestRunTree.GetItemText(hParent)+"\\";
		
	CTestResultsPlugin plug(pGraph->strPlugin, m_pPluginManager);
	plug.PerformMenuCommand(strRun,pGraph->GI.strName,pGraph->GI.arstrMenuItems[bResult-1]);

	Refresh();
}
