/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

////////////////////////////////////////////////////////////////////////////////
// Statistics implementation file.
//
// Return values: unless otherwise stated, a DWORD return value is a
// GetLastError() error code.
////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Stats.hpp"
#include <iomanip>

// Link the psapi.lib module.
#pragma comment(lib, "psapi.lib")

using namespace std;

///////////////////////////////// CTimeStats ///////////////////////////////

void CTimeStats::StartSnapshot()
{
    FILETIME ftDummy;
    GetSystemTime(&m_stTimerStart);
    GetProcessTimes(GetCurrentProcess(), &ftDummy, &ftDummy, &m_ftKernelTimeStart, &m_ftUserTimeStart);
}

void CTimeStats::EndSnapshot()
{
    FILETIME ftDummy, ftKernelTimeEnd, ftUserTimeEnd;
    SYSTEMTIME stTimerEnd;
    GetProcessTimes(GetCurrentProcess(), &ftDummy, &ftDummy, &ftKernelTimeEnd, &ftUserTimeEnd);
    GetSystemTime(&stTimerEnd);

    //// Process the results.

    // Global duration.
    FILETIME ftStart, ftGlobalDuration;
    SystemTimeToFileTime(&m_stTimerStart, &ftStart);
    SystemTimeToFileTime(&stTimerEnd, &ftGlobalDuration);
    __int64 qwGlobal = FileTimeToQuadWord(&ftGlobalDuration) - FileTimeToQuadWord(&ftStart);
    QuadWordToFileTime(qwGlobal, &ftGlobalDuration);
    FileTimeToSystemTime(&ftGlobalDuration, &m_stGlobalDuration);

    // Other times.
    __int64 qwKernelTimeElapsed, qwUserTimeElapsed, qwTotalTimeElapsed;
    qwKernelTimeElapsed = FileTimeToQuadWord(&ftKernelTimeEnd) - FileTimeToQuadWord(&m_ftKernelTimeStart);
    qwUserTimeElapsed   = FileTimeToQuadWord(&ftUserTimeEnd)   - FileTimeToQuadWord(&m_ftUserTimeStart);
    qwTotalTimeElapsed  = qwKernelTimeElapsed + qwUserTimeElapsed;
    FILETIME ftKernelElapsed, ftUserElapsed, ftTotalElapsed;
    QuadWordToFileTime(qwKernelTimeElapsed, &ftKernelElapsed);
    QuadWordToFileTime(qwUserTimeElapsed, &ftUserElapsed);
    QuadWordToFileTime(qwTotalTimeElapsed, &ftTotalElapsed);
    FileTimeToSystemTime(&ftKernelElapsed, &m_stKernelTime);
    FileTimeToSystemTime(&ftUserElapsed,   &m_stUserTime);
    FileTimeToSystemTime(&ftTotalElapsed,  &m_stTotalTime);

    // Copy the second snapshot to the first.
    m_stTimerStart = stTimerEnd;
    m_ftUserTimeStart = ftUserTimeEnd;
    m_ftKernelTimeStart = ftKernelTimeEnd;
}

ostream& operator<<(ostream& os, const CTimeStats& ts)
{
    os
        << setfill('0')

        << "Global duration:                  "
        << setw(2) << ts.m_stGlobalDuration.wHour   << ":"
        << setw(2) << ts.m_stGlobalDuration.wMinute << ":"
        << setw(2) << ts.m_stGlobalDuration.wSecond << "."
        << setw(3) << ts.m_stGlobalDuration.wMilliseconds
        << "\r\n"

        << "User mode cpu time:               "
        << setw(2) << ts.m_stUserTime.wHour   << ":"
        << setw(2) << ts.m_stUserTime.wMinute << ":"
        << setw(2) << ts.m_stUserTime.wSecond << "."
        << setw(3) << ts.m_stUserTime.wMilliseconds
        << "\r\n"

        << "Kernel mode cpu time:             "
        << setw(2) << ts.m_stKernelTime.wHour   << ":"
        << setw(2) << ts.m_stKernelTime.wMinute << ":"
        << setw(2) << ts.m_stKernelTime.wSecond << "."
        << setw(3) << ts.m_stKernelTime.wMilliseconds
        << "\r\n"

        << "Total cpu time (user + kernel):   "
        << setw(2) << ts.m_stTotalTime.wHour   << ":"
        << setw(2) << ts.m_stTotalTime.wMinute << ":"
        << setw(2) << ts.m_stTotalTime.wSecond << "."
        << setw(3) << ts.m_stTotalTime.wMilliseconds
        << "\r\n"

        << flush;

    return os;
}


///////////////////////////////// CMemoryStats ///////////////////////////////

void CMemoryStats::Snapshot()
{
    GetProcessMemoryInfo(GetCurrentProcess(), &m_pmc, sizeof(m_pmc));
}

ostream& operator<<(ostream& os, const CMemoryStats& ms)
{
    os
        << setfill(' ')
        << "Working set size (bytes):         " << setw(9) << ms.GetWorkingSetSize() 
        << "\r\n"

        << "Peak working set size (bytes):    " << setw(9) << ms.GetPeakWorkingSetSize()
        << "\r\n"

        << "Peak page file usage (bytes):     " << setw(9) << ms.GetPeakPageFileUsage()
        << "\r\n"

        << "Peak paged pool usage (bytes):    " << setw(9) << ms.GetPeakPagedPoolUsage()
        << "\r\n"

        << "Peak nonpaged pool usage (bytes): " << setw(9) << ms.GetPeakNonPagedPoolUsage()
        << "\r\n"

        << flush;

    return os;
}
