/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

//==================================================================< Pragmas >
#ifdef  WIN32
#  pragma warning(disable : 4710)
#endif  // WIN32


//=================================================================< Includes >
#ifndef __CYRTOOLS_TIME__
#  include <ctime.hpp>
#endif  // __CYRTOOLS_TIME__

#ifndef __GATEWAY_LISTEN_CLIENT__
#  include "gatewlc.hpp"
#endif  // __GATEWAY_LISTEN_CLIENT__

//====================================================================< Class >

///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////// Class CGGatewayClient //
///////////////////////////////////////////////////////////////////////////////

//============================================< Constructor(s) and destructor >
//-----------------------------------------------------------------------------
// CGGatewayClient::CGGatewayClient
//
// Constructor.
//
// Arguments : task : task associated.
// Return    : none.
//-----------------------------------------------------------------------------
CGGatewayClient::CGGatewayClient (CGTask& task)
        : task_ (task), gatewayInfo_ ( (*(CGGatewayListen*) (task_.getInitInfo () ) ).getGatewayInfo () ), pClient_ (NULL), pTarget_ (NULL), pLibrary_ (NULL), newGatewayParse_ (NULL), pGWParse_ (NULL), iConnectionStep_ (STEP_CONNECTED), pC2TBuffer_ (NULL), pT2CBuffer_ (NULL), lMaxBufferSize_ (INITBUFFERSIZE), lC2TBufferSize_ (-1L), lT2CBufferSize_ (-1L)
{
    // Set the task data.
    task_.setData (this) ;

    // Save this pointer.
    CTCurrentThread::setData (this) ;

    // Set the abort routine.
    CTCurrentThread::setAbortRoutine (abort) ;

} // CGGatewayClient::CGGatewayClient

//-----------------------------------------------------------------------------
// CGGatewayClient::~CGGatewayClient
//
// Destructor.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------
CGGatewayClient::~CGGatewayClient ()
{
    if (pClient_)
    {
        if (GpProcessMain->getTraceMode () )
        {
            CTLockGuard<CTMutex>    guard (GpProcessMain->getLogFileList ().getMutex () ) ;
            GpProcessMain->getLogFileList () << task_.getTaskName () << ": <" << pClient_->getPeerInetAddress ().getHostAddress () << ":" << pClient_->getPeerPort () << "> disconnected." << endl ;
        }
        else
        {
            CTLockGuard<CTMutex>    guard (GpProcessMain->getLogFileList ().getMutex () ) ;
            GpProcessMain->getLogFileList () << "Disconnection from <" << pClient_->getPeerInetAddress ().getHostAddress () << ":" << pClient_->getPeerPort () << ">." << endl ;
        }

        pClient_->close () ;
        delete pClient_ ;
        pClient_ = NULL ;
    }
    if (pTarget_)
    {
        pTarget_->close () ;
        delete pTarget_ ;
        pTarget_ = NULL ;
    }

    if (pC2TBuffer_)
    {
        delete [] pC2TBuffer_ ;
        pC2TBuffer_ = NULL ;
    }
    if (pT2CBuffer_)
    {
        delete [] pT2CBuffer_ ;
        pT2CBuffer_ = NULL ;
    }

    if (pGWParse_)
    {
        delete pGWParse_ ;
        pGWParse_ = NULL ;
    }

    if (pLibrary_)
    {   // Unload library.
        delete pLibrary_ ;
        pLibrary_ = NULL ;
    }
} // CGGatewayClient::~CGGatewayClient

//=========================================================< Control methodes >
//-----------------------------------------------------------------------------
// CGGatewayClient::init
//
// Initialize the connection.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------
void CGGatewayClient::init (void)
{
    // accept the connection.
    pClient_ = new CTSocket ( (*(CGGatewayListen*) (task_.getInitInfo () ) ).getSocket ().accept () ) ;
    if (pClient_ == NULL)
    {   // Error from new.
        throw CGTaskException ("init: new failed.", __FILE__, __LINE__) ;
    }

    // Connect to the targer server now or not ?
    if (gatewayInfo_.getTargetAddressAutoDetectionMode () )
    {   // Don't connect to the targer server, just create the socket.
        pTarget_ = new CTSocket ;
        if (pTarget_ == NULL)
        {   // Error from new.
            throw CGTaskException ("init: new failed.", __FILE__, __LINE__) ;
        }
        ( (CTInetAddress&) (pTarget_->getPeerInetAddress () ) ) = CTInetAddress (gatewayInfo_.getTargetAddress ().substr (0, gatewayInfo_.getTargetAddress ().find_first_of (':') ) ) ;
    }
    else
    {   // Connect to the targer server now.
        if (GpProcessMain->getTraceMode () )
        {
            CTLockGuard<CTMutex>    guard (GpProcessMain->getLogFileList ().getMutex () ) ;
            GpProcessMain->getLogFileList () << task_.getTaskName () << ": try to connect to target <" << gatewayInfo_.getTargetAddress () << ">." << endl ;
        }

        string  host (gatewayInfo_.getTargetAddress ().substr (0, gatewayInfo_.getTargetAddress ().find_first_of (':') ) ) ;
        string  port (gatewayInfo_.getTargetAddress ().substr (gatewayInfo_.getTargetAddress ().find_first_of (':') + 1, gatewayInfo_.getTargetAddress ().size () - host.size () - 1) ) ;
        port_t      tPort = (port_t) 0 ;

        sscanf (port.c_str (), "%hu", &tPort) ;
        if (tPort == 0)
        {
            tPort = CTInetAddress::getPortByName (port) ;
        }

        pTarget_ = new CTSocket (host, tPort) ;
        if (pTarget_ == NULL)
        {   // Error from new.
            throw CGTaskException ("init: new failed.", __FILE__, __LINE__) ;
        }
    }

    // Get the library.
    string  libraryName ;

    if (gatewayInfo_.getLibrary () != "")
    {
        libraryName = gatewayInfo_.getLibrary () ;
        if (GpProcessMain->getTraceMode () )
        {
            CTLockGuard<CTMutex>    guard (GpProcessMain->getLogFileList ().getMutex () ) ;
            GpProcessMain->getLogFileList () << task_.getTaskName () << ": try to load the library '" << CTLibrary::getSystemFileName (CTLibrary::getBaseFileName (libraryName) ) << "'." << endl ;
        }
        try
        {
            pLibrary_ = new CTLibrary (libraryName) ;
        }
        catch (CTxalloc&)
        {   // Error from new.
            throw CGTaskException (string ("init: fail to load the library '") + CTLibrary::getSystemFileName (CTLibrary::getBaseFileName (libraryName) ) + "': new CTLibrary (libraryName) failed.", __FILE__, __LINE__) ;
        }
        catch (CTException& e)
        {
            throw CGTaskException (string ("init: fail to load the library '") + CTLibrary::getSystemFileName (CTLibrary::getBaseFileName (libraryName) ) + "'.\n" + e.what (), __FILE__, __LINE__) ;
        }

        // Determinate if there is a new GWParse function.
        CTProcedure procNewGatewayParse (*pLibrary_, NEWGATEWAYPARSE) ;
        newGatewayParse_ = (CGWParse::fnNewGatewayParse) (procNewGatewayParse.getAddress () ) ;

        // Create a parse structure.
        newGatewayParse_ (CTCurrentThread::getThreadId (), task_.getTaskName (), GpProcessMain->getTraceMode (), GpProcessMain->getLogFileList (), pClient_, pTarget_, pGWParse_) ;

        // Determinate if we are in the pre-connection step.
        iConnectionStep_ = (int) (pGWParse_->getPreconnectionMode () ? STEP_PRECONNECTION : STEP_CONNECTED) ;
    }

    if (GpProcessMain->getTraceMode () )
    {
        CTLockGuard<CTMutex>    guard (GpProcessMain->getLogFileList ().getMutex () ) ;
        char szAddress1 [22] ;
        char szAddress2 [12] ;
        char szAddress3 [22] ;
        char szLine [80] ;
        if (pLibrary_)
        {
            sprintf (szLine, "<application client    -> '%-9.9s' -> target listener      >", CTLibrary::getSystemFileName (CTLibrary::getBaseFileName (libraryName) ).c_str () ) ;
        }
        else
        {
            sprintf (szLine, "<application client    -> gateway     -> target listener      >") ;
        }
        GpProcessMain->getLogFileList () << task_.getTaskName () << ": " << szLine << endl ;
        sprintf (szAddress1, "%s:%hu", pClient_->getPeerInetAddress ().getHostAddress ().c_str (), pClient_->getPeerPort () ) ;
        sprintf (szAddress2, "%hu/%hu", pClient_->getPort (), pTarget_->getPort () ) ;
        sprintf (szAddress3, "%s:%hu", pTarget_->getPeerInetAddress ().getHostAddress ().c_str (), pTarget_->getPeerPort () ) ;
        sprintf (szLine, "<%-21.21s -> %-11.11s -> %-21.21s>", szAddress1, szAddress2, szAddress3) ;
        GpProcessMain->getLogFileList () << task_.getTaskName () << ": " << szLine << endl ;
        if (gatewayInfo_.getTargetAddressAutoDetectionMode () )
        {
            GpProcessMain->getLogFileList () << task_.getTaskName () << ": target address auto detect mode is actif." << endl ;
        }
    }
    else
    {
        CTLockGuard<CTMutex>    guard (GpProcessMain->getLogFileList ().getMutex () ) ;
        GpProcessMain->getLogFileList () << "Connection from <" << pClient_->getPeerInetAddress ().getHostAddress () << ":" << pClient_->getPeerPort () << "> to " << (char*) (gatewayInfo_.getTargetAddressAutoDetectionMode () ? "default " : "") << "<" << pTarget_->getPeerInetAddress ().getHostAddress () << ":" << pTarget_->getPeerPort () << "> using " << CTLibrary::getSystemFileName(CTLibrary::getBaseFileName (libraryName) ) << "." << endl ;
    }

    // Allocate the communication buffers.
    pC2TBuffer_ = new char [2 * lMaxBufferSize_] ;
    if (pC2TBuffer_ == NULL)
    {   // Error from new.
        throw CGTaskException ("init: new failed.", __FILE__, __LINE__) ;
    }

    pT2CBuffer_ = new char [2 * lMaxBufferSize_] ;
    if (pT2CBuffer_ == NULL)
    {   // Error from new.
        throw CGTaskException ("init: new failed.", __FILE__, __LINE__) ;
    }

    if (iConnectionStep_ == STEP_CONNECTED)
    {   // Tell the parent task that initialization is terminated.
        task_.initializationTerminated () ;
    }
} // CGGatewayClient::init

//-----------------------------------------------------------------------------
// CGGatewayClient::processConnection
//
// Process connection.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------
void CGGatewayClient::processConnection (void)
{
    try
    {   // Process connection.
        CTSocketSelectorList    inList ;
        CTSocketSelectorList    outList ;
        timeval                 timeOut = {0, 0} ;
        int                     iTypeOfEvent = 0 ;
        bool                    bC2T ;
        bool                    bT2C ;
        bool                    bUrgentC2T ;
        bool                    bUrgentT2C ;
        string              newTargetListenAddress ;
        bool                    bGetNewMaxBufferSize = false ;
        long                    lNewMaxBufferSize = -1L ;
        CTSocket                newGatewayListen ;
        bool                    bNotConnectedToTarget = gatewayInfo_.getTargetAddressAutoDetectionMode () ;
        CTTime                  receiveTime ;
        CTTime                  sendTime ;

        // Add sockets to the selector.
        inList.reset () ;
        inList.add (*pClient_, _S_RECEIVE_ | _S_EXCEPTION_) ;
        inList.add (*pTarget_, _S_RECEIVE_ | _S_EXCEPTION_) ;

        // Set the polling waiting mode.
        inList.setTimeOut (&timeOut) ;

        do
        {
            do
            {
                // Initialize the flags.
                bC2T = bT2C = bUrgentC2T = bUrgentT2C = false ;
                iTypeOfEvent = 0 ;

                while (!task_.hasToBeReinitialized () && (!inList (outList, iTypeOfEvent) ) )
                {
                    CTCurrentThread::sleep (1) ;
                }

                if (pLibrary_)
                {   // Get the receive time.
                    receiveTime.now () ;
                }

                if (task_.hasToBeReinitialized () )
                {
                    task_.reinitialized () ;
                }

                // Select the socket(s) for receive.
                if (inList (outList, iTypeOfEvent) )
                {
                    int iTypeOfEventC2T ;
                    if (outList.contains (*pClient_, iTypeOfEventC2T) )
                    {
                        if (iTypeOfEventC2T & _S_RECEIVE_)
                        {
                            bC2T = true ;
                        }
                        if (iTypeOfEventC2T & _S_EXCEPTION_)
                        {
                            if (bC2T)
                            {
                                bUrgentC2T = true ;
                            }
                            else
                            {
                                throw CGTaskException ("processConnection: client socket exception.", __FILE__, __LINE__) ;
                            }
                        }
                    }
                    int iTypeOfEventT2C ;
                    if (outList.contains (*pTarget_, iTypeOfEventT2C) )
                    {
                        if (iTypeOfEventT2C & _S_RECEIVE_)
                        {
                            bT2C = true ;
                        }
                        else if (iTypeOfEventT2C & _S_EXCEPTION_)
                        {
                            if (bT2C)
                            {
                                bUrgentT2C = true ;
                            }
                            else
                            {
                                throw CGTaskException ("processConnection: target socket exception.", __FILE__, __LINE__) ;
                            }
                        }
                    }
                }


                if (bC2T)
                {   // Receive from client to target server.
                    lC2TBufferSize_ = pClient_->recvAvailableData (pC2TBuffer_, lMaxBufferSize_, bUrgentC2T) ;
                }
                else
                {
                    lC2TBufferSize_ = -1L ;
                }

                if (bT2C)
                {   // Receive from target server to client.
                    lT2CBufferSize_ = pTarget_->recvAvailableData (pT2CBuffer_, lMaxBufferSize_, bUrgentT2C) ;
                }
                else
                {
                    lT2CBufferSize_ = -1L ;
                }

                if (pLibrary_)
                {   // Pass buffers to the library.
                    if ( (bC2T) && (lC2TBufferSize_) )
                    {   // Parse buffers from client to target server.
                        long    lOffset = 0L ;
                        long    lMoreData = 0L ;
                        do
                        {
                            lOffset += pGWParse_->parseBuffer (FROM_CLIENT, bUrgentC2T, pC2TBuffer_, lC2TBufferSize_, lOffset, lMaxBufferSize_, lMoreData) ;
                            if (lMoreData > 0)
                            {
                                lC2TBufferSize_ += pClient_->recvAvailableData ( (void*) ( (char*) pC2TBuffer_ + lC2TBufferSize_), lMoreData, bUrgentC2T) ;
                                lMoreData = 0L ;
                            }
                            else if (lOffset < lC2TBufferSize_)
                            {   // Get the send time.
                                sendTime.now () ;

                                // Specify the receive time and the send time to the library.
                                pGWParse_->setReceiveAndSendTimes (receiveTime, sendTime) ;

                                // Get the receive time.
                                receiveTime = sendTime ;
                            }
                        }
                        while (lOffset < lC2TBufferSize_) ;
                        bC2T = (lC2TBufferSize_ != 0) ;
                        if (bC2T == false)
                        {
                            lC2TBufferSize_ = -1 ;
                        }
                    }
                    if ( (bT2C) && (lT2CBufferSize_) )
                    {   // Parse buffers from target server to client.
                        long    lOffset = 0L ;
                        long    lMoreData = 0L ;
                        do
                        {
                            lOffset += pGWParse_->parseBuffer (FROM_TARGET, bUrgentT2C, pT2CBuffer_, lT2CBufferSize_, lOffset, lMaxBufferSize_, lMoreData) ;
                            if (lMoreData > 0)
                            {
                                lT2CBufferSize_ += pTarget_->recvAvailableData ( (void*) ( (char*) pT2CBuffer_ + lT2CBufferSize_), lMoreData, bUrgentT2C) ;
                                lMoreData = 0L ;
                            }
                            else
                            {
                                if (lOffset < lT2CBufferSize_)
                                {   // Get the send time.
                                    sendTime.now () ;

                                    // Specify the receive time and the send time to the library.
                                    pGWParse_->setReceiveAndSendTimes (receiveTime, sendTime) ;

                                    // Get the receive time.
                                    receiveTime = sendTime ;
                                }
                                if (pGWParse_->getPreconnectionMode () )
                                {
                                    switch (iConnectionStep_)
                                    {
                                    case STEP_PRECONNECTION :
                                        if (pGWParse_->isConnected () )
                                        {
                                            iConnectionStep_ = STEP_CONNECTED ;

                                            // Tell the parent task that initialization is terminated.
                                            task_.initializationTerminated () ;
                                        }
                                        else if (newTargetListenAddress == "connection refused")
                                        {
                                            iConnectionStep_ = STEP_DISCONNECTED ;

                                            // Tell the parent task that initialization is terminated.
                                            task_.initializationTerminated () ;
                                        }
                                        else if (pGWParse_->getTargetListenAddress (newTargetListenAddress) )
                                        {
                                            if (newTargetListenAddress == "no preconnection phase")
                                            {
                                                iConnectionStep_ = STEP_CONNECTED ;

                                                // Tell the parent task that initialization is terminated.
                                                task_.initializationTerminated () ;
                                            }
                                            else
                                            {   // Create the new target socket.
                                                CTInetAddress   address (CTInetAddress::getLocalHostName () ) ;
                                                newGatewayListen.bindAnonymously (address) ;
                                                newGatewayListen.listen () ;

                                                // Change the target address in the packet.
                                                char zBuf [6] ;
                                                sprintf (zBuf, "%hu", newGatewayListen.getPort () ) ;
                                                string  newGatewayListenAddress = ( (CTInetAddress&) newGatewayListen.getInetAddress () ).getHostAddress () + ":" + zBuf ;
                                                long        lOldT2CBufferSize = lT2CBufferSize_ ;
                                                pGWParse_->setGatewayListenAddress (newGatewayListenAddress, pT2CBuffer_, lT2CBufferSize_, lMaxBufferSize_) ;
                                                lOffset += lT2CBufferSize_ - lOldT2CBufferSize ;
                                            }
                                        }
                                        break ;
                                    case STEP_CONNECTED :
                                        if (lNewMaxBufferSize == -1L)
                                        {
                                            lNewMaxBufferSize = pGWParse_->getNewMaxBufferSize () ;
                                        }
                                        break ;
                                    }
                                }
                            }
                        }
                        while (lOffset < lT2CBufferSize_) ;
                        bT2C = (lT2CBufferSize_ != 0) ;
                        if (bT2C == false)
                        {
                            lT2CBufferSize_ = -1 ;
                        }
                    }
                }

                // Connect to target if necessary.
                if (bNotConnectedToTarget && lC2TBufferSize_ != -1)
                {   // Not connected, so do it.
                    string  firstTargetListenAddress ;

                    if ( (pLibrary_) && (pGWParse_->getPreconnectionMode () ) )
                    {
                        pGWParse_->getTargetListenAddress (firstTargetListenAddress) ;
                    }

                    if (firstTargetListenAddress.empty () )
                    {
                        firstTargetListenAddress = gatewayInfo_.getTargetAddress () ;
                    }
                    if (firstTargetListenAddress.empty () )
                    {
                        if (pTarget_->getPeerPort() == 0)
                            throw CGTaskException ("processConnection: to target address found !", __FILE__, __LINE__) ;
                    }
                    else
                    {
                        size_t      sIndex = firstTargetListenAddress.find_first_of (':') ;
                        string  host (firstTargetListenAddress.substr (0, sIndex) ) ;
                        string  port (firstTargetListenAddress.substr (sIndex + 1, firstTargetListenAddress.length () - sIndex - 1) ) ;

                        if (GpProcessMain->getTraceMode () )
                        {
                            CTLockGuard<CTMutex>    guard (GpProcessMain->getLogFileList ().getMutex () ) ;
                            GpProcessMain->getLogFileList () << task_.getTaskName () << ": try to connect to target <" << host << "> on port " << port << "." << endl ;
                        }

                        port_t      tPort = 0 ;
                        sscanf (port.c_str (), "%hu", &tPort) ;
                        if (tPort == 0)
                        {
                            tPort = CTInetAddress::getPortByName (port) ;
                        }

                        pTarget_->connect (host, tPort) ;
                        bNotConnectedToTarget = false ;

                        if (GpProcessMain->getTraceMode () )
                        {
                            CTLockGuard<CTMutex>    guard (GpProcessMain->getLogFileList ().getMutex () ) ;
                            char szAddress1 [22] ;
                            char szAddress2 [12] ;
                            char szAddress3 [22] ;
                            char szLine [80] ;
                            if (pLibrary_)
                            {
                                sprintf (szLine, "<application client    -> '%-9.9s' -> target listener      >", CTLibrary::getSystemFileName (CTLibrary::getBaseFileName (pLibrary_->getName () ) ).c_str () ) ;
                            }
                            else
                            {
                                sprintf (szLine, "<application client    -> gateway     -> target listener      >") ;
                            }
                            GpProcessMain->getLogFileList () << task_.getTaskName () << ": " << szLine << endl ;
                            sprintf (szAddress1, "%s:%hu", pClient_->getPeerInetAddress ().getHostAddress ().c_str (), pClient_->getPeerPort () ) ;
                            sprintf (szAddress2, "%hu/%hu", pClient_->getPort (), pTarget_->getPort () ) ;
                            sprintf (szAddress3, "%s:%hu", pTarget_->getPeerInetAddress ().getHostAddress ().c_str (), pTarget_->getPeerPort () ) ;
                            sprintf (szLine, "<%-21.21s -> %-11.11s -> %-21.21s>", szAddress1, szAddress2, szAddress3) ;
                            GpProcessMain->getLogFileList () << task_.getTaskName () << ": " << szLine << endl ;
                        }
                        else
                        {
                            CTLockGuard<CTMutex>    guard (GpProcessMain->getLogFileList ().getMutex () ) ;
                            GpProcessMain->getLogFileList () << "Connection from <" << pClient_->getPeerInetAddress ().getHostAddress () << ":" << pClient_->getPeerPort () << "> to <" << pTarget_->getPeerInetAddress ().getHostAddress () << ":" << pTarget_->getPeerPort () << "> using " << pLibrary_->getName () << "." << endl ;
                        }
                    }
                }

                // Send buffers from each other.
                if (bC2T)
                {   // Send to target server from client.
                    pTarget_->sendBuffer (pC2TBuffer_, lC2TBufferSize_, "", bUrgentC2T) ;
                }
                if (bT2C)
                {   // Send to client from target server.
                    pClient_->sendBuffer (pT2CBuffer_, lT2CBufferSize_, "", bUrgentT2C) ;
                }

                if (pLibrary_)
                {   // Get the send time.
                    sendTime.now () ;

                    // Specify the receive time and the send time to the library.
                    pGWParse_->setReceiveAndSendTimes (receiveTime, sendTime) ;
                }

                if ( (pLibrary_) && (pGWParse_->getPreconnectionMode () ) && (iConnectionStep_ == STEP_CONNECTED) && (lNewMaxBufferSize != -1L) && (!bGetNewMaxBufferSize) )
                {   // Change the connection buffer size.
                    if (GpProcessMain->getTraceMode () )
                    {
                        CTLockGuard<CTMutex>    guard (GpProcessMain->getLogFileList ().getMutex () ) ;
                        GpProcessMain->getLogFileList () << task_.getTaskName () << ": new connection buffer size :" << lNewMaxBufferSize << "." << endl ;
                    }

                    lMaxBufferSize_ = (long) ( (lNewMaxBufferSize < MAXBUFFERSIZE) ? lNewMaxBufferSize : MAXBUFFERSIZE) ;

                    // Reallocate the communication buffers.
                    delete [] pC2TBuffer_ ;
                    pC2TBuffer_ = new char [2 * lMaxBufferSize_] ;
                    if (pC2TBuffer_ == NULL)
                    {   // Error from new.
                        throw CGTaskException ("processConnection: new failed.", __FILE__, __LINE__) ;
                    }

                    delete [] pT2CBuffer_ ;
                    pT2CBuffer_ = new char [2 * lMaxBufferSize_] ;
                    if (pT2CBuffer_ == NULL)
                    {   // Error from new.
                        throw CGTaskException ("processConnection: new failed.", __FILE__, __LINE__) ;
                    }

                    bGetNewMaxBufferSize = true ;
                }
            }
            while ( (lC2TBufferSize_ != 0L) && (lT2CBufferSize_ != 0L) ) ;

            if ( (pLibrary_) && (pGWParse_->getPreconnectionMode () ) && (iConnectionStep_ == STEP_PRECONNECTION) )
            {   // Change the sockets if it is a pre-connection step.

                // Reset the sockets list.
                inList.reset () ;

                // Set the polling waiting mode.
                inList.setTimeOut (&timeOut) ;

                // Close the sockets.
                pClient_->close () ;
                delete pClient_ ;
                pClient_ = NULL ;

                pTarget_->close () ;
                delete pTarget_ ;
                pTarget_ = NULL ;

                // Accept the new client connection.
                pClient_ = new CTSocket (newGatewayListen.accept () ) ;
                if (pClient_ == NULL)
                {   // Error from new.
                    throw CGTaskException ("processConnection: new failed.", __FILE__, __LINE__) ;
                }

                // Connect to the targer server.
                size_t      sIndex = newTargetListenAddress.find_first_of (":") ;
                string  host (newTargetListenAddress.substr (0, sIndex) ) ;
                string  port (newTargetListenAddress.substr (sIndex + 1, newTargetListenAddress.length () - sIndex - 1) ) ;

                if (GpProcessMain->getTraceMode () )
                {
                    CTLockGuard<CTMutex>    guard (GpProcessMain->getLogFileList ().getMutex () ) ;
                    GpProcessMain->getLogFileList () << task_.getTaskName () << ": try to reconnect to target <" << host << "> on port " << port << "." << endl ;
                }

                port_t      tPort = 0 ;
                sscanf (port.c_str (), "%hu", &tPort) ;
                if (tPort == 0)
                {
                    tPort = CTInetAddress::getPortByName (port) ;
                }

                pTarget_ = new CTSocket (host, tPort) ;
                if (pTarget_ == NULL)
                {   // Error from new.
                    throw CGTaskException ("processConnection: new failed.", __FILE__, __LINE__) ;
                }
                if (GpProcessMain->getTraceMode () )
                {
                    CTLockGuard<CTMutex>    guard (GpProcessMain->getLogFileList ().getMutex () ) ;
                    char szAddress1 [22] ;
                    char szAddress2 [12] ;
                    char szAddress3 [22] ;
                    char szLine [80] ;
                    sprintf (szLine, "<application client    -> reconnected -> target listener      >") ;
                    GpProcessMain->getLogFileList () << task_.getTaskName () << ": " << szLine << endl ;
                    sprintf (szAddress1, "%s:%hu", pClient_->getPeerInetAddress ().getHostAddress ().c_str (), pClient_->getPeerPort () ) ;
                    sprintf (szAddress2, "%hu/%hu", pClient_->getPort (), pTarget_->getPort () ) ;
                    sprintf (szAddress3, "%s:%hu", pTarget_->getPeerInetAddress ().getHostAddress ().c_str (), pTarget_->getPeerPort () ) ;
                    sprintf (szLine, "<%-21.21s -> %-11.11s -> %-21.21s>", szAddress1, szAddress2, szAddress3) ;
                    GpProcessMain->getLogFileList () << task_.getTaskName () << ": " << szLine << endl ;
                }
                else
                {
                    CTLockGuard<CTMutex>    guard (GpProcessMain->getLogFileList ().getMutex () ) ;
                    GpProcessMain->getLogFileList () << "Connection redirected : from <" << pClient_->getPeerInetAddress ().getHostAddress () << ":" << pClient_->getPeerPort () << "> to <" << pTarget_->getPeerInetAddress ().getHostAddress () << ":" << pTarget_->getPeerPort () << ">." << endl ;
                }

                // Add sockets to the selector.
                inList.add (*pClient_, _S_RECEIVE_ | _S_EXCEPTION_) ;
                inList.add (*pTarget_, _S_RECEIVE_ | _S_EXCEPTION_) ;
            }
            else
            {
                iConnectionStep_ = STEP_DISCONNECTED ;
            }
        }
        while (iConnectionStep_ != STEP_DISCONNECTED) ;
    }
    catch (CTSocketException& e)
    {
#ifdef  WIN32
        if ( (e.getNumber () != WSAEINTR) && (e.getNumber () != WSAECONNRESET) )
#endif  // WIN32
#ifdef  UNIX
            if ( (e.getNumber () != EINTR) && (e.getNumber () != ECONNRESET) )
#endif  // UNIX
            {
                throw e ;
            }
    }
    catch (CTException& e)
    {
        throw CTException (e) ;
    }
} // CGGatewayClient::processConnection

//-----------------------------------------------------------------------------
// CGGatewayClient::abort
//
// Abort Routine.
//
// Arguments : pStatus : pointer to the exit status.
// Return    : none.
//-----------------------------------------------------------------------------
void CGGatewayClient::abort (void*)
{
    // Delete the CGGatewayClient pointer.
    ( (CGGatewayClient*) CTCurrentThread::getData () )->~CGGatewayClient () ;
} // CGGatewayClient::abort

///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////// End of Class CGGatewayClient //
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////// Globals Functions //
///////////////////////////////////////////////////////////////////////////////

void gatewayClientTask (CGTask& task)
{
    try
    {
        if (GpProcessMain->getTraceMode () )
        {
            CTLockGuard<CTMutex>    guard (GpProcessMain->getLogFileList ().getMutex () ) ;
            GpProcessMain->getLogFileList () << task.getTaskName () << ": new gateway client task." << endl ;
        }

        // Destruction of this object must occur before killing the task.
        CGGatewayClient gatewayClient (task) ;
        gatewayClient.init () ;
        gatewayClient.processConnection () ;

        if (GpProcessMain->getTraceMode () )
        {
            CTLockGuard<CTMutex>    guard (GpProcessMain->getLogFileList ().getMutex () ) ;
            GpProcessMain->getLogFileList () << task.getTaskName () << ": gateway client task terminated." << endl ;
        }
    }
    catch (CTException& e)
    {
        if (GpProcessMain->getTraceMode () )
        {
            CTLockGuard<CTMutex>    guard (GpProcessMain->getLogFileList ().getMutex () ) ;
            GpProcessMain->getLogFileList () << task.getTaskName () << ": !! " << e.what () << endl ;
            GpProcessMain->getLogFileList () << task.getTaskName () << ": gateway client task terminated." << endl ;
        }
        else
        {
            CTLockGuard<CTMutex>    guard (GpProcessMain->getLogFileList ().getMutex () ) ;
            GpProcessMain->getLogFileList () << "!! " << e.what () << endl ;
        }
    }
    catch (...)
    {
        if (GpProcessMain->getTraceMode () )
        {
            CTLockGuard<CTMutex>    guard (GpProcessMain->getLogFileList ().getMutex () ) ;
            GpProcessMain->getLogFileList () << task.getTaskName () << ": !! unknown exception." << endl ;
            GpProcessMain->getLogFileList () << task.getTaskName () << ": gateway client task terminated." << endl ;
        }
        else
        {
            CTLockGuard<CTMutex>    guard (GpProcessMain->getLogFileList ().getMutex () ) ;
            GpProcessMain->getLogFileList () << "!! unknown exception." << endl ;
        }
    }
    if (task.initializationIsTerminated () )
    {   // Remove the task from the list.
        task.invalidate () ;
        for (int iIndex=0; iIndex<(int)task.getTasksList ().getTasksList ().size(); iIndex++)
        {
            if (task.getTasksList ().getTasksList ()[iIndex] == &task)
            {
                task.getTasksList ().killTask (iIndex) ;
                break ;
            }
        }
    }
    else
    {   // Tell the control task that initialization is terminated.
        task.initializationTerminated (false) ;
    }
} // gatewayClientTask


///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////// End of Globals Functions //
///////////////////////////////////////////////////////////////////////////////
