/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
#ifndef __GATEWAY_COMMAND__
#define __GATEWAY_COMMAND__


//==================================================================< Pragmas >
#ifdef  WIN32
#  pragma warning(disable : 4710)
#endif  // WIN32


//=================================================================< Includes >
#ifndef __DEFINITION__
#  include "definition.h"
#endif  // __DEFINITION__

#ifndef __EXCEPTION__
#  include <exception.hpp>
#endif  // __EXCEPTION__

#ifndef __MTSAFE__
#  include <mtsafe.hpp>
#endif  // __MTSAFE__

#ifndef __SOCKET__
#  include <socket.hpp>
#endif  // __SOCKET__

#include <list>
using std::list;
//===================================================================< Macros >
#ifndef DLLSPEC_GWCOMMAND
#  ifdef BUILDDLLGWORACLE
#    define DLLSPEC_GWCOMMAND __EXPORT__
#  else
#    define DLLSPEC_GWCOMMAND
#  endif    // BUILDDLLGWORACLE
#endif  // DLLSPEC_GWCOMMAND


//======================================================< Definition of class >

///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////// Class CGCmdException //
///////////////////////////////////////////////////////////////////////////////
// Class used to keep tasks exception.
class CGCmdException : public CTException
{
    //-----------------------------------------------------------------------------
public:                     // Constructor(s) and destructor.
    CGCmdException (const string& clMsg, const string& clFilename, short sLineno, long lNumber = -1L)
: CTException (clMsg, clFilename, sLineno, lNumber)
    { }


public:                     // Operators.
    void                    operator= (const CGCmdException& exception)
    {
        CTException::operator= (exception) ;
    }

    //-----------------------------------------------------------------------------
}

; // CGCmdException

///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////// End of Class CGCmdException //
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////// Class CGCmd //
///////////////////////////////////////////////////////////////////////////////
class DLLSPEC_GWCOMMAND CGCmd : public CTMTSafe
{
    //-----------------------------------------------------------------------------
public:                         // Types.
    enum enumCmd
    {
        CMD_NEW_LISTEN_TASK,
        CMD_KILL_TASK_ID,
        CMD_KILL_TASK_TYPE,
        CMD_KILL_TASK_DATA,
        CMD_SHUTDOWN,
        CMD_TRACE,
        CMD_WHO,
        CMD_QUIT,
        CMD_USER
    } ;

public:                         // Constructor(s) and destructor.
    CGCmd ()
: eCommand_ (CMD_USER), bResultOk_ (true)
    { }

    CGCmd (const enumCmd eCommand)
: eCommand_ (eCommand), bResultOk_ (true)
    { }

    CGCmd (const CGCmd& command)
: eCommand_ (command.eCommand_), bResultOk_ (command.bResultOk_)
    { }

    virtual ~CGCmd ()
    { }


public:                         // Operators.
    CGCmd&                      operator = (const CGCmd& command)
    {
        eCommand_ = command.eCommand_ ;
        return *this ;
    }
    bool                        operator == (const CGCmd& command)
    {
        return (eCommand_ == command.eCommand_) ;
    }

public:                         // Get methodes.
    enumCmd&                    getCommand (void)
    {
        return eCommand_ ;
    }
    bool&                       getResult (void)
    {
        return bResultOk_ ;
    }

    //-----------------------------------------------------------------------------
private:                        // Members.
    enumCmd                     eCommand_ ;
    bool                        bResultOk_ ;

    //-----------------------------------------------------------------------------
}

; // CGCmd

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////// End of Class CGCmd //
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////// Class CGCmdNewListenTask //
///////////////////////////////////////////////////////////////////////////////
class DLLSPEC_GWCOMMAND CGCmdNewListenTask : public CGCmd
{
    //-----------------------------------------------------------------------------
public:                         // Constructor(s) and destructor.
    CGCmdNewListenTask ()
: CGCmd (CMD_NEW_LISTEN_TASK), bAdministration_ (false), listenPort_ (0), bTargetAddressAutoDetection_ (true)
    { }

    CGCmdNewListenTask (const bool bAdministration, const string& targetAddress, const port_t listenPort, const string& library, const bool bTargetAddressAutoDetection)
: CGCmd (CMD_NEW_LISTEN_TASK), bAdministration_ (bAdministration), targetAddress_ (targetAddress), listenPort_ (listenPort), library_ (library), bTargetAddressAutoDetection_ (bTargetAddressAutoDetection || targetAddress.empty () )
    { }

    CGCmdNewListenTask (const CGCmdNewListenTask& newListenTask)
: CGCmd (CMD_NEW_LISTEN_TASK), bAdministration_ (newListenTask.bAdministration_), targetAddress_ (newListenTask.targetAddress_), listenPort_ (newListenTask.listenPort_), library_ (newListenTask.library_), bTargetAddressAutoDetection_ (newListenTask.bTargetAddressAutoDetection_)
    { }

    ~CGCmdNewListenTask ()
    { }


public:                         // Operators.
    CGCmdNewListenTask&         operator = (const CGCmdNewListenTask& newListenTask)
    {
        bAdministration_ = newListenTask.bAdministration_ ;
        targetAddress_ = newListenTask.targetAddress_ ;
        listenPort_ = newListenTask.listenPort_ ;
        library_ = newListenTask.library_ ;
        bTargetAddressAutoDetection_ = newListenTask.bTargetAddressAutoDetection_ ;
        return *this ;
    }
    bool                        operator == (const CGCmdNewListenTask& newListenTask)
    {
        return (listenPort_ == newListenTask.listenPort_) ;
    }

public:                         // Get methodes.
    bool&                       getIsAdministrationListen (void)
    {
        return bAdministration_ ;
    }
    string&                 getTargetAddress (void)
    {
        return targetAddress_ ;
    }
    port_t&                     getPort (void)
    {
        return listenPort_ ;
    }
    string&                 getLibrary (void)
    {
        return library_ ;
    }
    bool&                       getTargetAddressAutoDetectionMode (void)
    {
        return bTargetAddressAutoDetection_ ;
    }

    //-----------------------------------------------------------------------------
private:                        // Members.
    bool                        bAdministration_ ;
    string                  targetAddress_ ;
    port_t                      listenPort_ ;
    string                  library_ ;
    bool                        bTargetAddressAutoDetection_ ;

    //-----------------------------------------------------------------------------
}

; // CGCmdNewListenTask

///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////// End of Class CGCmdNewListenTask //
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////// Class CGCmdKillTaskId //
///////////////////////////////////////////////////////////////////////////////
class DLLSPEC_GWCOMMAND CGCmdKillTaskId : public CGCmd
{
    //-----------------------------------------------------------------------------
public:                         // Constructor(s) and destructor.
    CGCmdKillTaskId ()
: CGCmd (CMD_KILL_TASK_ID), lTaskId_ (0), bChildrenAlso_ (false)
    { }

    CGCmdKillTaskId (const long lTaskId, const bool bChildrenAlso = false)
: CGCmd (CMD_KILL_TASK_ID), lTaskId_ (lTaskId), bChildrenAlso_ (bChildrenAlso)
    { }

    CGCmdKillTaskId (const CGCmdKillTaskId& killTaskId)
: CGCmd (CMD_KILL_TASK_ID), lTaskId_ (killTaskId.lTaskId_), bChildrenAlso_ (killTaskId.bChildrenAlso_)
    { }

    ~CGCmdKillTaskId ()
    { }


public:                         // Operators.
    CGCmdKillTaskId&            operator = (const CGCmdKillTaskId& killTaskId)
    {
        lTaskId_ = killTaskId.lTaskId_ ;
        bChildrenAlso_ = killTaskId.bChildrenAlso_ ;
        return *this ;
    }
    bool                        operator == (const CGCmdKillTaskId& killTaskId)
    {
        return (lTaskId_ == killTaskId.lTaskId_) ;
    }

public:                         // Get methodes.
    long&                       getTaskId (void)
    {
        return lTaskId_ ;
    }
    bool&                       getChildrenAlso (void)
    {
        return bChildrenAlso_ ;
    }

    //-----------------------------------------------------------------------------
private:                        // Members.
    long                        lTaskId_ ;
    bool                        bChildrenAlso_ ;

    //-----------------------------------------------------------------------------
}

; // CGCmdKillTaskId

///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////// End of Class CGCmdKillTaskId //
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////// Class CGCmdKillTaskType //
///////////////////////////////////////////////////////////////////////////////
class DLLSPEC_GWCOMMAND CGCmdKillTaskType : public CGCmd
{
    //-----------------------------------------------------------------------------
public:                         // Types.
    enum enumKillType
    {
        KILL_CONTROL,
        KILL_ADMINISTRATION_LISTEN,
        KILL_ADMINISTRATION_CLIENT,
        KILL_GATEWAY_LISTEN,
        KILL_GATEWAY_CLIENT
    } ;

public:                         // Constructor(s) and destructor.
    CGCmdKillTaskType ()
: CGCmd (CMD_KILL_TASK_TYPE), eTaskType_ (KILL_CONTROL), bChildrenAlso_ (false)
    { }

    CGCmdKillTaskType (const enumKillType eTaskType, const bool bChildrenAlso = false)
: CGCmd (CMD_KILL_TASK_TYPE), eTaskType_ (eTaskType), bChildrenAlso_ (bChildrenAlso)
    { }

    CGCmdKillTaskType (const CGCmdKillTaskType& killTaskType)
: CGCmd (CMD_KILL_TASK_TYPE), eTaskType_ (killTaskType.eTaskType_), bChildrenAlso_ (killTaskType.bChildrenAlso_)
    { }

    ~CGCmdKillTaskType ()
    { }


public:                         // Operators.
    CGCmdKillTaskType&          operator = (const CGCmdKillTaskType& killTaskType)
    {
        eTaskType_ = killTaskType.eTaskType_ ;
        bChildrenAlso_ = killTaskType.bChildrenAlso_ ;
        return *this ;
    }

public:                         // Get methodes.
    enumKillType&               getTaskType (void)
    {
        return eTaskType_ ;
    }
    bool&                       getChildrenAlso (void)
    {
        return bChildrenAlso_ ;
    }

    //-----------------------------------------------------------------------------
private:                        // Members.
    enumKillType                eTaskType_ ;
    bool                        bChildrenAlso_ ;

    //-----------------------------------------------------------------------------
}

; // CGCmdKillTaskType

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////// End of Class CGCmdKillTaskType //
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////// Class CGCmdKillTaskData //
///////////////////////////////////////////////////////////////////////////////
class DLLSPEC_GWCOMMAND CGCmdKillTaskData : public CGCmd
{
    //-----------------------------------------------------------------------------
public:                         // Constructor(s) and destructor.
    CGCmdKillTaskData ()
: CGCmd (CMD_KILL_TASK_DATA), bAdministration_ (false), port_ (0), bTargetAddressAutoDetection_ (true), bChildrenAlso_ (false)
    { }

    CGCmdKillTaskData (const bool bAdministration, const string& targetAddress, const port_t listenPort, const string& library, const bool bTargetAddressAutoDetection, const bool bChildrenAlso = false)
: CGCmd (CMD_KILL_TASK_DATA), bAdministration_ (bAdministration), targetAddress_ (targetAddress), port_ (listenPort), library_ (library), bTargetAddressAutoDetection_ (bTargetAddressAutoDetection || targetAddress.empty () ), bChildrenAlso_ (bChildrenAlso)
    { }

    CGCmdKillTaskData (const CGCmdKillTaskData& killTaskData)
: CGCmd (CMD_KILL_TASK_DATA), bAdministration_ (killTaskData.bAdministration_), targetAddress_ (killTaskData.targetAddress_), port_ (killTaskData.port_), library_ (killTaskData.library_), bTargetAddressAutoDetection_ (killTaskData.bTargetAddressAutoDetection_), bChildrenAlso_ (killTaskData.bChildrenAlso_)
    { }

    ~CGCmdKillTaskData ()
    { }


public:                         // Operators.
    CGCmdKillTaskData&          operator = (const CGCmdKillTaskData& killTaskData)
    {
        bAdministration_ = killTaskData.bAdministration_ ;
        targetAddress_ = killTaskData.targetAddress_ ;
        port_ = killTaskData.port_ ;
        library_ = killTaskData.library_ ;
        bTargetAddressAutoDetection_ = killTaskData.bTargetAddressAutoDetection_ ;
        bChildrenAlso_ = killTaskData.bChildrenAlso_ ;
        return *this ;
    }

public:                         // Get methodes.
    bool&                       getIsAdministrationListen (void)
    {
        return bAdministration_ ;
    }
    string&                 getTargetAddress (void)
    {
        return targetAddress_ ;
    }
    port_t&                     getPort (void)
    {
        return port_ ;
    }
    string&                 getLibrary (void)
    {
        return library_ ;
    }
    bool&                       getTargetAddressAutoDetectionMode (void)
    {
        return bTargetAddressAutoDetection_ ;
    }
    bool&                       getChildrenAlso (void)
    {
        return bChildrenAlso_ ;
    }

    //-----------------------------------------------------------------------------
private:                        // Members.
    bool                        bAdministration_ ;
    string                  targetAddress_ ;
    port_t                      port_ ;
    string                  library_ ;
    bool                        bTargetAddressAutoDetection_ ;
    bool                        bChildrenAlso_ ;

    //-----------------------------------------------------------------------------
}

; // CGCmdKillTaskData

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////// End of Class CGCmdKillTaskData //
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////// Class CGCmdShutDown //
///////////////////////////////////////////////////////////////////////////////
class DLLSPEC_GWCOMMAND CGCmdShutDown : public CGCmd
{
    //-----------------------------------------------------------------------------
public:                         // Constructor(s) and destructor.
    CGCmdShutDown ()
: CGCmd (CMD_SHUTDOWN), bWithNoWait_ (false)
    { }

    CGCmdShutDown (const bool bWithNoWait)
: CGCmd (CMD_SHUTDOWN), bWithNoWait_ (bWithNoWait)
    { }

    CGCmdShutDown (const CGCmdShutDown& shutdown)
: CGCmd (CMD_SHUTDOWN), bWithNoWait_ (shutdown.bWithNoWait_)
    { }

    ~CGCmdShutDown ()
    { }


public:                         // Operators.
    CGCmdShutDown&              operator = (const CGCmdShutDown& shutdown)
    {
        bWithNoWait_ = shutdown.bWithNoWait_ ;
        return *this ;
    }

public:                         // Get methodes.
    bool&                       getWithNoWait (void)
    {
        return bWithNoWait_ ;
    }

    //-----------------------------------------------------------------------------
private:                        // Members.
    bool                        bWithNoWait_ ;

    //-----------------------------------------------------------------------------
}

; // CGCmdShutDown

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////// End of Class CGCmdShutDown //
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////// Class CGCmdTrace //
///////////////////////////////////////////////////////////////////////////////
class DLLSPEC_GWCOMMAND CGCmdTrace : public CGCmd
{
    //-----------------------------------------------------------------------------
public:                         // Constructor(s) and destructor.
    CGCmdTrace ()
: CGCmd (CMD_TRACE), bOn_ (false)
    { }

    CGCmdTrace (const bool bOn)
: CGCmd (CMD_TRACE), bOn_ (bOn)
    { }

    CGCmdTrace (const CGCmdTrace& trace)
: CGCmd (CMD_TRACE), bOn_ (trace.bOn_)
    { }

    ~CGCmdTrace ()
    { }


public:                         // Operators.
    CGCmdTrace&                 operator = (const CGCmdTrace& trace)
    {
        bOn_ = trace.bOn_ ;
        return *this ;
    }

public:                         // Get methodes.
    bool&                       getOn (void)
    {
        return bOn_ ;
    }

    //-----------------------------------------------------------------------------
private:                        // Members.
    bool                        bOn_ ;

    //-----------------------------------------------------------------------------
}

; // CGCmdTrace

///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////// End of Class CGCmdTrace //
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////// Class CGCmdWho //
///////////////////////////////////////////////////////////////////////////////
class CGCmdWho ;
typedef vector<CGCmdWho *> CGCmdWhoStlList ;
class DLLSPEC_GWCOMMAND CGCmdWho : public CGCmd
{
    //-----------------------------------------------------------------------------
public:                         // Types.
    enum enumWhoType
    {
        WHO_CONTROL,
        WHO_ADMINISTRATION_LISTEN,
        WHO_ADMINISTRATION_CLIENT,
        WHO_GATEWAY_LISTEN,
        WHO_GATEWAY_CLIENT
    } ;

public:                         // Constructor(s) and destructor.
    CGCmdWho ()
: CGCmd (CMD_WHO), eType_ (WHO_CONTROL), lId_ (0)
    { }

    CGCmdWho (const CGCmdWhoStlList& childrenList, enumWhoType eType, long lId, const string& clientAddress, const string& gatewayAddress, const string& targetAddress, const string& library)
: CGCmd (CMD_WHO), childrenList_ (childrenList), eType_ (eType), lId_ (lId), clientAddress_ (clientAddress), gatewayAddress_ (gatewayAddress), targetAddress_ (targetAddress), library_ (library)
    { }

    CGCmdWho (const CGCmdWho& who)
: CGCmd (CMD_WHO), childrenList_ (who.childrenList_), eType_ (who.eType_), lId_ (who.lId_), clientAddress_ (who.clientAddress_), gatewayAddress_ (who.gatewayAddress_), targetAddress_ (who.targetAddress_), library_ (who.library_)
    { }

    ~CGCmdWho ()
    {
        for (CGCmdWhoStlList::iterator itr = childrenList_.begin();itr != childrenList_.end(); itr++)
            delete *itr ;
        childrenList_.clear() ;
    }

public:                         // Operators.
    CGCmdWho&                   operator = (const CGCmdWho& who)
    {
        childrenList_ = who.childrenList_ ;
        eType_ = who.eType_ ;
        lId_ = who.lId_ ;
        clientAddress_ = who.clientAddress_ ;
        gatewayAddress_ = who.gatewayAddress_ ;
        targetAddress_ = who.targetAddress_ ;
        library_ = who.library_ ;
        return *this ;
    }
    bool                        operator == (const CGCmdWho& who)
    {
        return (lId_ == who.lId_) ;
    }

public:                         // Get methodes.
    CGCmdWhoStlList&        getChildrenList (void)
    {
        return childrenList_ ;
    }
    enumWhoType&                getType (void)
    {
        return eType_ ;
    }
    long&                       getId (void)
    {
        return lId_ ;
    }
    string&                 getClientAddress (void)
    {
        return clientAddress_ ;
    }
    string&                 getGatewayAddress (void)
    {
        return gatewayAddress_ ;
    }
    string&                 getTargetAddress (void)
    {
        return targetAddress_ ;
    }
    string&                 getLibrary (void)
    {
        return library_ ;
    }

    //-----------------------------------------------------------------------------
private:                        // Members.
    CGCmdWhoStlList     childrenList_ ;
    enumWhoType                 eType_ ;
    long                        lId_ ;
    string                  clientAddress_ ;
    string                  gatewayAddress_ ;
    string                  targetAddress_ ;
    string                  library_ ;

    //-----------------------------------------------------------------------------
}

; // CGCmdWho

///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////// End of Class CGCmdWho //
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////// Class CGCmdQuit //
///////////////////////////////////////////////////////////////////////////////
class DLLSPEC_GWCOMMAND CGCmdQuit : public CGCmd
{
    //-----------------------------------------------------------------------------
public:                         // Constructor(s) and destructor.
    CGCmdQuit ()
: CGCmd (CMD_QUIT)
    { }

    CGCmdQuit (const CGCmdQuit&)
: CGCmd (CMD_QUIT)
    { }

    ~CGCmdQuit ()
    { }


public:                         // Operators.
    CGCmdQuit&                  operator = (const CGCmdQuit&)
    {
        return *this ;
    }

    //-----------------------------------------------------------------------------
private:                        // Members.

    //-----------------------------------------------------------------------------
}

; // CGCmdQuit

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////// End of Class CGCmdQuit //
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////// Class CGCmdUser //
///////////////////////////////////////////////////////////////////////////////
class DLLSPEC_GWCOMMAND CGCmdUser : public CGCmd
{
    //-----------------------------------------------------------------------------
public:                         // Constructor(s) and destructor.
    CGCmdUser ()
: CGCmd (CMD_USER), lUserCommand_ (0)
    { }

    CGCmdUser (unsigned long lUserCommand)
: CGCmd (CMD_USER), lUserCommand_ (lUserCommand)
    { }

    CGCmdUser (const CGCmdUser& user)
: CGCmd (CMD_USER), lUserCommand_ (user.lUserCommand_)
    { }

    ~CGCmdUser ()
    { }


public:                         // Operators.
    CGCmdUser&                  operator = (const CGCmdUser& user)
    {
        lUserCommand_ = user.lUserCommand_ ;
        return *this ;
    }
    bool                        operator == (const CGCmdUser& user)
    {
        return (lUserCommand_ == user.lUserCommand_) ;
    }

public:                         // Get methodes.
    unsigned long&              getUserCommand (void)
    {
        return lUserCommand_ ;
    }

    //-----------------------------------------------------------------------------
private:                        // Members.
    unsigned long               lUserCommand_ ;

    //-----------------------------------------------------------------------------
}

; // CGCmdUser

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////// End of Class CGCmdUser //
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////// Class CGWParseCommand //
///////////////////////////////////////////////////////////////////////////////
class DLLSPEC_GWCOMMAND CGWParseCommand : public CTMTSafe
{
    //-----------------------------------------------------------------------------
    // Define Data.
#define NEWGATEWAYPARSECOMMAND  "LGW_NewGatewayParseCommand"    // Name of the library function to be used to parse command.
#define GATEWAYPARSEVERSION     "LGW_GWParseVersion"            // Name of the library version variable.

public:                         // Types.
    typedef void (*fnNewGatewayParseCommand) (const void*, const string&, bool&, CTOLogFileList&, CTSocket*, CGWParseCommand*&) ;


public:                         // Constructor(s) and destructor.
    CGWParseCommand (const void* pConnectionId, const string& taskName, bool& bTraceOn, CTOLogFileList& logFileList, CTSocket* pClient)
: pConnectionId_ ( (void*) pConnectionId), taskName_ (taskName), bTraceOn_ (bTraceOn), logFileList_ (logFileList), client_ (*pClient)
    { }
    ;
    virtual ~CGWParseCommand ()
    { }
    ;

public:                         // Operators.
    CGWParseCommand&            operator = (const CGWParseCommand&) ;

public:                         // Control Methodes.
    void                        traceString (const string& str)
    {
        if (bTraceOn_)
        {
            CTLockGuard<CTMutex> guard (logFileList_.getMutex () ) ;
            logFileList_ << str ;
        }
    }
    void                        traceLine (const string& str)
    {
        if (bTraceOn_)
        {
            CTLockGuard<CTMutex> guard (logFileList_.getMutex () ) ;
            logFileList_ << str << endl ;
        }
    }
    void                        traceFlush (const string& str)
    {
        if (bTraceOn_)
        {
            CTLockGuard<CTMutex> guard (logFileList_.getMutex () ) ;
            logFileList_ << str << flush ;
        }
    }
    virtual void                parseCommand (const string& command, CGCmd*& pCmd) = 0 ;
    virtual void                commandResult (CGCmd*& pCmd) = 0 ;

protected:                      // Get Methodes.

    //-----------------------------------------------------------------------------
protected:                      // Members.
    void*                       pConnectionId_ ;
    string                  taskName_ ;
    bool&                       bTraceOn_ ;
    CTOLogFileList&             logFileList_ ;
    CTSocket&                   client_ ;

    //-----------------------------------------------------------------------------
}

; // CGWParseCommand

///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////// End of Class CGWParseCommand //
///////////////////////////////////////////////////////////////////////////////

#endif  // __GATEWAY_COMMAND__
