/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// SCLWriter.h: interface for the SCLWriter class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SCLWRITER_H__52E94885_FB16_11D3_BD85_00C04F89E392__INCLUDED_)
#define AFX_SCLWRITER_H__52E94885_FB16_11D3_BD85_00C04F89E392__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <afx.h>

class SCLWriter
{
public:
    typedef enum { enGet,enHead,enPost} RequestTypes;
public:
    SCLWriter();
    virtual ~SCLWriter();

    void WriteConnectString(const char* szHostName,int nConId);
    void WriteWaitString(int nDelay);
    void WriteDisconnectString(int nConId,int nOptions);
    void WriteRequestString(int nConId,RequestTypes RequestType,const char* szHttpVersion,const char* szHeaders,const char* szBody,const char* szCertificate,int nOption=0);
    void WriteCommentString(const char* szComment);
    void WriteLoadResponseInfoString(int nConId,int nType,const char* szIdentifier,const char *szName);
    void WriteVariableDef(const char *szName,int nOption,int nSize);
    void WriteTimer(const char *szName,bool bStart);
    void WriteTimerDef(const char *szName);
    void WriteSynchronize();
	void WriteSetVar(const char *szName,const char *szValue);
	void WriteBAuthentication(int nType,int nTPArg,const char* szUserName,const char* szPassword,const char* szDomain,const char* szBlob,const char* szInto);
private:
    void GetEnvironmentString(CString& strEnvironmentStrings);
    void GetDefinitionsString(CString& strDefinitionsString);
    void GetDefaultHeadersString(CString& strDefHeadersString);
    void WriteExitString();
    void WriteString(const CString& psz);
    void ValidateString(CString& strSCL,int nNumTabs);
	void WritePageTimer();

    int m_nBufferLen;
    int m_nBufferPos;
    char *m_pszBuffer;
private:
    BOOL m_bHeadersInitialised;
    CMapStringToString m_mapDefaultHeaders;
    CStringArray m_arVariables;
    CString m_strUserAgent;

    BOOL m_bOnPrimaryGet;
    BOOL m_bLastWriteWasConnect;
    CString m_strDefIndent;
	int m_nPageTimer;
	bool m_bMatchedTimers;
	CString m_strPageTimerName;

    CString m_strPrevLine;
    void ProcessHeaders(const char* szHeaders,CString& strHeader);
    void ProcessIniFile(const CString& strIniFile);
};

#endif // !defined(AFX_SCLWRITER_H__52E94885_FB16_11D3_BD85_00C04F89E392__INCLUDED_)
