/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CSerialization.h: interface for the CCSerialization class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_CSERIALIZATION_H__6DCF04C6_F7C3_11D2_AC19_00105A483E3C__INCLUDED_)
#define AFX_CSERIALIZATION_H__6DCF04C6_F7C3_11D2_AC19_00105A483E3C__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <afxtempl.h>

//Write string data to the registry
BOOL WriteExtendedProfileString(HKEY _hKey, LPCTSTR _lpSubKey,
                                LPCTSTR _szEntry, LPCTSTR _pszValue) ;
//Write binary data to the registry
BOOL WriteExtendedProfileBinary(HKEY _hKey, LPCTSTR _lpSubKey,
                                LPCTSTR _szEntry, LPBYTE _lValue , int nSize ) ;

//Write DWORD data to the registry
BOOL WriteExtendedProfileDword(HKEY _hKey, LPCTSTR _lpSubKey,
                               LPCTSTR _szEntry, DWORD dwValue ) ;

//Read string data to the registry
CString GetExtendedProfileString(HKEY _hKey, LPCTSTR _lpSubKey,
                                 LPCTSTR _szEntry , LPCTSTR _szDefault ) ;

//Read binary data to the registry
LPBYTE GetExtendedProfileBinary(HKEY _hKey, LPCTSTR _lpSubKey,
                                LPCTSTR _szEntry , LPBYTE _pValue ) ;

//Read DWORD data to the registry
DWORD GetExtendedProfileDword(HKEY _hKey, LPCTSTR _lpSubKey,
                              LPCTSTR _szEntry , DWORD dwDefault ) ;




class CCPersistentObject
{
    //Attribute
private:

    CString                     m_strLocation ;         //location where data will be stored
    int                         m_nID ;                 //entry for one data
    CArray<CString , CString&>  m_ArrayValue ;          //values for one data
    int                         m_nArrayValueIndex ;    //iterator for reading or consulting data
    CCPersistentObject          *m_pNext ;              //pointer in the next elt;

    //Construction
public :

    CCPersistentObject() ;
    ~CCPersistentObject() ;

protected:
    void Init() ;                                   //Init data
    BOOL FillValues( const CString & pczValues ) ;  //fill values in the string array
    //one state = one value
    void SaveValues( CString &strStoreValues ) ;    //save values from array to string
    // as val1|val2|val3|


    //Operation
public:
    void ResetEnum() ;                              //Reset iterator

    BOOL    GetBool() ;                                 //Get boolean value
    void    SetBool( BOOL bSet ) ;                      //Set boolean value
    CString GetString() ;                               //Get string value
    void    SetString( CString strValue ) ;             //Set string value
    int     GetInt() ;                                  //Get integer value
    void    SetInt( int nSet ) ;                        //Set integer value
    long    GetLong() ;                                 //Get long value
    void    SetLong( long lSet ) ;                      //Set long value
    float   GetFloat() ;                                //Get float value
    void    SetFloat( float lSet ) ;                    //Set float value


    BOOL Read() ;                                   //Read data from the registry
    void Save() ;                                   //Save data from the registry




    inline void     SetID( int nID )
    {
        m_nID = nID ;
    }
    inline int      GetID()
    {
        return m_nID ;
    }
    inline void     SetLocation( const CString& strLocation )
    {
        m_strLocation = strLocation ;
    }
    inline CString  GetLocation()
    {
        return m_strLocation ;
    }
    inline void     SetNext( CCPersistentObject *pNext )
    {
        m_pNext = pNext ;
    }
    inline          CCPersistentObject* GetNext()
    {
        return m_pNext ;
    }
} ;

class CCSerialization
{

    CString             m_strLocation ;     //location common to all item in the persistence list
    CCPersistentObject  *m_pFirstElt ;      // first persistent elt in the list
    HKEY                m_hKey ;            // key of the location


public:
    CCSerialization();
    virtual ~CCSerialization();


    void SetLocation( LPCSTR pczLocation ) ;        //Set location where the data will be stored
    CCPersistentObject *GetObject( int nID ) ;      //find an object in the list
    // if exists return the object
    //else return NULL

    CCPersistentObject *AddObject( int nID ) ;      //Add an object in the list
    void Read() ;                                   //read all the data in the registry
    // and create the persistent objects list
    void Save() ;                                   //save all the persistent objects in the registry

}
;

#endif // !defined(AFX_CSERIALIZATION_H__6DCF04C6_F7C3_11D2_AC19_00105A483E3C__INCLUDED_)
