// SSDataDlg.cpp : implementation file
//

#include "stdafx.h"
#include "SSDataDlg.h"
#include "HttpSingleStepFile.h"
#include "SSDetailsDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSSDataDlg dialog

extern HINSTANCE m_hInstance;
CSSDataDlg::CSSDataDlg(CHttpSingleStepFile* pFile,CWnd* pParent /*=NULL*/)
	: CDialog(CSSDataDlg::IDD, pParent),
	m_pSSFile(pFile)
{
	//{{AFX_DATA_INIT(CSSDataDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


void CSSDataDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CSSDataDlg)
	DDX_Control(pDX, IDC_REQUEST_LIST, m_ctlRequestList);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CSSDataDlg, CDialog)
	//{{AFX_MSG_MAP(CSSDataDlg)
	ON_WM_SIZE()
	ON_NOTIFY(NM_DBLCLK, IDC_REQUEST_LIST, OnDblclkRequestList)
	ON_WM_DESTROY()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSSDataDlg message handlers

BOOL CSSDataDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	if(!m_pSSFile->IsOpen())
		return FALSE;

	SetupRequestList();

	LoadRequests();

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CSSDataDlg::SetupRequestList()
{
	CString strName;
	int nWidth = 80;
	for(int nCol=0;nCol<enLastCol;nCol++)
	{
		switch(nCol)
		{
		case enRequestCol:	strName = "Request";		nWidth=100;		break;
		case enVUIdCol:		strName = "VU ID";			nWidth=40;		break;
		case enConIdCol:	strName = "Con ID";			nWidth=40;		break;
		case enBinaryCol:	strName = "Binary Body";	nWidth=60;		break;
		case enBodyLenCol:	strName = "Body Length";	nWidth=60;		break;
		}
		m_ctlRequestList.InsertColumn(nCol,strName,LVCFMT_LEFT,nWidth,nCol);
	}
	m_ctlRequestList.SetExtendedStyle(LVS_EX_FULLROWSELECT|LVS_EX_GRIDLINES);
}

void CSSDataDlg::LoadRequests()
{
	CString str;
	int nItem =0;
	CBaseSingleStepData *pData = NULL;
	CMap <CString,LPCTSTR,int,int> mapVUToItem;
	
	m_pSSFile->SeekToBegin();
	while(pData = m_pSSFile->Read())
	{
		switch(pData->GetType())
		{
		case CBaseSingleStepData::enNextRequest:
		{
			CNextRequestData *pNRData = (CNextRequestData*)pData;

			nItem = m_ctlRequestList.GetItemCount();
			nItem = m_ctlRequestList.InsertItem(nItem,pNRData->GetUrl());

			RequestData *pRD = new RequestData;
			pRD->NRData = *pNRData;
			m_mapItemToData.SetAt(nItem,pRD);
				
			GetUserKey(pNRData->GetVUId(),pNRData->GetConId(),str);

//			OutputDebugString("NR: " + str + "\n");
			mapVUToItem.SetAt(str,nItem);
						
			str.Format("%d",pNRData->GetVUId()+1);
			m_ctlRequestList.SetItemText(nItem,enVUIdCol,str);
			
			str.Format("%d",pNRData->GetConId());
			m_ctlRequestList.SetItemText(nItem,enConIdCol,str);
		}
		break;
		case CBaseSingleStepData::enRequestComplete:
		{
			CRequestCompleteData *pRCData = (CRequestCompleteData*)pData;

			GetUserKey(pRCData->GetVUId(),pRCData->GetConId(),str);

//			OutputDebugString("RC: " + str + "\n");

			if(mapVUToItem.Lookup(str,nItem))
			{
				mapVUToItem.RemoveKey(str);
			
				m_ctlRequestList.SetItemText(nItem,enBinaryCol, (pRCData->IsBinaryData() ? "Yes":"No") );
					
				str.Format("%d",pRCData->GetBodyLength());
				m_ctlRequestList.SetItemText(nItem,enBodyLenCol,str);

				RequestData* pRD;
				if(m_mapItemToData.Lookup(nItem,pRD))
				{
					pRD->RCData = *pRCData;
				}
			}
		}
		break;
		}
	}
}

void CSSDataDlg::GetUserKey(int nVU,int nConId, CString& strKey)
{
	strKey.Format("%d-%d",nVU,nConId);
}

void CSSDataDlg::OnSize(UINT nType, int cx, int cy) 
{
	CDialog::OnSize(nType, cx, cy);
	
	if(m_ctlRequestList.GetSafeHwnd())
		m_ctlRequestList.MoveWindow(0,0,cx,cy);
}

void CSSDataDlg::OnDblclkRequestList(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NMLISTVIEW *pLV = (NM_LISTVIEW*)pNMHDR;

	RequestData *pData=NULL;
	if(m_mapItemToData.Lookup(pLV->iItem,pData) && pData)
	{
		HINSTANCE hInst = AfxGetResourceHandle();
		AfxSetResourceHandle(m_hInstance);
		CSSDetailsDlg dlg(pData);
		dlg.DoModal();
		AfxSetResourceHandle(hInst);
	}
	
	
	*pResult = 0;
}

void CSSDataDlg::OnDestroy() 
{
	int nItem;
	RequestData *pData;
	POSITION pos = m_mapItemToData.GetStartPosition();
	while(pos)
	{
		m_mapItemToData.GetNextAssoc(pos,nItem,pData);
		delete pData;
	}
	m_mapItemToData.RemoveAll();

	CDialog::OnDestroy();
}
 