	/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// DOMView.cpp : implementation file
//

#include "stdafx.h"
#include "Modeller.h"
#include "ModellerDoc.h"
#include "Constantes.h"
#include "VarAddrNameDlg.h"
#include "DOMView.h"
#include "VariousObject.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif



//----------------------------------------------------------------------------

BEGIN_MESSAGE_MAP(CDOMView, CTreeView)
//{{AFX_MSG_MAP(CDOMView)
ON_COMMAND(IDM_ADDRESS, OnAddressing)
ON_WM_CONTEXTMENU()
ON_NOTIFY_REFLECT(NM_RCLICK, OnRclick)
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//----------------------------------------------------------------------------

IMPLEMENT_DYNCREATE(CDOMView, CTreeView)

//----------------------------------------------------------------------------

CDOMView::CDOMView()
{}

//----------------------------------------------------------------------------


CDOMView::~CDOMView()
{}


/////////////////////////////////////////////////////////////////////////////
// CDOMView drawing
//----------------------------------------------------------------------------


void CDOMView::OnDraw(CDC* pDC)
{
    CDocument* pDoc = GetDocument();
    // TODO: add draw code here
}

/////////////////////////////////////////////////////////////////////////////
// CDOMView diagnostics
//----------------------------------------------------------------------------


#ifdef _DEBUG
void CDOMView::AssertValid() const
{
    CTreeView::AssertValid();
}

//----------------------------------------------------------------------------

void CDOMView::Dump(CDumpContext& dc) const
{
    CTreeView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CDOMView message handlers
//----------------------------------------------------------------------------

void CDOMView::OnInitialUpdate()
{
    CTreeView::OnInitialUpdate();
    //-----
    InitTreeCtrlDisplay() ;
}

//----------------------------------------------------------------------------

void CDOMView::InitTreeCtrlDisplay()
{
    CBitmap bmp;
    //----


    m_ImageList.Create( 16, 16, ILC_COLOR, 1, 1 );

    VERIFY(bmp.LoadBitmap(IDB_DOM));
    m_nidDOM = m_ImageList.Add(&bmp, RGB(0,0,0));
    bmp.DeleteObject();


    VERIFY(bmp.LoadBitmap(IDB_LINK));
    m_nidLink = m_ImageList.Add(&bmp, RGB(0,0,0));
    bmp.DeleteObject();

    VERIFY(bmp.LoadBitmap(IDB_SCRIPT));
    m_nidScript = m_ImageList.Add(&bmp, RGB(0,0,0));
    bmp.DeleteObject();

    VERIFY(bmp.LoadBitmap(IDB_FRAME));
    m_nidFrame = m_ImageList.Add(&bmp, RGB(0,0,0));
    bmp.DeleteObject();

    VERIFY(bmp.LoadBitmap(IDB_IMAGE));
    m_nidImage = m_ImageList.Add(&bmp, RGB(0,0,0));
    bmp.DeleteObject();


    VERIFY(bmp.LoadBitmap(IDB_INPUT));
    m_nidInput = m_ImageList.Add(&bmp, RGB(0,0,0));
    bmp.DeleteObject();

    VERIFY(bmp.LoadBitmap(IDB_SELECT));
    m_nidCombo = m_ImageList.Add(&bmp, RGB(0,0,0));
    bmp.DeleteObject();

    VERIFY(bmp.LoadBitmap(IDB_TABLE));
    m_nidTable = m_ImageList.Add(&bmp, RGB(0,0,0));
    bmp.DeleteObject();

    VERIFY(bmp.LoadBitmap(IDB_CHECKBOX));
    m_nidChkBox = m_ImageList.Add(&bmp, RGB(0,0,0));
    bmp.DeleteObject();

    VERIFY(bmp.LoadBitmap(IDB_RADIO));
    m_nidRadio = m_ImageList.Add(&bmp, RGB(0,0,0));
    bmp.DeleteObject();

    GetTreeCtrl().SetImageList(&m_ImageList , TVSIL_NORMAL);
	GetTreeCtrl().ModifyStyle(0,TVS_HASLINES|TVS_HASBUTTONS|TVS_LINESATROOT);
}

//----------------------------------------------------------------------------

void CDOMView::FillTreeCtrl()
{
	m_hLink = GetTreeCtrl().InsertItem("Link"  ,m_nidDOM, m_nidDOM);
	m_hScript = GetTreeCtrl().InsertItem("Script",m_nidDOM, m_nidDOM);
	m_hFrame = GetTreeCtrl().InsertItem("Frame" ,m_nidDOM, m_nidDOM);
	m_hImage = GetTreeCtrl().InsertItem("Image" ,m_nidDOM, m_nidDOM);
	m_hInput = GetTreeCtrl().InsertItem("Input" ,m_nidDOM, m_nidDOM);
	m_hSelect = GetTreeCtrl().InsertItem("Combo/Listbox" ,m_nidDOM, m_nidDOM);
	m_hRadio = GetTreeCtrl().InsertItem("Radio Button" ,m_nidDOM, m_nidDOM);
	m_hChkBox = GetTreeCtrl().InsertItem("Checkbox" ,m_nidDOM, m_nidDOM);
//	m_hTable = GetTreeCtrl().InsertItem("Table" ,m_nidDOM, m_nidDOM);
//	m_hOther = GetTreeCtrl().InsertItem("Other" ,m_nidDOM, m_nidDOM);
}

//----------------------------------------------------------------------------


void CDOMView::ShowDow()
{
	CTag* pTag = static_cast<CModellerDoc*> (GetDocument() )->GetHtmlRootTag() ;
	if(pTag)
		InsertTag(pTag) ;
}

//----------------------------------------------------------------------------

void CDOMView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint)
{
    if( lHint == ID_DISPLAY_DOM_URL )
    {
        GetTreeCtrl().DeleteAllItems() ;

		CLongObject *pObj =  (CLongObject*)pHint;
        if(!pObj->m_lObject)
        {
			FillTreeCtrl()  ;
			ShowDow();
			ExpandAllChildren(TVI_ROOT);
		}
    }
    else
        CTreeView::OnUpdate(pSender, lHint, pHint);

}

//----------------------------------------------------------------------------

void CDOMView::InsertTag( CTag* pTag)
{
	InsertTagInTree( pTag) ;

	for(CTag::TagList::const_iterator it = pTag->sons_.begin(); it != pTag->sons_.end(); it++)
		InsertTag(*it) ;
}

//----------------------------------------------------------------------------

void CDOMView::InsertTagInTree( CTag* pTag)
{
	switch(pTag->eTagType_)
	{
    case TAG_A:			InsertLink( pTag);		break;
    case TAG_SCRIPT:	InsertScript( pTag);	break;
    case TAG_FRAME:		InsertFrame( pTag);		break;
    case TAG_IMG:		InsertImage( pTag);		break;
	case TAG_INPUT:		InsertInput( pTag);		break;
    case TAG_SELECT:	InsertSelect( pTag);	break;
    case TAG_TABLE:		InsertTable( pTag);		break;
	default:
	{
//		CString str;
//		str.Format("Tag %d\n",pTag->eTagType_);
//		OutputDebugString(str);
	}
	break;
	}
}

//----------------------------------------------------------------------------

void CDOMView::InsertLink( CTag* pTag)
{
	CTag::AttributeList& AttributeList = pTag->attributes_ ;

	for(CTag::AttributeList::const_iterator it = AttributeList.begin(); it != AttributeList.end(); it++)
	{
		if(stricmp((*it).strLabel_.c_str(), STR_HREF)==0)
		{
			HTREEITEM hItem = GetTreeCtrl().InsertItem((*it).strText_.c_str(),m_nidLink,m_nidLink, m_hLink);
			GetTreeCtrl().SetItemData(hItem,(DWORD)&(*it));
		}
	}
}

//----------------------------------------------------------------------------

void CDOMView::InsertScript( CTag* pTag)
{
    CString strText;

	CTag::NodeList& NodeList = pTag->scripts_;
	for(CTag::NodeList::const_iterator it = NodeList.begin(); it != NodeList.end(); it++)
	{
		HTREEITEM hItem = GetTreeCtrl().InsertItem((*it).strText_.c_str(),m_nidScript,m_nidScript, m_hScript);
		GetTreeCtrl().SetItemData(hItem,(DWORD)&(*it));
	}
}

//----------------------------------------------------------------------------

void CDOMView::InsertFrame( CTag* pTag)
{
	CTag::AttributeList& AttributeList = pTag->attributes_;
	for(CTag::AttributeList::const_iterator it = AttributeList.begin(); it != AttributeList.end(); it++)
	{
		if(stricmp((*it).strLabel_.c_str(), STR_SRC)==0)
		{
	        HTREEITEM hItem = GetTreeCtrl().InsertItem((*it).strText_.c_str(),m_nidFrame,m_nidFrame,m_hFrame);
			GetTreeCtrl().SetItemData(hItem,(DWORD)&(*it));
		}
	}
}

//----------------------------------------------------------------------------

void CDOMView::InsertImage( CTag* pTag)
{
	CTag::AttributeList& AttributeList = pTag->attributes_;

	for(CTag::AttributeList::const_iterator it = AttributeList.begin(); it != AttributeList.end(); it++)
	{
		if(stricmp((*it).strLabel_.c_str(), STR_SRC)==0)
		{
	        HTREEITEM hItem = GetTreeCtrl().InsertItem((*it).strText_.c_str(),m_nidImage,m_nidImage,m_hImage);
			GetTreeCtrl().SetItemData(hItem,(DWORD)&(*it));
		}
	}
}

//----------------------------------------------------------------------------

void CDOMView::InsertInput( CTag* pTag)
{
	CTag::AttributeList& AttributeList = pTag->attributes_;

	for(CTag::AttributeList::const_iterator it = AttributeList.begin(); it != AttributeList.end(); it++)
	{
		if(stricmp((*it).strLabel_.c_str(), STR_TYPE)==0)
		{
			if( stricmp( (*it).strText_.c_str()  , STR_TYPE_TEXT ) == 0 ||
					stricmp( (*it).strText_.c_str()  , STR_TYPE_TEXT2 ) == 0 ||
					stricmp( (*it).strText_.c_str()  , STR_TYPE_PWD ) == 0   ||
					stricmp( (*it).strText_.c_str()  , STR_TYPE_PWD2 ) == 0 )
				InsertInputText( pTag) ;
			else
			if( stricmp( (*it).strText_.c_str()  , STR_TYPE_CHECKBOX ) == 0 ||
					stricmp( (*it).strText_.c_str()  , STR_TYPE_CHECKBOX2 ) == 0 )
				InsertInputCheckbox( pTag) ;
			else
			if( stricmp( (*it).strText_.c_str()  , STR_TYPE_RADIO ) == 0 ||
					stricmp( (*it).strText_.c_str()  , STR_TYPE_RADIO2 ) == 0 )
				InsertInputRadio( pTag) ;
			return;
		}
	}
}

//----------------------------------------------------------------------------

void CDOMView::InsertSelect( CTag* pTag)
{
	CTag::AttributeList& AttributeList = pTag->attributes_;
	for(CTag::AttributeList::const_iterator it = AttributeList.begin(); it != AttributeList.end(); it++)
	{
		if(stricmp((*it).strLabel_.c_str(), STR_NAME)==0)
		{
	        HTREEITEM hItem = GetTreeCtrl().InsertItem((*it).strText_.c_str(),m_nidLink,m_nidLink,m_hSelect);
			GetTreeCtrl().SetItemData(hItem,(DWORD)&(*it));
		}
	}
}

//----------------------------------------------------------------------------

void CDOMView::InsertTable( CTag* pTag)
{
}

//----------------------------------------------------------------------------


void CDOMView::InsertInputText( CTag* pTag)
{
	CTag::AttributeList& AttributeList = pTag->attributes_;
	for(CTag::AttributeList::const_iterator it = AttributeList.begin(); it != AttributeList.end(); it++)
	{
		if(stricmp((*it).strLabel_.c_str(), STR_NAME)==0)
		{
	        HTREEITEM hItem = GetTreeCtrl().InsertItem((*it).strText_.c_str(),m_nidInput,m_nidInput,m_hInput);
			GetTreeCtrl().SetItemData(hItem,(DWORD)&(*it));
		}
	}
}

//----------------------------------------------------------------------------

void CDOMView::InsertInputCheckbox( CTag* pTag)
{
	CTag::AttributeList& AttributeList = pTag->attributes_;
	for(CTag::AttributeList::const_iterator it = AttributeList.begin(); it != AttributeList.end(); it++)
	{
		if(stricmp((*it).strLabel_.c_str(), STR_NAME)==0)
		{
	        HTREEITEM hItem = GetTreeCtrl().InsertItem((*it).strText_.c_str(),m_nidChkBox,m_nidChkBox,m_hChkBox);
			GetTreeCtrl().SetItemData(hItem,(DWORD)&(*it));
		}
	}
}

//----------------------------------------------------------------------------

void CDOMView::InsertInputRadio( CTag* pTag)
{
	CTag::AttributeList& AttributeList = pTag->attributes_;

	for(CTag::AttributeList::const_iterator it = AttributeList.begin(); it != AttributeList.end(); it++)
	{
		if(stricmp((*it).strLabel_.c_str(), STR_NAME)==0)
		{
	        HTREEITEM hItem = GetTreeCtrl().InsertItem((*it).strText_.c_str(),m_nidRadio,m_nidRadio,m_hRadio);
			GetTreeCtrl().SetItemData(hItem,(DWORD)&(*it));
		}
	}
}

//----------------------------------------------------------------------------

void CDOMView::OnAddressing()
{
	if(!m_hMenuSelection)
		return;

    CVarResponse    VarResponse ;
    CModellerDoc *  pDoc ;
    //---
    VarResponse.m_ResponseType = RT_BODY ;

    CVarAddrNameDlg Dlg( &VarResponse ) ;
    if( Dlg.DoModal() == IDCANCEL )
	{
		m_hMenuSelection = NULL;
		return;
	}

	CTag::CNode *pNode = (CTag::CNode *)GetTreeCtrl().GetItemData( m_hMenuSelection);
    VarResponse.m_strIdentifier = pNode->strAddress_.c_str();
	m_hMenuSelection = NULL;
    pDoc = static_cast<CModellerDoc*>(GetDocument());
    pDoc->Adressing( &VarResponse ) ;
}


void CDOMView::OnContextMenu(CWnd* pWnd, CPoint point) 
{
	if (point.x == -1 && point.y == -1)
		point = (CPoint) GetMessagePos();

	ScreenToClient(&point);

	CMenu menu;
	menu.LoadMenu(IDR_DOM_VIEW);
    CMenu* pPopup = menu.GetSubMenu(0);
	pPopup->RemoveMenu(IDM_SEARCH,MF_BYCOMMAND);

	UINT uFlags;
	m_hMenuSelection = GetTreeCtrl().HitTest( point, &uFlags );
	if(m_hMenuSelection == NULL || GetTreeCtrl().GetItemData(m_hMenuSelection)==0)
	{
		m_hMenuSelection = NULL;
		return;
	}
	
	ClientToScreen(&point);
    pPopup->TrackPopupMenu(TPM_LEFTALIGN, point.x, point.y,this);
}

void CDOMView::OnRclick(NMHDR* pNMHDR, LRESULT* pResult) 
{
	// Send WM_CONTEXTMENU to self
	SendMessage(WM_CONTEXTMENU, (WPARAM) m_hWnd, GetMessagePos());
	// Mark message as handled and suppress default handling
	*pResult = 1;
}

void CDOMView::ExpandAllChildren(HTREEITEM hItem)
{
	HTREEITEM hChild = GetTreeCtrl().GetChildItem(hItem);
	while (hChild != NULL)
	{
		GetTreeCtrl().Expand(hChild,TVE_EXPAND);
		ExpandAllChildren(hChild);
		hChild = GetTreeCtrl().GetNextItem(hChild, TVGN_NEXT);
	}
}