/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// OutputReplayView.cpp : implementation file
//

#include "stdafx.h"
#include "Modeller.h"
#include "Constantes.h"
#include "ModellerDoc.h"
#include "VariousObject.h"
#include "OutputReplayView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CModellerApp theApp ;

#define WM_ADDTEXT      WM_APP+1
#define WM_ADDERROR     WM_APP+2
#define WM_REPLAYSTOP   WM_APP+3

//---------------------------------------------------------------------------

BEGIN_MESSAGE_MAP(COutputReplayExtendedEditCtrl, COSEditCtrl)
//{{AFX_MSG_MAP(COutputReplayExtendedEditCtrl)
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//---------------------------------------------------------------------------

COutputReplayExtendedEditCtrl::COutputReplayExtendedEditCtrl(LPCSTR pczRegistryPath)
        : COSEditCtrl(pczRegistryPath)
{}

//---------------------------------------------------------------------------


COutputReplayExtendedEditCtrl::~COutputReplayExtendedEditCtrl()
{}

//-------------------------------------------------------------------


void COutputReplayExtendedEditCtrl::InitComments()
{
    m_OSLanguage.SetComments("!");
}
//---------------------------------------------------------------------------

void COutputReplayExtendedEditCtrl::DoOwnerContextMenu( CMenu *pMenu )
{
    /*$$$OSCExtendedEditCtrl::DoOwnerContextMenu( pMenu ) ;
    //---
    int nCount = pMenu->GetMenuItemCount( ) ;
    pMenu->RemoveMenu( nCount - 2 , MF_BYPOSITION );
    pMenu->RemoveMenu( nCount - 2 , MF_BYPOSITION );*/
}

//---------------------------------------------------------------------------

IMPLEMENT_DYNCREATE(COutputReplayView, CEditView)

//---------------------------------------------------------------------------

BEGIN_MESSAGE_MAP(COutputReplayView, CView)
//{{AFX_MSG_MAP(COutputReplayView)
ON_MESSAGE( WM_ADDTEXT ,    OnAddText)
ON_MESSAGE( WM_ADDERROR,    OnAddError)
ON_MESSAGE( WM_REPLAYSTOP,  OnStopReplay)
ON_WM_CREATE()
ON_WM_SIZE()
ON_COMMAND(ID_EDIT_COPY, OnEditCopy)
ON_COMMAND(ID_EDIT_CUT, OnEditCut)
ON_COMMAND(ID_EDIT_FIND, OnEditFind)
ON_COMMAND(ID_EDIT_PASTE, OnEditPaste)
ON_COMMAND(ID_EDIT_REDO, OnEditRedo)
ON_COMMAND(ID_EDIT_REPLACE, OnEditReplace)
ON_COMMAND(ID_EDIT_UNDO, OnEditUndo)
ON_UPDATE_COMMAND_UI(ID_EDIT_COPY, OnUpdateEditCopy)
ON_UPDATE_COMMAND_UI(ID_EDIT_CUT, OnUpdateEditCut)
ON_UPDATE_COMMAND_UI(ID_EDIT_FIND, OnUpdateEditFind)
ON_UPDATE_COMMAND_UI(ID_EDIT_PASTE, OnUpdateEditPaste)
ON_UPDATE_COMMAND_UI(ID_EDIT_REDO, OnUpdateEditRedo)
ON_UPDATE_COMMAND_UI(ID_EDIT_REPLACE, OnUpdateEditReplace)
ON_UPDATE_COMMAND_UI(ID_EDIT_UNDO, OnUpdateEditUndo)
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//---------------------------------------------------------------------------

COutputReplayView::COutputReplayView()
        :   m_Editor(theApp.GetRegistryPath() )
{}


//---------------------------------------------------------------------------


COutputReplayView::~COutputReplayView()
{}

//---------------------------------------------------------------------------


void COutputReplayView::OnInitialUpdate()
{
    CView::OnInitialUpdate();

    static_cast<CModellerDoc*>(GetDocument())->SetOutputReplayWnd(m_hWnd);
}



//---------------------------------------------------------------------------

void COutputReplayView::OnDraw(CDC* pDC)
{
    CDocument* pDoc = GetDocument();
    // TODO: add draw code here
}


#ifdef _DEBUG

//---------------------------------------------------------------------------

void COutputReplayView::AssertValid() const
{
    CView::AssertValid();
}

//---------------------------------------------------------------------------

void COutputReplayView::Dump(CDumpContext& dc) const
{
    CView::Dump(dc);
}
#endif //_DEBUG


//---------------------------------------------------------------------------

void COutputReplayView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint)
{
    if( lHint == ID_REPLAY_SCL )
        GetEditCtrl()->SetWindowText(((CStringObject*)pHint)->m_strObject ) ;
}


//---------------------------------------------------------------------------

COSEditCtrl* COutputReplayView::GetEditCtrl()
{
    return &m_Editor ;
}


//---------------------------------------------------------------------------

LRESULT COutputReplayView::OnAddText( WPARAM w , LPARAM pText )
{
    CWaitCursor wc;

    BOOL bSaveReadOnly = m_Editor.IsReadOnly();
    m_Editor.SetReadOnly(FALSE);

    long lNewLine = m_Editor.GetLineCount();
    //---

    m_Editor.GoToLine( lNewLine ) ;

    std::string* pStrText = (std::string*)pText;
    char* zText = new char[pStrText->size() + 1];
    lstrcpy(zText, pStrText->c_str());
    delete pStrText; pStrText = NULL;
    char *pszToken = strtok(zText, "\r\n");
    if ( pszToken )
    {
        while(pszToken != NULL)
        {
            m_Editor.InsertLine(lNewLine,pszToken);

            pszToken = strtok(NULL,"\r\n");
            lNewLine++;
            m_Editor.GoToLine(lNewLine);
        }
    }

    delete zText; zText = NULL;
    m_Editor.SetReadOnly(bSaveReadOnly);
    return TRUE;
}

//---------------------------------------------------------------------------

LRESULT COutputReplayView::OnAddError( WPARAM w , LPARAM pText )
{
    //

    BOOL bSaveReadOnly = m_Editor.IsReadOnly();
    m_Editor.SetReadOnly(FALSE);

    CString strText;
    strText = _T("!");
    std::string* pStrText = (std::string*)pText;
    strText += pStrText->c_str();
    delete pStrText; pStrText = NULL;

    int newLine = m_Editor.GetLineCount();
    m_Editor.InsertLine( newLine, strText.GetBuffer(0) );
    m_Editor.GoToLine(newLine);

    m_Editor.SetReadOnly(bSaveReadOnly);


    return TRUE;
}

//---------------------------------------------------------------------------

LRESULT COutputReplayView::OnStopReplay( WPARAM w , LPARAM pText )
{
    BOOL bSaveReadOnly = m_Editor.IsReadOnly();
    m_Editor.SetReadOnly(FALSE);

    CString strText;
    strText = _T("!");
    std::string* pstrMsg = (std::string*)pText;
    strText += pstrMsg->c_str();
    delete pstrMsg; pstrMsg = NULL;

    int newLine = m_Editor.GetLineCount();
    m_Editor.InsertLine( newLine, strText.GetBuffer(0) );
    m_Editor.GoToLine(newLine+1);

    m_Editor.SetReadOnly(bSaveReadOnly);

    // Synchronize with the end of the thread pool's main thread.
    HANDLE hTPThread = (HANDLE)w;
    WaitForSingleObject(hTPThread, INFINITE);
    CloseHandle(hTPThread);

    static_cast<CModellerDoc*>(GetDocument())->OnReplayStopped();
    return TRUE;
}

//---------------------------------------------------------------------------

BOOL COutputReplayView::OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo)
{
    if(m_Editor.OnCmdMsg(nID,nCode,pExtra,pHandlerInfo))
        return true;

    return CView::OnCmdMsg(nID, nCode, pExtra, pHandlerInfo);
}

//---------------------------------------------------------------------------

int COutputReplayView::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
    if (CView::OnCreate(lpCreateStruct) == -1)
        return -1;

    if ( m_Editor.Create( WS_CHILD | WS_VISIBLE | WS_VSCROLL, CRect( 0, 0, 100, 100 ), this, 100 ) == -1 )
        return -1;

    m_Editor.SetReadOnly(TRUE);

    return 0;
}

//---------------------------------------------------------------------------

void COutputReplayView::OnSize(UINT nType, int cx, int cy)
{
    CView::OnSize(nType, cx, cy);

    if ( m_Editor.m_hWnd )
        m_Editor.SetWindowPos( NULL, -1, -1, cx, cy, SWP_NOZORDER | SWP_NOMOVE );

}

//---------------------------------------------------------------------------

void COutputReplayView::OnEditCopy()
{
    m_Editor.Copy();

}

//---------------------------------------------------------------------------

void COutputReplayView::OnEditCut()
{
    m_Editor.Cut();
}

//---------------------------------------------------------------------------

void COutputReplayView::OnEditFind()
{
    m_Editor.SendMessage(CMM_EXECUTECMD, CMD_FIND, 0 );
}

//---------------------------------------------------------------------------

void COutputReplayView::OnEditPaste()
{
    m_Editor.Paste();
}

//---------------------------------------------------------------------------

void COutputReplayView::OnEditRedo()
{
    m_Editor.Redo();
}

//---------------------------------------------------------------------------

void COutputReplayView::OnEditReplace()
{
    ::SendMessage( m_Editor.m_hWnd, CMM_EXECUTECMD, CMD_FINDREPLACE, 0 );
}

//---------------------------------------------------------------------------

void COutputReplayView::OnEditUndo()
{
    m_Editor.Undo();

}

//---------------------------------------------------------------------------

void COutputReplayView::OnUpdateEditCopy(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(m_Editor.CanCopy());
}

//---------------------------------------------------------------------------

void COutputReplayView::OnUpdateEditCut(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(m_Editor.CanCut());
}

//---------------------------------------------------------------------------

void COutputReplayView::OnUpdateEditFind(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(TRUE);
}

//---------------------------------------------------------------------------

void COutputReplayView::OnUpdateEditPaste(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(m_Editor.CanPaste());

}

//---------------------------------------------------------------------------

void COutputReplayView::OnUpdateEditRedo(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(m_Editor.CanRedo());
}

//---------------------------------------------------------------------------

void COutputReplayView::OnUpdateEditReplace(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(!m_Editor.IsReadOnly());
}

//---------------------------------------------------------------------------

void COutputReplayView::OnUpdateEditUndo(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(m_Editor.CanUndo());
}
