/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// VarDBColSelPPage.cpp : implementation file
//

#include "stdafx.h"
#include "VarDBColSelPPage.h"
#include "htmlhelp.h"
#include "Help\VarDBColSelPPage.inl"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//---------------------------------------------------------------------------

IMPLEMENT_DYNCREATE(CVarDBColSelPPage, CPropertyPage)

//---------------------------------------------------------------------------

BEGIN_MESSAGE_MAP(CVarDBColSelPPage, CPropertyPage)
//{{AFX_MSG_MAP(CVarDBColSelPPage)
ON_NOTIFY(LVN_ITEMCHANGED, ID_TABLE_LIST, OnItemchangedTableList)
ON_WM_CTLCOLOR()
ON_CBN_SELCHANGE(ID_OWNER_COMBO, OnSelchangeOwnerCombo)
ON_WM_HELPINFO()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//---------------------------------------------------------------------------

CVarDBColSelPPage::CVarDBColSelPPage(CVarDBParam * _VarDBParam)
        :   CPropertyPage(CVarDBColSelPPage::IDD),
        m_DBTableList(&(_VarDBParam->m_VarDBSession.m_Database)),
        m_DBColumnList(&(_VarDBParam->m_VarDBSession.m_Database))

{
    m_pVarDBParam       = _VarDBParam ;
    m_bComingFromBack   = FALSE ;

}

//---------------------------------------------------------------------------

CVarDBColSelPPage::~CVarDBColSelPPage()
{}

//---------------------------------------------------------------------------


BOOL CVarDBColSelPPage::OnInitDialog()
{
    CPropertyPage::OnInitDialog();

    m_TableList.SubclassDlgItem( ID_TABLE_LIST , this ) ;
    m_ColumnList.SubclassDlgItem( ID_COLUMN_LIST , this ) ;
    m_OwnerList.SubclassDlgItem( ID_OWNER_COMBO , this ) ;

    LV_COLUMN lvcA ;
    lvcA.mask = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM ;
    lvcA.iSubItem = 0 ;
    lvcA.pszText = _T("Table") ;
    lvcA.cx = 125 ;
    lvcA.fmt = LVCFMT_LEFT ;
    m_TableList.InsertColumn(1,&lvcA) ;

    LV_COLUMN lvcS ;
    lvcS.mask = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM ;
    lvcS.iSubItem = 0 ;
    lvcS.pszText = _T("Column") ;
    lvcS.cx = 125 ;
    lvcS.fmt = LVCFMT_LEFT ;
    m_ColumnList.InsertColumn(1,&lvcS) ;

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}

//---------------------------------------------------------------------------


void CVarDBColSelPPage::Ctrl2Values()
{
    POSITION pos ;
    int nItem, nRow ;
    pDB_ELEMENT_SET_FIELDS pdbFields ;
    pDB_COLUMN_SET_FIELDS pdbColFields ;

    //Get Selected Table Item
    pos = m_TableList.GetFirstSelectedItemPosition() ;

    ASSERT(pos!=NULL) ;

    nItem   = m_TableList.GetNextSelectedItem(pos) ;
    nRow    = m_TableList.GetItemData(nItem) ;

    pdbFields =  m_DBTableList.m_aData.GetAt(nRow) ;

    m_pVarDBParam->m_strTableName    = pdbFields->m_strName ;
    m_pVarDBParam->m_strTableOwner   = pdbFields->m_strOwner ;

    //Get Selected Column Item
    pos = m_ColumnList.GetFirstSelectedItemPosition() ;

    ASSERT(pos!=NULL) ;

    nItem   = m_ColumnList.GetNextSelectedItem(pos) ;
    nRow    = m_ColumnList.GetItemData(nItem) ;

    pdbColFields =  m_DBColumnList.m_aData.GetAt(nRow) ;
    m_pVarDBParam->m_strColumnName = pdbColFields->strColumnName ;

}

//---------------------------------------------------------------------------

void CVarDBColSelPPage::Values2Ctrl()
{
    long    nCount ;
    int     nItem;
    //---
    m_OwnerList.ResetContent();

    m_DBTableList.Open(FALSE) ;
    m_DBTableList.Load() ;
    m_DBTableList.Close() ;

    nCount = m_DBTableList.m_astrOwner.GetSize();
    for (int nRow = 0; nRow < nCount; nRow++)
    {
        nItem = m_OwnerList.AddString( m_DBTableList.m_astrOwner.GetAt(nRow) ) ;
        m_OwnerList.SetItemData(nItem, nRow);
    }

    m_OwnerList.SetCurSel(0);
    OnSelchangeOwnerCombo();
}

//---------------------------------------------------------------------------

void CVarDBColSelPPage::ShowTables(int _nOwnerId)
{
    long    nCount ;
    CString strCurOwner ;
    int     nItem;
    //---

    m_TableList.SetRedraw( FALSE );

    m_TableList.DeleteAllItems() ;

    strCurOwner = m_DBTableList.m_astrOwner.GetAt( _nOwnerId ) ;

    nCount = m_DBTableList.m_aData.GetSize() ;
    for (int nRow = 0; nRow < nCount; nRow++)
    {
        pDB_ELEMENT_SET_FIELDS pdbFields =  m_DBTableList.m_aData.GetAt(nRow) ;

        if( pdbFields->m_strOwner == strCurOwner )
        {
            nItem = m_TableList.InsertItem(0, pdbFields->m_strName) ;
            m_TableList.SetItemData(nItem, nRow);
        }
    }

    m_TableList.SetItemState(0, LVIS_SELECTED, LVIS_SELECTED) ;

    m_TableList.SetRedraw();
}

//---------------------------------------------------------------------------

void CVarDBColSelPPage::ShowColumns(int _nTableId)
{
    long    nCount ;
    CString strQualifierParam ;
    CString strOwnerParam ;
    int     nRow;
    int     nItem;

    pDB_ELEMENT_SET_FIELDS pdbFields =  m_DBTableList.m_aData.GetAt(_nTableId) ;

    m_DBColumnList.Open(FALSE) ;
    m_DBColumnList.Load(pdbFields->m_strName,pdbFields->m_strQualifier,pdbFields->m_strOwner) ;
    m_DBColumnList.Close() ;

    m_ColumnList.SetRedraw( FALSE );
    m_ColumnList.DeleteAllItems();
    ASSERT(m_ColumnList.GetItemCount() == 0);

    nCount = m_DBColumnList.m_aData.GetSize() ;
    for (nRow = 0; nRow < nCount; nRow++)
    {
        pDB_COLUMN_SET_FIELDS pdbFields =  m_DBColumnList.m_aData.GetAt(nRow) ;

        nItem = m_ColumnList.InsertItem(0, pdbFields->strColumnName) ;
        m_ColumnList.SetItemData(nItem, nRow) ;
    }

    m_ColumnList.SetItemState(0, LVIS_SELECTED, LVIS_SELECTED) ;
    m_ColumnList.SetRedraw();
}

//---------------------------------------------------------------------------

LRESULT CVarDBColSelPPage::OnWizardBack()
{
    m_bComingFromBack = FALSE ;

    return IDD_VAR_DBREQ_PPAGE ;
}

//---------------------------------------------------------------------------

LRESULT CVarDBColSelPPage::OnWizardNext()
{
    POSITION pos ;

    pos = m_TableList.GetFirstSelectedItemPosition() ;
    if (pos == NULL) return -1 ;

    pos = m_ColumnList.GetFirstSelectedItemPosition() ;
    if (pos == NULL) return -1 ;

    int sel = m_OwnerList.GetCurSel() ;
    if (sel == CB_ERR) return -1 ;

    Ctrl2Values() ;

    m_bComingFromBack = TRUE ;

    return IDD_VAR_DBROWSEL_PPAGE ;
}

//---------------------------------------------------------------------------

void CVarDBColSelPPage::OnItemchangedTableList(NMHDR* pNMHDR, LRESULT* pResult)
{
    NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;

    if(!pNMListView->uOldState) FillColumnList() ;

    *pResult = 0 ;
}

//---------------------------------------------------------------------------

void CVarDBColSelPPage::FillColumnList()
{
    POSITION pos ;
    int nItem, nRow ;

    pos = m_TableList.GetFirstSelectedItemPosition( ) ;

    if (pos == NULL)
        return ;

    nItem   = m_TableList.GetNextSelectedItem(pos) ;
    nRow    = m_TableList.GetItemData(nItem);
    ShowColumns(nRow) ;
}

//---------------------------------------------------------------------------

BOOL CVarDBColSelPPage::OnSetActive()
{
    if( !m_bComingFromBack )
        Values2Ctrl() ;

    return CPropertyPage::OnSetActive();
}

//---------------------------------------------------------------------------

HBRUSH CVarDBColSelPPage::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor)
{
    HBRUSH hbr = CPropertyPage::OnCtlColor(pDC, pWnd, nCtlColor);

    if( nCtlColor == CTLCOLOR_STATIC && pWnd->m_hWnd == GetDlgItem( IDC_TITLE_STATIC )->m_hWnd )
    {
        pDC->SetTextColor( RGB( 0 , 0 , 255 ) ) ;
    }

    return hbr;
}

//---------------------------------------------------------------------------

void CVarDBColSelPPage::OnSelchangeOwnerCombo()
{
    CExtendedComboBox*  pCombo ;
    int nOwnerId ;
    //---
    pCombo      = static_cast<CExtendedComboBox*>(GetDlgItem(ID_OWNER_COMBO)) ;
	int nCurSel = pCombo->GetCurSel();
	if(nCurSel != CB_ERR)
	{
		nOwnerId    = pCombo->GetItemData( nCurSel );
		ShowTables( nOwnerId );
	}
}


BOOL CVarDBColSelPPage::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp;
    contextHelp.LoadString(IDS_MODELLER_HELP_FILE);
    contextHelp= getenv("OPENSTA_HOME")+contextHelp+"::whatisthis.txt";



    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

	if(pWnd)
	    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);

    return true;
}
