/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// SclParser.h: interface for the SclParser class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SCLPARSER_H__B76A7945_77DF_4EF4_8CE6_2FF40D82F117__INCLUDED_)
#define AFX_SCLPARSER_H__B76A7945_77DF_4EF4_8CE6_2FF40D82F117__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "RequestInfo.h"
#include "VariableInfo.h"
#include "TimerInfo.h"
#include "CommentInfo.h"
#include "CallScriptInfo.h"
#include "WaitInfo.h"

class CScriptItemInfo;

class CSclParser  
{
public:
	CSclParser();
	virtual ~CSclParser();

	void Parse(const char *szScript);

	const CScriptItemInfo *GetItemOnLine(int nLine) const ;
	int GetPrimaryUrls(UrlArray& arUrls) const ;
	int GetVariableInsertLine() const ;

	POSITION GetFirstRequestPosition();
	CRequestInfo* GetNextRequest(POSITION& pos);

	POSITION GetFirstItemPosition();
	CScriptItemInfo* GetNextItem(POSITION& pos);

#ifdef _DEBUG
	void Dump() const;
#endif

protected:	
	char *StripWhiteSpace(char *szLine);
	BOOL ProcessWait(char *szLine,int nStartLine,int nEndLine);
	BOOL ProcessCallScript(char *szLine,int nStartLine,int nEndLine);
	BOOL ProcessVariable(char *szLine,int nStartLine,int nEndLine);
	BOOL ProcessRequest(char *szLine,int nStartLine,int nEndLine);
	BOOL ProcessTimer(char *szLine,int nStartLine,int nEndLine);
	void ProcessLine(char *szLine,int nStartLine , int nEndLine);
	void SetDefinitionsLine(int nLine){m_nDefinitionsLine = nLine;}
	void Reset();

	void AddScriptItem(CScriptItemInfo *pItem)
	{
		m_listScriptItems.AddTail(pItem);	
	}

private:
	CList < CScriptItemInfo*,CScriptItemInfo*> m_listScriptItems;
	int m_nDefinitionsLine;
};

#endif // !defined(AFX_SCLPARSER_H__B76A7945_77DF_4EF4_8CE6_2FF40D82F117__INCLUDED_)
