/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
// GraphTypes.h: interface for the CTimerGraph class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_GRAPHTYPES_H__24021AB3_F437_11D3_BD83_00C04F89E392__INCLUDED_)
#define AFX_GRAPHTYPES_H__24021AB3_F437_11D3_BD83_00C04F89E392__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "NTPerfGraphs.h"
#define WMU_DATA_CHANGED (WM_APP+1)

#define GRAPH_MENU_ITEM_ID_START 0x8000
#define GRAPH_MENU_ITEM_ID_END 0x8005

class CXChartCtrl;
//CGraphTypeBase is the base class for all the graph types and
//controls the basic style of the graph,eg background , title
class CGraphTypeBase
{
protected:
    CGraphTypeBase(CXChartCtrl *pChart);

public:
    virtual ~CGraphTypeBase();
    virtual void SetGraphType(int nType);
    virtual int GetGraphType();

    //Called to add a legend to the graph display
    virtual void DisplayLegend();

    virtual BOOL RemoveTestRun(int nRun);

    BOOL ExportToExcel();

    struct MenuItems
    {
        CString strItem;
        UINT uiCommandID;
    };
    typedef CArray<MenuItems,MenuItems&> MenuItemArray;

    virtual void GetMenuItems(MenuItemArray& arMenuItems);
    virtual void OnCommand(UINT uiCommandID,CWnd *pParent);

	virtual BOOL DoesGraphHaveFilter(){return m_bHasFilter;}
	virtual void ShowFilter(){}
	CString GetElapsedTimeString(DWORD dwElapsedTime,DWORD dwTotalDuration);

    //Add all the data from another graph to this one
//    void AddDataFromGraph(CGraphTypeBase *m_pView);

protected:
    int m_nNextGroup;

	CXChartCtrl* m_pChart;
    CString m_strTitle;
	BOOL m_bHasFilter;
};

//CNTPGraph is the base class of all the graphs which use OTA data
class CNTPGraph : public CGraphTypeBase
{
public:
    CNTPGraph(CXChartCtrl *pChart);
    virtual ~CNTPGraph();

	BOOL AddTestRun(NTPHeaderList* pHeader,DWORD dwTestStartTime);
    
    virtual void GetMenuItems(MenuItemArray& arMenuItems);
    virtual void OnCommand(UINT uiCommandID,CWnd *pParent);
	virtual void ShowFilter();

protected:
    
    typedef enum{enMenuItemFilter=GRAPH_MENU_ITEM_ID_START}MenuItemIDs;

	NTPHeaderList *m_pListHeaders;
	DWORD m_dwTestStartTime;
};

#endif // !defined(AFX_GRAPHTYPES_H__24021AB3_F437_11D3_BD83_00C04F89E392__INCLUDED_)


